@@ -16,20 +16,28 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""convert .ini files to Gettext PO localization files"""
+"""Convert .ini files to Gettext PO localization files.
+
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/ini2po.html
+for examples and usage instructions.
+"""
 
 import sys
+import logging
+
 from translate.storage import po
 
+logger = logging.getLogger(__name__)
+
 class ini2po:
     """convert a .ini file to a .po file for handling the translation..."""
+
     def convert_store(self, input_store, duplicatestyle="msgctxt"):
         """converts a .ini file to a .po file..."""
         output_store = po.pofile()
-        output_header = output_store.init_headers(charset="UTF-8", encoding="8bit")
+        output_header = output_store.header()
         output_header.addnote("extracted from %s" % input_store.filename, "developer")
 
         for input_unit in input_store.units:
@@ -42,7 +50,7 @@ class ini2po:
     def merge_store(self, template_store, input_store, blankmsgstr=False, duplicatestyle="msgctxt"):
         """converts two .ini files to a .po file..."""
         output_store = po.pofile()
-        output_header = output_store.init_headers(charset="UTF-8", encoding="8bit")
+        output_header = output_store.header()
         output_header.addnote("extracted from %s, %s" % (template_store.filename, input_store.filename), "developer")
 
         input_store.makeindex()
@@ -61,7 +69,8 @@ class ini2po:
                     origpo.target = translatedpo.source
                 output_store.addunit(origpo)
             elif translatedpo is not None:
-                print >> sys.stderr, "error converting original ini definition %s" % origpo.name
+                logger.error("error converting original ini definition %s",
+                             origpo.name)
         output_store.removeduplicates(duplicatestyle)
         return output_store
 
@@ -77,8 +86,10 @@ class ini2po:
         output_unit.target = ""
         return output_unit
 
+
 def convertini(input_file, output_file, template_file, pot=False, duplicatestyle="msgctxt", dialect="default"):
-    """Reads in L{input_file} using ini, converts using L{ini2po}, writes to L{output_file}"""
+    """Reads in *input_file* using ini, converts using :class:`ini2po`,
+    writes to *output_file*."""
     from translate.storage import ini
     input_store = ini.inifile(input_file, dialect=dialect)
     convertor = ini2po()
@@ -92,9 +103,11 @@ def convertini(input_file, output_file, template_file, pot=False, duplicatestyle
     output_file.write(str(output_store))
     return 1
 
+
 def convertisl(input_file, output_file, template_file, pot=False, duplicatestyle="msgctxt", dialect="inno"):
     return convertini(input_file, output_file, template_file, pot=False, duplicatestyle="msgctxt", dialect=dialect)
 
+
 def main(argv=None):
     from translate.convert import convert
     formats = {
@@ -107,6 +120,6 @@ def main(argv=None):
     parser.passthrough.append("pot")
     parser.run(argv)
 
+
 if __name__ == '__main__':
     main()
-
