@@ -16,20 +16,22 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 
-"""convert OpenDocument (ODF) files to XLIFF localization files"""
+"""Convert OpenDocument (ODF) files to XLIFF localization files.
+
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/odf2xliff.html
+for examples and usage instructions.
+"""
 
-# Import from ttk
 from translate.storage import factory
-
 from translate.misc.contextlib import contextmanager
 from translate.misc.context import with_
 from translate.storage import odf_io
 
-def convertodf(inputfile, outputfile, templates, engine):
+
+def convertodf(inputfile, outputfile, templates, engine='toolkit'):
     """reads in stdin using fromfileclass, converts using convertorclass,
        writes to stdout
     """
@@ -43,10 +45,10 @@ def convertodf(inputfile, outputfile, templates, engine):
 
         contents = odf_io.open_odf(inputfile)
         for data in contents.values():
-            parse_state = extract.ParseState(odf_shared.no_translate_content_elements, 
+            parse_state = extract.ParseState(odf_shared.no_translate_content_elements,
                                              odf_shared.inline_elements)
             extract.build_store(cStringIO.StringIO(data), store, parse_state)
-    
+
     def itools_implementation(store):
         from itools.handlers import get_handler
         from itools.gettext.po import encode_source
@@ -58,8 +60,7 @@ def convertodf(inputfile, outputfile, templates, engine):
         try:
             get_units = handler.get_units
         except AttributeError:
-            message = 'error: the file "%s" could not be processed'
-            raise AttributeError, message % filename
+            raise AttributeError('error: the file "%s" could not be processed' % filename)
 
         # Make the XLIFF file
         for source, context, line in get_units():
@@ -92,7 +93,30 @@ def convertodf(inputfile, outputfile, templates, engine):
     return True
 
 
+# For formats see OpenDocument 1.2 draft 7 Appendix C
+formats = {
+    "sxw": ("xlf", convertodf),
+    "odt": ("xlf", convertodf),  # Text
+    "ods": ("xlf", convertodf),  # Spreadsheet
+    "odp": ("xlf", convertodf),  # Presentation
+    "odg": ("xlf", convertodf),  # Drawing
+    "odc": ("xlf", convertodf),  # Chart
+    "odf": ("xlf", convertodf),  # Formula
+    "odi": ("xlf", convertodf),  # Image
+    "odm": ("xlf", convertodf),  # Master Document
+    "ott": ("xlf", convertodf),  # Text template
+    "ots": ("xlf", convertodf),  # Spreadsheet template
+    "otp": ("xlf", convertodf),  # Presentation template
+    "otg": ("xlf", convertodf),  # Drawing template
+    "otc": ("xlf", convertodf),  # Chart template
+    "otf": ("xlf", convertodf),  # Formula template
+    "oti": ("xlf", convertodf),  # Image template
+    "oth": ("xlf", convertodf),  # Web page template
+}
+
+
 def main(argv=None):
+
     def add_options(parser):
         parser.add_option("", "--engine", dest="engine", default="toolkit",
                           type="choice", choices=["toolkit", "itools"],
@@ -100,30 +124,12 @@ def main(argv=None):
                           should be used as the engine to convert an ODF file to an XLIFF file.""")
         parser.passthrough = ['engine']
         return parser
-    
+
     from translate.convert import convert
-    # For formats see OpenDocument 1.2 draft 7 Appendix C
-    formats = {"sxw": ("xlf", convertodf),
-               "odt": ("xlf", convertodf), # Text
-               "ods": ("xlf", convertodf), # Spreadsheet
-               "odp": ("xlf", convertodf), # Presentation
-               "odg": ("xlf", convertodf), # Drawing
-               "odc": ("xlf", convertodf), # Chart
-               "odf": ("xlf", convertodf), # Formula
-               "odi": ("xlf", convertodf), # Image
-               "odm": ("xlf", convertodf), # Master Document
-               "ott": ("xlf", convertodf), # Text template
-               "ots": ("xlf", convertodf), # Spreadsheet template
-               "otp": ("xlf", convertodf), # Presentation template
-               "otg": ("xlf", convertodf), # Drawing template
-               "otc": ("xlf", convertodf), # Chart template
-               "otf": ("xlf", convertodf), # Formula template
-               "oti": ("xlf", convertodf), # Image template
-               "oth": ("xlf", convertodf), # Web page template
-              }
     parser = convert.ConvertOptionParser(formats, description=__doc__)
     add_options(parser)
     parser.run(argv)
 
+
 if __name__ == '__main__':
     main()
