@@ -2,34 +2,40 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright 2008 Mozilla Corporation, Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-""" Convert .po files to TikiWiki's language.php files. """
+"""Convert Gettext PO files to TikiWiki's language.php files.
+
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/tiki2po.html
+for examples and usage instructions.
+"""
 
 import sys
+
 from translate.storage import tiki
 from translate.storage import po
 
+
 class po2tiki:
+
     def convertstore(self, thepofile):
         """Converts a given (parsed) po file to a tiki file.
 
-        @param thepofile: a pofile pre-loaded with input data
+        :param thepofile: a pofile pre-loaded with input data
         """
         thetargetfile = tiki.TikiStore()
         for unit in thepofile.units:
@@ -49,12 +55,13 @@ class po2tiki:
                 thetargetfile.addunit(newunit)
         return thetargetfile
 
+
 def convertpo(inputfile, outputfile, template=None):
     """Converts from po file format to tiki.
 
-    @param inputfile: file handle of the source
-    @param outputfile: file handle to write to
-    @param template: unused
+    :param inputfile: file handle of the source
+    :param outputfile: file handle to write to
+    :param template: unused
     """
     inputstore = po.pofile(inputfile)
     if inputstore.isempty():
@@ -64,16 +71,18 @@ def convertpo(inputfile, outputfile, template=None):
     outputfile.write(str(outputstore))
     return True
 
+
 def main(argv=None):
     """Will convert from .po to tiki style .php"""
     from translate.convert import convert
     from translate.misc import stdiotell
     sys.stdout = stdiotell.StdIOWrapper(sys.stdout)
 
-    formats = {"po":("tiki", convertpo)}
+    formats = {"po": ("tiki", convertpo)}
 
     parser = convert.ConvertOptionParser(formats, description=__doc__)
     parser.run(argv)
 
+
 if __name__ == '__main__':
     main()
