@@ -2,38 +2,40 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2006 Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""convert Gettext PO localization files to plain text (.txt) files
+"""Convert Gettext PO localization files to plain text (.txt) files.
 
-see: http://translate.sourceforge.net/wiki/toolkit/po2txt for examples and 
-usage instructions
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/txt2po.html
+for examples and usage instructions.
 """
 
-from translate.storage import factory
 try:
     import textwrap
 except ImportError:
     textwrap = None
 
+from translate.convert import convert
+from translate.storage import factory
+
+
 class po2txt:
     """po2txt can take a po file and generate txt. best to give it a template file otherwise will just concat msgstrs"""
+
     def __init__(self, wrap=None):
         self.wrap = wrap
 
@@ -54,7 +56,7 @@ class po2txt:
             else:
                 txtresult += self.wrapmessage(unit.source) + "\n" + "\n"
         return txtresult.rstrip()
- 
+
     def mergestore(self, inputstore, templatetext, includefuzzy):
         """converts a file to txt format"""
         txtresult = templatetext
@@ -70,9 +72,15 @@ class po2txt:
                     txtresult = txtresult.replace(txtsource, txttarget)
         return txtresult
 
-def converttxt(inputfile, outputfile, templatefile, wrap=None, includefuzzy=False, encoding='utf-8'):
+
+def converttxt(inputfile, outputfile, templatefile, wrap=None, includefuzzy=False, encoding='utf-8',
+               outputthreshold=None):
     """reads in stdin using fromfileclass, converts using convertorclass, writes to stdout"""
     inputstore = factory.getobject(inputfile)
+
+    if not convert.should_output_store(inputstore, outputthreshold):
+        return False
+
     convertor = po2txt(wrap=wrap)
     if templatefile is None:
         outputstring = convertor.convertstore(inputstore, includefuzzy)
@@ -82,12 +90,12 @@ def converttxt(inputfile, outputfile, templatefile, wrap=None, includefuzzy=Fals
     outputfile.write(outputstring.encode('utf-8'))
     return 1
 
+
 def main(argv=None):
-    from translate.convert import convert
     from translate.misc import stdiotell
     import sys
     sys.stdout = stdiotell.StdIOWrapper(sys.stdout)
-    formats = {("po", "txt"):("txt", converttxt), ("po"):("txt", converttxt), ("xlf", "txt"):("txt", converttxt), ("xlf"):("txt", converttxt)}
+    formats = {("po", "txt"): ("txt", converttxt), ("po"): ("txt", converttxt), ("xlf", "txt"): ("txt", converttxt), ("xlf"): ("txt", converttxt)}
     parser = convert.ConvertOptionParser(formats, usetemplates=True, description=__doc__)
     parser.add_option("", "--encoding", dest="encoding", default='utf-8', type="string",
             help="The encoding of the template file (default: UTF-8)")
@@ -96,6 +104,7 @@ def main(argv=None):
         parser.add_option("-w", "--wrap", dest="wrap", default=None, type="int",
                 help="set number of columns to wrap text at", metavar="WRAP")
         parser.passthrough.append("wrap")
+    parser.add_threshold_option()
     parser.add_fuzzy_option()
     parser.run(argv)
 
