@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2009 Zuza Software Foundation
+# Copyright 2009-2010 Zuza Software Foundation
 #
 # This file is part of translate.
 #
@@ -19,20 +19,25 @@
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 # (c) 2009 Dominic König (dominic@nursix.org)
-#
 
-"""convert GNU/gettext PO files to web2py translation dictionaries (.py)"""
+"""Convert GNU/gettext PO files to web2py translation dictionaries (.py).
+
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/web2py2po.html
+for examples and usage instructions.
+"""
 
 from translate.storage import factory
 
+
 class po2pydict:
+
     def __init__(self):
         return
 
     def convertstore(self, inputstore, includefuzzy):
         from StringIO import StringIO
         str_obj = StringIO()
-        
+
         mydict = dict()
         for unit in inputstore.units:
             if unit.isheader():
@@ -40,8 +45,10 @@ class po2pydict:
             if unit.istranslated() or (includefuzzy and unit.isfuzzy()):
                 mydict[unit.source] = unit.target
             else:
-                mydict[unit.source] = '*** ' + unit.source
-        
+                mydict[unit.source] = unit.source
+                # The older convention is to prefix with "*** ":
+                #mydict[unit.source] = '*** ' + unit.source
+
         str_obj.write('{\n')
         for source_str in mydict:
             str_obj.write("%s:%s,\n" % (repr(str(source_str)), repr(str(mydict[source_str]))))
@@ -49,22 +56,30 @@ class po2pydict:
         str_obj.seek(0)
         return str_obj
 
-def convertpy(inputfile, outputfile, templatefile=None, includefuzzy=False):
+
+def convertpy(inputfile, outputfile, templatefile=None, includefuzzy=False,
+              outputthreshold=None):
     inputstore = factory.getobject(inputfile)
+
+    if not convert.should_output_store(inputstore, outputthreshold):
+        return False
+
     convertor = po2pydict()
     outputstring = convertor.convertstore(inputstore, includefuzzy)
     outputfile.write(outputstring.read())
     return 1
 
+
 def main(argv=None):
-    from translate.convert import convert
     from translate.misc import stdiotell
     import sys
     sys.stdout = stdiotell.StdIOWrapper(sys.stdout)
-    formats = {("po", "py"):("py", convertpy), ("po"):("py", convertpy)}
+    formats = {("po", "py"): ("py", convertpy), ("po"): ("py", convertpy)}
     parser = convert.ConvertOptionParser(formats, usetemplates=False, description=__doc__)
+    parser.add_threshold_option()
     parser.add_fuzzy_option()
     parser.run(argv)
 
+
 if __name__ == '__main__':
     main()
