@@ -18,19 +18,30 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""convert .rc files to Gettext PO localization files"""
+"""Convert Windows RC files to Gettext PO localization files.
+
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/rc2po.html
+for examples and usage instructions.
+"""
 
 import sys
+import logging
+
 from translate.storage import po
 from translate.storage import rc
 
+logger = logging.getLogger(__name__)
+
 class rc2po:
     """Convert a .rc file to a .po file for handling the translation."""
 
     def convert_store(self, input_store, duplicatestyle="msgctxt"):
         """converts a .rc file to a .po file..."""
         output_store = po.pofile()
-        output_header = output_store.init_headers(charset="UTF-8", encoding="8bit")
+        output_header = output_store.init_headers(
+                x_accelerator_marker="&",
+                x_merge_on="location",
+        )
         output_header.addnote("extracted from %s" % input_store.filename, "developer")
         for input_unit in input_store.units:
             output_unit = self.convert_unit(input_unit, "developer")
@@ -42,7 +53,10 @@ class rc2po:
     def merge_store(self, template_store, input_store, blankmsgstr=False, duplicatestyle="msgctxt"):
         """converts two .rc files to a .po file..."""
         output_store = po.pofile()
-        output_header = output_store.init_headers(charset="UTF-8", encoding="8bit")
+        output_header = output_store.init_headers(
+                x_accelerator_marker="&",
+                x_merge_on="location",
+        )
         output_header.addnote("extracted from %s, %s" % (template_store.filename, input_store.filename), "developer")
         input_store.makeindex()
         for template_unit in template_store.units:
@@ -60,7 +74,8 @@ class rc2po:
                     origpo.target = translatedpo.source
                 output_store.addunit(origpo)
             elif translatedpo is not None:
-                print >> sys.stderr, "error converting original rc definition %s" % template_unit.name
+                logging.error("error converting original rc definition %s",
+                              template_unit.name)
         output_store.removeduplicates(duplicatestyle)
         return output_store
 
@@ -76,6 +91,7 @@ class rc2po:
         output_unit.target = ""
         return output_unit
 
+
 def convertrc(input_file, output_file, template_file, pot=False, duplicatestyle="msgctxt", charset=None, lang=None, sublang=None):
     """reads in input_file using rc, converts using rc2po, writes to output_file"""
     input_store = rc.rcfile(input_file, lang, sublang, encoding=charset)
@@ -90,6 +106,7 @@ def convertrc(input_file, output_file, template_file, pot=False, duplicatestyle=
     output_file.write(str(output_store))
     return 1
 
+
 def main(argv=None):
     from translate.convert import convert
     formats = {"rc": ("po", convertrc), ("rc", "rc"): ("po", convertrc),
@@ -111,5 +128,6 @@ def main(argv=None):
     parser.passthrough.append("sublang")
     parser.run(argv)
 
+
 if __name__ == '__main__':
     main()
