@@ -16,19 +16,27 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""convert subtitle files to Gettext PO localization files"""
+"""Convert subtitle files to Gettext PO localization files.
+
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/sub2po.html
+for examples and usage instructions.
+"""
 
 import sys
+import logging
+
 from translate.storage import po
 
+logger = logging.getLogger(__name__)
+
 def convert_store(input_store, duplicatestyle="msgctxt"):
     """converts a subtitle file to a .po file..."""
     output_store = po.pofile()
-    output_header = output_store.init_headers(charset="UTF-8", encoding="8bit")
-    output_header.addnote("extracted from %s" % input_store.filename, "developer")
+    output_header = output_store.header()
+    output_header.addnote("extracted from %s" % input_store.filename,
+                          "developer")
 
     for input_unit in input_store.units:
         output_unit = convert_unit(input_unit, "developer")
@@ -37,11 +45,15 @@ def convert_store(input_store, duplicatestyle="msgctxt"):
     output_store.removeduplicates(duplicatestyle)
     return output_store
 
-def merge_store(template_store, input_store, blankmsgstr=False, duplicatestyle="msgctxt"):
+
+def merge_store(template_store, input_store, blankmsgstr=False,
+                duplicatestyle="msgctxt"):
     """converts two subtitle files to a .po file..."""
     output_store = po.pofile()
-    output_header = output_store.init_headers(charset="UTF-8", encoding="8bit")
-    output_header.addnote("extracted from %s, %s" % (template_store.filename, input_store.filename), "developer")
+    output_header = output_store.headers()
+    output_header.addnote("extracted from %s, %s" % \
+                          (template_store.filename, input_store.filename),
+                          "developer")
 
     input_store.makeindex()
     for template_unit in template_store.units:
@@ -59,10 +71,12 @@ def merge_store(template_store, input_store, blankmsgstr=False, duplicatestyle="
                 origpo.target = translatedpo.source
             output_store.addunit(origpo)
         elif translatedpo is not None:
-            print >> sys.stderr, "error converting original subtitle definition %s" % origini.name
+            logger.error("error converting original subtitle definition %s",
+                         origini.name)
     output_store.removeduplicates(duplicatestyle)
     return output_store
 
+
 def convert_unit(input_unit, commenttype):
     """Converts a subtitle unit to a .po unit. Returns None if empty
     or not for translation."""
@@ -76,27 +90,41 @@ def convert_unit(input_unit, commenttype):
     output_unit.target = ""
     return output_unit
 
-def convertsub(input_file, output_file, template_file=None, pot=False, duplicatestyle="msgctxt"):
-    """Reads in L{input_file} using translate.subtitles, converts using L{sub2po}, writes to L{output_file}"""
+
+def convertsub(input_file, output_file, template_file=None, pot=False,
+               duplicatestyle="msgctxt"):
+    """Reads in *input_file* using translate.subtitles, converts using
+    :class:`sub2po`, writes to *output_file*."""
     from translate.storage import subtitles
     input_store = subtitles.SubtitleFile(input_file)
     if template_file is None:
-        output_store = convert_store(input_store, duplicatestyle=duplicatestyle)
+        output_store = convert_store(input_store,
+                                     duplicatestyle=duplicatestyle)
     else:
         template_store = subtitles.SubtitleFile(template_file)
-        output_store = merge_store(template_store, input_store, blankmsgstr=pot, duplicatestyle=duplicatestyle)
+        output_store = merge_store(template_store, input_store,
+                                   blankmsgstr=pot,
+                                   duplicatestyle=duplicatestyle)
     if output_store.isempty():
         return 0
     output_file.write(str(output_store))
     return 1
 
+
 def main(argv=None):
     from translate.convert import convert
-    formats = {"srt": ("po", convertsub), ("srt", "srt"): ("po", convertsub)}
-    parser = convert.ConvertOptionParser(formats, usetemplates=True, usepots=True, description=__doc__)
+    formats = {
+          "srt": ("po", convertsub), ("srt", "srt"): ("po", convertsub),
+          "sub": ("po", convertsub), ("sub", "sub"): ("po", convertsub),
+          "ssa": ("po", convertsub), ("ssa", "ssa"): ("po", convertsub),
+          "ass": ("po", convertsub), ("ass", "ass"): ("po", convertsub),
+    }
+    parser = convert.ConvertOptionParser(formats, usetemplates=True,
+                                         usepots=True, description=__doc__)
     parser.add_duplicates_option()
     parser.passthrough.append("pot")
     parser.run(argv)
 
+
 if __name__ == '__main__':
     main()
