@@ -1,21 +1,18 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from translate.convert import convert
 import os
 import sys
-from py import test
-try:
-    import psyco
-except Exception:
-    psyco = None
+
+import pytest
+
+from translate.convert import convert
+
 
 class TestConvertCommand:
     """Tests running actual commands on files"""
     convertmodule = convert
     defaultoptions = {"progress": "none"}
-    if psyco:
-        defaultoptions["psyco"] = "none"
 
     def setup_method(self, method):
         """creates a clean test directory for the given method"""
@@ -37,7 +34,8 @@ class TestConvertCommand:
                     os.remove(os.path.join(dirpath, name))
                 for name in subdirs:
                     os.rmdir(os.path.join(dirpath, name))
-        if os.path.exists(self.testdir): os.rmdir(self.testdir)
+        if os.path.exists(self.testdir):
+            os.rmdir(self.testdir)
         assert not os.path.exists(self.testdir)
 
     def run_command(self, *argv, **kwargs):
@@ -88,7 +86,7 @@ class TestConvertCommand:
 
     def help_check(self, options, option, last=False):
         """check that a help string occurs and remove it"""
-        assert option in options 
+        assert option in options
         newoptions = []
         for line in options.splitlines():
             if option in line or not line.lstrip().startswith("-"):
@@ -104,7 +102,7 @@ class TestConvertCommand:
         helpfile = self.open_testfile("help.txt", "w")
         sys.stdout = helpfile
         try:
-            test.raises(SystemExit, self.run_command, help=True)
+            pytest.raises(SystemExit, self.run_command, help=True)
         finally:
             sys.stdout = stdout
         helpfile.close()
@@ -114,8 +112,8 @@ class TestConvertCommand:
         # the convertsummary might be wrapped. this will probably unwrap it
         assert convertsummary in help_string.replace("\n", " ")
         usageline = help_string[:help_string.find("\n")]
-        # Different versions of optparse might contain either upper or 
-        # lowercase versions of 'Usage:' and 'Options:', so we need to take 
+        # Different versions of optparse might contain either upper or
+        # lowercase versions of 'Usage:' and 'Options:', so we need to take
         # that into account
         assert (usageline.startswith("Usage: ") or usageline.startswith("usage: ")) \
             and "[--version] [-h|--help]" in usageline
@@ -126,10 +124,8 @@ class TestConvertCommand:
         options = self.help_check(options, "-h, --help")
         options = self.help_check(options, "--manpage")
         options = self.help_check(options, "--errorlevel=ERRORLEVEL")
-        if psyco:
-            options = self.help_check(options, "--psyco=MODE")
         options = self.help_check(options, "-i INPUT, --input=INPUT")
         options = self.help_check(options, "-x EXCLUDE, --exclude=EXCLUDE")
         options = self.help_check(options, "-o OUTPUT, --output=OUTPUT")
+        options = self.help_check(options, "-S, --timestamp")
         return options
-
