@@ -1,13 +1,17 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+from pytest import mark
+
 from translate.convert import dtd2po
 from translate.convert import test_convert
 from translate.misc import wStringIO
 from translate.storage import po
 from translate.storage import dtd
 
+
 class TestDTD2PO:
+
     def dtd2po(self, dtdsource, dtdtemplate=None):
         """helper that converts dtd source to po source without requiring files"""
         inputfile = wStringIO.StringIO(dtdsource)
@@ -90,6 +94,13 @@ class TestDTD2PO:
         assert "credit.translation" in str(pounit)
         assert 'msgctxt "credit.translation"' in str(pounit)
 
+    def test_two_empty_entities(self):
+        """checks that two empty entitu definitions have correct context (bug 2190)."""
+        dtdsource = '<!ENTITY community.exp.start "">\n<!ENTITY contribute.end "">\n'
+        pofile = self.dtd2po(dtdsource)
+        assert pofile.units[-2].getcontext() == u"community.exp.start"
+        assert pofile.units[-1].getcontext() == u"contribute.end"
+
     def test_emptyentity_translated(self):
         """checks that if we translate an empty entity it makes it into the PO, bug 101"""
         dtdtemplate = '<!ENTITY credit.translation "">\n'
@@ -114,7 +125,7 @@ class TestDTD2PO:
         pofile = self.dtd2po(dtdsource)
         posource = str(pofile)
         print posource
-        assert posource.count('#.') == 5 # 1 Header extracted from, 3 comment lines, 1 autoinserted comment
+        assert posource.count('#.') == 5  # 1 Header extracted from, 3 comment lines, 1 autoinserted comment
 
     def test_localisation_note_merge(self):
         """test that LOCALIZATION NOTES are added properly as #. comments and disambiguated with msgctxt entries"""
@@ -283,7 +294,7 @@ Some other text
         assert pofile.units[2].target == "M"
 
     def test_changed_labels_and_accelerators(self):
-        """test to ensure that when the template changes an entity name we can still manage the accelerators""" 
+        """test to ensure that when the template changes an entity name we can still manage the accelerators"""
         dtdtemplate = '''<!ENTITY  managecerts.caption      "Manage Certificates">
 <!ENTITY  managecerts.text         "Use the Certificate Manager to manage your personal certificates, as well as those of other people and certificate authorities.">
 <!ENTITY  managecerts.button       "Manage Certificates...">
@@ -299,7 +310,8 @@ Some other text
         assert pofile.units[4].source == "M"
         assert pofile.units[4].target == u"ﺩ"
 
-    def wtest_accelerator_keys_not_in_sentence(self):
+    @mark.xfail(reason="Not Implemented")
+    def test_accelerator_keys_not_in_sentence(self):
         """tests to ensure that we can manage accelerator keys that are not part of the transated sentence eg in Chinese"""
         dtdtemplate = '''<!ENTITY useAutoScroll.label             "Use autoscrolling">
 <!ENTITY useAutoScroll.accesskey         "a">'''
@@ -352,6 +364,7 @@ Some other text
         print pofile
         assert self.countelements(pofile) == 0
 
+
 class TestDTD2POCommand(test_convert.TestConvertCommand, TestDTD2PO):
     """Tests running actual dtd2po commands on files"""
     convertmodule = dtd2po
