@@ -1,16 +1,20 @@
 #!/usr/bin/env python
 
+from pytest import mark
+
 from translate.convert import html2po
 from translate.convert import po2html
 from translate.convert import test_convert
 from translate.misc import wStringIO
 
+
 class TestHTML2PO:
+
     def html2po(self, markup, includeuntagged=False, duplicatestyle="msgctxt", keepcomments=False):
         """Helper to convert html to po without a file."""
         inputfile = wStringIO.StringIO(markup)
         convertor = html2po.html2po()
-        outputpo = convertor.convertfile(inputfile, "test", False, includeuntagged, duplicatestyle, keepcomments)
+        outputpo = convertor.convertfile(inputfile, "test", includeuntagged, duplicatestyle, keepcomments)
         return outputpo
 
     def po2html(self, posource, htmltemplate):
@@ -52,9 +56,9 @@ class TestHTML2PO:
     def check_phpsnippet(self, php):
         """Given a snippet of php, put it into an HTML shell and see
         if the results are as expected"""
-        self.check_single('<html><head></head><body><p><a href="'+php+'/site.html">Body text</a></p></body></html>', "Body text")
-        self.check_single('<html><head></head><body><p>More things in <a href="'+php+'/site.html">Body text</a></p></body></html>', 'More things in <a href="'+php+'/site.html">Body text</a>')
-        self.check_null('<html><head></head><body><p>'+php+'</p></body></html>')
+        self.check_single('<html><head></head><body><p><a href="' + php + '/site.html">Body text</a></p></body></html>', "Body text")
+        self.check_single('<html><head></head><body><p>More things in <a href="' + php + '/site.html">Body text</a></p></body></html>', 'More things in <a href="' + php + '/site.html">Body text</a>')
+        self.check_null('<html><head></head><body><p>' + php + '</p></body></html>')
 
     def test_htmllang(self):
         """test to ensure that we no longer use the lang attribure"""
@@ -171,7 +175,7 @@ newlines.</p></body></html>
 
     def test_table_simple(self):
         """Test that we can fully extract a simple table."""
-        markup = '''<html><head></head><body><table><tr><th>Heading One</th><th>Heading Two</th><tr><td>One</td><td>Two</td></tr></table></body></html>'''
+        markup = '''<html><head></head><body><table><tr><th>Heading One</th><th>Heading Two</th></tr><tr><td>One</td><td>Two</td></tr></table></body></html>'''
         pofile = self.html2po(markup)
         self.countunits(pofile, 4)
         self.compareunit(pofile, 1, "Heading One")
@@ -180,7 +184,7 @@ newlines.</p></body></html>
         self.compareunit(pofile, 4, "Two")
 
     def test_table_complex(self):
-        markup = '''<table summary="This is the summary"><caption>A caption</caption><thead><tr><th abbr="Head 1">Heading One</th><th>Heading Two</th></thead><tfoot><tr><td>Foot One</td><td>Foot Two</td></tr></tfoot><tbody><tr><td>One</td><td>Two</td></tr></tbody></table>'''
+        markup = '''<table summary="This is the summary"><caption>A caption</caption><thead><tr><th abbr="Head 1">Heading One</th><th>Heading Two</th></tr></thead><tfoot><tr><td>Foot One</td><td>Foot Two</td></tr></tfoot><tbody><tr><td>One</td><td>Two</td></tr></tbody></table>'''
         pofile = self.html2po(markup)
         self.countunits(pofile, 9)
         self.compareunit(pofile, 1, "This is the summary")
@@ -195,7 +199,7 @@ newlines.</p></body></html>
 
     def test_table_empty(self):
         """Test that we ignore tables that are empty.
-        
+
         A table is deemed empty if it has no translatable content.
         """
 
@@ -262,12 +266,13 @@ newlines.</p></body></html>
         self.compareunit(pofile, 1, "Duplicate")
         self.compareunit(pofile, 2, "Duplicate")
 
-    def wtest_multiline_reflow(self):
+    def test_multiline_reflow(self):
         """check that we reflow multiline content to make it more readable for translators"""
         self.check_single('''<td valign="middle" width="96%"><font class="headingwhite">South
-                  Africa</font></td>''', '''<font class="headingwhite">South Africa</font>''')
+                  Africa</font></td>''', '''South Africa''')
 
-    def wtest_nested_tags(self):
+    @mark.xfail(reason="Not Implemented")
+    def test_nested_tags(self):
         """check that we can extract items within nested tags"""
         markup = "<div><p>Extract this</p>And this</div>"
         pofile = self.html2po(markup)
@@ -332,23 +337,25 @@ years has helped to bridge the digital divide to a limited extent.</p> \r
 <body>
 <table width="100%"  border="0" cellpadding="0" cellspacing="0">
   <tr align="left" valign="top">
-    <td width="150" height="556"> 
+    <td width="150" height="556">
       <table width="157" height="100%" border="0" cellspacing="0" id="leftmenubg-color">
       <tr>
-          <td align="left" valign="top" height="555"> 
+          <td align="left" valign="top" height="555">
             <table width="100%" border="0" cellspacing="0" cellpadding="2">
-              <tr align="left" valign="top" bgcolor="#660000"> 
+              <tr align="left" valign="top" bgcolor="#660000">
                 <td width="4%"><strong></strong></td>
                 <td width="96%"><strong><font class="headingwhite">Projects</font></strong></td>
               </tr>
-              <tr align="left" valign="top"> 
+              <tr align="left" valign="top">
                 <td valign="middle" width="4%"><img src="images/arrow.gif" width="8" height="8"></td>
                 <td width="96%"><a href="index.html">Home Page</a></td>
               </tr>
             </table>
           </td>
       </tr>
-    </table></td>
+      </table>
+    </td>
+  </tr>
 </table>
 </body>
 </html>
@@ -367,10 +374,11 @@ years has helped to bridge the digital divide to a limited extent.</p> \r
         snippet = '<td width="96%"><a href="index.html">Tuisblad</a></td>'
         assert snippet in htmlresult
 
+    @mark.xfail(reason="Performing major HTML surgery")
     def test_php(self):
         """Test that PHP snippets don't interfere"""
 
-        # A simple string 
+        # A simple string
         self.check_phpsnippet('''<?=$phpvariable?>''')
 
         # Contains HTML tag charcters (< and >)
@@ -381,13 +389,13 @@ years has helped to bridge the digital divide to a limited extent.</p> \r
 
     def test_multiple_php(self):
         """Test multiple PHP snippets in a string to make sure they get restored properly"""
-        php1 = '''<?=$phpvariable?>''' 
+        php1 = '''<?=$phpvariable?>'''
         php2 = '''<?=($a < $b ? $foo : ($b > c ? $bar : $cat))?>'''
         php3 = '''<? asdfghjklqwertyuiop1234567890!@#$%^&*()-=_+[]\{}|;':",./<>? ?>'''
 
         # Put 3 different strings into an html string
-        innertext = '<a href="'+php1+'/site.html">Body text</a> and some '+php2+' more text '+php2+php3
-        htmlsource = '<html><head></head><body><p>'+innertext+'</p></body></html>'
+        innertext = '<a href="' + php1 + '/site.html">Body text</a> and some ' + php2 + ' more text ' + php2 + php3
+        htmlsource = '<html><head></head><body><p>' + innertext + '</p></body></html>'
         self.check_single(htmlsource, innertext)
 
     def test_php_multiline(self):
@@ -398,14 +406,14 @@ def
 ghi ?>'''
 
         # Scatter the php strings throughout the file, and show what the translation should be
-        innertext = '<a href="'+php1+'/site.html">Body text</a> and some '+php1+' more text '+php1+php1
-        innertrans = '<a href="'+php1+'/site.html">Texte de corps</a> et encore de '+php1+' plus de texte '+php1+php1
+        innertext = '<a href="' + php1 + '/site.html">Body text</a> and some ' + php1 + ' more text ' + php1 + php1
+        innertrans = '<a href="' + php1 + '/site.html">Texte de corps</a> et encore de ' + php1 + ' plus de texte ' + php1 + php1
 
-        htmlsource = '<html><head></head><body><p>'+innertext+'</p></body></html>' # Current html file
-        transsource = '<html><head></head><body><p>'+innertrans+'</p></body></html>' # Expected translation
+        htmlsource = '<html><head></head><body><p>' + innertext + '</p></body></html>'  # Current html file
+        transsource = '<html><head></head><body><p>' + innertrans + '</p></body></html>'  # Expected translation
 
         pofile = self.html2po(htmlsource)
-        pofile.units[1].target = innertrans # Register the translation in the PO file
+        pofile.units[1].target = innertrans  # Register the translation in the PO file
         htmlresult = self.po2html(pofile, htmlsource)
         assert htmlresult == transsource
 
@@ -413,9 +421,10 @@ ghi ?>'''
         """Test that HTML comments are converted to translator notes in output"""
         pofile = self.html2po('<!-- comment outside block --><p><!-- a comment -->A paragraph<!-- with another comment -->.</p>', keepcomments=True)
         self.compareunit(pofile, 1, 'A paragraph.')
-        notes = pofile.getunits()[0].getnotes()
+        notes = pofile.getunits()[-1].getnotes()
         assert unicode(notes) == ' a comment \n with another comment '
 
+
 class TestHTML2POCommand(test_convert.TestConvertCommand, TestHTML2PO):
     """Tests running actual html2po commands on files"""
     convertmodule = html2po
