@@ -1,10 +1,14 @@
 #!/usr/bin/env python
 
+from pytest import mark
+
 from translate.convert import po2html
 from translate.convert import test_convert
 from translate.misc import wStringIO
 
+
 class TestPO2Html:
+
     def converthtml(self, posource, htmltemplate):
         """helper to exercise the command line function"""
         inputfile = wStringIO.StringIO(posource)
@@ -56,7 +60,8 @@ sin.
 </body>'''
         assert htmlexpected.replace("\n", " ") in self.converthtml(posource, htmlsource).replace("\n", " ")
 
-    def xtest_entities(self):
+    @mark.xfail(reason="Not Implemented")
+    def test_entities(self):
         """Tests that entities are handled correctly"""
         htmlsource = '<p>5 less than 6</p>'
         posource = '#:html:3\nmsgid "5 less than 6"\nmsgstr "5 < 6"\n'
@@ -68,7 +73,8 @@ sin.
         htmlexpected = '<p>Vis &amp; skyfies</p>'
         assert htmlexpected in self.converthtml(posource, htmlsource)
 
-    def xtest_escapes(self):
+    @mark.xfail(reason="Not Implemented")
+    def test_escapes(self):
         """Tests that PO escapes are correctly handled"""
         htmlsource = '<div>Row 1<br />Row 2</div>'
         posource = '#: html:3\nmsgid "Row 1\\n"\n"Row 2"\nmsgstr "Ry 1\\n"\n"Ry 2"\n'
@@ -80,6 +86,19 @@ sin.
         htmlexpected = '<p>"ek is dom"</p>'
         assert htmlexpected in self.converthtml(posource, htmlsource)
 
+    def test_fuzzy_strings(self):
+        """Test that we use source when a string is fuzzy
+
+        This fixes :bug:`3145`
+        """
+        htmlsource = '<div>aaa</div>'
+        posource = '#: html:3\nmsgid "aaa"\nmsgstr "bbb"\n'
+        posource_fuzzy = '#: html:3\n#, fuzzy\nmsgid "aaa"\nmsgstr "bbb"\n'
+        htmlexpected = '<div>bbb</div>'
+        assert htmlexpected in self.converthtml(posource, htmlsource)
+        assert htmlexpected not in self.converthtml(posource_fuzzy, htmlsource)
+        assert htmlsource in self.converthtml(posource_fuzzy, htmlsource)
+
 
 class TestPO2HtmlCommand(test_convert.TestConvertCommand, TestPO2Html):
     """Tests running actual po2oo commands on files"""
@@ -89,11 +108,6 @@ class TestPO2HtmlCommand(test_convert.TestConvertCommand, TestPO2Html):
         """tests getting help"""
         options = test_convert.TestConvertCommand.test_help(self)
         options = self.help_check(options, "-t TEMPLATE, --template=TEMPLATE")
-        options = self.help_check(options, "-w WRAP, --wrap=WRAP")
+        options = self.help_check(options, "--threshold=PERCENT")
         options = self.help_check(options, "--fuzzy")
-        try:
-            import tidy
-            options = self.help_check(options, "--notidy")
-        except ImportError, e:
-            pass
         options = self.help_check(options, "--nofuzzy", last=True)
