@@ -3,9 +3,11 @@
 from translate.convert import po2moz
 from translate.convert import test_convert
 
+
 class TestPO2Moz:
     pass
 
+
 class TestPO2MozCommand(test_convert.TestConvertCommand, TestPO2Moz):
     """Tests running actual po2moz commands on files"""
     convertmodule = po2moz
@@ -16,6 +18,7 @@ class TestPO2MozCommand(test_convert.TestConvertCommand, TestPO2Moz):
         options = test_convert.TestConvertCommand.test_help(self)
         options = self.help_check(options, "-t TEMPLATE, --template=TEMPLATE")
         options = self.help_check(options, "-l LOCALE, --locale=LOCALE")
-        options = self.help_check(options, "--clonexpi=CLONEXPI")
+        options = self.help_check(options, "--removeuntranslated")
+        options = self.help_check(options, "--threshold=PERCENT")
         options = self.help_check(options, "--fuzzy")
         options = self.help_check(options, "--nofuzzy", last=True)
