@@ -1,12 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+from pytest import mark
+
 from translate.convert import po2php
 from translate.convert import test_convert
 from translate.misc import wStringIO
 from translate.storage import po
 
+
 class TestPO2Php:
+
     def po2php(self, posource):
         """helper that converts po source to .php source without requiring files"""
         inputfile = wStringIO.StringIO(posource)
@@ -21,8 +25,8 @@ class TestPO2Php:
         inputpo = po.pofile(inputfile)
         templatefile = wStringIO.StringIO(phpsource)
         #templatephp = php.phpfile(templatefile)
-        convertor = po2php.rephp(templatefile)
-        outputphp = convertor.convertstore(inputpo)
+        convertor = po2php.rephp(templatefile, inputpo)
+        outputphp = convertor.convertstore()
         print outputphp
         return outputphp
 
@@ -59,7 +63,7 @@ msgstr ""'''
 
     def test_merging_fuzzy(self):
         """check merging a fuzzy translation"""
-        posource = '''#: $lang['name']\n#, fuzzy\nmsgid "value"\nmsgstr "waarde"\n'''
+        posource = '''#: %24lang%5B+%27name%27+%5D\n#, fuzzy\nmsgid "value"\nmsgstr "waarde"\n'''
         phptemplate = '''$lang['name']  =  'value';\n'''
         phpexpected = '''$lang['name']  =  'value';\n'''
         phpfile = self.merge2php(phptemplate, posource)
@@ -68,7 +72,7 @@ msgstr ""'''
 
     def test_locations_with_spaces(self):
         """check that a location with spaces in php but spaces removed in PO is used correctly"""
-        posource = '''#: $lang['name']\nmsgid "value"\nmsgstr "waarde"\n'''
+        posource = '''#: %24lang%5B+%27name%27+%5D\nmsgid "value"\nmsgstr "waarde"\n'''
         phptemplate = '''$lang[ 'name' ]  =  'value';\n'''
         phpexpected = '''$lang[ 'name' ]  =  'waarde';\n'''
         phpfile = self.merge2php(phptemplate, posource)
@@ -77,7 +81,7 @@ msgstr ""'''
 
     def test_inline_comments(self):
         """check that we include inline comments from the template.  Bug 590"""
-        posource = '''#: $lang['name']\nmsgid "value"\nmsgstr "waarde"\n'''
+        posource = '''#: %24lang%5B+%27name%27+%5D\nmsgid "value"\nmsgstr "waarde"\n'''
         phptemplate = '''$lang[ 'name' ]  =  'value'; //inline comment\n'''
         phpexpected = '''$lang[ 'name' ]  =  'waarde'; //inline comment\n'''
         phpfile = self.merge2php(phptemplate, posource)
@@ -139,14 +143,16 @@ msgstr "stringetjie"
         print phpfile
         assert "".join(phpfile) == phpexpected
 
-#    def test_merging_propertyless_template(self):
-#        """check that when merging with a template with no property values that we copy the template"""
-#        posource = ""
-#        proptemplate = "# A comment\n"
-#        propexpected = proptemplate
-#        propfile = self.merge2prop(proptemplate, posource)
-#        print propfile
-#        assert propfile == [propexpected]
+    @mark.xfail(reason="Need to review if we want this behaviour")
+    def test_merging_propertyless_template(self):
+        """check that when merging with a template with no property values that we copy the template"""
+        posource = ""
+        proptemplate = "# A comment\n"
+        propexpected = proptemplate
+        propfile = self.merge2prop(proptemplate, posource)
+        print propfile
+        assert propfile == [propexpected]
+
 
 class TestPO2PhpCommand(test_convert.TestConvertCommand, TestPO2Php):
     """Tests running actual po2php commands on files"""
@@ -157,6 +163,6 @@ class TestPO2PhpCommand(test_convert.TestConvertCommand, TestPO2Php):
         """tests getting help"""
         options = test_convert.TestConvertCommand.test_help(self)
         options = self.help_check(options, "-t TEMPLATE, --template=TEMPLATE")
+        options = self.help_check(options, "--threshold=PERCENT")
         options = self.help_check(options, "--fuzzy")
         options = self.help_check(options, "--nofuzzy", last=True)
-
