@@ -1,12 +1,18 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
+
+import warnings
+
+from pytest import mark
 
 from translate.convert import pot2po
 from translate.convert import test_convert
 from translate.misc import wStringIO
 from translate.storage import po
-import warnings
+
 
 class TestPOT2PO:
+
     def setup_method(self, method):
         warnings.resetwarnings()
 
@@ -100,7 +106,8 @@ msgstr[1] "%d handleidings."
         newpo = self.convertpot(potsource, posource)
         assert str(self.singleunit(newpo)) == poexpected
 
-    def xtest_merging_msgid_change(self):
+    @mark.xfail(reason="Not implemented - review if this is even correct")
+    def test_merging_msgid_change(self):
         """tests that if the msgid changes but the location stays the same that we merge"""
         potsource = '''#: simple.label\n#: simple.accesskey\nmsgid "Its &hard coding a newline.\\n"\nmsgstr ""\n'''
         posource = '''#: simple.label\n#: simple.accesskey\nmsgid "A &hard coded newline.\\n"\nmsgstr "&Hart gekoeerde nuwe lyne\\n"\n'''
@@ -119,7 +126,9 @@ msgstr[1] "%d handleidings."
         assert str(self.singleunit(newpo)) == poexpected
 
     def test_merging_location_and_whitespace_change(self):
-        """test that even if the location changes that if the msgid only has whitespace changes we can still merge"""
+        """test that even if the location changes that if the msgid
+        only has whitespace changes we can still merge"""
+
         potsource = '''#: singlespace.label%ssinglespace.accesskey\nmsgid "&We have spaces"\nmsgstr ""\n''' % po.lsep
         posource = '''#: doublespace.label%sdoublespace.accesskey\nmsgid "&We  have  spaces"\nmsgstr "&One  het  spasies"\n''' % po.lsep
         poexpected = '''#: singlespace.label%ssinglespace.accesskey\n#, fuzzy\nmsgid "&We have spaces"\nmsgstr "&One  het  spasies"\n''' % po.lsep
@@ -128,7 +137,7 @@ msgstr[1] "%d handleidings."
         assert str(self.singleunit(newpo)) == poexpected
 
     def test_merging_location_ambiguous_with_disambiguous(self):
-        """test that when we have a PO in ambiguous (Gettext form) and merge with disamabiguous (KDE comment form) 
+        """test that when we have a PO in ambiguous (Gettext form) and merge with disamabiguous (KDE comment form)
         that we don't duplicate the location #: comments"""
         potsource = '''#: location.c:1\nmsgid ""\n"_: location.c:1\\n"\n"Source"\nmsgstr ""\n\n''' + \
                     '''#: location.c:10\nmsgid ""\n"_: location.c:10\\n"\n"Source"\nmsgstr ""\n'''
@@ -140,7 +149,8 @@ msgstr[1] "%d handleidings."
         assert str(newpo.units[1]) == poexpected1
         assert str(newpo.units[2]) == poexpected2
 
-    def wtest_merging_accelerator_changes(self):
+    @mark.xfail(reason="Not Implemented - needs review")
+    def test_merging_accelerator_changes(self):
         """test that a change in the accelerator localtion still allows merging"""
         potsource = '''#: someline.c\nmsgid "A&bout"\nmsgstr ""\n'''
         posource = '''#: someline.c\nmsgid "&About"\nmsgstr "&Info"\n'''
@@ -149,7 +159,8 @@ msgstr[1] "%d handleidings."
         print newpo
         assert str(self.singleunit(newpo)) == poexpected
 
-    def xtest_lines_cut_differently(self):
+    @mark.xfail(reason="Not Implemented - review if this is even correct")
+    def test_lines_cut_differently(self):
         """Checks that the correct formatting is preserved when pot an po lines differ."""
         potsource = '''#: simple.label\nmsgid "Line split "\n"differently"\nmsgstr ""\n'''
         posource = '''#: simple.label\nmsgid "Line"\n" split differently"\nmsgstr "Lyne verskillend gesny"\n'''
@@ -262,13 +273,13 @@ msgstr "Sertifikate"
 
     def test_merging_plurals(self):
         """ensure that we can merge plural messages"""
-        potsource = '''msgid "One"\nmsgid_plural "Two"\nmsgstr[0] ""\nmsgstr[1] ""\n''' 
+        potsource = '''msgid "One"\nmsgid_plural "Two"\nmsgstr[0] ""\nmsgstr[1] ""\n'''
         posource = '''msgid "One"\nmsgid_plural "Two"\nmsgstr[0] "Een"\nmsgstr[1] "Twee"\nmsgstr[2] "Drie"\n'''
         newpo = self.convertpot(potsource, posource)
         print newpo
         newpounit = self.singleunit(newpo)
         assert str(newpounit) == posource
-        
+
     def test_merging_obsoleting_messages(self):
         """check that we obsolete messages no longer present in the new file"""
         #add emtpy msgid line to help factory identify format
@@ -388,7 +399,7 @@ msgstr ""
 
     def test_merging_comments(self):
         """Test that we can merge comments correctly"""
-        potsource = '''#. Don't do it!\n#: file.py:1\nmsgid "One"\nmsgstr ""\n''' 
+        potsource = '''#. Don't do it!\n#: file.py:1\nmsgid "One"\nmsgstr ""\n'''
         posource = '''#. Don't do it!\n#: file.py:2\nmsgid "One"\nmsgstr "Een"\n'''
         poexpected = '''#. Don't do it!\n#: file.py:1\nmsgid "One"\nmsgstr "Een"\n'''
         newpo = self.convertpot(potsource, posource)
@@ -398,7 +409,7 @@ msgstr ""
 
     def test_merging_typecomments(self):
         """Test that we can merge with typecomments"""
-        potsource = '''#: file.c:1\n#, c-format\nmsgid "%d pipes"\nmsgstr ""\n''' 
+        potsource = '''#: file.c:1\n#, c-format\nmsgid "%d pipes"\nmsgstr ""\n'''
         posource = '''#: file.c:2\nmsgid "%d pipes"\nmsgstr "%d pype"\n'''
         poexpected = '''#: file.c:1\n#, c-format\nmsgid "%d pipes"\nmsgstr "%d pype"\n'''
         newpo = self.convertpot(potsource, posource)
@@ -406,7 +417,7 @@ msgstr ""
         print newpounit
         assert str(newpounit) == poexpected
 
-        potsource = '''#: file.c:1\n#, c-format\nmsgid "%d computers"\nmsgstr ""\n''' 
+        potsource = '''#: file.c:1\n#, c-format\nmsgid "%d computers"\nmsgstr ""\n'''
         posource = '''#: file.c:2\n#, c-format\nmsgid "%s computers "\nmsgstr "%s-rekenaars"\n'''
         poexpected = '''#: file.c:1\n#, fuzzy, c-format\nmsgid "%d computers"\nmsgstr "%s-rekenaars"\n'''
         newpo = self.convertpot(potsource, posource)
@@ -454,6 +465,162 @@ msgstr "teks"
         print newpo
         assert poexpected in str(newpo)
 
+    def test_msgctxt_multiline(self):
+        """Test multiline msgctxt fields."""
+        pot_source = r'''#. |1MV
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_21\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr ""
+'''
+
+        po_source = r'''#. |1MV
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_21\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr "A szükséges lemezterület kiszámítása"
+'''
+
+        new_po = self.convertpot(pot_source, po_source)
+        assert new_po.units[0].isheader()
+        unit = new_po.units[1]
+        assert not unit.isfuzzy()
+        assert po_source == str(unit)
+
+    def test_msgid_merge_on_location(self):
+        """Tests that unit merges rely on location-based matching."""
+        pot_source = r'''
+msgid ""
+msgstr ""
+"X-Merge-On: location\n"
+
+#. $:am
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_11\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr ""
+
+#. NjJ3
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_12\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr ""
+'''
+
+        po_source = r'''
+#. $:am
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_11\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr "A szükséges lemezterület kiszámítása"
+
+#. NjJ3
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_12\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr "A szükséges lemezterület kiszámítása"
+'''
+
+        new_po = self.convertpot(pot_source, po_source)
+
+        assert len(new_po.units) == 3
+        assert new_po.units[0].isheader()
+
+        assert not new_po.units[1].isfuzzy()
+        assert new_po.units[2].isfuzzy()
+
+    def test_msgid_merge_on_id(self):
+        """Tests that unit merges rely on location-based matching."""
+        pot_source = r'''
+msgid ""
+msgstr ""
+"X-Merge-On: id\n"
+
+#. $:am
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_11\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr ""
+
+#. NjJ3
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_12\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr ""
+'''
+        pot_source_noheader = r'''
+#. $:am
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_11\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr ""
+
+#. NjJ3
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_12\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr ""
+'''
+
+        po_source = r'''
+#. $:am
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_11\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr "A szükséges lemezterület kiszámítása"
+
+#. NjJ3
+#: ActionTe.ulf
+msgctxt ""
+"ActionTe.ulf\n"
+"OOO_ACTIONTEXT_12\n"
+"LngText.text"
+msgid "Computing space requirements"
+msgstr "A szükséges lemezterület kiszámítása"
+'''
+
+        for pot in (pot_source, pot_source_noheader):
+            new_po = self.convertpot(pot, po_source)
+
+            assert len(new_po.units) == 3
+            assert new_po.units[0].isheader()
+
+            assert not new_po.units[1].isfuzzy()
+            assert not new_po.units[2].isfuzzy()
+
     def test_empty_msgid(self):
         """Test that we handle empty msgids correctly."""
         #TODO: this test will fail if we don't have the gettext location
@@ -482,17 +649,30 @@ msgstr "trans"
         """Test that we migrate correctly from msgidcomments to msgctxt.
 
         This is needed for our move away from using msgidcomments for mozilla."""
-        potsource = '#: bla\nmsgctxt "bla"\nmsgid ""\nmsgstr ""'
-        posource = r"""
+        potsource = r'''
 msgid ""
-"Project-Id-Version: Pootle 0.10\n"
 msgstr ""
+"X-Merge-On: location\n"
+"X-Accelerator-Marker: &\n"
+
+
+#: bla
+msgctxt "bla"
+msgid ""
+msgstr ""
+'''
+        posource = r'''
+msgid ""
+msgstr ""
+"Project-Id-Version: Pootle 0.10\n"
+"X-Accelerator-Marker: &\n"
+
 
 #: bla
 msgid ""
 "_: bla\n"
 msgstr "trans"
-"""
+'''
         newpo = self.convertpot(potsource, posource)
         print newpo
         assert len(newpo.units) == 2
@@ -547,6 +727,8 @@ msgstr ""
 "Content-Transfer-Encoding: 8bit\n"
 "Plural-Forms: nplurals=INTEGER; plural=EXPRESSION;\n"
 "X-Generator: Translate Toolkit 0.10rc2\n"
+"X-Merge-On: location\n"
+"X-Accelerator-Marker: &\n"
 
 #: new_disassociated_mozilla_accesskey
 msgid "R"
@@ -565,6 +747,7 @@ msgstr ""
 "Content-Transfer-Encoding: 8bit\n"
 "Plural-Forms: nplurals=2; plural=(n != 1);\n"
 "X-Generator: Translate Toolkit 0.9\n"
+"X-Accelerator-Marker: &\n"
 
 #: old_disassociated_mozilla_accesskey
 msgid "R"
@@ -583,6 +766,8 @@ msgstr ""
 "Content-Transfer-Encoding: 8bit\n"
 "Plural-Forms: nplurals=2; plural=(n != 1);\n"
 "X-Generator: Translate Toolkit 0.10rc2\n"
+"X-Merge-On: location\n"
+"X-Accelerator-Marker: &\n"
 
 #: new_disassociated_mozilla_accesskey
 msgid "R"
@@ -606,4 +791,3 @@ class TestPOT2POCommand(test_convert.TestConvertCommand, TestPOT2PO):
         options = self.help_check(options, "--tm")
         options = self.help_check(options, "-s MIN_SIMILARITY, --similarity=MIN_SIMILARITY")
         options = self.help_check(options, "--nofuzzymatching", last=True)
-
