@@ -1,11 +1,29 @@
 # -*- coding: utf-8 -*-
+
+from pytest import mark
+
 from translate.filters import checks
 from translate.lang import data
-from translate.storage import po
+from translate.storage import po, xliff
+
 
 def strprep(str1, str2, message=None):
     return data.normalized_unicode(str1), data.normalized_unicode(str2), data.normalized_unicode(message)
 
+
+def check_category(filterfunction):
+    """Checks whether ``filterfunction`` has defined a category or not."""
+    has_category = []
+    classes = (checks.TeeChecker, checks.UnitChecker)
+
+    for klass in classes:
+        categories = getattr(klass, 'categories', None)
+        has_category.append(categories is not None and \
+                            filterfunction.__name__ in categories)
+
+    return True in has_category
+
+
 def passes(filterfunction, str1, str2):
     """returns whether the given strings pass on the given test, handling FilterFailures"""
     str1, str2, no_message = strprep(str1, str2)
@@ -13,8 +31,12 @@ def passes(filterfunction, str1, str2):
         filterresult = filterfunction(str1, str2)
     except checks.FilterFailure, e:
         filterresult = False
+
+    filterresult = filterresult and check_category(filterfunction)
+
     return filterresult
 
+
 def fails(filterfunction, str1, str2, message=None):
     """returns whether the given strings fail on the given test, handling only FilterFailures"""
     str1, str2, message = strprep(str1, str2, message)
@@ -24,13 +46,17 @@ def fails(filterfunction, str1, str2, message=None):
         filterresult = True
     except checks.FilterFailure, e:
         if message:
-            exc_message = e.args[0]
+            exc_message = e.messages[0]
             filterresult = exc_message != message
             print exc_message.encode('utf-8')
         else:
             filterresult = False
+
+    filterresult = filterresult and check_category(filterfunction)
+
     return not filterresult
 
+
 def fails_serious(filterfunction, str1, str2, message=None):
     """returns whether the given strings fail on the given test, handling only SeriousFilterFailures"""
     str1, str2, message = strprep(str1, str2, message)
@@ -38,11 +64,14 @@ def fails_serious(filterfunction, str1, str2, message=None):
         filterresult = filterfunction(str1, str2)
     except checks.SeriousFilterFailure, e:
         if message:
-            exc_message = e.args[0]
+            exc_message = e.messages[0]
             filterresult = exc_message != message
             print exc_message.encode('utf-8')
         else:
             filterresult = False
+
+    filterresult = filterresult and check_category(filterfunction)
+
     return not filterresult
 
 
@@ -54,6 +83,7 @@ def test_defaults():
     stdchecker = checks.StandardChecker()
     assert stdchecker.config.varmatches == []
 
+
 def test_construct():
     """tests that the checkers can be constructed"""
     stdchecker = checks.StandardChecker()
@@ -62,6 +92,7 @@ def test_construct():
     gnomechecker = checks.GnomeChecker()
     kdechecker = checks.KdeChecker()
 
+
 def test_accelerator_markers():
     """test that we have the correct accelerator marker for the various default configs"""
     stdchecker = checks.StandardChecker()
@@ -75,12 +106,14 @@ def test_accelerator_markers():
     kdechecker = checks.KdeChecker()
     assert kdechecker.config.accelmarkers == ["&"]
 
+
 def test_messages():
     """test that our helpers can check for messages and that these error messages can contain Unicode"""
     stdchecker = checks.StandardChecker(checks.CheckerConfig(validchars='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'))
-    assert fails(stdchecker.validchars, "Some unexpected characters", "©", "invalid chars: '©' (\\u00a9)")
+    assert fails(stdchecker.validchars, "Some unexpected characters", "©", "Invalid characters: '©' (\\u00a9)")
     stdchecker = checks.StandardChecker()
-    assert fails_serious(stdchecker.escapes, r"A tab", r"'n Ṱab\t", r"""escapes in original () don't match escapes in translation ('Ṱab\t')""")
+    assert fails_serious(stdchecker.escapes, r"A tab", r"'n Ṱab\t", r"""Escapes in original () don't match escapes in translation ('Ṱab\t')""")
+
 
 def test_accelerators():
     """tests accelerators"""
@@ -119,10 +152,27 @@ def test_accelerators():
     # Bug 289: accept accented accelerator characters
     afchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&", targetlanguage="fi"))
     assert passes(afchecker.accelerators, "&Reload Frame", "P&äivitä kehys")
+
+    trchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&", targetlanguage="tr"))
+    assert passes(trchecker.accelerators, "&Download", "&İndir")
+    assert passes(trchecker.accelerators, "&Business", "İ&ş")
+    assert passes(trchecker.accelerators, "&Remove", "Kald&ır")
+    assert passes(trchecker.accelerators, "&Three", "&Üç")
+    assert passes(trchecker.accelerators, "&Three", "Ü&ç")
+    assert passes(trchecker.accelerators, "&Before", "&Önce")
+    assert passes(trchecker.accelerators, "Fo&ur", "D&ört")
+    assert passes(trchecker.accelerators, "Mo&dern", "Ça&ğdaş")
+    assert passes(trchecker.accelerators, "Mo&dern", "&Çağdaş")
+    assert passes(trchecker.accelerators, "&February", "&Şubat")
+    assert passes(trchecker.accelerators, "P&lain", "D&üz")
+    assert passes(trchecker.accelerators, "GAR&DEN", "BA&Ğ")
+
     # Problems:
     # Accelerator before variable - see test_acceleratedvariables
 
-def xtest_acceleratedvariables():
+
+@mark.xfail(reason="Accelerated variables needs a better implementation")
+def test_acceleratedvariables():
     """test for accelerated variables"""
     # FIXME: disabled since acceleratedvariables has been removed, but these checks are still needed
     mozillachecker = checks.MozillaChecker()
@@ -132,6 +182,7 @@ def xtest_acceleratedvariables():
     assert fails(ooochecker.acceleratedvariables, "%PRODUCTNAME% ~Options", "~%PRODUCTNAME% Ikhetho")
     assert passes(ooochecker.acceleratedvariables, "%PRODUCTNAME% ~Options", "%PRODUCTNAME% ~Ikhetho")
 
+
 def test_acronyms():
     """tests acronyms"""
     stdchecker = checks.StandardChecker()
@@ -148,12 +199,14 @@ def test_acronyms():
     # Test '-W' (bug 283)
     assert passes(stdchecker.acronyms, "%s: option `-W %s' is ambiguous", "%s: opsie '-W %s' is dubbelsinnig")
 
+
 def test_blank():
     """tests blank"""
     stdchecker = checks.StandardChecker()
     assert fails(stdchecker.blank, "Save as", " ")
     assert fails(stdchecker.blank, "_: KDE comment\\n\nSimple string", "  ")
 
+
 def test_brackets():
     """tests brackets"""
     stdchecker = checks.StandardChecker()
@@ -164,6 +217,7 @@ def test_brackets():
     assert fails(stdchecker.brackets, "For ]sic[ numbers", "Vier getalle")
     assert passes(stdchecker.brackets, "{[(", "[({")
 
+
 def test_compendiumconflicts():
     """tests compendiumconflicts"""
     stdchecker = checks.StandardChecker()
@@ -172,6 +226,7 @@ Leer nie gestoor gestoor nie\n
 #-#-#-#-# file1.po #-#-#-#-#\n
 Leer nie gestoor""")
 
+
 def test_doublequoting():
     """tests double quotes"""
     stdchecker = checks.StandardChecker()
@@ -186,6 +241,8 @@ def test_doublequoting():
     assert fails(frchecker.doublequoting, "Do \"this\"", "Do \"this\"")
     assert passes(frchecker.doublequoting, "Do \"this\"", "Do « this »")
     assert fails(frchecker.doublequoting, "Do \"this\"", "Do « this » « this »")
+    # This used to fail because we strip variables, and was left with an empty quotation that was not converted
+    assert passes(frchecker.doublequoting, u"Copying `%s' to `%s'", u"Copie de « %s » vers « %s »")
 
     vichecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="vi"))
     assert passes(vichecker.doublequoting, 'Save "File"', u"Lưu « Tập tin »")
@@ -195,6 +252,7 @@ def test_doublequoting():
     assert passes(eschecker.doublequoting, "<![CDATA[ Enter the name of the Windows workgroup that this server should appear in. ]]>",
             "<![CDATA[ Ingrese el nombre del grupo de trabajo de Windows en el que debe aparecer este servidor. ]]>")
 
+
 def test_doublespacing():
     """tests double spacing"""
     stdchecker = checks.StandardChecker()
@@ -206,6 +264,7 @@ def test_doublespacing():
     assert passes(ooochecker.doublespacing, "Execute %PROGRAMNAME Calc", "Blah %PROGRAMNAME Calc")
     assert passes(ooochecker.doublespacing, "Execute %PROGRAMNAME Calc", "Blah % PROGRAMNAME Calc")
 
+
 def test_doublewords():
     """tests doublewords"""
     stdchecker = checks.StandardChecker()
@@ -214,12 +273,16 @@ def test_doublewords():
     # Double variables are not an error
     stdchecker = checks.StandardChecker(checks.CheckerConfig(varmatches=[("%", 1)]))
     assert passes(stdchecker.doublewords, "%s %s installation", "tsenyo ya %s %s")
+    # Double XML tags are not an error
+    stdchecker = checks.StandardChecker()
+    assert passes(stdchecker.doublewords, "Line one <br> <br> line two", "Lyn een <br> <br> lyn twee")
     # In some language certain double words are not errors
     st_checker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="st"))
     assert passes(st_checker.doublewords, "Color to draw the name of a message you sent.", "Mmala wa ho taka bitso la molaetsa oo o o rometseng.")
     assert passes(st_checker.doublewords, "Ten men", "Banna ba ba leshome")
     assert passes(st_checker.doublewords, "Give SARS the tax", "Lekgetho le le fe SARS")
 
+
 def test_endpunc():
     """tests endpunc"""
     stdchecker = checks.StandardChecker()
@@ -248,16 +311,20 @@ def test_endpunc():
     assert passes(stdchecker.endpunc, "Header:", u"En-tête :")
     assert passes(stdchecker.endpunc, "Header:", u"En-tête\u00a0:")
 
+
 def test_endwhitespace():
     """tests endwhitespace"""
     stdchecker = checks.StandardChecker()
+    assert passes(stdchecker.endwhitespace, "A setence.", "I'm correct.")
     assert passes(stdchecker.endwhitespace, "A setence. ", "I'm correct. ")
     assert fails(stdchecker.endwhitespace, "A setence. ", "'I'm incorrect.")
+    assert passes(stdchecker.endwhitespace, "Problem with something: %s\n", "Probleem met iets: %s\n")
 
     zh_checker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='zh'))
     # This should pass since the space is not needed in Chinese
     assert passes(zh_checker.endwhitespace, "Init. Limit: ", "起始时间限制：")
 
+
 def test_escapes():
     """tests escapes"""
     stdchecker = checks.StandardChecker()
@@ -277,6 +344,7 @@ def test_escapes():
     ooochecker = checks.OpenOfficeChecker()
     assert passes(ooochecker.escapes, ",\t44\t;\t59\t:\t58\t{Tab}\t9\t{space}\t32", ",\t44\t;\t59\t:\t58\t{Tab}\t9\t{space}\t32")
 
+
 def test_newlines():
     """tests newlines"""
     stdchecker = checks.StandardChecker()
@@ -291,10 +359,14 @@ def test_newlines():
     assert fails(stdchecker.newlines, "A file\n", "'n Leer\r\n")
     assert fails(stdchecker.newlines, "A file\r\n", "'n Leer\n")
     assert fails(stdchecker.newlines, "blah.\nA file", "bleah. 'n leer")
+    # msgfmt errors
+    assert fails(stdchecker.newlines, "One two\n", "Een\ntwee")
+    assert fails(stdchecker.newlines, "\nOne two", "Een\ntwee")
     # Real example
     ooochecker = checks.OpenOfficeChecker()
     assert fails(ooochecker.newlines, "The arrowhead was modified without saving.\nWould you like to save the arrowhead now?", "Ṱhoho ya musevhe yo khwinifhadzwa hu si na u seiva.Ni khou ṱoda u seiva thoho ya musevhe zwino?")
 
+
 def test_tabs():
     """tests tabs"""
     stdchecker = checks.StandardChecker()
@@ -306,11 +378,14 @@ def test_tabs():
     ooochecker = checks.OpenOfficeChecker()
     assert passes(ooochecker.tabs, ",\t44\t;\t59\t:\t58\t{Tab}\t9\t{space}\t32", ",\t44\t;\t59\t:\t58\t{Tab}\t9\t{space}\t32")
 
+
 def test_filepaths():
     """tests filepaths"""
     stdchecker = checks.StandardChecker()
     assert passes(stdchecker.filepaths, "%s to the file /etc/hosts on your system.", "%s na die leer /etc/hosts op jou systeem.")
     assert fails(stdchecker.filepaths, "%s to the file /etc/hosts on your system.", "%s na die leer /etc/gasheer op jou systeem.")
+    assert passes(stdchecker.filepaths, "Text with <br />line break", "Teks met <br /> lynbreuk")
+
 
 def test_kdecomments():
     """tests kdecomments"""
@@ -322,6 +397,7 @@ A string to translate""", r"""_: Ek is 'n commment\n
 'n String om te vertaal""")
     assert fails(stdchecker.kdecomments, """_: I am a comment\\n\n""", """_: I am a comment\\n\n""")
 
+
 def test_long():
     """tests long messages"""
     stdchecker = checks.StandardChecker()
@@ -329,6 +405,8 @@ def test_long():
     assert fails(stdchecker.long, "Short.", "Kort.......................................................................................")
     assert fails(stdchecker.long, "a", "bc")
 
+
+@mark.xfail(reason="FIXME: All fails() tests are not working")
 def test_musttranslatewords():
     """tests stopwords"""
     stdchecker = checks.StandardChecker(checks.CheckerConfig(musttranslatewords=[]))
@@ -358,6 +436,7 @@ def test_musttranslatewords():
     # Unicode
     assert fails(stdchecker.musttranslatewords, "Click OK", u"Kiḽikani OK")
 
+
 def test_notranslatewords():
     """tests stopwords"""
     stdchecker = checks.StandardChecker(checks.CheckerConfig(notranslatewords=[]))
@@ -391,6 +470,7 @@ def test_notranslatewords():
     stdchecker = checks.StandardChecker(checks.CheckerConfig(notranslatewords=[u"\u1e3cike"]))
     assert passes(stdchecker.notranslatewords, u"You \u1e3cike me", u"Ek \u004c\u032dike jou")
 
+
 def test_numbers():
     """test numbers"""
     stdchecker = checks.StandardChecker()
@@ -413,6 +493,10 @@ def test_numbers():
     assert fails(stdchecker.numbers, "1,000.00", "1 000,00")
     # You should be able to reorder numbers
     assert passes(stdchecker.numbers, "40-bit RC2 encryption with RSA and an MD5", "Umbhalo ocashile i-RC2 onamabhithi angu-40 one-RSA ne-MD5")
+    # Don't fail the numbers check if the entry is a dialogsize entry
+    mozillachecker = checks.MozillaChecker()
+    assert passes(mozillachecker.numbers, 'width: 12em;', 'width: 20em;')
+
 
 def test_options():
     """tests command line options e.g. --option"""
@@ -426,6 +510,7 @@ def test_options():
     assert passes(stdchecker.options, "Hello! -- Hi", "Hallo! &mdash; Haai")
     assert passes(stdchecker.options, "--blank--", "--vide--")
 
+
 def test_printf():
     """tests printf style variables"""
     # This should really be a subset of the variable checks
@@ -441,7 +526,14 @@ def test_printf():
     assert passes(stdchecker.printf, "String %s and number %d", "String %1$s en nommer %2$d")
     assert passes(stdchecker.printf, "String %1$s and number %2$d", "String %1$s en nommer %2$d")
     assert passes(stdchecker.printf, "String %s and number %d", "Nommer %2$d and string %1$s")
+    assert passes(stdchecker.printf, "String %s and real number %f and number %d", "String %1$s en nommer %3$d en reële getal %2$f")
+    assert passes(stdchecker.printf, "String %1$s and real number %2$f and number %3$d", "String %1$s en nommer %3$d en reële getal %2$f")
+    assert passes(stdchecker.printf, "Real number %2$f and string %1$s and number %3$d", "String %1$s en nommer %3$d en reële getal %2$f")
     assert fails(stdchecker.printf, "String %s and number %d", "Nommer %1$d and string %2$s")
+    assert fails(stdchecker.printf, "String %s and real number %f and number %d", "String %1$s en nommer %3$d en reële getal %2$d")
+    assert fails(stdchecker.printf, "String %s and real number %f and number %d", "String %1$s en nommer %3$d en reële getal %4$f")
+    assert fails(stdchecker.printf, "String %s and real number %f and number %d", "String %2$s en nommer %3$d en reële getal %2$f")
+    assert fails(stdchecker.printf, "Real number %2$f and string %1$s and number %3$d", "String %1$f en nommer %3$d en reële getal %2$f")
     # checking python format strings
     assert passes(stdchecker.printf, "String %(1)s and number %(2)d", "Nommer %(2)d en string %(1)s")
     assert passes(stdchecker.printf, "String %(str)s and number %(num)d", "Nommer %(num)d en string %(str)s")
@@ -450,6 +542,12 @@ def test_printf():
     # checking omitted plural format string placeholder %.0s
     stdchecker.hasplural = 1
     assert passes(stdchecker.printf, "%d plurals", "%.0s plural")
+    # checking Objective-C %@ format specification
+    assert fails(stdchecker.printf, "I am %@", "Ek is @%")  # typo
+    assert fails(stdchecker.printf, "Object %@ and object %@", "String %1$@ en string %3$@")  # out of bounds
+    assert fails(stdchecker.printf, "I am %@", "Ek is %s")  # wrong specification
+    assert passes(stdchecker.printf, "Object %@ and string %s", "Object %1$@ en string %2$s")  # correct sentence
+
 
 def test_puncspacing():
     """tests spacing after punctuation"""
@@ -465,6 +563,7 @@ def test_puncspacing():
     assert passes(frchecker.puncspacing, u"Do \"this\"", u"Do «\u00a0this\u00a0»")
     assert fails(frchecker.puncspacing, "Do \"this\"", "Do «this»")
 
+
 def test_purepunc():
     """tests messages containing only punctuation"""
     stdchecker = checks.StandardChecker()
@@ -475,13 +574,17 @@ def test_purepunc():
     assert fails(stdchecker.purepunc, "'", "Find")
     assert passes(stdchecker.purepunc, "year measurement template|2000", "2000")
 
+
 def test_sentencecount():
     """tests sentencecount messages"""
     stdchecker = checks.StandardChecker()
     assert passes(stdchecker.sentencecount, "One. Two. Three.", "Een. Twee. Drie.")
     assert passes(stdchecker.sentencecount, "One two three", "Een twee drie.")
     assert fails(stdchecker.sentencecount, "One. Two. Three.", "Een Twee. Drie.")
-    assert passes(stdchecker.sentencecount, "Sentence with i.e. in it.", "Sin met d.w.s. in dit.") # bug 178, description item 8
+    assert passes(stdchecker.sentencecount, "Sentence with i.e. in it.", "Sin met d.w.s. in dit.")  # bug 178, description item 8
+    el_checker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='el'))
+    assert fails(el_checker.sentencecount, "First sentence. Second sentence.", "Πρώτη πρόταση. δεύτερη πρόταση.")
+
 
 def test_short():
     """tests short messages"""
@@ -490,6 +593,7 @@ def test_short():
     assert fails(stdchecker.short, "I am a very long sentence", "Ek")
     assert fails(stdchecker.short, "abcde", "c")
 
+
 def test_singlequoting():
     """tests single quotes"""
     stdchecker = checks.StandardChecker()
@@ -522,6 +626,7 @@ def test_singlequoting():
     assert passes(vichecker.singlequoting, "Save 'File'", u"Lưu « Tập tin »")
     assert passes(vichecker.singlequoting, "Save `File'", u"Lưu « Tập tin »")
 
+
 def test_simplecaps():
     """tests simple caps"""
     # Simple caps is a very vauge test so the checks here are mostly for obviously fixable problem
@@ -547,6 +652,8 @@ def test_simplecaps():
     assert passes(ooochecker.simplecaps, "%STAROFFICE Image", "Tshifanyiso tsha %STAROFFICE")
     assert passes(stdchecker.simplecaps, "Flies, flies, everywhere! Ack!", u"Vlieë, oral vlieë! Jig!")
 
+
+@mark.xfail(reason="FIXME: spell checking test not working")
 def test_spellcheck():
     """tests spell checking"""
     stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="af"))
@@ -559,9 +666,13 @@ def test_spellcheck():
     stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="af"))
     assert fails(stdchecker.spellcheck, "Mozilla is wonderful", "Mozillaaa is wonderlik")
     # We should pass the test if the "error" occurs in the English
-    assert passes(stdchecker.spellcheck, "Mozilla is wonderful", "Mozilla is wonderlik")
+    assert passes(stdchecker.spellcheck, "Mozillaxxx is wonderful", "Mozillaxxx is wonderlik")
     stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="af", notranslatewords=["Mozilla"]))
     assert passes(stdchecker.spellcheck, "Mozilla is wonderful", "Mozilla is wonderlik")
+    # Some variables were still being spell checked
+    mozillachecker = checks.MozillaChecker(checkerconfig=checks.CheckerConfig(targetlanguage="af"))
+    assert passes(mozillachecker.spellcheck, "&brandShortName.labels; is wonderful", "&brandShortName.label; is wonderlik")
+
 
 def test_startcaps():
     """tests starting capitals"""
@@ -590,24 +701,28 @@ def test_startcaps():
     # Unicode further down the Unicode tables
     assert passes(stdchecker.startcaps, "A text enclosed...", u"Ḽiṅwalwa ḽo katelwaho...")
     assert fails(stdchecker.startcaps, "A text enclosed...", u"ḽiṅwalwa ḽo katelwaho...")
-
     # Accelerators
     stdchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&"))
     assert passes(stdchecker.startcaps, "&Find", "Vi&nd")
+    # Numbers - we really can't tell what should happen with numbers, so ignore
+    # source or target that start with a number
+    assert passes(stdchecker.startcaps, "360 degrees", "Grade 360")
+    assert passes(stdchecker.startcaps, "360 degrees", "grade 360")
 
     # Language specific stuff
-    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='af'))
-    assert passes(stdchecker.startcaps, "A cow", "'n Koei")
-    assert passes(stdchecker.startcaps, "A list of ", "'n Lys van ")
+    afchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='af'))
+    assert passes(afchecker.startcaps, "A cow", "'n Koei")
+    assert passes(afchecker.startcaps, "A list of ", "'n Lys van ")
     # should pass:
-    #assert passes(stdchecker.startcaps, "A 1k file", u"'n 1k-lêer")
-    assert passes(stdchecker.startcaps, "'Do it'", "'Doen dit'")
-    assert fails(stdchecker.startcaps, "'Closer than'", "'nader as'")
-    assert passes(stdchecker.startcaps, "List", "Lys")
-    assert passes(stdchecker.startcaps, "a cow", "'n koei")
-    assert fails(stdchecker.startcaps, "a cow", "'n Koei")
-    assert passes(stdchecker.startcaps, "(A cow)", "('n Koei)")
-    assert fails(stdchecker.startcaps, "(a cow)", "('n Koei)")
+    #assert passes(afchecker.startcaps, "A 1k file", u"'n 1k-lêer")
+    assert passes(afchecker.startcaps, "'Do it'", "'Doen dit'")
+    assert fails(afchecker.startcaps, "'Closer than'", "'nader as'")
+    assert passes(afchecker.startcaps, "List", "Lys")
+    assert passes(afchecker.startcaps, "a cow", "'n koei")
+    assert fails(afchecker.startcaps, "a cow", "'n Koei")
+    assert passes(afchecker.startcaps, "(A cow)", "('n Koei)")
+    assert fails(afchecker.startcaps, "(a cow)", "('n Koei)")
+
 
 def test_startpunc():
     """tests startpunc"""
@@ -626,12 +741,14 @@ def test_startpunc():
     stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='af'))
     assert passes(stdchecker.startpunc, "A human?", u"'n Mens?")
 
+
 def test_startwhitespace():
     """tests startwhitespace"""
     stdchecker = checks.StandardChecker()
     assert passes(stdchecker.startwhitespace, "A setence.", "I'm correct.")
     assert fails(stdchecker.startwhitespace, " A setence.", "I'm incorrect.")
 
+
 def test_unchanged():
     """tests unchanged entries"""
     stdchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&"))
@@ -639,12 +756,12 @@ def test_unchanged():
     assert fails(stdchecker.unchanged, "&Unchanged", "Un&changed")
     assert passes(stdchecker.unchanged, "Unchanged", "Changed")
     assert passes(stdchecker.unchanged, "1234", "1234")
-    assert passes(stdchecker.unchanged, "2×2", "2×2") # bug 178, description item 14
+    assert passes(stdchecker.unchanged, "2×2", "2×2")  # bug 178, description item 14
     assert passes(stdchecker.unchanged, "I", "I")
     assert passes(stdchecker.unchanged, "   ", "   ")  # bug 178, description item 5
     assert passes(stdchecker.unchanged, "???", "???")  # bug 178, description item 15
-    assert passes(stdchecker.unchanged, "&ACRONYM", "&ACRONYM") # bug 178, description item 7
-    assert passes(stdchecker.unchanged, "F1", "F1") # bug 178, description item 20
+    assert passes(stdchecker.unchanged, "&ACRONYM", "&ACRONYM")  # bug 178, description item 7
+    assert passes(stdchecker.unchanged, "F1", "F1")  # bug 178, description item 20
     assert fails(stdchecker.unchanged, "Two words", "Two words")
     #TODO: this still fails
 #    assert passes(stdchecker.unchanged, "NOMINAL", "NOMİNAL")
@@ -653,12 +770,19 @@ def test_unchanged():
     # Variable only and variable plus punctuation messages should be ignored
     mozillachecker = checks.MozillaChecker()
     assert passes(mozillachecker.unchanged, "$ProgramName$", "$ProgramName$")
-    assert passes(mozillachecker.unchanged, "$file$ : $dir$", "$file$ : $dir$") # bug 178, description item 13
+    assert passes(mozillachecker.unchanged, "$file$ : $dir$", "$file$ : $dir$")  # bug 178, description item 13
     assert fails(mozillachecker.unchanged, "$file$ in $dir$", "$file$ in $dir$")
     assert passes(mozillachecker.unchanged, "&brandShortName;", "&brandShortName;")
     # Don't translate words should be ignored
     stdchecker = checks.StandardChecker(checks.CheckerConfig(notranslatewords=["Mozilla"]))
-    assert passes(stdchecker.unchanged, "Mozilla", "Mozilla") # bug 178, description item 10
+    assert passes(stdchecker.unchanged, "Mozilla", "Mozilla")  # bug 178, description item 10
+    # Don't fail unchanged if the entry is a dialogsize, quite plausible that you won't change it
+    mozillachecker = checks.MozillaChecker()
+    assert passes(mozillachecker.unchanged, 'width: 12em;', 'width: 12em;')
+    assert fails(stdchecker.unchanged, 'width: 12em;', 'width: 12em;')
+    assert passes(mozillachecker.unchanged, '7em', '7em')
+    assert fails(stdchecker.unchanged, '7em', '7em')
+
 
 def test_untranslated():
     """tests untranslated entries"""
@@ -668,6 +792,7 @@ def test_untranslated():
     # KDE comments that make it into translations should not mask untranslated test
     assert fails(stdchecker.untranslated, "_: KDE comment\\n\nI am untranslated", "_: KDE comment\\n\n")
 
+
 def test_validchars():
     """tests valid characters"""
     stdchecker = checks.StandardChecker(checks.CheckerConfig())
@@ -685,6 +810,7 @@ def test_validchars():
     assert passes(stdchecker.validchars, "This sentence contains valid chars", u"\u1e3c")
     assert passes(stdchecker.validchars, "This sentence contains valid chars", u"\u004c\u032d")
 
+
 def test_variables_kde():
     """tests variables in KDE translations"""
     # GNOME variables
@@ -692,6 +818,7 @@ def test_variables_kde():
     assert passes(kdechecker.variables, "%d files of type %s saved.", "%d leers van %s tipe gestoor.")
     assert fails_serious(kdechecker.variables, "%d files of type %s saved.", "%s leers van %s tipe gestoor.")
 
+
 def test_variables_gnome():
     """tests variables in GNOME translations"""
     # GNOME variables
@@ -701,6 +828,7 @@ def test_variables_gnome():
     assert passes(gnomechecker.variables, "Save $(file)", "Stoor $(file)")
     assert fails_serious(gnomechecker.variables, "Save $(file)", "Stoor $(leer)")
 
+
 def test_variables_mozilla():
     """tests variables in Mozilla translations"""
     # Mozilla variables
@@ -723,6 +851,8 @@ def test_variables_mozilla():
     assert fails_serious(mozillachecker.variables, "About $_CLICK and more", "Oor $_KLIK en meer")
     assert passes(mozillachecker.variables, "About $(^NameDA)", "Oor $(^NameDA)")
     assert fails_serious(mozillachecker.variables, "About $(^NameDA)", "Oor $(^NaamDA)")
+    assert passes(mozillachecker.variables, "Open {{pageCount}} pages", "Make {{pageCount}} bladsye oop")
+    assert fails_serious(mozillachecker.variables, "Open {{pageCount}} pages", "Make {{bladTelling}} bladsye oop")
     # Double variable problem
     assert fails_serious(mozillachecker.variables, "Create In &lt;&lt;", "Etsa ka Ho &lt;lt;")
     # Variables at the end of a sentence
@@ -738,6 +868,7 @@ def test_variables_mozilla():
     # We shouldn't mask variables within variables.  This should highlight &brandShortName as missing and &amp as extra
     assert fails_serious(mozillachecker.variables, "&brandShortName;", "&amp;brandShortName;")
 
+
 def test_variables_openoffice():
     """tests variables in OpenOffice translations"""
     # OpenOffice.org variables
@@ -772,9 +903,12 @@ def test_variables_openoffice():
     assert fails_serious(ooochecker.variables, "Save [file]", "Stoor [leer]")
     assert passes(ooochecker.variables, "Save $file", "Stoor $file")
     assert fails_serious(ooochecker.variables, "Save $file", "Stoor $leer")
+    assert passes(ooochecker.variables, "Use @EXTENSION@", "Gebruik @EXTENSION@")
+    assert fails_serious(ooochecker.variables, "Use @EXTENSUION@", "Gebruik @UITBRUIDING@")
     # Same variable name twice
     assert fails_serious(ooochecker.variables, r"""Start %PROGRAMNAME% as %PROGRAMNAME%""", "Begin %PROGRAMNAME%")
 
+
 def test_variables_cclicense():
     """Tests variables in Creative Commons translations."""
     checker = checks.CCLicenseChecker()
@@ -786,6 +920,7 @@ def test_variables_cclicense():
     assert fails_serious(checker.variables, "The @license_name_full@ is", "Die @iiilicense_name_full@ is")
     assert fails_serious(checker.variables, "A @ccvar@", "'n @ccvertaaldeveranderlike@")
 
+
 def test_xmltags():
     """tests xml tags"""
     stdchecker = checks.StandardChecker()
@@ -800,13 +935,34 @@ def test_xmltags():
 
     assert fails(stdchecker.xmltags, "Click <a href=\"page.html\">", "Klik <a hverw=\"page.html\">")
     assert passes(stdchecker.xmltags, "Click <a xml-lang=\"en\" href=\"page.html\">", "Klik <a xml-lang=\"af\" href=\"page.html\">")
+    assert passes(stdchecker.xmltags, "Click <div lang=\"en\" dir=\"ltr\">", "Klik <div lang=\"ar\" dir=\"rtl\">")
     assert fails(stdchecker.xmltags, "Click <a href=\"page.html\" target=\"koei\">", "Klik <a href=\"page.html\">")
     assert fails(stdchecker.xmltags, "<b>Current Translation</b>", "<b>Traducción Actual:<b>")
     assert passes(stdchecker.xmltags, "<Error>", "<Fout>")
     assert fails(stdchecker.xmltags, "%d/%d translated\n(%d blank, %d fuzzy)", "<br>%d/%d μεταφρασμένα\n<br>(%d κενά, %d ασαφή)")
+    assert fails(stdchecker.xmltags, '(and <a href="http://www.schoolforge.net/education-software" class="external">other open source software</a>)', '(en <a href="http://www.schoolforge.net/education-software" class="external">ander Vry Sagteware</a)')
+    assert fails(stdchecker.xmltags, 'Because Tux Paint (and <a href="http://www.schoolforge.net/education-software" class="external">other open source software</a>) is free of cost and not limited in any way, a school can use it <i>today</i>, without waiting for procurement or a budget!', 'Omdat Tux Paint (en <a href="http://www.schoolforge.net/education-software" class="external">ander Vry Sagteware</a)gratis is en nie beperk is op enige manier nie, kan \'n skool dit vandag</i> gebruik sonder om te wag vir goedkeuring of \'n begroting!')
+    assert fails(stdchecker.xmltags, "test <br />", "test <br>")
+    assert fails(stdchecker.xmltags, "test <img src='foo.jpg'/ >", "test <img src='foo.jpg'  >")
+
+    # This used to cause an error (traceback), because of mismatch between
+    # different regular expressions (because of the newlines)
+    assert passes(stdchecker.xmltags, '''<markup>
+<span weight="bold" size="large"
+style="oblique">
+Can't create server !
+</span>
+</markup>''',
+                                    '''<markup>
+<span weight="bold" size="large"
+style="oblique">
+No s'ha pogut crear el servidor
+</span>
+</markup>''')
     frchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="fr"))
     assert fails(frchecker.xmltags, "Click <a href=\"page.html\">", "Klik <a href=« page.html »>")
 
+
 def test_ooxmltags():
     """Tests the xml tags in OpenOffice.org translations for quality as done in gsicheck"""
     ooochecker = checks.OpenOfficeChecker()
@@ -823,6 +979,12 @@ def test_ooxmltags():
     assert passes(ooochecker.xmltags, "<link name=\"John\">", "<link name=\"Jan\">")
     assert fails(ooochecker.xmltags, "<link name=\"John\">", "<link naam=\"Jan\">")
 
+    # Reported OOo error
+    ## Bug 1910
+    assert fails(ooochecker.xmltags, u"""<variable id="FehlendesElement">In a database file window, click the <emph>Queries</emph> icon, then choose <emph>Edit - Edit</emph>. When referenced fields no longer exist, you see this dialog</variable>""", u"""<variable id="FehlendesElement">Dans  une fenêtre de fichier de base de données, cliquez sur l'icône <emph>Requêtes</emph>, puis choisissez <emph>Éditer - Éditer</emp>. Lorsque les champs de référence n'existent plus, vous voyez cette boîte de dialogue</variable>""")
+    assert fails(ooochecker.xmltags, "<variable> <emph></emph> <emph></emph> </variable>", "<variable> <emph></emph> <emph></emp> </variable>")
+
+
 def test_functions():
     """tests to see that funtions() are not translated"""
     stdchecker = checks.StandardChecker()
@@ -837,18 +999,21 @@ def test_functions():
     assert passes(stdchecker.functions, "or domain().", "domain() verwag.")
     assert passes(stdchecker.functions, "Expected url(), url-prefix(), or domain().", "url(), url-prefix() of domain() verwag.")
 
+
 def test_emails():
     """tests to see that email addresses are not translated"""
     stdchecker = checks.StandardChecker()
     assert fails(stdchecker.emails, "blah bob@example.net blah", "blee kobus@voorbeeld.net blee")
     assert passes(stdchecker.emails, "blah bob@example.net blah", "blee bob@example.net blee")
 
+
 def test_urls():
     """tests to see that URLs are not translated"""
     stdchecker = checks.StandardChecker()
     assert fails(stdchecker.urls, "blah http://translate.org.za blah", "blee http://vertaal.org.za blee")
     assert passes(stdchecker.urls, "blah http://translate.org.za blah", "blee http://translate.org.za blee")
 
+
 def test_simpleplurals():
     """test that we can find English style plural(s)"""
     stdchecker = checks.StandardChecker()
@@ -861,6 +1026,7 @@ def test_simpleplurals():
     assert passes(stdchecker.simpleplurals, "computer(s)", u"Máy tính")
     assert fails(stdchecker.simpleplurals, "computer(s)", u"Máy tính(s)")
 
+
 def test_nplurals():
     """Test that we can find the wrong number of plural forms. Note that this
     test uses a UnitChecker, not a translation checker."""
@@ -901,6 +1067,7 @@ def test_nplurals():
     unit.target = [u"%d ឯកសារ", u"%d lêers"]
     assert not checker.nplurals(unit)
 
+
 def test_credits():
     """tests credits"""
     stdchecker = checks.StandardChecker()
@@ -922,6 +1089,7 @@ def test_credits():
     assert passes(gnomechecker.credits, "Your names", "Ekke, ekke!")
     assert passes(gnomechecker.credits, "ROLES_OF_TRANSLATORS", "Ekke, ekke!")
 
+
 def test_gconf():
     """test GNOME gconf errors"""
     gnomechecker = checks.GnomeChecker()
@@ -933,3 +1101,46 @@ def test_gconf():
     gnomechecker.locations = ['file.schemas.in.h:24']
     assert passes(gnomechecker.gconf, 'Blah "gconf_setting"', 'Bleh "gconf_setting"')
     assert fails(gnomechecker.gconf, 'Blah "gconf_setting"', 'Bleh "gconf_steling"')
+    # redo the same, but with the new location comment:
+    gnomechecker.locations = ['file.gschema.xml.in.in.h:24']
+    assert passes(gnomechecker.gconf, 'Blah "gconf_setting"', 'Bleh "gconf_setting"')
+    assert fails(gnomechecker.gconf, 'Blah "gconf_setting"', 'Bleh "gconf_steling"')
+
+
+def test_hassuggestion():
+    """test that hassuggestion() works"""
+    checker = checks.StandardUnitChecker()
+
+    po_store = po.pofile()
+    po_store.addsourceunit("koeie")
+    assert checker.hassuggestion(po_store.units[-1])
+
+    xliff_store = xliff.xlifffile.parsestring('''
+<xliff version='1.2'
+       xmlns='urn:oasis:names:tc:xliff:document:1.2'>
+<file original='hello.txt' source-language='en' target-language='fr' datatype='plaintext'>
+<body>
+    <trans-unit id='hi'>
+        <source>Hello world</source>
+        <target>Bonjour le monde</target>
+        <alt-trans>
+            <target xml:lang='es'>Hola mundo</target>
+        </alt-trans>
+    </trans-unit>
+</body>
+</file>
+</xliff>
+''')
+    assert not checker.hassuggestion(xliff_store.units[0])
+
+
+def test_dialogsizes():
+    """test Mozilla dialog sizes"""
+    mozillachecker = checks.MozillaChecker()
+    assert passes(mozillachecker.dialogsizes, 'width: 12em;', 'width: 12em;')
+    assert passes(mozillachecker.dialogsizes, 'width: 12em; height: 36em', 'width: 12em; height: 36em')
+    assert fails(mozillachecker.dialogsizes, 'height: 12em;', 'hoogde: 12em;')
+    assert passes(mozillachecker.dialogsizes, 'height: 12em;', 'height: 24px;')
+    assert fails(mozillachecker.dialogsizes, 'height: 12em;', 'height: 24xx;')
+    assert fails(mozillachecker.dialogsizes, 'height: 12.5em;', 'height: 12,5em;')
+    assert fails(mozillachecker.dialogsizes, 'width: 36em; height: 18em;', 'width: 30em; min-height: 20em;')
