@@ -16,33 +16,38 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""This module represents Afrikaans language.
+"""This module represents the Afrikaans language.
 
-For more information, see U{http://en.wikipedia.org/wiki/Afrikaans_language}
+.. seealso:: http://en.wikipedia.org/wiki/Afrikaans_language
 """
 
-from translate.lang import common
 import re
 
+from translate.lang import common
+
 articlere = re.compile(r"'n\b")
 
 
 class af(common.Common):
     """This class represents Afrikaans."""
+    validdoublewords = [u"u"]
 
     punctuation = u"".join([common.Common.commonpunc, common.Common.quotes,
                             common.Common.miscpunc])
     sentenceend = u".!?…"
-    sentencere = re.compile(r"""(?s)    #make . also match newlines
-                            .*?         #anything, but match non-greedy
-                            [%s]        #the puntuation for sentence ending
-                            \s+         #the spacing after the puntuation
-                            (?='n\s[A-Z]|[^'a-z\d]|'[^n])
-                            #lookahead that next part starts with caps or 'n followed by caps
-                            """ % sentenceend, re.VERBOSE)
+    sentencere = re.compile(r"""
+        (?s)        # make . also match newlines
+        .*?         # anything, but match non-greedy
+        [%s]        # the puntuation for sentence ending
+        \s+         # the spacing after the puntuation
+        (?='n\s[A-Z]|[^'a-z\d]|'[^n])
+        # lookahead that next part starts with caps or 'n followed by caps
+        """ % sentenceend, re.VERBOSE
+    )
+
+    specialchars = u"ëïêôûáéíóúý"
 
     def capsstart(cls, text):
         """Modify this for the indefinite article ('n)."""
@@ -64,8 +69,8 @@ cyr2lat = {
    u"Г": "G", u"г": "g",  # see rule 3 and 4
    u"Д": "D", u"д": "d",
    u"ДЖ": "Dj", u"дж": "dj",
-   u"Е": "Je", u"е": "je", # Sometimes e need to check when/why see rule 5.
-   u"Ё": "Jo", u"ё": "jo", # see rule 6
+   u"Е": "Je", u"е": "je",  # Sometimes e need to check when/why see rule 5.
+   u"Ё": "Jo", u"ё": "jo",  # see rule 6
    u"ЕЙ": "Ei", u"ей": "ei",
    u"Ж": "Zj", u"ж": "zj",
    u"З": "Z", u"з": "z",
@@ -78,18 +83,18 @@ cyr2lat = {
    u"О": "O", u"о": "o",
    u"П": "P", u"п": "p",
    u"Р": "R", u"р": "r",
-   u"С": "S", u"с": "s", # see note 12
+   u"С": "S", u"с": "s",  # see note 12
    u"Т": "T", u"т": "t",
    u"У": "Oe", u"у": "oe",
    u"Ф": "F", u"ф": "f",
-   u"Х": "Ch", u"х": "ch", # see rule 12
+   u"Х": "Ch", u"х": "ch",  # see rule 12
    u"Ц": "Ts", u"ц": "ts",
    u"Ч": "Tj", u"ч": "tj",
    u"Ш": "Sj", u"ш": "sj",
    u"Щ": "Sjtsj", u"щ": "sjtsj",
-   u"Ы": "I", u"ы": "i", # see note 13
+   u"Ы": "I", u"ы": "i",  # see note 13
    u"Ъ": "", u"ъ": "",  # See note 14
-   u"Ь": "", u"ь": "", # this letter is not in the AWS we assume it is left out as in the previous letter
+   u"Ь": "", u"ь": "",  # this letter is not in the AWS we assume it is left out as in the previous letter
    u"Э": "E", u"э": "e",
    u"Ю": "Joe", u"ю": "joe",
    u"Я": "Ja", u"я": "ja",
