@@ -1,73 +1,81 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
+#
 # Copyright 2007-2008 Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """This module contains all the common features for languages.
 
-   Supported features
-   ==================
-     - language code (km, af)
-     - language name (Khmer, Afrikaans)
-     - Plurals
-       - Number of plurals (nplurals)
-       - Plural equation
-     - pofilter tests to ignore
-   
-   Segmentation
-   ------------
-     - characters
-     - words
-     - sentences
-   
-   TODOs and Ideas for possible features
-   =====================================
-     - Language-Team information
-     - Segmentation
-       - phrases
-   
-   Punctuation
-   -----------
-     - End of sentence
-     - Start of sentence
-     - Middle of sentence
-     - Quotes
-       - single
-       - double
-   
-     - Valid characters
-     - Accelerator characters
-     - Special characters
-     - Direction (rtl or ltr)
+Supported features:
+
+- language code (km, af)
+- language name (Khmer, Afrikaans)
+- Plurals
+
+  - Number of plurals (nplurals)
+  - Plural equation
+
+- pofilter tests to ignore
+
+Segmentation:
+
+- characters
+- words
+- sentences
+
+Punctuation:
+
+- End of sentence
+- Start of sentence
+- Middle of sentence
+- Quotes
+
+  - single
+  - double
+
+- Valid characters
+- Accelerator characters
+- Special characters
+- Direction (rtl or ltr)
+
+TODOs and Ideas for possible features:
+
+- Language-Team information
+- Segmentation
+
+  - phrases
 """
 
-from translate.lang import data
 import re
+import logging
+
+from translate.lang import data
+
+logger = logging.getLogger(__name__)
 
 class Common(object):
     """This class is the common parent class for all language classes."""
-    
+
     code = ""
-    """The ISO 639 language code, possibly with a country specifier or other 
+    """The ISO 639 language code, possibly with a country specifier or other
     modifier.
-    
+
     Examples::
+
         km
         pt_BR
         sr_YU@Latn
@@ -76,51 +84,64 @@ class Common(object):
     fullname = ""
     """The full (English) name of this language.
 
-       Dialect codes should have the form of 
-         - Khmer
-         - Portugese (Brazil)
-         - TODO: sr_YU@Latn?
+    Dialect codes should have the form of:
+
+    - Khmer
+    - Portugese (Brazil)
+    - TODO: sr_YU@Latn?
     """
-    
+
     nplurals = 0
     """The number of plural forms of this language.
-    
+
     0 is not a valid value - it must be overridden.
     Any positive integer is valid (it should probably be between 1 and 6)
-    @see: L{data}
+
+    .. seealso:: :mod:`translate.lang.data`
     """
-    
+
     pluralequation = "0"
-    """The plural equation for selection of plural forms. 
+    """The plural equation for selection of plural forms.
 
     This is used for PO files to fill into the header.
-    @see: U{Gettext manual<http://www.gnu.org/software/gettext/manual/html_node/gettext_150.html#Plural-forms>}
-    @see: L{data}
+
+    .. seealso::
+
+       `Gettext manual <http://www.gnu.org/software/gettext/manual/html_node/gettext_150.html#Plural-forms>`_, :mod:`translate.lang.data`
     """
     # Don't change these defaults of nplurals or pluralequation willy-nilly:
     # some code probably depends on these for unrecognised languages
-    
+
+    mozilla_nplurals = 0
+    mozilla_pluralequation = "0"
+    """This of languages that has different plural formula in Mozilla than the
+    standard one in Gettext."""
+
     listseperator = u", "
-    """This string is used to separate lists of textual elements. Most 
+    """This string is used to separate lists of textual elements. Most
     languages probably can stick with the default comma, but Arabic and some
     Asian languages might want to override this."""
-    
+
+    specialchars = u""
+    """Characters used by the language that might not be easy to input with
+    common keyboard layouts"""
+
     commonpunc = u".,;:!?-@#$%^*_()[]{}/\\'`\"<>"
-    """These punctuation marks are common in English and most languages that 
+    """These punctuation marks are common in English and most languages that
     use latin script."""
 
     quotes = u"‘’‛“”„‟′″‴‵‶‷‹›«»"
     """These are different quotation marks used by various languages."""
 
     invertedpunc = u"¿¡"
-    """Inveted punctuation sometimes used at the beginning of sentences in 
+    """Inverted punctuation sometimes used at the beginning of sentences in
     Spanish, Asturian, Galician, and Catalan."""
 
     rtlpunc = u"،؟؛÷"
     """These punctuation marks are used by Arabic and Persian, for example."""
 
     CJKpunc = u"。、，；！？「」『』【】"
-    """These punctuation marks are used in certain circumstances with CJK 
+    """These punctuation marks are used in certain circumstances with CJK
     languages."""
 
     indicpunc = u"।॥॰"
@@ -134,28 +155,30 @@ class Common(object):
 
     punctuation = u"".join([commonpunc, quotes, invertedpunc, rtlpunc, CJKpunc,\
             indicpunc, ethiopicpunc, miscpunc])
-    """We include many types of punctuation here, simply since this is only 
-    meant to determine if something is punctuation. Hopefully we catch some 
-    languages which might not be represented with modules. Most languages won't 
+    """We include many types of punctuation here, simply since this is only
+    meant to determine if something is punctuation. Hopefully we catch some
+    languages which might not be represented with modules. Most languages won't
     need to override this."""
 
-    sentenceend = u".!?…։؟।。！？።"
-    """These marks can indicate a sentence end. Once again we try to account 
+    sentenceend = u".!?…։؟।。！？።\u06d4"
+    """These marks can indicate a sentence end. Once again we try to account
     for many languages. Most langauges won't need to override this."""
 
-    #The following tries to account for a lot of things. For the best idea of 
-    #what works, see test_common.py. We try to ignore abbreviations, for 
-    #example, by checking that the following sentence doesn't start with lower 
+    #The following tries to account for a lot of things. For the best idea of
+    #what works, see test_common.py. We try to ignore abbreviations, for
+    #example, by checking that the following sentence doesn't start with lower
     #case or numbers.
-    sentencere = re.compile(r"""(?s)    #make . also match newlines
-                            .*?         #anything, but match non-greedy
-                            [%s]        #the puntuation for sentence ending
-                            \s+         #the spacing after the puntuation
-                            (?=[^a-z\d])#lookahead that next part starts with caps
-                            """ % sentenceend, re.VERBOSE)
-    
+    sentencere = re.compile(ur"""
+        (?s)        # make . also match newlines
+        .*?         # anything, but match non-greedy
+        [%s]        # the puntuation for sentence ending
+        \s+         # the spacing after the puntuation
+        (?=[^a-zа-џ\d])  # lookahead that next part starts with caps
+        """ % sentenceend, re.VERBOSE | re.UNICODE
+    )
+
     puncdict = {}
-    """A dictionary of punctuation transformation rules that can be used by 
+    """A dictionary of punctuation transformation rules that can be used by
     punctranslate()."""
 
     ignoretests = []
@@ -169,8 +192,8 @@ class Common(object):
     _languages = {}
 
     validaccel = None
-    """Characters that can be used as accelerators (access keys) i.e. Alt+X 
-    where X is the accelerator.  These can include combining diacritics as 
+    """Characters that can be used as accelerators (access keys) i.e. Alt+X
+    where X is the accelerator.  These can include combining diacritics as
     long as they are accessible from the users keyboard in a single keystroke,
     but normally they would be at least precomposed characters. All characters,
     lower and upper, are included in the list."""
@@ -180,7 +203,7 @@ class Common(object):
     of such words."""
 
     def __new__(cls, code):
-        """This returns the language class for the given code, following a 
+        """This returns the language class for the given code, following a
         singleton like approach (only one object per language)."""
         code = code or ""
         # First see if a language object for this code already exists
@@ -191,14 +214,12 @@ class Common(object):
 
         language.code = code
         while code:
-            langdata = data.languages.get(code, None)
+            langdata = data.get_language(code)
             if langdata:
-                language.fullname, language.nplurals, language.pluralequation = langdata
+                language.fullname, language.nplurals, \
+                    language.pluralequation = langdata
                 break
             code = data.simplercode(code)
-        if not code:
-#            print >> sys.stderr, "Warning: No information found about language code %s" % code
-            pass
         return language
 
     def __deepcopy__(self, memo={}):
@@ -206,7 +227,7 @@ class Common(object):
         return self
 
     def __repr__(self):
-        """Give a simple string representation without address information to 
+        """Give a simple string representation without address information to
         be able to store it in text for comparison later."""
         detail = ""
         if self.code:
@@ -214,9 +235,9 @@ class Common(object):
         return "<class 'translate.lang.common.Common%s'>" % detail
 
     def punctranslate(cls, text):
-        """Converts the punctuation in a string according to the rules of the 
+        """Converts the punctuation in a string according to the rules of the
         language."""
-#        TODO: look at po::escapeforpo() for performance idea
+        #TODO: look at po::escapeforpo() for performance idea
         if not text:
             return text
         ellipses_end = text.endswith(u"...")
@@ -229,12 +250,13 @@ class Common(object):
                 text += cls.puncdict[u"..."]
             else:
                 text += u"..."
-        # Let's account for cases where a punctuation symbol plus a space is 
-        # replaced, but the space won't exist at the end of a message.
+        # Let's account for cases where a punctuation symbol plus a space is
+        # replaced, but the space won't exist at the end of the source message.
         # As a simple improvement for messages ending in ellipses (...), we
         # test that the last character is different from the second last
         # This is only relevant if the string has two characters or more
-        if (text[-1] + u" " in cls.puncdict) and (len(text) < 2 or text[-2] != text[-1]):
+        if ((text[-1] + u" " in cls.puncdict) and
+            (len(text) < 2 or text[-2] != text[-1])):
             text = text[:-1] + cls.puncdict[text[-1] + u" "].rstrip()
         return text
     punctranslate = classmethod(punctranslate)
@@ -252,16 +274,17 @@ class Common(object):
                 break
             code = data.simplercode(code)
         else:
-            expansion_factor = 0.1 # default
-        constant = max(5, int(40*expansion_factor))
+            expansion_factor = 0.1  # default
+        constant = max(5, int(40 * expansion_factor))
         # The default: return 5 + length/10
         return constant + int(expansion_factor * length)
     length_difference = classmethod(length_difference)
 
     def alter_length(cls, text):
-        """Converts the given string by adding or removing characters as an 
-        estimation of translation length (with English assumed as source 
+        """Converts the given string by adding or removing characters as an
+        estimation of translation length (with English assumed as source
         language)."""
+
         def alter_it(text):
             l = len(text)
             if l > 9:
@@ -328,7 +351,7 @@ class Common(object):
     sentence_iter = classmethod(sentence_iter)
 
     def sentences(cls, text, strip=True):
-        """Returns a list of senteces in text."""
+        """Returns a list of sentences in text."""
         return [s for s in cls.sentence_iter(text, strip=strip)]
     sentences = classmethod(sentences)
 
@@ -338,3 +361,8 @@ class Common(object):
         return stripped and stripped[0].isupper()
     capsstart = classmethod(capsstart)
 
+    def numstart(cls, text):
+        """Determines whether the text starts with a numeric value."""
+        stripped = text.lstrip().lstrip(cls.punctuation)
+        return stripped and stripped[0].isnumeric()
+    numstart = classmethod(numstart)
