@@ -3,6 +3,9 @@
 
 from translate.lang import common
 
+from pytest import mark
+
+
 def test_characters():
     """Test the basic characters segmentation"""
     language = common.Common
@@ -12,6 +15,7 @@ def test_characters():
     # Spaces are compacted, source has 2 returned has only one
     assert language.characters(u"A  B") == [u"A", u" ", u"B"]
 
+
 def test_words():
     """Tests basic functionality of word segmentation."""
     language = common.Common
@@ -21,10 +25,6 @@ def test_words():
     words = language.words(u"test sentence.")
     assert words == [u"test", u"sentence"]
 
-    # Let's test Khmer with zero width space (\u200b)
-    words = language.words(u"ផ្ដល់​យោបល់")
-    assert words == [u"ផ្ដល់", u"យោបល់"]
-
     words = language.words(u"This is a weird test .")
     assert words == [u"This", u"is", u"a", u"weird", u"test"]
 
@@ -34,6 +34,21 @@ def test_words():
     words = language.words(u"Don’t send e-mail!")
     assert words == [u"Don’t", u"send", u"e-mail"]
 
+
+@mark.xfail("sys.version_info >= (2, 6)",
+            reason="ZWS "
+	    "is not considered a space in Python 2.6+. Khmer should extend "
+	    "words() to include \\u200b in addition to other word breakers.")
+def test_word_khmer():
+    language = common.Common
+    # Let's test Khmer with zero width space (\u200b)
+    words = language.words(u"ផ្ដល់​យោបល់")
+    print u"ផ្ដល់​យោបល់"
+    print language.words(u"ផ្ដល់<200b>យោបល់")
+    print [u"ផ្ដល់", u"យោបល់"]
+    assert words == [u"ផ្ដល់", u"យោបល់"]
+
+
 def test_sentences():
     """Tests basic functionality of sentence segmentation."""
     language = common.Common
@@ -68,6 +83,7 @@ def test_sentences():
     sentences = language.sentences(u"Doen dit d.m.v. koeie.")
     assert sentences == [u"Doen dit d.m.v. koeie."]
 
+
 def test_capsstart():
     """Tests for basic sane behaviour in startcaps()."""
     language = common.Common
@@ -77,9 +93,33 @@ def test_capsstart():
     assert not language.capsstart(":")
     assert not language.capsstart("")
 
+
+def test_numstart():
+    """Tests for basic sane behaviour in startcaps()."""
+    language = common.Common
+    assert language.numstart("360 degress")
+    assert language.numstart("3D file")
+    assert not language.numstart("Open 360 degrees")
+    assert not language.numstart(":")
+    assert not language.numstart("")
+
+
 def test_punctranslate():
     """Test the basic punctranslate function"""
     language = common.Common
     assert not language.punctranslate(u"A...") == u"A…"
     language.puncdict = {u"...": u"…"}
     assert language.punctranslate(u"A...") == u"A…"
+
+
+def test_length_difference():
+    """Test the heuristics of the length difference function"""
+    # Expansion with no code
+    assert common.Common.length_difference(10) == 6
+    assert common.Common.length_difference(100) == 15
+    assert common.Common.length_difference(300) == 35
+
+
+def test_alter_length():
+    """Test that we create the correct length by adding or removing characters"""
+    assert common.Common.alter_length("One two three") == "One twOne two three"
