@@ -1,25 +1,26 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-"""Implements a case-insensitive (on keys) dictionary and order-sensitive dictionary"""
+"""Implements a case-insensitive (on keys) dictionary and
+order-sensitive dictionary"""
 
 # Copyright 2002, 2003 St James Software
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
+
 
 def generalupper(str):
     """this uses the object's upper method - works with string and unicode"""
@@ -27,15 +28,18 @@ def generalupper(str):
         return str
     return str.upper()
 
+
 class cidict(dict):
-    def __init__(self, fromdict = None):
+
+    def __init__(self, fromdict=None):
         """constructs the cidict, optionally using another dict to do so"""
         if fromdict is not None:
             self.update(fromdict)
 
     def __getitem__(self, key):
         if type(key) != str and type(key) != unicode:
-            raise TypeError, "cidict can only have str or unicode as key (got %r)" % type(key)
+            raise TypeError("cidict can only have str or unicode as key (got %r)" %
+                            type(key))
         for akey in self.iterkeys():
             if akey.lower() == key.lower():
                 return dict.__getitem__(self, akey)
@@ -43,20 +47,23 @@ class cidict(dict):
 
     def __setitem__(self, key, value):
         if type(key) != str and type(key) != unicode:
-            raise TypeError, "cidict can only have str or unicode as key (got %r)" % type(key)
+            raise TypeError("cidict can only have str or unicode as key (got %r)" %
+                            type(key))
         for akey in self.iterkeys():
             if akey.lower() == key.lower():
                 return dict.__setitem__(self, akey, value)
         return dict.__setitem__(self, key, value)
 
     def update(self, updatedict):
-        """D.update(E) -> None.  Update D from E: for k in E.keys(): D[k] = E[k]"""
+        """D.update(E) -> None.
+           Update D from E: for k in E.keys(): D[k] = E[k]"""
         for key, value in updatedict.iteritems():
             self[key] = value
 
     def __delitem__(self, key):
         if type(key) != str and type(key) != unicode:
-            raise TypeError, "cidict can only have str or unicode as key (got %r)" % type(key)
+            raise TypeError("cidict can only have str or unicode as key (got %r)" %
+                            type(key))
         for akey in self.iterkeys():
             if akey.lower() == key.lower():
                 return dict.__delitem__(self, akey)
@@ -64,7 +71,8 @@ class cidict(dict):
 
     def __contains__(self, key):
         if type(key) != str and type(key) != unicode:
-            raise TypeError, "cidict can only have str or unicode as key (got %r)" % type(key)
+            raise TypeError("cidict can only have str or unicode as key (got %r)" %
+                            type(key))
         for akey in self.iterkeys():
             if akey.lower() == key.lower():
                 return 1
@@ -74,19 +82,23 @@ class cidict(dict):
         return self.__contains__(key)
 
     def get(self, key, default=None):
-        if self.has_key(key):
+        if key in self:
             return self[key]
         else:
             return default
 
+
 class ordereddict(dict):
-    """a dictionary which remembers its keys in the order in which they were given"""
+    """a dictionary which remembers its keys in the order in which they
+    were given"""
+
     def __init__(self, *args):
         if len(args) == 0:
             super(ordereddict, self).__init__()
             self.order = []
         elif len(args) > 1:
-            raise TypeError("ordereddict() takes at most 1 argument (%d given)" % len(args))
+            raise TypeError("ordereddict() takes at most 1 argument (%d given)" %
+                            len(args))
         else:
             initarg = args[0]
             apply(super(ordereddict, self).__init__, args)
@@ -109,7 +121,8 @@ class ordereddict(dict):
         return result
 
     def update(self, updatedict):
-        """D.update(E) -> None.  Update D from E: for k in E.keys(): D[k] = E[k]"""
+        """D.update(E) -> None.
+        Update D from E: for k in E.keys(): D[k] = E[k]"""
         for key, value in updatedict.iteritems():
             self[key] = value
 
@@ -152,7 +165,8 @@ class ordereddict(dict):
         return self.order[:]
 
     def popitem(self):
-        """D.popitem() -> (k, v), remove and return some (key, value) pair as a 2-tuple; but raise KeyError if D is empty"""
+        """D.popitem() -> (k, v), remove and return some (key, value) pair
+        as a 2-tuple; but raise KeyError if D is empty"""
         if len(self.order) == 0:
             raise KeyError("popitem(): ordered dictionary is empty")
         k = self.order.pop()
