@@ -1,36 +1,39 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
+#
 # Copyright 2002-2006 Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
+import re
 import sys
 import os.path
 import fnmatch
+import logging
 import traceback
 import optparse
-from translate.misc import progressbar
-from translate import __version__
 try:
     from cStringIO import StringIO
 except ImportError:
     from StringIO import StringIO
 
+from translate.misc import progressbar
+from translate import __version__
+
+
 class ManPageOption(optparse.Option, object):
     ACTIONS = optparse.Option.ACTIONS + ("manpage",)
 
@@ -39,10 +42,13 @@ class ManPageOption(optparse.Option, object):
         if action == "manpage":
             parser.print_manpage()
             sys.exit(0)
-        return super(ManPageOption, self).take_action(action, dest, opt, value, values, parser)
+        return super(ManPageOption, self).take_action(action, dest, opt, value,
+                                                      values, parser)
 
-class ManHelpFormatter (optparse.HelpFormatter):
-    def __init__ (self,
+
+class ManHelpFormatter(optparse.HelpFormatter):
+
+    def __init__(self,
             indent_increment=0,
             max_help_position=0,
             width=80,
@@ -50,7 +56,7 @@ class ManHelpFormatter (optparse.HelpFormatter):
         optparse.HelpFormatter.__init__(
             self, indent_increment, max_help_position, width, short_first)
 
-    def format_option_strings (self, option):
+    def format_option_strings(self, option):
         """Return a comma-separated list of option strings & metavariables."""
         if option.takes_value():
             metavar = option.metavar or option.dest.upper()
@@ -68,32 +74,37 @@ class ManHelpFormatter (optparse.HelpFormatter):
 
         return '\\fB%s\\fP' % ("\\fR, \\fP".join(opts))
 
+
 class RecursiveOptionParser(optparse.OptionParser, object):
     """A specialized Option Parser for recursing through directories."""
 
-    def __init__(self, formats, usetemplates=False, allowmissingtemplate=False, description=None):
+    def __init__(self, formats, usetemplates=False, allowmissingtemplate=False,
+                 description=None):
         """Construct the specialized Option Parser.
 
-        @type formats: Dictionary
-        @param formats: See L{setformats()} for an explanation of the formats parameter.
-
+        :type formats: Dictionary
+        :param formats: See :meth:`~.RecursiveOptionParser.setformats`
+        for an explanation of the formats parameter.
         """
 
-        optparse.OptionParser.__init__(self, version="%prog "+__version__.sver, description=description)
+        optparse.OptionParser.__init__(self, version="%prog " + __version__.sver,
+                                       description=description)
         self.setmanpageoption()
         self.setprogressoptions()
         self.seterrorleveloptions()
         self.setformats(formats, usetemplates)
-        self.setpsycooption()
         self.passthrough = []
         self.allowmissingtemplate = allowmissingtemplate
+        logging.basicConfig(format="%(name)s: %(levelname)s: %(message)s")
 
     def get_prog_name(self):
         return os.path.basename(sys.argv[0])
 
     def setmanpageoption(self):
-        """creates a manpage option that allows the optionparser to generate a manpage"""
-        manpageoption = ManPageOption(None, "--manpage", dest="manpage", default=False, action="manpage",
+        """creates a manpage option that allows the optionparser to generate a
+        manpage"""
+        manpageoption = ManPageOption(None, "--manpage", dest="manpage",
+                                      default=False, action="manpage",
             help="output a manpage based on the help")
         self.define_option(manpageoption)
 
@@ -108,24 +119,26 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             formatToolkit(self.version),
             formatToolkit(self.version)))
         result.append('.SH NAME\n')
-        result.append('%s \\- %s\n' % (self.get_prog_name(), self.description.split('\n\n')[0]))
+        result.append('%s \\- %s\n' % (self.get_prog_name(),
+                                       self.description.split('\n\n')[0]))
         result.append('.SH SYNOPSIS\n')
         result.append('.PP\n')
-        usage  = "\\fB%prog "
+        usage = "\\fB%prog "
         usage += " ".join([self.getusageman(option) for option in self.option_list])
         usage += "\\fP"
         result.append('%s\n' % formatprog(usage))
         description_lines = self.description.split('\n\n')[1:]
         if description_lines:
             result.append('.SH DESCRIPTION\n')
-            result.append('\n'.join(description_lines))
+            result.append('\n\n'.join([re.sub('\.\. note::', 'Note:', l)
+                                              for l in description_lines]))
         result.append('.SH OPTIONS\n')
         ManHelpFormatter().store_option_strings(self)
         result.append('.PP\n')
         for option in self.option_list:
             result.append('.TP\n')
-            result.append('%s\n'%option)
-            result.append('%s\n'%option.help)
+            result.append('%s\n' % str(option).replace('-', '\-'))
+            result.append('%s\n' % option.help.replace('-', '\-'))
         return "".join(result)
 
     def print_manpage(self, file=None):
@@ -134,40 +147,9 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             file = sys.stdout
         file.write(self.format_manpage())
 
-    def setpsycooption(self):
-        try:
-            import psyco # pylint: disable-msg=W0612
-        except Exception:
-            return
-        psycomodes = ["none", "full", "profile"]
-        psycooption = optparse.Option(None, "--psyco", dest="psyco", default=None,
-            choices=psycomodes, metavar="MODE",
-            help="use psyco to speed up the operation, modes: %s" % (", ".join(psycomodes)))
-        self.define_option(psycooption)
-
-    def usepsyco(self, options):
-        # options.psyco == None means the default, which is "full", but don't give a warning...
-        # options.psyco == "none" means don't use psyco at all...
-        if getattr(options, "psyco", "none") == "none":
-            return
-        try:
-            import psyco
-        except Exception:
-            if options.psyco is not None:
-                self.warning("psyco unavailable", options, sys.exc_info())
-            return
-        if options.psyco is None:
-            options.psyco = "full"
-        if options.psyco == "full":
-            psyco.full()
-        elif options.psyco == "profile":
-            psyco.profile()
-        # tell psyco the functions it cannot compile, to prevent warnings
-        import encodings
-        psyco.cannotcompile(encodings.search_function)
-
     def set_usage(self, usage=None):
-        """sets the usage string - if usage not given, uses getusagestring for each option"""
+        """sets the usage string - if usage not given, uses getusagestring for
+        each option"""
         if usage is None:
             self.usage = "%prog " + " ".join([self.getusagestring(option) for option in self.option_list])
         else:
@@ -177,7 +159,8 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         """Print a warning message incorporating 'msg' to stderr and exit."""
         if options:
             if options.errorlevel == "traceback":
-                errorinfo = "\n".join(traceback.format_exception(exc_info[0], exc_info[1], exc_info[2]))
+                errorinfo = "\n".join(traceback.format_exception(exc_info[0],
+                                      exc_info[1], exc_info[2]))
             elif options.errorlevel == "exception":
                 errorinfo = "\n".join(traceback.format_exception_only(exc_info[0], exc_info[1]))
             elif options.errorlevel == "message":
@@ -186,7 +169,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 errorinfo = ""
             if errorinfo:
                 msg += ": " + errorinfo
-        print >> sys.stderr, "\n%s: warning: %s" % (self.get_prog_name(), msg)
+        logging.getLogger(self.get_prog_name()).warning(msg)
 
     def getusagestring(self, option):
         """returns the usage string for the given option"""
@@ -213,7 +196,8 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             return "\\fR[\\fP%s\\fR]\\fP" % optionstring
 
     def define_option(self, option):
-        """defines the given option, replacing an existing one of the same short name if neccessary..."""
+        """Defines the given option, replacing an existing one of the same short
+        name if neccessary..."""
         for short_opt in option._short_opts:
             if self.has_option(short_opt):
                 self.remove_option(short_opt)
@@ -225,13 +209,18 @@ class RecursiveOptionParser(optparse.OptionParser, object):
     def setformats(self, formats, usetemplates):
         """Sets the format options using the given format dictionary.
 
-        @type formats: Dictionary
-        @param formats: The dictionary I{keys} should be:
-            - single strings (or 1-tuples) containing an input format (if not usetemplates)
-            - tuples containing an input format and template format (if usetemplates)
-            - formats can be None to indicate what to do with standard input
-        The dictionary I{values} should be tuples of outputformat (string) and processor method.
-        
+        :type formats: Dictionary
+        :param formats: The dictionary *keys* should be:
+
+                        - Single strings (or 1-tuples) containing an
+                          input format (if not *usetemplates*)
+                        - Tuples containing an input format and
+                          template format (if *usetemplates*)
+                        - Formats can be *None* to indicate what to do
+                          with standard input
+
+                        The dictionary *values* should be tuples of
+                        outputformat (string) and processor method.
         """
 
         inputformats = []
@@ -261,17 +250,20 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             self.outputoptions[(inputformat, templateformat)] = (outputformat, processor)
         self.inputformats = inputformats
         inputformathelp = self.getformathelp(inputformats)
-        inputoption = optparse.Option("-i", "--input", dest="input", default=None, metavar="INPUT",
+        inputoption = optparse.Option("-i", "--input", dest="input",
+                default=None, metavar="INPUT",
                 help="read from INPUT in %s" % (inputformathelp))
         inputoption.optionalswitch = True
         inputoption.required = True
         self.define_option(inputoption)
-        excludeoption = optparse.Option("-x", "--exclude", dest="exclude", action="append",
-                type="string", default=["CVS", ".svn", "_darcs", ".git", ".hg", ".bzr"], metavar="EXCLUDE",
+        excludeoption = optparse.Option("-x", "--exclude", dest="exclude",
+                action="append", type="string", metavar="EXCLUDE",
+                default=["CVS", ".svn", "_darcs", ".git", ".hg", ".bzr"],
                 help="exclude names matching EXCLUDE from input paths")
         self.define_option(excludeoption)
         outputformathelp = self.getformathelp(outputformats)
-        outputoption = optparse.Option("-o", "--output", dest="output", default=None, metavar="OUTPUT",
+        outputoption = optparse.Option("-o", "--output", dest="output",
+                default=None, metavar="OUTPUT",
                 help="write to OUTPUT in %s" % (outputformathelp))
         outputoption.optionalswitch = True
         outputoption.required = True
@@ -279,34 +271,38 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         if self.usetemplates:
             self.templateformats = templateformats
             templateformathelp = self.getformathelp(self.templateformats)
-            templateoption = optparse.Option("-t", "--template", dest="template", default=None, metavar="TEMPLATE",
+            templateoption = optparse.Option("-t", "--template",
+                dest="template", default=None, metavar="TEMPLATE",
                 help="read from TEMPLATE in %s" % (templateformathelp))
             self.define_option(templateoption)
 
     def setprogressoptions(self):
-        """sets the progress options"""
+        """Sets the progress options."""
         self.progresstypes = {
                 "none": progressbar.NoProgressBar,
                 "bar": progressbar.HashProgressBar,
                 "dots": progressbar.DotsProgressBar,
                 "names": progressbar.MessageProgressBar,
-                "verbose": progressbar.VerboseProgressBar
+                "verbose": progressbar.VerboseProgressBar,
         }
-        progressoption = optparse.Option(None, "--progress", dest="progress", default="bar",
-                choices = self.progresstypes.keys(), metavar="PROGRESS",
+        progressoption = optparse.Option(None, "--progress", dest="progress",
+                default="bar",
+                choices=self.progresstypes.keys(), metavar="PROGRESS",
                 help="show progress as: %s" % (", ".join(self.progresstypes)))
         self.define_option(progressoption)
 
     def seterrorleveloptions(self):
-        """sets the errorlevel options"""
+        """Sets the errorlevel options."""
         self.errorleveltypes = ["none", "message", "exception", "traceback"]
-        errorleveloption = optparse.Option(None, "--errorlevel", dest="errorlevel", default="message",
-                choices = self.errorleveltypes, metavar="ERRORLEVEL",
-                help="show errorlevel as: %s" % (", ".join(self.errorleveltypes)))
+        errorleveloption = optparse.Option(None, "--errorlevel",
+                dest="errorlevel", default="message",
+                choices=self.errorleveltypes, metavar="ERRORLEVEL",
+                help="show errorlevel as: %s" % \
+                     (", ".join(self.errorleveltypes)))
         self.define_option(errorleveloption)
 
     def getformathelp(self, formats):
-        """make a nice help string for describing formats..."""
+        """Make a nice help string for describing formats..."""
         if None in formats:
             formats = filter(lambda format: format is not None, formats)
         if len(formats) == 0:
@@ -317,7 +313,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             return "%s formats" % (", ".join(formats))
 
     def isrecursive(self, fileoption, filepurpose='input'):
-        """checks if fileoption is a recursive file"""
+        """Checks if fileoption is a recursive file."""
         if fileoption is None:
             return False
         elif isinstance(fileoption, list):
@@ -326,9 +322,11 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             return os.path.isdir(fileoption)
 
     def parse_args(self, args=None, values=None):
-        """parses the command line options, handling implicit input/output args"""
+        """Parses the command line options, handling implicit input/output
+        args."""
         (options, args) = super(RecursiveOptionParser, self).parse_args(args, values)
-        # some intelligent as to what reasonable people might give on the command line
+        # some intelligent as to what reasonable people might give on the
+        # command line
         if args and not options.input:
             if len(args) > 1:
                 options.input = args[:-1]
@@ -350,7 +348,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         return (options, args)
 
     def getpassthroughoptions(self, options):
-        """get the options required to pass to the filtermethod..."""
+        """Get the options required to pass to the filtermethod..."""
         passthroughoptions = {}
         for optionname in dir(options):
             if optionname in self.passthrough:
@@ -358,7 +356,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         return passthroughoptions
 
     def getoutputoptions(self, options, inputpath, templatepath):
-        """works out which output format and processor method to use..."""
+        """Works out which output format and processor method to use..."""
         if inputpath:
             inputbase, inputext = self.splitinputext(inputpath)
         else:
@@ -388,11 +386,14 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 if inputext is None:
                     raise ValueError("don't know what to do with input format (no file extension), no template file")
                 elif templateext is None:
-                    raise ValueError("don't know what to do with input format %s, no template file" % (os.extsep + inputext))
+                    raise ValueError("don't know what to do with input format %s, no template file" %
+                                     (os.extsep + inputext))
                 else:
-                    raise ValueError("don't know what to do with input format %s, template format %s" % (os.extsep + inputext, os.extsep + templateext))
+                    raise ValueError("don't know what to do with input format %s, template format %s" %
+                                     (os.extsep + inputext, os.extsep + templateext))
             else:
-                raise ValueError("don't know what to do with input format %s" % os.extsep + inputext)
+                raise ValueError("don't know what to do with input format %s" %
+                                 (os.extsep + inputext))
         if outputformat == "*":
             if inputext:
                 outputformat = inputext
@@ -402,58 +403,68 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 outputformat = None
             else:
                 if self.usetemplates:
-                    if templateext is None:
-                        raise ValueError("don't know what to do with input format %s, no template file" % (os.extsep + inputext))
-                    else:
-                        raise ValueError("don't know what to do with input format %s, template format %s" % (os.extsep + inputext, os.extsep + templateext))
+                    raise ValueError("don't know what to do with input format (no file extension), no template file")
                 else:
-                    raise ValueError("don't know what to do with input format %s" % os.extsep + inputext)
+                    raise ValueError("don't know what to do with input format (no file extension)")
         return outputformat, fileprocessor
 
     def initprogressbar(self, allfiles, options):
-        """sets up a progress bar appropriate to the options and files"""
+        """Sets up a progress bar appropriate to the options and files."""
         if options.progress in ('bar', 'verbose'):
-            self.progressbar = self.progresstypes[options.progress](0, len(allfiles))
-            print >> sys.stderr, "processing %d files..." % len(allfiles)
+            self.progressbar = \
+                self.progresstypes[options.progress](0, len(allfiles))
+            # should use .getChild("progress") but that is only in 2.7
+            logger = logging.getLogger(self.get_prog_name() + ".progress")
+            logger.setLevel(logging.INFO)
+            logger.propagate = False
+            handler = logging.StreamHandler()
+            handler.setLevel(logging.INFO)
+            handler.setFormatter(logging.Formatter())
+            logger.addHandler(handler)
+            logger.info("processing %d files...", len(allfiles))
         else:
             self.progressbar = self.progresstypes[options.progress]()
 
     def getfullinputpath(self, options, inputpath):
-        """gets the absolute path to an input file"""
+        """Gets the absolute path to an input file."""
         if options.input:
             return os.path.join(options.input, inputpath)
         else:
             return inputpath
 
     def getfulloutputpath(self, options, outputpath):
-        """gets the absolute path to an output file"""
+        """Gets the absolute path to an output file."""
         if options.recursiveoutput and options.output:
             return os.path.join(options.output, outputpath)
         else:
             return outputpath
 
     def getfulltemplatepath(self, options, templatepath):
-        """gets the absolute path to a template file"""
+        """Gets the absolute path to a template file."""
         if not options.recursivetemplate:
             return templatepath
-        elif templatepath is not None and self.usetemplates and options.template:
+        elif (templatepath is not None and
+              self.usetemplates and options.template):
             return os.path.join(options.template, templatepath)
         else:
             return None
 
     def run(self):
-        """parses the arguments, and runs recursiveprocess with the resulting options..."""
+        """Parses the arguments, and runs recursiveprocess with the resulting
+        options..."""
         (options, args) = self.parse_args()
-        # this is so derived classes can modify the inputformats etc based on the options
+        # this is so derived classes can modify the inputformats etc based on
+        # the options
         options.inputformats = self.inputformats
         options.outputoptions = self.outputoptions
-        self.usepsyco(options)
         self.recursiveprocess(options)
 
     def recursiveprocess(self, options):
-        """recurse through directories and process files"""
+        """Recurse through directories and process files."""
         if self.isrecursive(options.input, 'input') and getattr(options, "allowrecursiveinput", True):
             if not self.isrecursive(options.output, 'output'):
+                if not options.output:
+                    self.error(optparse.OptionValueError("No output directory given"))
                 try:
                     self.warning("Output directory does not exist. Attempting to create")
                     os.mkdir(options.output)
@@ -469,20 +480,26 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 options.input = os.path.dirname(options.input)
             else:
                 inputfiles = [options.input]
-        options.recursiveoutput = self.isrecursive(options.output, 'output') and getattr(options, "allowrecursiveoutput", True)
-        options.recursivetemplate = self.usetemplates and self.isrecursive(options.template, 'template') and getattr(options, "allowrecursivetemplate", True)
+        options.recursiveoutput = (self.isrecursive(options.output, 'output') and
+                                   getattr(options, "allowrecursiveoutput", True))
+        options.recursivetemplate = (self.usetemplates and
+                                     self.isrecursive(options.template, 'template') and
+                                     getattr(options, "allowrecursivetemplate", True))
         self.initprogressbar(inputfiles, options)
         for inputpath in inputfiles:
             try:
                 templatepath = self.gettemplatename(options, inputpath)
-                # If we have a recursive template, but the template doesn't have this
-                # input file, let's drop it.
-                if options.recursivetemplate and templatepath is None and not self.allowmissingtemplate:
-                    self.warning("No template at %s. Skipping %s." % (templatepath, inputpath))
+                # If we have a recursive template, but the template doesn't
+                # have this input file, let's drop it.
+                if (options.recursivetemplate and templatepath is None and
+                    not self.allowmissingtemplate):
+                    self.warning("No template at %s. Skipping %s." %
+                                 (templatepath, inputpath))
                     continue
                 outputformat, fileprocessor = self.getoutputoptions(options, inputpath, templatepath)
                 fullinputpath = self.getfullinputpath(options, inputpath)
-                fulltemplatepath = self.getfulltemplatepath(options, templatepath)
+                fulltemplatepath = self.getfulltemplatepath(options,
+                                                            templatepath)
                 outputpath = self.getoutputname(options, inputpath, outputformat)
                 fulloutputpath = self.getfulloutputpath(options, outputpath)
                 if options.recursiveoutput and outputpath:
@@ -490,36 +507,41 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             except Exception, error:
                 if isinstance(error, KeyboardInterrupt):
                     raise
-                self.warning("Couldn't handle input file %s" % inputpath, options, sys.exc_info())
+                self.warning("Couldn't handle input file %s" %
+                             inputpath, options, sys.exc_info())
                 continue
             try:
-                success = self.processfile(fileprocessor, options, fullinputpath, fulloutputpath, fulltemplatepath)
+                success = self.processfile(fileprocessor, options,
+                                           fullinputpath, fulloutputpath,
+                                           fulltemplatepath)
             except Exception, error:
                 if isinstance(error, KeyboardInterrupt):
                     raise
-                self.warning("Error processing: input %s, output %s, template %s" % (fullinputpath, fulloutputpath, fulltemplatepath), options, sys.exc_info())
+                self.warning("Error processing: input %s, output %s, template %s" %
+                             (fullinputpath, fulloutputpath,
+                              fulltemplatepath), options, sys.exc_info())
                 success = False
             self.reportprogress(inputpath, success)
         del self.progressbar
 
     def openinputfile(self, options, fullinputpath):
-        """opens the input file"""
+        """Opens the input file."""
         if fullinputpath is None:
             return sys.stdin
         return open(fullinputpath, 'r')
 
     def openoutputfile(self, options, fulloutputpath):
-        """opens the output file"""
+        """Opens the output file."""
         if fulloutputpath is None:
             return sys.stdout
         return open(fulloutputpath, 'w')
 
     def opentempoutputfile(self, options, fulloutputpath):
-        """opens a temporary output file"""
+        """Opens a temporary output file."""
         return StringIO()
 
     def finalizetempoutputfile(self, options, outputfile, fulloutputpath):
-        """write the temp outputfile to its final destination"""
+        """Write the temp outputfile to its final destination."""
         outputfile.reset()
         outputstring = outputfile.read()
         outputfile = self.openoutputfile(options, fulloutputpath)
@@ -527,7 +549,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         outputfile.close()
 
     def opentemplatefile(self, options, fulltemplatepath):
-        """opens the template file (if required)"""
+        """Opens the template file (if required)."""
         if fulltemplatepath is not None:
             if os.path.isfile(fulltemplatepath):
                 return open(fulltemplatepath, 'r')
@@ -535,10 +557,12 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 self.warning("missing template file %s" % fulltemplatepath)
         return None
 
-    def processfile(self, fileprocessor, options, fullinputpath, fulloutputpath, fulltemplatepath):
-        """process an individual file"""
+    def processfile(self, fileprocessor, options, fullinputpath,
+                    fulloutputpath, fulltemplatepath):
+        """Process an individual file."""
         inputfile = self.openinputfile(options, fullinputpath)
-        if fulloutputpath and fulloutputpath in (fullinputpath, fulltemplatepath):
+        if (fulloutputpath and
+            fulloutputpath in (fullinputpath, fulltemplatepath)):
             outputfile = self.opentempoutputfile(options, fulloutputpath)
             tempoutput = True
         else:
@@ -546,10 +570,12 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             tempoutput = False
         templatefile = self.opentemplatefile(options, fulltemplatepath)
         passthroughoptions = self.getpassthroughoptions(options)
-        if fileprocessor(inputfile, outputfile, templatefile, **passthroughoptions):
+        if fileprocessor(inputfile, outputfile, templatefile,
+                         **passthroughoptions):
             if tempoutput:
                 self.warning("writing to temporary output...")
-                self.finalizetempoutputfile(options, outputfile, fulloutputpath)
+                self.finalizetempoutputfile(options, outputfile,
+                                            fulloutputpath)
             return True
         else:
             # remove the file if it is a file (could be stdout etc)
@@ -559,14 +585,15 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             return False
 
     def reportprogress(self, filename, success):
-        """shows that we are progressing..."""
+        """Shows that we are progressing..."""
         self.progressbar.amount += 1
         self.progressbar.show(filename)
 
     def mkdir(self, parent, subdir):
-        """makes a subdirectory (recursively if neccessary)"""
+        """Makes a subdirectory (recursively if neccessary)."""
         if not os.path.isdir(parent):
-            raise ValueError("cannot make child directory %r if parent %r does not exist" % (subdir, parent))
+            raise ValueError("cannot make child directory %r if parent %r does not exist" %
+                             (subdir, parent))
         currentpath = parent
         subparts = subdir.split(os.sep)
         for part in subparts:
@@ -575,13 +602,14 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 os.mkdir(currentpath)
 
     def checkoutputsubdir(self, options, subdir):
-        """checks to see if subdir under options.output needs to be created, creates if neccessary"""
+        """Checks to see if subdir under options.output needs to be created,
+        creates if neccessary."""
         fullpath = os.path.join(options.output, subdir)
         if not os.path.isdir(fullpath):
             self.mkdir(options.output, subdir)
 
     def isexcluded(self, options, inputpath):
-        """checks if this path has been excluded"""
+        """Checks if this path has been excluded."""
         basename = os.path.basename(inputpath)
         for excludename in options.exclude:
             if fnmatch.fnmatch(basename, excludename):
@@ -589,22 +617,23 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         return False
 
     def recurseinputfilelist(self, options):
-        """use a list of files, and find a common base directory for them"""
-        # find a common base directory for the files to do everything relative to
+        """Use a list of files, and find a common base directory for them."""
+        # find a common base directory for the files to do everything
+        # relative to
         commondir = os.path.dirname(os.path.commonprefix(options.input))
         inputfiles = []
         for inputfile in options.input:
             if self.isexcluded(options, inputfile):
                 continue
-            if inputfile.startswith(commondir+os.sep):
-                inputfiles.append(inputfile.replace(commondir+os.sep, "", 1))
+            if inputfile.startswith(commondir + os.sep):
+                inputfiles.append(inputfile.replace(commondir + os.sep, "", 1))
             else:
                 inputfiles.append(inputfile.replace(commondir, "", 1))
         options.input = commondir
         return inputfiles
 
     def recurseinputfiles(self, options):
-        """recurse through directories and return files to be processed..."""
+        """Recurse through directories and return files to be processed."""
         dirstack = ['']
         join = os.path.join
         inputfiles = []
@@ -622,41 +651,42 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                     dirs.append(inputpath)
                 elif os.path.isfile(fullinputpath):
                     if not self.isvalidinputname(options, name):
-                        # only handle names that match recognized input file extensions
+                        # only handle names that match recognized input
+                        # file extensions
                         continue
                     inputfiles.append(inputpath)
-            # make sure the directories are processed next time round...
+            # make sure the directories are processed next time round.
             dirs.reverse()
             dirstack.extend(dirs)
         return inputfiles
 
     def splitext(self, pathname):
-        """Splits L{pathname} into name and ext, and removes the extsep
-    
-        @param pathname: A file path
-        @type pathname: string
-        @return: root, ext
-        @rtype: tuple
+        """Splits *pathname* into name and ext, and removes the extsep.
+
+        :param pathname: A file path
+        :type pathname: string
+        :return: root, ext
+        :rtype: tuple
         """
         root, ext = os.path.splitext(pathname)
         ext = ext.replace(os.extsep, "", 1)
         return (root, ext)
 
     def splitinputext(self, inputpath):
-        """splits an inputpath into name and extension"""
+        """Splits an *inputpath* into name and extension."""
         return self.splitext(inputpath)
 
     def splittemplateext(self, templatepath):
-        """splits a templatepath into name and extension"""
+        """Splits a *templatepath* into name and extension."""
         return self.splitext(templatepath)
 
     def templateexists(self, options, templatepath):
-        """returns whether the given template exists..."""
+        """Returns whether the given template exists..."""
         fulltemplatepath = self.getfulltemplatepath(options, templatepath)
         return os.path.isfile(fulltemplatepath)
 
     def gettemplatename(self, options, inputname):
-        """gets an output filename based on the input filename"""
+        """Gets an output filename based on the input filename."""
         if not self.usetemplates:
             return None
         if not inputname or not options.recursivetemplate:
@@ -683,7 +713,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         return None
 
     def getoutputname(self, options, inputname, outputformat):
-        """gets an output filename based on the input filename"""
+        """Gets an output filename based on the input filename."""
         if not inputname or not options.recursiveoutput:
             return options.output
         inputbase, inputext = self.splitinputext(inputname)
@@ -693,7 +723,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         return outputname
 
     def isvalidinputname(self, options, inputname):
-        """checks if this is a valid input filename"""
+        """Checks if this is a valid input filename."""
         inputbase, inputext = self.splitinputext(inputname)
-        return (inputext in options.inputformats) or ("*" in options.inputformats)
-
+        return ((inputext in options.inputformats) or
+                ("*" in options.inputformats))
