@@ -1,8 +1,11 @@
 #!/usr/bin/env python
 
+import pytest
+
 from translate.misc import multistring
 from translate.misc import test_autoencode
 
+
 class TestMultistring(test_autoencode.TestAutoencode):
     type2test = multistring.multistring
 
@@ -17,6 +20,7 @@ class TestMultistring(test_autoencode.TestAutoencode):
         assert s2 == "test"
         assert s2.strings == ["test", "me"]
         assert s2 != s1
+        pytest.raises(ValueError, t, [])
 
     def test_replace(self):
         t = self.type2test
@@ -32,11 +36,19 @@ class TestMultistring(test_autoencode.TestAutoencode):
         result = s1.replace("e", u"\xe9")
         assert result == t([u"abcd\xe9f", u"d\xe9f"])
 
-	result = s1.replace("e", "\n")
-	assert result == t([u"abcd\nf", u"d\nf"])
+        result = s1.replace("e", "\n")
+        assert result == t([u"abcd\nf", u"d\nf"])
 
-	result = result.replace("\n", "\\n")
-	assert result == t([u"abcd\\nf", u"d\\nf"])
-	
-	result = result.replace("\\n", "\n")
-	assert result == t([u"abcd\nf", u"d\nf"])
+        result = result.replace("\n", "\\n")
+        assert result == t([u"abcd\\nf", u"d\\nf"])
+
+        result = result.replace("\\n", "\n")
+        assert result == t([u"abcd\nf", u"d\nf"])
+
+        s2 = t(["abcdeef", "deef"])
+
+        result = s2.replace("e", "g")
+        assert result == t([u"abcdggf", u"dggf"])
+
+        result = s2.replace("e", "g", 1)
+        assert result == t([u"abcdgef", u"dgef"])
