@@ -1,15 +1,15 @@
 #!/usr/bin/env python
 
-from translate.misc import optrecurse
 import os
 
+from translate.misc import optrecurse
+
+
 class TestRecursiveOptionParser:
 
-    def __init__(self):
-        self.parser = optrecurse.RecursiveOptionParser({"txt":("po", None)})
-
     def test_splitext(self):
-        """test the L{optrecurse.splitext} function"""
+        """test the ``optrecurse.splitext`` function"""
+        self.parser = optrecurse.RecursiveOptionParser({"txt": ("po", None)})
         name = "name"
         extension = "ext"
         filename = name + os.extsep + extension
