@@ -2,38 +2,57 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright 2008 Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 
 
-import __init__ as indexing
-import CommonIndexer
 import os
 import sys
 import shutil
+import pytest
+
+import __init__ as indexing
+import CommonIndexer
+
+# following block only needs running under pytest; unclear how to detect it?
+
+# check whether any indexer is present at all
+noindexer = True
+for indexer in [ "lucene", "PyLucene", "xapian" ]:
+    try:
+        __import__(indexer)
+    except ImportError:
+        continue
+    noindexer = False
+    break
+# mark entire module as skipped for pytest if no indexer available
+pytestmark = pytest.mark.skipif("noindexer")
+
+# FIXME (bug 2819) need to rename most test_* functions, add new "parametrized"
+# test_indexer function to normalize operation whether run directly as script
+# or via pytest
 
 DATABASE = "tmp-index"
 
 # overwrite this value to change the preferred indexing engine
-default_engine = "" 
+default_engine = ""
 
 # order of tests to be done
-ORDER_OF_TESTS = [ "XapianIndexer", "PyLuceneIndexer", "PyLuceneIndexer1" ]
+ORDER_OF_TESTS = ["XapianIndexer", "PyLuceneIndexer", "PyLuceneIndexer1"]
 
 
 def _get_indexer(location):
@@ -42,13 +61,14 @@ def _get_indexer(location):
 
     create an indexer based on the preference order 'default_engine'
 
-    @param location: the path of the database to be created/opened
-    @type location: str
-    @return: the resulting indexing engine instance
-    @rtype: CommonIndexer.CommonDatabase
+    :param location: the path of the database to be created/opened
+    :type location: str
+    :return: the resulting indexing engine instance
+    :rtype: CommonIndexer.CommonDatabase
     """
     return indexing.get_indexer(location, [default_engine])
 
+
 def clean_database():
     """remove an existing database"""
     dbase_dir = os.path.abspath(DATABASE)
@@ -58,13 +78,14 @@ def clean_database():
     # recursively remove the directory
     shutil.rmtree(dbase_dir)
 
+
 def create_example_content(database):
     """add some defined documents to the database
 
     this may be used to check some specific queries
 
-    @param database: a indexing database object
-    @type database: CommonIndexer.CommonDatabase
+    :param database: a indexing database object
+    :type database: CommonIndexer.CommonDatabase
     """
     # a reasonable foo-bar entry
     database.index_document(["foo", "bar", "med"])
@@ -76,7 +97,7 @@ def create_example_content(database):
     database.index_document({"fname1": "foo_field1", "fname2": "foo_field2"})
     database.index_document({"fname1": "bar_field1", "fname2": "foo_field2",
             None: ["HELO", "foo"]})
-    database.index_document({ None: "med" })
+    database.index_document({None: "med"})
     # for tokenizing tests
     database.set_field_analyzers({
             "fname1": database.ANALYZER_PARTIAL | database.ANALYZER_TOKENIZE,
@@ -87,8 +108,10 @@ def create_example_content(database):
     database.index_document({"fname2": "foo-bar.po"})
     # add a list of terms for a keyword
     database.index_document({"multiple": ["foo", "bar"]})
+    database.flush()
     assert _get_number_of_docs(database) == 10
 
+
 def test_create_database():
     """create a new database from scratch"""
     # clean up everything first
@@ -99,6 +122,7 @@ def test_create_database():
     # clean up
     clean_database()
 
+
 def test_open_database():
     """open an existing database"""
     # clean up everything first
@@ -112,6 +136,7 @@ def test_open_database():
     # clean up
     clean_database()
 
+
 def test_make_queries():
     """create a simple query from a plain string"""
     # clean up everything first
@@ -130,6 +155,7 @@ def test_make_queries():
         new_db.make_query("bar")], require_all=False)
     assert str(q_combined_or) != str(q_combined_and)
 
+
 def test_partial_text_matching():
     """check if implicit and explicit partial text matching works"""
     # clean up everything first
@@ -140,16 +166,16 @@ def test_partial_text_matching():
     # this query should return three matches (disabled partial matching)
     q_plain_partial1 = new_db.make_query("bar",
             analyzer=(new_db.analyzer ^ new_db.ANALYZER_PARTIAL))
-    r_plain_partial1 = new_db.get_query_result(q_plain_partial1).get_matches(0,10)
+    r_plain_partial1 = new_db.get_query_result(q_plain_partial1).get_matches(0, 10)
     assert r_plain_partial1[0] == 2
     # this query should return three matches (wildcard works)
     q_plain_partial2 = new_db.make_query("bar", analyzer=new_db.ANALYZER_PARTIAL)
-    r_plain_partial2 = new_db.get_query_result(q_plain_partial2).get_matches(0,10)
+    r_plain_partial2 = new_db.get_query_result(q_plain_partial2).get_matches(0, 10)
     assert r_plain_partial2[0] == 3
     # return two matches (the wildcard is ignored without PARTIAL)
     q_plain_partial3 = new_db.make_query("bar*",
             analyzer=(new_db.analyzer ^ new_db.ANALYZER_PARTIAL))
-    r_plain_partial3 = new_db.get_query_result(q_plain_partial3).get_matches(0,10)
+    r_plain_partial3 = new_db.get_query_result(q_plain_partial3).get_matches(0, 10)
     assert r_plain_partial3[0] == 2
     # partial matching at the start of the string
     # TODO: enable this as soon, as partial matching works at the beginning of text
@@ -171,31 +197,32 @@ def test_field_matching():
     create_example_content(new_db)
     # do a field search with a tuple argument
     q_field1 = new_db.make_query(("fname1", "foo_field1"))
-    r_field1 = new_db.get_query_result(q_field1).get_matches(0,10)
+    r_field1 = new_db.get_query_result(q_field1).get_matches(0, 10)
     assert r_field1[0] == 1
     # do a field search with a dict argument
-    q_field2 = new_db.make_query({"fname1":"bar_field1"})
-    r_field2 = new_db.get_query_result(q_field2).get_matches(0,10)
+    q_field2 = new_db.make_query({"fname1": "bar_field1"})
+    r_field2 = new_db.get_query_result(q_field2).get_matches(0, 10)
     assert r_field2[0] == 1
     # do an incomplete field search with a dict argument - should fail
-    q_field3 = new_db.make_query({"fname2":"foo_field"})
-    r_field3 = new_db.get_query_result(q_field3).get_matches(0,10)
+    q_field3 = new_db.make_query({"fname2": "foo_field"})
+    r_field3 = new_db.get_query_result(q_field3).get_matches(0, 10)
     assert r_field3[0] == 0
     # do an AND field search with a dict argument
-    q_field4 = new_db.make_query({"fname1":"foo_field1", "fname2":"foo_field2"}, require_all=True)
-    r_field4 = new_db.get_query_result(q_field4).get_matches(0,10)
+    q_field4 = new_db.make_query({"fname1": "foo_field1", "fname2": "foo_field2"}, require_all=True)
+    r_field4 = new_db.get_query_result(q_field4).get_matches(0, 10)
     assert r_field4[0] == 1
     # do an OR field search with a dict argument
-    q_field5 = new_db.make_query({"fname1":"foo_field1", "fname2":"foo_field2"}, require_all=False)
-    r_field5 = new_db.get_query_result(q_field5).get_matches(0,10)
+    q_field5 = new_db.make_query({"fname1": "foo_field1", "fname2": "foo_field2"}, require_all=False)
+    r_field5 = new_db.get_query_result(q_field5).get_matches(0, 10)
     assert r_field5[0] == 2
     # do an incomplete field search with a partial field analyzer
-    q_field6 = new_db.make_query({"fname1":"foo_field"}, analyzer=new_db.ANALYZER_PARTIAL)
-    r_field6 = new_db.get_query_result(q_field6).get_matches(0,10)
+    q_field6 = new_db.make_query({"fname1": "foo_field"}, analyzer=new_db.ANALYZER_PARTIAL)
+    r_field6 = new_db.get_query_result(q_field6).get_matches(0, 10)
     assert r_field6[0] == 1
     # clean up
     clean_database()
 
+
 def test_field_analyzers():
     """test if we can change the analyzer of specific fields"""
     # clean up everything first
@@ -204,26 +231,27 @@ def test_field_analyzers():
     new_db = _get_indexer(DATABASE)
     create_example_content(new_db)
     # do an incomplete field search with partial analyzer (configured for this field)
-    q_field1 = new_db.make_query({"fname1":"bar_field"})
-    r_field1 = new_db.get_query_result(q_field1).get_matches(0,10)
+    q_field1 = new_db.make_query({"fname1": "bar_field"})
+    r_field1 = new_db.get_query_result(q_field1).get_matches(0, 10)
     assert r_field1[0] == 1
     # check the get/set field analyzer functions
     old_analyzer = new_db.get_field_analyzers("fname1")
-    new_db.set_field_analyzers({"fname1":new_db.ANALYZER_EXACT})
+    new_db.set_field_analyzers({"fname1": new_db.ANALYZER_EXACT})
     assert new_db.get_field_analyzers("fname1") == new_db.ANALYZER_EXACT
-    new_db.set_field_analyzers({"fname1":new_db.ANALYZER_PARTIAL})
+    new_db.set_field_analyzers({"fname1": new_db.ANALYZER_PARTIAL})
     assert new_db.get_field_analyzers("fname1") == new_db.ANALYZER_PARTIAL
     # restore previous setting
-    new_db.set_field_analyzers({"fname1":old_analyzer})
+    new_db.set_field_analyzers({"fname1": old_analyzer})
     # check if ANALYZER_TOKENIZE is the default
     assert (new_db.get_field_analyzers("thisFieldDoesNotExist") & new_db.ANALYZER_TOKENIZE) > 0
     # do an incomplete field search - now we use the partial analyzer
-    q_field2 = new_db.make_query({"fname1":"bar_field"}, analyzer=new_db.ANALYZER_PARTIAL)
-    r_field2 = new_db.get_query_result(q_field2).get_matches(0,10)
+    q_field2 = new_db.make_query({"fname1": "bar_field"}, analyzer=new_db.ANALYZER_PARTIAL)
+    r_field2 = new_db.get_query_result(q_field2).get_matches(0, 10)
     assert r_field2[0] == 1
     # clean up
     clean_database()
 
+
 def test_and_queries():
     """test if AND queries work as expected"""
     # clean up everything first
@@ -234,21 +262,22 @@ def test_and_queries():
     # do an AND query (partial matching disabled)
     q_and1 = new_db.make_query("foo bar",
             analyzer=(new_db.analyzer ^ new_db.ANALYZER_PARTIAL))
-    r_and1 = new_db.get_query_result(q_and1).get_matches(0,10)
+    r_and1 = new_db.get_query_result(q_and1).get_matches(0, 10)
     assert r_and1[0] == 2
     # do the same AND query in a different way
     q_and2 = new_db.make_query(["foo", "bar"],
             analyzer=(new_db.analyzer ^ new_db.ANALYZER_PARTIAL))
-    r_and2 = new_db.get_query_result(q_and2).get_matches(0,10)
+    r_and2 = new_db.get_query_result(q_and2).get_matches(0, 10)
     assert r_and2[0] == 2
     # do an AND query without results
     q_and3 = new_db.make_query(["HELO", "bar", "med"],
             analyzer=(new_db.analyzer ^ new_db.ANALYZER_PARTIAL))
-    r_and3 = new_db.get_query_result(q_and3).get_matches(0,10)
+    r_and3 = new_db.get_query_result(q_and3).get_matches(0, 10)
     assert r_and3[0] == 0
     # clean up
     clean_database()
 
+
 def test_or_queries():
     """test if OR queries work as expected"""
     # clean up everything first
@@ -258,19 +287,41 @@ def test_or_queries():
     create_example_content(new_db)
     # do an OR query
     q_or1 = new_db.make_query("foo bar", require_all=False)
-    r_or1 = new_db.get_query_result(q_or1).get_matches(0,10)
+    r_or1 = new_db.get_query_result(q_or1).get_matches(0, 10)
     assert r_or1[0] == 4
     # do the same or query in a different way
     q_or2 = new_db.make_query(["foo", "bar"], require_all=False)
-    r_or2 = new_db.get_query_result(q_or2).get_matches(0,10)
+    r_or2 = new_db.get_query_result(q_or2).get_matches(0, 10)
     assert r_or2[0] == r_or1[0]
     # do an OR query with lots of results
     q_or3 = new_db.make_query(["HELO", "bar", "med"], require_all=False)
-    r_or3 = new_db.get_query_result(q_or3).get_matches(0,10)
+    r_or3 = new_db.get_query_result(q_or3).get_matches(0, 10)
     assert r_or3[0] == 5
     # clean up
     clean_database()
 
+def test_string_queries():
+    """test if string queries work as expected"""
+    # clean up everything first
+    clean_database()
+    # initialize the database with example content
+    new_db = _get_indexer(DATABASE)
+    create_example_content(new_db)
+    # do string query
+    q_string1 = new_db.make_query("foo bar")
+    r_string1 = new_db.get_query_result(q_string1).get_matches(0, 10)
+    assert r_string1[0] == 3
+    # do string query with non contagious words
+    q_string2 = new_db.make_query("foo HELO")
+    r_string2 = new_db.get_query_result(q_string2).get_matches(0, 10)
+    assert r_string2[0] == 3
+    # do string query with a named field
+    q_string3 = new_db.make_query({"multiple": "foo bar"})
+    r_string3 = new_db.get_query_result(q_string3).get_matches(0, 10)
+    assert r_string3[0] == 1
+    # clean up
+    clean_database()
+
 def test_lower_upper_case():
     """test if case is ignored for queries and for indexed terms"""
     # clean up everything first
@@ -281,24 +332,25 @@ def test_lower_upper_case():
     # use upper case search terms for lower case indexed terms
     q_case1 = new_db.make_query("BAR",
             analyzer=(new_db.analyzer ^ new_db.ANALYZER_PARTIAL))
-    r_case1 = new_db.get_query_result(q_case1).get_matches(0,10)
+    r_case1 = new_db.get_query_result(q_case1).get_matches(0, 10)
     assert r_case1[0] == 2
     # use lower case search terms for upper case indexed terms
     q_case2 = new_db.make_query("helo")
-    r_case2 = new_db.get_query_result(q_case2).get_matches(0,10)
+    r_case2 = new_db.get_query_result(q_case2).get_matches(0, 10)
     assert r_case2[0] == 3
     # use lower case search terms for lower case indexed terms
     q_case3 = new_db.make_query("bar",
             analyzer=(new_db.analyzer ^ new_db.ANALYZER_PARTIAL))
-    r_case3 = new_db.get_query_result(q_case3).get_matches(0,10)
+    r_case3 = new_db.get_query_result(q_case3).get_matches(0, 10)
     assert r_case3[0] == 2
     # use upper case search terms for upper case indexed terms
     q_case4 = new_db.make_query("HELO")
-    r_case4 = new_db.get_query_result(q_case4).get_matches(0,10)
+    r_case4 = new_db.get_query_result(q_case4).get_matches(0, 10)
     assert r_case4[0] == 3
     # clean up
     clean_database()
 
+
 def test_tokenizing():
     """test if the TOKENIZE analyzer field setting is honoured"""
     # clean up everything first
@@ -308,25 +360,26 @@ def test_tokenizing():
     create_example_content(new_db)
     # check if the plain term was tokenized
     q_token1 = new_db.make_query("rfv")
-    r_token1 = new_db.get_query_result(q_token1).get_matches(0,10)
+    r_token1 = new_db.get_query_result(q_token1).get_matches(0, 10)
     assert r_token1[0] == 2
     # check if the field term was tokenized
-    q_token2 = new_db.make_query({"fname1":"wsx"})
-    r_token2 = new_db.get_query_result(q_token2).get_matches(0,10)
+    q_token2 = new_db.make_query({"fname1": "wsx"})
+    r_token2 = new_db.get_query_result(q_token2).get_matches(0, 10)
     assert r_token2[0] == 1
     # check that the other field term was not tokenized
-    q_token3 = new_db.make_query({"fname2":"wsx"})
-    r_token3 = new_db.get_query_result(q_token3).get_matches(0,10)
+    q_token3 = new_db.make_query({"fname2": "wsx"})
+    r_token3 = new_db.get_query_result(q_token3).get_matches(0, 10)
     assert r_token3[0] == 0
     # check that the other field term was not tokenized
-    q_token4 = new_db.make_query({"fname2":"foo-bar.po"})
+    q_token4 = new_db.make_query({"fname2": "foo-bar.po"})
     #q_token4 = new_db.make_query("poo-foo.po")
-    r_token4 = new_db.get_query_result(q_token4).get_matches(0,10)
+    r_token4 = new_db.get_query_result(q_token4).get_matches(0, 10)
     # problem can be fixed by adding "TOKENIZE" to the field before populating the database -> this essentially splits the document term into pieces
     assert r_token4[0] == 1
     # clean up
     clean_database()
 
+
 def test_searching():
     """test if searching (retrieving specified field values) works"""
     # clean up everything first
@@ -338,11 +391,11 @@ def test_searching():
     r_search1 = new_db.search(q_search1, ["fname2", None])
     assert len(r_search1) == 1
     dict_search1 = r_search1[0]
-    assert dict_search1.has_key("fname2") and \
+    assert "fname2" in dict_search1 and \
             (dict_search1["fname2"] == ["foo_field2"])
     # a stupid way for checking, if the second field list is also correct
     # (without caring for the order of the list)
-    assert dict_search1.has_key(None)
+    assert None in dict_search1
     # TODO: for now PyLucene cares for case, while xapian doesn't - FIXME
     list_search1_sorted = [item.lower() for item in dict_search1[None]]
     list_search1_sorted.sort()
@@ -361,11 +414,11 @@ def test_multiple_terms():
     # check for the first item ("foo")
     q_multiple1 = new_db.make_query({"multiple": "f"},
             analyzer=new_db.ANALYZER_PARTIAL)
-    r_multiple1 = new_db.get_query_result(q_multiple1).get_matches(0,10)
+    r_multiple1 = new_db.get_query_result(q_multiple1).get_matches(0, 10)
     assert r_multiple1[0] == 1
     # check for the second item ("bar")
     q_multiple2 = new_db.make_query({"multiple": "bar"})
-    r_multiple2 = new_db.get_query_result(q_multiple2).get_matches(0,10)
+    r_multiple2 = new_db.get_query_result(q_multiple2).get_matches(0, 10)
     assert r_multiple2[0] == 1
     # clean up
     clean_database()
@@ -373,7 +426,7 @@ def test_multiple_terms():
 
 def show_database(database):
     """print the complete database - for debugging purposes"""
-    if hasattr(database, "database"):
+    if database.INDEX_DIRECTORY_NAME == "xapian":
         _show_database_xapian(database)
     else:
         _show_database_pylucene(database)
@@ -385,14 +438,15 @@ def _show_database_pylucene(database):
     for index in range(reader.maxDoc()):
         print reader.document(index).toString().encode("charmap")
 
+
 def _show_database_xapian(database):
     import xapian
-    doccount = database.database.get_doccount()
-    max_doc_index = database.database.get_lastdocid()
+    doccount = database.reader.get_doccount()
+    max_doc_index = database.reader.get_lastdocid()
     print "Database overview: %d items up to index %d" % (doccount, max_doc_index)
-    for index in range(1, max_doc_index+1):
+    for index in range(1, max_doc_index + 1):
         try:
-            document = database.database.get_document(index)
+            document = database.reader.get_document(index)
         except xapian.DocNotFoundError:
             continue
         # print the document's terms and their positions
@@ -402,17 +456,20 @@ def _show_database_xapian(database):
 
 
 def _get_number_of_docs(database):
-    if hasattr(database, "database"):
+    if database.INDEX_DIRECTORY_NAME == "xapian":
         # xapian
-        return database.database.get_lastdocid()
+        return database.reader.get_doccount()
     else:
         # pylucene
-        database.flush()
+        database._writer_close()
+        database._index_refresh()
         return database.reader.numDocs()
 
+
 def get_engine_name(database):
     return database.__module__
 
+
 def report_whitelisted_success(db, name):
     """ Output a warning message regarding a successful unittest, that was
     supposed to fail for a specific indexing engine.
@@ -421,6 +478,7 @@ def report_whitelisted_success(db, name):
     print "the test '%s' works again for '%s' - please remove the exception" \
             % (name, get_engine_name(db))
 
+
 def report_whitelisted_failure(db, name):
     """ Output a warning message regarding a unittest, that was supposed to fail
     for a specific indexing engine.
@@ -430,6 +488,7 @@ def report_whitelisted_failure(db, name):
     print "the test '%s' fails - as expected for '%s'" % (name,
             get_engine_name(db))
 
+
 def assert_whitelisted(db, assert_value, white_list_engines, name_of_check):
     """ Do an assertion, but ignoring failure for specific indexing engines.
     This can be used for almost-complete implementations, that just need
@@ -472,6 +531,7 @@ if __name__ == "__main__":
         test_field_analyzers()
         test_and_queries()
         test_or_queries()
+        test_string_queries()
         test_lower_upper_case()
         test_tokenizing()
         test_searching()
@@ -480,4 +540,3 @@ if __name__ == "__main__":
         # TODO: add test for transaction handling
         # TODO: add test for multiple engine/database handling in "get_indexer"
     clean_database()
-
