@@ -18,16 +18,19 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""A class to calculate a similarity based on the Levenshtein 
-distance. See http://en.wikipedia.org/wiki/Levenshtein_distance.
+"""A class to calculate a similarity based on the Levenshtein
+distance.
 
-If available, the python-Levenshtein package will be used which will provide
-better performance as it is implemented natively. See
-http://trific.ath.cx/python/levenshtein/
+See http://en.wikipedia.org/wiki/Levenshtein_distance.
+
+If available, the `python-Levenshtein
+<https://pypi.python.org/pypi/python-Levenshtein>`_ will be used which will
+provide better performance as it is implemented natively.
 """
 
 import math
 
+
 def python_distance(a, b, stopvalue=-1):
     """Calculates the distance for use in similarity calculation. Python
     version."""
@@ -35,9 +38,9 @@ def python_distance(a, b, stopvalue=-1):
     l2 = len(b)
     if stopvalue == -1:
         stopvalue = l2
-    current = range(l1+1)
-    for i in range(1, l2+1):
-        previous, current = current, [i]+[0]*l1
+    current = range(l1 + 1)
+    for i in range(1, l2 + 1):
+        previous, current = current, ([i] + [0] * l1)
         least = l2
         for j in range(1, l1 + 1):
             change = previous[j-1]
@@ -55,23 +58,26 @@ def python_distance(a, b, stopvalue=-1):
 
     return current[l1]
 
+
 def native_distance(a, b, stopvalue=0):
-    """Same as python_distance in functionality. This uses the fast C 
+    """Same as python_distance in functionality. This uses the fast C
     version if we detected it earlier.
 
-    Note that this does not support arbitrary sequence types, but only 
+    Note that this does not support arbitrary sequence types, but only
     string types."""
     return Levenshtein.distance(a, b)
 
 try:
     import Levenshtein as Levenshtein
     distance = native_distance
-except Exception:
+except ImportError:
     import logging
     logging.warning("Python-Levenshtein not found. Continuing with built-in (slower) fuzzy matching.")
     distance = python_distance
 
+
 class LevenshteinComparer:
+
     def __init__(self, max_len=200):
         self.MAX_LEN = max_len
 
@@ -105,7 +111,7 @@ class LevenshteinComparer:
                  at the end will therefore seem to match better than they should. See the
                  use of the variable penalty to lessen the effect of this.
                - Strings with widely different lengths give the opportunity for shortcut.
-                 This is by definition of the Levenshtein distance: the distance will be 
+                 This is by definition of the Levenshtein distance: the distance will be
                  at least as much as the difference in string length.
                - Calculation is stopped as soon as a similarity of stoppercentage becomes
                  unattainable. See the use of the variable stopvalue.
@@ -122,7 +128,7 @@ class LevenshteinComparer:
 
         #maxsimilarity is the maximum similarity that can be attained as constrained
         #by the difference in string length
-        maxsimilarity = 100 - 100.0*abs(l1 - l2)/l2
+        maxsimilarity = 100 - 100.0 * (l2 - l1) / l2
         if maxsimilarity < stoppercentage:
             return maxsimilarity * 1.0
 
@@ -138,16 +144,16 @@ class LevenshteinComparer:
                 penalty += 7
 
         #The actual value in the array that would represent a giveup situation:
-        stopvalue = math.ceil((100.0 - stoppercentage)/100 * l2)
+        stopvalue = math.ceil((100.0 - stoppercentage) / 100 * l2)
         dist = distance(a, b, stopvalue)
         if dist > stopvalue:
             return stoppercentage - 1.0
 
-        #If MAX_LEN came into play, we consider the calculated distance to be 
+        #If MAX_LEN came into play, we consider the calculated distance to be
         #representative of the distance between the whole, untrimmed strings
         if dist != 0:
             penalty = 0
-        return 100 - (dist*1.0/l2)*100 - penalty
+        return 100 - (dist * 1.0 / l2) * 100 - penalty
 
 
 if __name__ == "__main__":
