@@ -18,8 +18,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""Class to perform translation memory matching from a store of translation units"""
+"""Class to perform translation memory matching from a store of
+translation units."""
 
+import itertools
 import heapq
 import re
 
@@ -31,12 +33,23 @@ from translate.misc.multistring import multistring
 
 
 def sourcelen(unit):
-    """Returns the length of the source string"""
+    """Returns the length of the source string."""
     return len(unit.source)
 
 
+def _sort_matches(matches, match_info):
+
+    def _matches_cmp(x, y):
+        # This function will sort a list of matches according to the match's starting
+        # position, putting the one with the longer source text first, if two are the same.
+        c = cmp(match_info[x.source]['pos'], match_info[y.source]['pos'])
+        return c and c or cmp(len(y.source), len(x.source))
+    matches.sort(_matches_cmp)
+
+
 class matcher(object):
-    """A class that will do matching and store configuration for the matching process"""
+    """A class that will do matching and store configuration for the
+    matching process."""
 
     sort_reverse = False
 
@@ -85,17 +98,16 @@ class matcher(object):
     def extendtm(self, units, store=None, sort=True):
         """Extends the memory with extra unit(s).
 
-        @param units: The units to add to the TM.
-        @param store: Optional store from where some metadata can be retrieved
-        and associated with each unit.
-        @param sort:  Optional parameter that can be set to False to supress
-        sorting of the candidates list. This should probably only be used in
-        inittm().
+        :param units: The units to add to the TM.
+        :param store: Optional store from where some metadata can be retrieved
+                      and associated with each unit.
+        :param sort: Optional parameter that can be set to False to supress
+                     sorting of the candidates list. This should probably
+                     only be used in :meth:`matcher.inittm`.
         """
         if isinstance(units, base.TranslationUnit):
             units = [units]
-        candidates = filter(self.usable, units)
-        for candidate in candidates:
+        for candidate in itertools.ifilter(self.usable, units):
             simpleunit = base.TranslationUnit("")
             # We need to ensure that we don't pass multistrings futher, since
             # some modules (like the native Levenshtein) can't use it.
@@ -128,24 +140,25 @@ class matcher(object):
     def getstoplength(self, min_similarity, text):
         """Calculates a length beyond which we are not interested.
         The extra fat is because we don't use plain character distance only."""
-        return min(len(text) / (min_similarity/100.0), self.MAX_LENGTH)
+        return min(len(text) / (min_similarity / 100.0), self.MAX_LENGTH)
 
     def getstartlength(self, min_similarity, text):
         """Calculates the minimum length we are interested in.
         The extra fat is because we don't use plain character distance only."""
-        return max(len(text) * (min_similarity/100.0), 1)
+        return max(len(text) * (min_similarity / 100.0), 1)
 
     def matches(self, text):
         """Returns a list of possible matches for given source text.
 
-        @type text: String
-        @param text: The text that will be search for in the translation memory
-        @rtype: list
-        @return: a list of units with the source and target strings from the
-        translation memory. If self.addpercentage is true (default) the match
-        quality is given as a percentage in the notes.
+        :type text: String
+        :param text: The text that will be search for in the translation memory
+        :rtype: list
+        :return: a list of units with the source and target strings from the
+                 translation memory. If :attr:`self.addpercentage` is
+                 *True* (default) the match quality is given as a
+                 percentage in the notes.
         """
-        bestcandidates = [(0.0, None)]*self.MAX_CANDIDATES
+        bestcandidates = [(0.0, None)] * self.MAX_CANDIDATES
         #We use self.MIN_SIMILARITY, but if we already know we have max_candidates
         #that are better, we can adjust min_similarity upwards for speedup
         min_similarity = self.MIN_SIMILARITY
@@ -196,7 +209,8 @@ class matcher(object):
         return self.buildunits(bestcandidates)
 
     def buildunits(self, candidates):
-        """Builds a list of units conforming to base API, with the score in the comment"""
+        """Builds a list of units conforming to base API, with the score
+        in the comment."""
         units = []
         for score, candidate in candidates:
             if hasattr(candidate, "orig_source"):
@@ -222,16 +236,18 @@ class matcher(object):
 # The tuples define a regular expression to search for, and with what it
 # should be replaced.
 ignorepatterns = [
-    ("y\s*$", "ie"),          #category/categories, identify/identifies, apply/applied
-    ("[\s-]+", ""),           #down time / downtime, pre-order / preorder
-    ("-", " "),               #pre-order / pre order
-    (" ", "-"),               #pre order / pre-order
+    ("y\s*$", "ie"),          # category/categories, identify/identifies, apply/applied
+    ("[\s-]+", ""),           # down time / downtime, pre-order / preorder
+    ("-", " "),               # pre-order / pre order
+    (" ", "-"),               # pre order / pre-order
 ]
+ignorepatterns_re = [(re.compile(a), b) for (a, b) in ignorepatterns]
 
 context_re = re.compile("\s+\(.*\)\s*$")
 
+
 class terminologymatcher(matcher):
-    """A matcher with settings specifically for terminology matching"""
+    """A matcher with settings specifically for terminology matching."""
 
     sort_reverse = True
 
@@ -248,9 +264,11 @@ class terminologymatcher(matcher):
         extras = []
         for unit in self.candidates.units:
             source = unit.source = context_re.sub("", unit.source).lower()
-            for ignorepattern in ignorepatterns:
-                (newterm, occurrences) = re.subn(ignorepattern[0], ignorepattern[1], source)
-                if occurrences:
+            for ignorepattern_re, replacement in ignorepatterns_re:
+                (newterm, occurrences) = ignorepattern_re.subn(replacement, source)
+                # we'll add it as long as we only replaced one thing, but not
+                # something like "are-you-sure-you-want-to" due to (" ", "-")
+                if occurrences == 1:
                     new_unit = type(unit).buildfromunit(unit)
                     new_unit.source = newterm
                     # We mark it fuzzy to indicate that it isn't pristine
@@ -282,30 +300,85 @@ class terminologymatcher(matcher):
     def matches(self, text):
         """Normal matching after converting text to lower case. Then replace
         with the original unit to retain comments, etc."""
+        text_l = len(text)
+        if text_l < self.getstartlength(0, ''):  # parameters unused
+            # impossible to return anything
+            return []
         text = text.lower()
         comparer = self.comparer
         comparer.match_info = {}
+        match_info = {}
         matches = []
         known = set()
-        for cand in self.candidates.units:
-            if (cand.source, cand.target) in known:
-                continue
+
+        # We want to limit our search in self.candidates, so we want to ignore
+        # all units with a source string that is too long. We use binary search
+        # to find the first string short enough to occur in text, from where we
+        # start our search in the candidates.
+
+        # the maximum possible length is text_l
+        startindex = 0
+        endindex = len(self.candidates.units)
+        while startindex < endindex:
+            mid = (startindex + endindex) // 2
+            if sourcelen(self.candidates.units[mid]) > text_l:
+                startindex = mid + 1
+            else:
+                endindex = mid
+
+        for cand in self.candidates.units[startindex:]:
             source = cand.source
+            if (source, cand.target) in known:
+                continue
             if comparer.similarity(text, source, self.MIN_SIMILARITY):
-                self.match_info[source] = {'pos': comparer.match_info[source]['pos']}
+                match_info[source] = {'pos': comparer.match_info[source]['pos']}
                 matches.append(cand)
-                known.add((cand.source, cand.target))
-        return matches
+                known.add((source, cand.target))
+
+        final_matches = []
+        lastend = 0
+        _sort_matches(matches, match_info)
+        for match in matches:
+            start_pos = match_info[match.source]['pos']
+            if start_pos < lastend:
+                continue
+            end = start_pos + len(match.source)
+
+            final_matches.append(match)
+
+            # Get translations for the placeable
+            for m in matches:
+                if m is match:
+                    continue
+                m_info = match_info[m.source]
+                m_end = m_info['pos']
+                if m_end > start_pos:
+                    # we past valid possibilities in the list
+                    break
+                m_end += len(m.source)
+                if start_pos == m_info['pos'] and end == m_end:
+                    # another match for the same term
+                    final_matches.append(m)
+
+            lastend = end
+        if final_matches:
+            self.match_info = match_info
+        return final_matches
 
 
 # utility functions used by virtaal and tmserver to convert matching units in easily marshallable dictionaries
 def unit2dict(unit):
     """converts a pounit to a simple dict structure for use over the web"""
-    return {"source": unit.source, "target": unit.target,
-            "quality": _parse_quality(unit.getnotes()), "context": unit.getcontext()}
+    return {
+        "source": unit.source,
+        "target": unit.target,
+        "quality": _parse_quality(unit.getnotes()),
+        "context": unit.getcontext()
+    }
+
 
 def _parse_quality(comment):
-    """extracts match quality from po comments"""
+    """Extracts match quality from po comments."""
     quality = re.search('([0-9]+)%', comment)
     if quality:
         return quality.group(1)
