@@ -18,22 +18,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""Base classes for storage interfaces.
-
-@organization: Zuza Software Foundation
-@copyright: 2006-2009 Zuza Software Foundation
-@license: U{GPL <http://www.fsf.org/licensing/licenses/gpl.html>}
-"""
+"""Base classes for storage interfaces."""
 
+import logging
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
 from exceptions import NotImplementedError
+
 import translate.i18n
-from translate.storage.placeables import StringElem, general, parse as rich_parse
-from translate.misc.typecheck import accepts, Self, IsOneOf
 from translate.misc.multistring import multistring
+from translate.misc.typecheck import accepts, Self, IsOneOf
+from translate.storage.placeables import (StringElem, general,
+                                          parse as rich_parse)
+from translate.storage.workflow import StateEnum as states
 
 
 def force_override(method, baseclass):
@@ -47,8 +46,7 @@ def force_override(method, baseclass):
     if actualclass != baseclass:
         raise NotImplementedError(
             "%s does not reimplement %s as required by %s" % \
-            (actualclass.__name__, method.__name__, baseclass.__name__)
-        )
+            (actualclass.__name__, method.__name__, baseclass.__name__))
 
 
 class ParseError(Exception):
@@ -63,53 +61,91 @@ class ParseError(Exception):
 class TranslationUnit(object):
     """Base class for translation units.
 
-    Our concept of a I{translation unit} is influenced heavily by XLIFF:
-    U{http://www.oasis-open.org/committees/xliff/documents/xliff-specification.htm}
+    Our concept of a *translation unit* is influenced heavily by `XLIFF
+    <http://docs.oasis-open.org/xliff/xliff-core/xliff-core.html>`_.
 
-    As such most of the method- and variable names borrows from XLIFF terminology.
+    As such most of the method- and variable names borrows from XLIFF
+    terminology.
 
     A translation unit consists of the following:
-      - A I{source} string. This is the original translatable text.
-      - A I{target} string. This is the translation of the I{source}.
-      - Zero or more I{notes} on the unit. Notes would typically be some
-        comments from a translator on the unit, or some comments originating from
-        the source code.
-      - Zero or more I{locations}. Locations indicate where in the original
-        source code this unit came from.
-      - Zero or more I{errors}. Some tools (eg. L{pofilter <filters.pofilter>}) can run checks on
-        translations and produce error messages.
 
-    @group Source: *source*
-    @group Target: *target*
-    @group Notes: *note*
-    @group Locations: *location*
-    @group Errors: *error*
+      - A *source* string. This is the original translatable text.
+      - A *target* string. This is the translation of the *source*.
+      - Zero or more *notes* on the unit. Notes would typically be some
+        comments from a translator on the unit, or some comments originating
+        from the source code.
+      - Zero or more *locations*. Locations indicate where in the original
+        source code this unit came from.
+      - Zero or more *errors*. Some tools (eg.
+        :mod:`~translate.filters.pofilter`)
+        can run checks on translations and produce error messages.
+
     """
 
     rich_parsers = []
-    """A list of functions to use for parsing a string into a rich string tree."""
+    """A list of functions to use for parsing a string into a rich string
+    tree."""
 
-    def __init__(self, source):
+    # State constants
+    S_OBSOLETE = states.OBSOLETE
+    S_EMPTY = states.EMPTY
+    S_NEEDS_WORK = states.NEEDS_WORK
+    S_REJECTED = states.REJECTED
+    S_NEEDS_REVIEW = states.NEEDS_REVIEW
+    S_UNREVIEWED = states.UNREVIEWED
+    S_FINAL = states.FINAL
+
+    # Elaborate state support could look something like this:
+    # STATE = {
+    #     S_OBSOLETE: (states.OBSOLETE, states.EMPTY),
+    #     S_EMPTY: (states.EMPTY, states.NEEDS_WORK),
+    #     S_NEEDS_WORK: (states.NEEDS_WORK, states.REJECTED),
+    #     S_REJECTED: (states.REJECTED, states.NEEDS_REVIEW),
+    #     S_NEEDS_REVIEW: (states.NEEDS_REVIEW, states.UNREVIEWED),
+    #     S_UNREVIEWED: (states.UNREVIEWED, states.FINAL),
+    #     S_FINAL: (states.FINAL, states.MAX),
+    # }
+    # """
+    # Default supported states:
+    #     * obsolete: The unit is not to be used.
+    #     * empty: The unit has not been translated before.
+    #     * needs work: Some translation has been done, but is not complete.
+    #     * rejected: The unit has been reviewed, but was rejected.
+    #     * needs review: The unit has been translated, but review was requested.
+    #     * unreviewed: The unit has been translated, but not reviewed.
+    #     * final: The unit is translated, reviewed and accepted.
+    # """
+    #
+    # ... but by default a format will not support state:
+    STATE = {}
+
+    _store = None
+    _source = None
+    _target = None
+    _rich_source = None
+    _rich_target = None
+    _state_n = 0
+    notes = ""
+
+    def __init__(self, source=None):
         """Constructs a TranslationUnit containing the given source string."""
-        self.notes = ""
-        self._store = None
-        self.source = source
-        self._target = None
-        self._rich_source = None
-        self._rich_target = None
+        if source is not None:
+            self.source = source
 
     def __eq__(self, other):
         """Compares two TranslationUnits.
 
-        @type other: L{TranslationUnit}
-        @param other: Another L{TranslationUnit}
-        @rtype: Boolean
-        @return: Returns True if the supplied TranslationUnit equals this unit.
+        :type other: :class:`TranslationUnit`
+        :param other: Another :class:`TranslationUnit`
+        :rtype: Boolean
+        :return: Returns *True* if the supplied :class:`TranslationUnit`
+                 equals this unit.
         """
         return self.source == other.source and self.target == other.target
 
     def __str__(self):
-        """Converts to a string representation that can be parsed back using L{parsestring()}."""
+        """Converts to a string representation that can be parsed back using
+        :meth:`~.TranslationStore.parsestring`."""
         # no point in pickling store object, so let's hide it for a while.
         store = getattr(self, "_store", None)
         self._store = None
@@ -117,8 +153,9 @@ class TranslationUnit(object):
         self._store = store
         return dump
 
+    @classmethod
     def rich_to_multistring(cls, elem_list):
-        """Convert a "rich" string tree to a C{multistring}:
+        """Convert a "rich" string tree to a ``multistring``:
 
            >>> from translate.storage.placeables.interfaces import X
            >>> rich = [StringElem(['foo', X(id='xxx', sub=[' ']), 'bar'])]
@@ -126,7 +163,6 @@ class TranslationUnit(object):
            multistring(u'foo bar')
         """
         return multistring([unicode(elem) for elem in elem_list])
-    rich_to_multistring = classmethod(rich_to_multistring)
 
     def multistring_to_rich(self, mulstring):
         """Convert a multistring to a list of "rich" string trees:
@@ -142,13 +178,13 @@ class TranslationUnit(object):
         return [rich_parse(mulstring, self.rich_parsers)]
 
     def setsource(self, source):
-        """Sets the source string to the given value."""
+        """Set the source string to the given value."""
         self._rich_source = None
         self._source = source
     source = property(lambda self: self._source, setsource)
 
     def settarget(self, target):
-        """Sets the target string to the given value."""
+        """Set the target string to the given value."""
         self._rich_target = None
         self._target = target
     target = property(lambda self: self._target, settarget)
@@ -170,8 +206,9 @@ class TranslationUnit(object):
         if self.source != multi:
             self.source = multi
     rich_source = property(_get_rich_source, _set_rich_source)
-    """ @see: rich_to_multistring
-        @see: multistring_to_rich"""
+    """
+    .. seealso:: :meth:`.rich_to_multistring`, :meth:`multistring_to_rich`
+    """
 
     def _get_rich_target(self):
         if self._rich_target is None:
@@ -188,14 +225,18 @@ class TranslationUnit(object):
         self._rich_target = list(value)
         self.target = self.rich_to_multistring(value)
     rich_target = property(_get_rich_target, _set_rich_target)
-    """ @see: rich_to_multistring
-        @see: multistring_to_rich"""
+    """
+    .. seealso:: :meth:`.rich_to_multistring`, :meth:`.multistring_to_rich`
+    """
 
     def gettargetlen(self):
         """Returns the length of the target string.
 
-        @note: Plural forms might be combined.
-        @rtype: Integer
+        :rtype: Integer
+
+        .. note::
+
+           Plural forms might be combined.
         """
         length = len(self.target or "")
         strings = getattr(self.target, "strings", [])
@@ -206,8 +247,8 @@ class TranslationUnit(object):
     def getid(self):
         """A unique identifier for this unit.
 
-        @rtype: string
-        @return: an identifier for this unit that is unique in the store
+        :rtype: string
+        :return: an identifier for this unit that is unique in the store
 
         Derived classes should override this in a way that guarantees a unique
         identifier for each unit in the store.
@@ -224,24 +265,34 @@ class TranslationUnit(object):
     def getlocations(self):
         """A list of source code locations.
 
-        @note: Shouldn't be implemented if the format doesn't support it.
-        @rtype: List
+        :rtype: List
+
+        .. note::
+
+           Shouldn't be implemented if the format doesn't support it.
         """
         return []
 
     def addlocation(self, location):
         """Add one location to the list of locations.
 
-        @note: Shouldn't be implemented if the format doesn't support it.
+        .. note::
+
+           Shouldn't be implemented if the format doesn't support it.
         """
         pass
 
     def addlocations(self, location):
         """Add a location or a list of locations.
 
-        @note: Most classes shouldn't need to implement this,
-               but should rather implement L{addlocation()}.
-        @warning: This method might be removed in future.
+        .. note::
+
+           Most classes shouldn't need to implement this, but should rather
+           implement :meth:`TranslationUnit.addlocation`.
+
+        .. warning::
+
+           This method might be removed in future.
         """
         if isinstance(location, list):
             for item in location:
@@ -262,20 +313,21 @@ class TranslationUnit(object):
 
         It will probably be freeform text or something reasonable that can be
         synthesised by the format.
-        It should not include location comments (see L{getlocations()}).
+        It should not include location comments (see
+        :meth:`~.TranslationUnit.getlocations`).
         """
         return getattr(self, "notes", "")
 
     def addnote(self, text, origin=None, position="append"):
         """Adds a note (comment).
 
-        @type text: string
-        @param text: Usually just a sentence or two.
-        @type origin: string
-        @param origin: Specifies who/where the comment comes from.
+        :type text: string
+        :param text: Usually just a sentence or two.
+        :type origin: string
+        :param origin: Specifies who/where the comment comes from.
                        Origin can be one of the following text strings:
-                         - 'translator'
-                         - 'developer', 'programmer', 'source code' (synonyms)
+                       - 'translator'
+                       - 'developer', 'programmer', 'source code' (synonyms)
         """
         if position == "append" and getattr(self, "notes", None):
             self.notes += '\n' + text
@@ -289,25 +341,25 @@ class TranslationUnit(object):
     def adderror(self, errorname, errortext):
         """Adds an error message to this unit.
 
-        @type errorname: string
-        @param errorname: A single word to id the error.
-        @type errortext: string
-        @param errortext: The text describing the error.
+        :type errorname: string
+        :param errorname: A single word to id the error.
+        :type errortext: string
+        :param errortext: The text describing the error.
         """
         pass
 
     def geterrors(self):
         """Get all error messages.
 
-        @rtype: Dictionary
+        :rtype: Dictionary
         """
         return {}
 
     def markreviewneeded(self, needsreview=True, explanation=None):
         """Marks the unit to indicate whether it needs review.
 
-        @keyword needsreview: Defaults to True.
-        @keyword explanation: Adds an optional explanation as a note.
+        :keyword needsreview: Defaults to True.
+        :keyword explanation: Adds an optional explanation as a note.
         """
         pass
 
@@ -326,7 +378,7 @@ class TranslationUnit(object):
         This should be used to distinguish real units for translation from
         header, obsolete, binary or other blank units.
         """
-        return True
+        return bool(self.source)
 
     def isfuzzy(self):
         """Indicates whether this unit is fuzzy."""
@@ -355,8 +407,11 @@ class TranslationUnit(object):
     def isblank(self):
         """Used to see if this unit has no source or target string.
 
-        @note: This is probably used more to find translatable units,
-        and we might want to move in that direction rather and get rid of this.
+        .. note::
+
+           This is probably used more to find translatable units,
+           and we might want to move in that direction rather and
+           get rid of this.
         """
         return not (self.source or self.target)
 
@@ -366,12 +421,13 @@ class TranslationUnit(object):
         return False
 
     def getsourcelanguage(self):
-        return getattr(self._store, "sourcelanguage", "en")
+        return self._store.getsourcelanguage()
 
     def gettargetlanguage(self):
-        return getattr(self._store, "targetlanguage", None)
+        return self._store.gettargetlanguage()
 
-    def merge(self, otherunit, overwrite=False, comments=True, authoritative=False):
+    def merge(self, otherunit, overwrite=False, comments=True,
+              authoritative=False):
         """Do basic format agnostic merging."""
         if not self.target or overwrite:
             self.rich_target = otherunit.rich_target
@@ -384,6 +440,7 @@ class TranslationUnit(object):
         """This unit in a list."""
         return [self]
 
+    @classmethod
     def buildfromunit(cls, unit):
         """Build a native unit from a foreign unit, preserving as much
         information as possible."""
@@ -399,14 +456,39 @@ class TranslationUnit(object):
         if notes:
             newunit.addnote(notes)
         return newunit
-    buildfromunit = classmethod(buildfromunit)
 
     xid = property(lambda self: None, lambda self, value: None)
     rid = property(lambda self: None, lambda self, value: None)
 
+    def get_state_id(self, n=None):
+        if n is None:
+            n = self.get_state_n()
+        for state_id, state_range in self.STATE.iteritems():
+            if state_range[0] <= n < state_range[1]:
+                return state_id
+        if self.STATE:
+            raise ValueError('No state containing value %s' % (n))
+        else:
+            return n
+
+    def get_state_n(self):
+        if self.STATE:
+            return self._state_n
+        else:
+            return self.istranslated() and self.S_UNREVIEWED or self.S_EMPTY
+
+    def set_state_n(self, value):
+        self._state_n = value
+
+    def infer_state(self):
+        """Empty method that should be overridden in sub-classes to infer the
+            current state(_n) of the unit from its current state."""
+        pass
+
 
 class TranslationStore(object):
-    """Base class for stores for multiple translation units of type UnitClass."""
+    """Base class for stores for multiple translation units of type
+    UnitClass."""
 
     UnitClass = TranslationUnit
     """The class of units that will be instantiated and used by this class"""
@@ -419,33 +501,42 @@ class TranslationStore(object):
     _binary = False
     """Indicates whether a file should be accessed as a binary file."""
     suggestions_in_format = False
-    """Indicates if format can store suggestions and alternative translation for a unit"""
+    """Indicates if format can store suggestions and alternative translation
+    for a unit"""
+
+    sourcelanguage = None
+    targetlanguage = None
 
     def __init__(self, unitclass=None):
-        """Constructs a blank TranslationStore."""
+        """Construct a blank TranslationStore."""
         self.units = []
-        self.sourcelanguage = None
-        self.targetlanguage = None
         if unitclass:
             self.UnitClass = unitclass
-        super(TranslationStore, self).__init__()
 
     def getsourcelanguage(self):
-        """Gets the source language for this store"""
+        """Get the source language for this store."""
         return self.sourcelanguage
 
     def setsourcelanguage(self, sourcelanguage):
-        """Sets the source language for this store"""
+        """Set the source language for this store."""
         self.sourcelanguage = sourcelanguage
 
     def gettargetlanguage(self):
-        """Gets the target language for this store"""
+        """Get the target language for this store."""
         return self.targetlanguage
 
     def settargetlanguage(self, targetlanguage):
-        """Sets the target language for this store"""
+        """Set the target language for this store."""
         self.targetlanguage = targetlanguage
 
+    def getprojectstyle(self):
+        """Get the project type for this store."""
+        return getattr(self, '_project_style', None)
+
+    def setprojectstyle(self, project_style):
+        """Set the project type for this store."""
+        self._project_style = project_style
+
     def unit_iter(self):
         """Iterator over all the units in this store."""
         for unit in self.units:
@@ -456,21 +547,21 @@ class TranslationStore(object):
         return [unit for unit in self.unit_iter()]
 
     def addunit(self, unit):
-        """Appends the given unit to the object's list of units.
+        """Append the given unit to the object's list of units.
 
         This method should always be used rather than trying to modify the
         list manually.
 
-        @type unit: L{TranslationUnit}
-        @param unit: The unit that will be added.
+        :type unit: :class:`TranslationUnit`
+        :param unit: The unit that will be added.
         """
         unit._store = self
         self.units.append(unit)
 
     def addsourceunit(self, source):
-        """Adds and returns a new unit with the given source string.
+        """Add and returns a new unit with the given source string.
 
-        @rtype: L{TranslationUnit}
+        :rtype: :class:`TranslationUnit`
         """
         unit = self.UnitClass(source)
         self.addunit(unit)
@@ -482,9 +573,9 @@ class TranslationStore(object):
         return self.id_index.get(id, None)
 
     def findunit(self, source):
-        """Finds the unit with the given source string.
+        """Find the unit with the given source string.
 
-        @rtype: L{TranslationUnit} or None
+        :rtype: :class:`TranslationUnit` or None
         """
         if len(getattr(self, "sourceindex", [])):
             if source in self.sourceindex:
@@ -496,9 +587,9 @@ class TranslationStore(object):
         return None
 
     def findunits(self, source):
-        """Finds the units with the given source string.
+        """Find the units with the given source string.
 
-        @rtype: L{TranslationUnit} or None
+        :rtype: :class:`TranslationUnit` or None
         """
         if len(getattr(self, "sourceindex", [])):
             if source in self.sourceindex:
@@ -514,9 +605,9 @@ class TranslationStore(object):
         return None
 
     def translate(self, source):
-        """Returns the translated string for a given source string.
+        """Return the translated string for a given source string.
 
-        @rtype: String or None
+        :rtype: String or None
         """
         unit = self.findunit(source)
         if unit and unit.target:
@@ -543,7 +634,8 @@ class TranslationStore(object):
             remove_unit(unit.source)
 
         for location in unit.getlocations():
-            if location in self.locationindex and self.locationindex[location] is not None \
+            if location in self.locationindex \
+                   and self.locationindex[location] is not None \
                    and self.locationindex[location] == unit:
                 del(self.locationindex[location])
 
@@ -572,13 +664,14 @@ class TranslationStore(object):
                 self.locationindex[location] = unit
 
     def makeindex(self):
-        """Indexes the items in this store. At least .sourceindex should be usefull."""
+        """Indexes the items in this store. At least .sourceindex should be
+        useful."""
         self.locationindex = {}
         self.sourceindex = {}
         self.id_index = {}
         for index, unit in enumerate(self.units):
             unit.index = index
-            if unit.istranslatable():
+            if not (unit.isheader() or unit.isblank()):
                 self.add_unit_to_index(unit)
 
     def require_index(self):
@@ -602,7 +695,8 @@ class TranslationStore(object):
             self.fileobj = open(self.filename)
 
     def __str__(self):
-        """Converts to a string representation that can be parsed back using L{parsestring()}."""
+        """Converts to a string representation that can be parsed back using
+        :meth:`~.TranslationStore.parsestring`."""
         # We can't pickle fileobj if it is there, so let's hide it for a while.
         fileobj = getattr(self, "fileobj", None)
         self.fileobj = None
@@ -611,7 +705,7 @@ class TranslationStore(object):
         return dump
 
     def isempty(self):
-        """Returns True if the object doesn't contain any translation units."""
+        """Return True if the object doesn't contain any translation units."""
         if len(self.units) == 0:
             return True
         for unit in self.units:
@@ -624,24 +718,71 @@ class TranslationStore(object):
         assigns it to .filename."""
         fileobj = getattr(self, "fileobj", None)
         if fileobj:
-            filename = getattr(fileobj, "name", getattr(fileobj, "filename", None))
+            filename = getattr(fileobj, "name",
+                               getattr(fileobj, "filename", None))
             if filename:
                 self.filename = filename
 
+    @classmethod
     def parsestring(cls, storestring):
-        """Converts the string representation back to an object."""
+        """Convert the string representation back to an object."""
         newstore = cls()
         if storestring:
             newstore.parse(storestring)
         return newstore
-    parsestring = classmethod(parsestring)
+
+    def detect_encoding(self, text, default_encodings=None):
+        if not default_encodings:
+            default_encodings = ['utf-8']
+        try:
+            import chardet
+            # many false complaints with ellipse (see bug 1825)
+            detected_encoding = chardet.detect(text.replace("…", ""))
+            if detected_encoding['confidence'] < 0.48:
+                detected_encoding = None
+            elif detected_encoding['encoding'] == 'ascii':
+                detected_encoding['encoding'] = 'utf-8'
+        except ImportError:
+            detected_encoding = None
+
+        encodings = []
+        if self.encoding == 'auto':
+            if detected_encoding and detected_encoding['encoding'] not in encodings:
+                encodings.append(detected_encoding['encoding'])
+            for encoding in default_encodings:
+                if encoding not in encodings:
+                    encodings.append(encoding)
+        else:
+            encodings.append(self.encoding)
+            if (detected_encoding and
+                    detected_encoding['encoding'] != self.encoding and
+                    detected_encoding['confidence'] != 1.0):
+                logging.warn("trying to parse %s with encoding: %s but "
+                             "detected encoding is %s (confidence: %s)",
+                             self.filename, self.encoding,
+                             detected_encoding['encoding'],
+                             detected_encoding['confidence'])
+            encodings.append(self.encoding)
+
+        for encoding in encodings:
+            try:
+                r_text = unicode(text, encoding)
+                r_encoding = encoding
+                break
+            except UnicodeDecodeError:
+                r_text = None
+                r_encoding = None
+        if r_encoding == 'ascii':
+            r_encoding = 'utf-8'
+        return r_text, r_encoding
 
     def parse(self, data):
         """parser to process the given source string"""
         self.units = pickle.loads(data).units
 
     def savefile(self, storefile):
-        """Writes the string representation to the given file (or filename)."""
+        """Write the string representation to the given file (or filename)."""
+        storestring = str(self)
         if isinstance(storefile, basestring):
             mode = 'w'
             if self._binary:
@@ -649,12 +790,12 @@ class TranslationStore(object):
             storefile = open(storefile, mode)
         self.fileobj = storefile
         self._assignname()
-        storestring = str(self)
         storefile.write(storestring)
         storefile.close()
 
     def save(self):
-        """Save to the file that data was originally read from, if available."""
+        """Save to the file that data was originally read from, if
+        available."""
         fileobj = getattr(self, "fileobj", None)
         mode = 'w'
         if self._binary:
@@ -665,14 +806,17 @@ class TranslationStore(object):
                 fileobj = file(filename, mode)
         else:
             fileobj.close()
-            filename = getattr(fileobj, "name", getattr(fileobj, "filename", None))
+            filename = getattr(fileobj, "name",
+                               getattr(fileobj, "filename", None))
             if not filename:
                 raise ValueError("No file or filename to save to")
             fileobj = fileobj.__class__(filename, mode)
         self.savefile(fileobj)
 
+    @classmethod
     def parsefile(cls, storefile):
-        """Reads the given file (or opens the given filename) and parses back to an object."""
+        """Reads the given file (or opens the given filename) and parses back
+        to an object."""
         mode = 'r'
         if cls._binary:
             mode = 'rb'
@@ -689,4 +833,12 @@ class TranslationStore(object):
         newstore.fileobj = storefile
         newstore._assignname()
         return newstore
-    parsefile = classmethod(parsefile)
+
+    @property
+    def merge_on(self):
+        """The matching criterion to use when merging on.
+
+        :return: The default matching criterion for all the subclasses.
+        :rtype: string
+        """
+        return "id"
