@@ -15,18 +15,22 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-from translate.storage import factory
-import os
 import cProfile
+import os
 import pstats
 import random
 import sys
+import argparse
+
+from translate.storage import factory
+from translate.storage import placeables
+
 
 class TranslateBenchmarker:
     """class to aid in benchmarking Translate Toolkit stores"""
+
     def __init__(self, test_dir, storeclass):
         """sets up benchmarking on the test directory"""
         self.test_dir = os.path.abspath(test_dir)
@@ -34,6 +38,7 @@ class TranslateBenchmarker:
         self.extension = self.StoreClass.Extensions[0]
         self.project_dir = os.path.join(self.test_dir, "benchmark")
         self.file_dir = os.path.join(self.project_dir, "zxx")
+        self.parsedfiles = []
 
     def clear_test_dir(self):
         """removes the given directory"""
@@ -70,26 +75,58 @@ class TranslateBenchmarker:
                     sample_unit.target = " ".join(["drow%d" % (random.randint(0, strings_per_file) * i) for i in range(target_words_per_string)])
                 sample_file.savefile(os.path.join(dirname, "file_%d.%s" % (filenum, self.extension)))
 
-    def parse_file(self):
+    def parse_files(self, file_dir=None):
         """parses all the files in the test directory into memory"""
         count = 0
-        for dirpath, subdirs, filenames in os.walk(self.file_dir, topdown=False):
+        self.parsedfiles = []
+        if file_dir is None:
+            file_dir = self.file_dir
+        for dirpath, subdirs, filenames in os.walk(file_dir, topdown=False):
             for name in filenames:
                 pofilename = os.path.join(dirpath, name)
                 parsedfile = self.StoreClass(open(pofilename, 'r'))
                 count += len(parsedfile.units)
+                self.parsedfiles.append(parsedfile)
         print "counted %d units" % count
 
+    def parse_placeables(self):
+        """parses placeables"""
+        count = 0
+        for parsedfile in self.parsedfiles:
+            for unit in parsedfile.units:
+                placeables.parse(unit.source, placeables.general.parsers)
+                placeables.parse(unit.target, placeables.general.parsers)
+            count += len(parsedfile.units)
+        print "counted %d units" % count
+
+
 if __name__ == "__main__":
-    storetype = "po"
-    if len(sys.argv) > 1:
-        storetype = sys.argv[1]
-    if storetype in factory.classes:
-        storeclass = factory.classes[storetype]
+    parser = argparse.ArgumentParser(description='Process some integers.')
+    parser.add_argument('podir', metavar='DIR', type=str, nargs='?',
+                        help='PO dir to use (default: create sample files)')
+    parser.add_argument('--store-type', dest='storetype',
+                        action='store_const', const='po', default="po",
+                        help='type of the store to benchmark (default: po)')
+    parser.add_argument('--check-parsing', dest='check_parsing',
+                        action='store_true',
+                        help='benchmark parsing files')
+    parser.add_argument('--check-placeables', dest='check_placeables',
+                        action='store_true',
+                        help='benchmark placeables')
+    args = parser.parse_args()
+
+    storetype = args.storetype
+
+    if storetype in factory.classes_str:
+        _module, _class = factory.classes_str[storetype]
+        module = __import__("translate.storage.%s" % _module,
+                            globals(), fromlist=_module)
+        storeclass = getattr(module, _class)
     else:
         print "StoreClass: '%s' is not a base class that the class factory can load" % storetype
         sys.exit()
-    for sample_file_sizes in [
+
+    sample_files = [
       # num_dirs, files_per_dir, strings_per_file, source_words_per_string, target_words_per_string
       # (1, 1, 2, 2, 2),
       (1, 1, 10000, 5, 10),   # Creat 1 very large file with German like ratios or source to target
@@ -99,20 +136,30 @@ if __name__ == "__main__":
       # (5, 10, 10, 10, 10),
       # (5, 10, 100, 20, 20),
       # (10, 20, 100, 10, 10),
-      # (10, 20, 100, 10, 10),   
+      # (10, 20, 100, 10, 10),
       # (100, 2, 140, 3, 3),  # OpenOffice.org approximate ratios
-      ]:
+    ]
+
+    for sample_file_sizes in sample_files:
         benchmarker = TranslateBenchmarker("BenchmarkDir", storeclass)
         benchmarker.clear_test_dir()
-        benchmarker.create_sample_files(*sample_file_sizes)
-        methods = [("create_sample_files", "*sample_file_sizes"), ("parse_file", ""), ]
+        if args.podir is None:
+            benchmarker.create_sample_files(*sample_file_sizes)
+        benchmarker.parse_files(file_dir=args.podir)
+        methods = [] # [("create_sample_files", "*sample_file_sizes")]
+
+        if args.check_parsing:
+            methods.append(("parse_files", ""))
+
+        if args.check_placeables:
+            methods.append(("parse_placeables", ""))
+
         for methodname, methodparam in methods:
-            print methodname, "%d dirs, %d files, %d strings, %d/%d words" % sample_file_sizes
+            #print methodname, "%d dirs, %d files, %d strings, %d/%d words" % sample_file_sizes
             print "_______________________________________________________"
             statsfile = "%s_%s" % (methodname, storetype) + '_%d_%d_%d_%d_%d.stats' % sample_file_sizes
             cProfile.run('benchmarker.%s(%s)' % (methodname, methodparam), statsfile)
             stats = pstats.Stats(statsfile)
-            stats.sort_stats('cumulative').print_stats(20)
+            stats.sort_stats('time').print_stats(20)
             print "_______________________________________________________"
-        #benchmarker.clear_test_dir()
-
+        benchmarker.clear_test_dir()
