@@ -16,8 +16,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """Classes that hold units of .po files (pounit) or entire files (pofile).
 
@@ -30,21 +29,27 @@ to have a look at gettext-tools/libgettextpo/gettext-po.h from the gettext
 package for the public API of the library.
 """
 
-from translate.misc.multistring import multistring
-from translate.storage import pocommon
-from translate.storage import pypo
-from translate.storage.pocommon import encodingToUse
-from translate.lang import data
-from ctypes import c_int, c_uint, c_char_p, c_long, CFUNCTYPE, POINTER
+from __future__ import with_statement
+
+from ctypes import c_size_t, c_int, c_uint, c_char_p, c_long, CFUNCTYPE, POINTER
 from ctypes import Structure, cdll
 import ctypes.util
 import os
 import re
 import sys
+import logging
 import tempfile
 
+from translate.lang import data
+from translate.misc.multistring import multistring
+from translate.storage import base, pocommon
+from translate.storage import pypo
+from translate.storage.pocommon import encodingToUse
+
+logger = logging.getLogger(__name__)
+
 lsep = " "
-"""Seperator for #: entries"""
+"""Separator for #: entries"""
 
 STRING = c_char_p
 
@@ -54,8 +59,12 @@ class po_message(Structure):
     _fields_ = []
 
 # Function prototypes
-xerror_prototype = CFUNCTYPE(None, c_int, POINTER(po_message), STRING, c_uint, c_uint, c_int, STRING)
-xerror2_prototype = CFUNCTYPE(None, c_int, POINTER(po_message), STRING, c_uint, c_uint, c_int, STRING, POINTER(po_message), STRING, c_uint, c_uint, c_int, STRING)
+xerror_prototype = CFUNCTYPE(None, c_int, POINTER(po_message), STRING, c_uint,
+                             c_uint, c_int, STRING)
+xerror2_prototype = CFUNCTYPE(None, c_int, POINTER(po_message), STRING,
+                              c_uint, c_uint, c_int, STRING,
+                              POINTER(po_message), STRING, c_uint, c_uint,
+                              c_int, STRING)
 
 
 # Structures (error handler)
@@ -74,17 +83,62 @@ class po_error_handler(Structure):
 
 
 # Callback functions for po_xerror_handler
-def xerror_cb(severity, message, filename, lineno, column, multilint_p, message_text):
-    print >> sys.stderr, "xerror_cb", severity, message, filename, lineno, column, multilint_p, message_text
+def xerror_cb(severity, message, filename, lineno, column, multiline_p,
+              message_text):
+    logger.error("xerror_cb" + severity + message +
+                 filename + lineno + column + multiline_p + message_text)
     if severity >= 1:
         raise ValueError(message_text)
 
 
-def xerror2_cb(severity, message1, filename1, lineno1, column1, multiline_p1, message_text1, message2, filename2, lineno2, column2, multiline_p2, message_text2):
-    print >> sys.stderr, "xerror2_cb", severity, message1, filename1, lineno1, column1, multiline_p1, message_text1, message2, filename2, lineno2, column2, multiline_p2, message_text2
+def xerror2_cb(severity, message1, filename1, lineno1, column1, multiline_p1,
+               message_text1, message2, filename2, lineno2, column2,
+               multiline_p2, message_text2):
+    logger.error("xerror2_cb" + severity +
+                 message1 +
+                 filename1 + lineno1 + column1 + multiline_p1 + message_text1,
+                 filename2 + lineno2 + column2 + multiline_p2 + message_text2)
     if severity >= 1:
         raise ValueError(message_text1)
 
+# Setup return and parameter types
+def setup_call_types(gpo):
+    # File access
+    gpo.po_file_read_v3.argtypes = [STRING, POINTER(po_xerror_handler)]
+    gpo.po_file_write_v2.argtypes = [c_int, STRING, POINTER(po_xerror_handler)]
+    gpo.po_file_write_v2.retype = c_int
+
+    # Header
+    gpo.po_file_domain_header.restype = STRING
+    gpo.po_header_field.restype = STRING
+    gpo.po_header_field.argtypes = [STRING, STRING]
+
+    # Locations (filepos)
+    gpo.po_filepos_file.restype = STRING
+    gpo.po_message_filepos.restype = c_int
+    gpo.po_message_filepos.argtypes = [c_int, c_int]
+    gpo.po_message_add_filepos.argtypes = [c_int, STRING, c_size_t]
+
+    # Message (get methods)
+    gpo.po_message_comments.restype = STRING
+    gpo.po_message_extracted_comments.restype = STRING
+    gpo.po_message_prev_msgctxt.restype = STRING
+    gpo.po_message_prev_msgid.restype = STRING
+    gpo.po_message_prev_msgid_plural.restype = STRING
+    gpo.po_message_is_format.restype = c_int
+    gpo.po_message_is_format.argtypes = [c_int, STRING]
+    gpo.po_message_set_format.argtypes = [c_int, STRING, c_int]
+    gpo.po_message_msgctxt.restype = STRING
+    gpo.po_message_msgid.restype = STRING
+    gpo.po_message_msgid_plural.restype = STRING
+    gpo.po_message_msgstr.restype = STRING
+    gpo.po_message_msgstr_plural.restype = STRING
+
+    # Message (set methods)
+    gpo.po_message_set_comments.argtypes = [c_int, STRING]
+    gpo.po_message_set_extracted_comments.argtypes = [c_int, STRING]
+    gpo.po_message_set_fuzzy.argtypes = [c_int, c_int]
+    gpo.po_message_set_msgctxt.argtypes = [c_int, STRING]
 
 # Load libgettextpo
 gpo = None
@@ -98,51 +152,22 @@ for name in names:
         if gpo:
             break
 else:
-    # Now we are getting desperate, so let's guess a unix type DLL that might
-    # be in LD_LIBRARY_PATH or loaded with LD_PRELOAD
-    try:
-        gpo = cdll.LoadLibrary('libgettextpo.so')
-    except OSError, e:
-        raise ImportError("gettext PO library not found")
+    # Don't raise exception in Sphinx autodoc [where xml is Mock()ed]. There is
+    # nothing special about use of xml here - any of the Mock classes set up
+    # in docs/conf.py would work as well, but xml is likely always to be there.
+    gpo = None
+    if 'xml' not in sys.modules or sys.modules['xml'].__path__ != '/dev/null':
 
-# Setup return and paramater types
-# File access
-gpo.po_file_read_v3.argtypes = [STRING, POINTER(po_xerror_handler)]
-gpo.po_file_write_v2.argtypes = [c_int, STRING, POINTER(po_xerror_handler)]
-gpo.po_file_write_v2.retype = c_int
-
-# Header
-gpo.po_file_domain_header.restype = STRING
-gpo.po_header_field.restype = STRING
-gpo.po_header_field.argtypes = [STRING, STRING]
-
-# Locations (filepos)
-gpo.po_filepos_file.restype = STRING
-gpo.po_message_filepos.restype = c_int
-gpo.po_message_filepos.argtypes = [c_int, c_int]
-gpo.po_message_add_filepos.argtypes = [c_int, STRING, c_int]
-
-# Message (get methods)
-gpo.po_message_comments.restype = STRING
-gpo.po_message_extracted_comments.restype = STRING
-gpo.po_message_prev_msgctxt.restype = STRING
-gpo.po_message_prev_msgid.restype = STRING
-gpo.po_message_prev_msgid_plural.restype = STRING
-gpo.po_message_is_format.restype = c_int
-gpo.po_message_is_format.argtypes = [c_int, STRING]
-gpo.po_message_set_format.argtypes = [c_int, STRING, c_int]
-gpo.po_message_msgctxt.restype = STRING
-gpo.po_message_msgid.restype = STRING
-gpo.po_message_msgid_plural.restype = STRING
-gpo.po_message_msgstr.restype = STRING
-gpo.po_message_msgstr_plural.restype = STRING
-
-# Message (set methods)
-gpo.po_message_set_comments.argtypes = [c_int, STRING]
-gpo.po_message_set_extracted_comments.argtypes = [c_int, STRING]
-gpo.po_message_set_fuzzy.argtypes = [c_int, c_int]
-gpo.po_message_set_msgctxt.argtypes = [c_int, STRING]
+        # Now we are getting desperate, so let's guess a unix type DLL that
+        # might be in LD_LIBRARY_PATH or loaded with LD_PRELOAD
+        try:
+            gpo = cdll.LoadLibrary('libgettextpo.so')
+        except OSError:
+            raise ImportError("gettext PO library not found")
 
+if gpo:
+    setup_call_types(gpo)
+    
 # Setup the po_xerror_handler
 xerror_handler = po_xerror_handler()
 xerror_handler.xerror = xerror_prototype(xerror_cb)
@@ -164,30 +189,59 @@ def unquotefrompo(postr):
 def get_libgettextpo_version():
     """Returns the libgettextpo version
 
-       @rtype: three-value tuple
-       @return: libgettextpo version in the following format::
+       :rtype: three-value tuple
+       :return: libgettextpo version in the following format::
            (major version, minor version, subminor version)
     """
     libversion = c_long.in_dll(gpo, 'libgettextpo_version')
     major = libversion.value >> 16
-    minor = libversion.value >> 8
+    minor = (libversion.value >> 8) & 0xff
     subminor = libversion.value - (major << 16) - (minor << 8)
     return major, minor, subminor
 
 
 class pounit(pocommon.pounit):
 
+    #: fixed encoding that is always used for cPO structure (self._gpo_message)
+    CPO_ENC = 'utf-8'
+
     def __init__(self, source=None, encoding='utf-8', gpo_message=None):
         self._rich_source = None
         self._rich_target = None
-        self._encoding = encoding
+        encoding = encoding or 'utf-8'
         if not gpo_message:
             self._gpo_message = gpo.po_message_create()
         if source or source == "":
             self.source = source
             self.target = ""
         elif gpo_message:
+            if encoding.lower() != self.CPO_ENC:
+                features = ['msgctxt', 'msgid', 'msgid_plural']
+                features += ['prev_' + x for x in features]
+                features += ['comments', 'extracted_comments',
+                             'msgstr', 'msgstr_plural']
+                for feature in features:
+                    text = getattr(gpo, 'po_message_' + feature)(gpo_message)
+                    if text:
+                        getattr(gpo, 'po_message_set_' + feature)(
+                            gpo_message,
+                            text.decode(encoding).encode(self.CPO_ENC))
             self._gpo_message = gpo_message
+        self.infer_state()
+
+    def infer_state(self):
+        #FIXME: do obsolete
+        if gpo.po_message_is_obsolete(self._gpo_message):
+            if gpo.po_message_is_fuzzy(self._gpo_message):
+                self.set_state_n(self.STATE[self.S_FUZZY_OBSOLETE][0])
+            else:
+                self.set_state_n(self.STATE[self.S_OBSOLETE][0])
+        elif gpo.po_message_is_fuzzy(self._gpo_message):
+            self.set_state_n(self.STATE[self.S_FUZZY][0])
+        elif self.gettarget():
+            self.set_state_n(self.STATE[self.S_TRANSLATED][0])
+        else:
+            self.set_state_n(self.STATE[self.S_UNTRANSLATED][0])
 
     def setmsgid_plural(self, msgid_plural):
         if isinstance(msgid_plural, list):
@@ -208,11 +262,11 @@ class pounit(pocommon.pounit):
                     return u""
             else:
                 return text
-        singular = remove_msgid_comments(gpo.po_message_msgid(self._gpo_message).decode(self._encoding))
+        singular = remove_msgid_comments((gpo.po_message_msgid(self._gpo_message) or "").decode(self.CPO_ENC))
         if singular:
             if self.hasplural():
-                multi = multistring(singular, self._encoding)
-                pluralform = gpo.po_message_msgid_plural(self._gpo_message).decode(self._encoding)
+                multi = multistring(singular, self.CPO_ENC)
+                pluralform = (gpo.po_message_msgid_plural(self._gpo_message) or "").decode(self.CPO_ENC)
                 multi.strings.append(pluralform)
                 return multi
             else:
@@ -224,11 +278,11 @@ class pounit(pocommon.pounit):
         if isinstance(source, multistring):
             source = source.strings
         if isinstance(source, unicode):
-            source = source.encode(self._encoding)
+            source = source.encode(self.CPO_ENC)
         if isinstance(source, list):
-            gpo.po_message_set_msgid(self._gpo_message, source[0].encode(self._encoding))
+            gpo.po_message_set_msgid(self._gpo_message, source[0].encode(self.CPO_ENC))
             if len(source) > 1:
-                gpo.po_message_set_msgid_plural(self._gpo_message, source[1].encode(self._encoding))
+                gpo.po_message_set_msgid_plural(self._gpo_message, source[1].encode(self.CPO_ENC))
         else:
             gpo.po_message_set_msgid(self._gpo_message, source)
             gpo.po_message_set_msgid_plural(self._gpo_message, None)
@@ -240,15 +294,15 @@ class pounit(pocommon.pounit):
             nplural = 0
             plural = gpo.po_message_msgstr_plural(self._gpo_message, nplural)
             while plural:
-                plurals.append(plural.decode(self._encoding))
+                plurals.append(plural.decode(self.CPO_ENC))
                 nplural += 1
                 plural = gpo.po_message_msgstr_plural(self._gpo_message, nplural)
             if plurals:
-                multi = multistring(plurals, encoding=self._encoding)
+                multi = multistring(plurals, encoding=self.CPO_ENC)
             else:
                 multi = multistring(u"")
         else:
-            multi = (gpo.po_message_msgstr(self._gpo_message) or "").decode(self._encoding)
+            multi = (gpo.po_message_msgstr(self._gpo_message) or "").decode(self.CPO_ENC)
         return multi
 
     def settarget(self, target):
@@ -281,7 +335,7 @@ class pounit(pocommon.pounit):
             for i in range(len(target)):
                 targetstring = target[i]
                 if isinstance(targetstring, unicode):
-                    targetstring = targetstring.encode(self._encoding)
+                    targetstring = targetstring.encode(self.CPO_ENC)
                 gpo.po_message_set_msgstr_plural(self._gpo_message, i, targetstring)
         # add the values of a dict
         elif isinstance(target, dict):
@@ -290,7 +344,7 @@ class pounit(pocommon.pounit):
         # add a single string
         else:
             if isinstance(target, unicode):
-                target = target.encode(self._encoding)
+                target = target.encode(self.CPO_ENC)
             if target is None:
                 gpo.po_message_set_msgstr(self._gpo_message, "")
             else:
@@ -298,9 +352,9 @@ class pounit(pocommon.pounit):
     target = property(gettarget, settarget)
 
     def getid(self):
-        """The unique identifier for this unit according to the convensions in
+        """The unique identifier for this unit according to the conventions in
         .mo files."""
-        id = (gpo.po_message_msgid(self._gpo_message) or "").decode(self._encoding)
+        id = (gpo.po_message_msgid(self._gpo_message) or "").decode(self.CPO_ENC)
         # Gettext does not consider the plural to determine duplicates, only
         # the msgid. For generation of .mo files, we might want to use this
         # code to generate the entry for the hash table, but for now, it is
@@ -310,7 +364,7 @@ class pounit(pocommon.pounit):
 #            id = '%s\0%s' % (id, plural)
         context = gpo.po_message_msgctxt(self._gpo_message)
         if context:
-            id = u"%s\04%s" % (context.decode(self._encoding), id)
+            id = u"%s\04%s" % (context.decode(self.CPO_ENC), id)
         return id
 
     def getnotes(self, origin=None):
@@ -325,9 +379,9 @@ class pounit(pocommon.pounit):
             raise ValueError("Comment type not valid")
 
         if comments and get_libgettextpo_version() < (0, 17, 0):
-            comments = "\n".join([line.strip() for line in comments.split("\n")])
+            comments = "\n".join([line for line in comments.split("\n")])
         # Let's drop the last newline
-        return comments[:-1].decode(self._encoding)
+        return comments[:-1].decode(self.CPO_ENC)
 
     def addnote(self, text, origin=None, position="append"):
         # ignore empty strings and strings without non-space characters
@@ -343,7 +397,7 @@ class pounit(pocommon.pounit):
                 if oldnotes != text:
                     oldnoteslist = oldnotes.split("\n")
                     for newline in text.split("\n"):
-                        newline = newline.rstrip()
+                        newline = newline.rstrip("\r")
                         # avoid duplicate comment lines (this might cause some problems)
                         if newline not in oldnotes or len(newline) < 5:
                             oldnoteslist.append(newline)
@@ -351,7 +405,7 @@ class pounit(pocommon.pounit):
             else:
                 newnotes = text + '\n' + oldnotes
         else:
-            newnotes = "\n".join([line.rstrip() for line in text.split("\n")])
+            newnotes = "\n".join([line.rstrip("\r") for line in text.split("\n")])
 
         if newnotes:
             newlines = []
@@ -361,7 +415,7 @@ class pounit(pocommon.pounit):
                     newlines.append(" " + line)
                 else:
                     newlines.append(line)
-            newnotes = "\n".join(newlines).encode(self._encoding)
+            newnotes = "\n".join(newlines).encode(self.CPO_ENC)
             if origin in ["programmer", "developer", "source code"]:
                 gpo.po_message_set_extracted_comments(self._gpo_message, newnotes)
             else:
@@ -434,19 +488,18 @@ class pounit(pocommon.pounit):
     def isfuzzy(self):
         return gpo.po_message_is_fuzzy(self._gpo_message)
 
-    def markfuzzy(self, present=True):
+    def _domarkfuzzy(self, present=True):
         gpo.po_message_set_fuzzy(self._gpo_message, present)
 
-    def isobsolete(self):
-        return gpo.po_message_is_obsolete(self._gpo_message)
-
     def makeobsolete(self):
         # FIXME: libgettexpo currently does not reset other data, we probably want to do that
         # but a better solution would be for libgettextpo to output correct data on serialisation
         gpo.po_message_set_obsolete(self._gpo_message, True)
+        self.infer_state()
 
     def resurrect(self):
         gpo.po_message_set_obsolete(self._gpo_message, False)
+        self.infer_state()
 
     def hasplural(self):
         return gpo.po_message_msgid_plural(self._gpo_message) is not None
@@ -454,11 +507,11 @@ class pounit(pocommon.pounit):
     def _extract_msgidcomments(self, text=None):
         """Extract KDE style msgid comments from the unit.
 
-        @rtype: String
-        @return: Returns the extracted msgidcomments found in this unit's msgid.
+        :rtype: String
+        :return: Returns the extracted msgidcomments found in this unit's msgid.
         """
         if not text:
-            text = gpo.po_message_msgid(self._gpo_message).decode(self._encoding)
+            text = (gpo.po_message_msgid(self._gpo_message) or "").decode(self.CPO_ENC)
         if text:
             return pocommon.extract_msgid_comment(text)
         return u""
@@ -469,7 +522,7 @@ class pounit(pocommon.pounit):
     msgidcomment = property(_extract_msgidcomments, setmsgidcomment)
 
     def __str__(self):
-        pf = pofile()
+        pf = pofile(noheader=True)
         pf.addunit(self)
         return str(pf)
 
@@ -478,48 +531,54 @@ class pounit(pocommon.pounit):
         i = 0
         location = gpo.po_message_filepos(self._gpo_message, i)
         while location:
-            locname = gpo.po_filepos_file(location)
+            locname = gpo.po_filepos_file(location).decode(self.CPO_ENC)
             locline = gpo.po_filepos_start_line(location)
             if locline == -1:
                 locstring = locname
             else:
-                locstring = locname + ":" + str(locline)
-            locations.append(locstring)
+                locstring = locname + u":" + unicode(locline)
+            locations.append(pocommon.unquote_plus(locstring))
             i += 1
             location = gpo.po_message_filepos(self._gpo_message, i)
         return locations
 
     def addlocation(self, location):
-        for loc in location.split():
-            parts = loc.split(":")
+        if location.find(" ") != -1:
+            location = pocommon.quote_plus(location)
+        parts = location.split(":")
+        if len(parts) == 2 and parts[1].isdigit():
             file = parts[0]
-            if len(parts) == 2:
-                line = int(parts[1] or "0")
-            else:
-                line = -1
-            gpo.po_message_add_filepos(self._gpo_message, file, line)
+            line = int(parts[1] or "0")
+        else:
+            file = location
+            line = -1
+        gpo.po_message_add_filepos(self._gpo_message, file, line)
 
     def getcontext(self):
         msgctxt = gpo.po_message_msgctxt(self._gpo_message)
         if msgctxt:
-            return msgctxt.decode(self._encoding)
+            return msgctxt.decode(self.CPO_ENC)
         else:
             msgidcomment = self._extract_msgidcomments()
             return msgidcomment
 
-    def buildfromunit(cls, unit):
+    def setcontext(self, context):
+        context = data.forceunicode(context)
+        gpo.po_message_set_msgctxt(self._gpo_message, context.encode(self.CPO_ENC))
+
+    def buildfromunit(cls, unit, encoding=None):
         """Build a native unit from a foreign unit, preserving as much
         information as possible."""
         if type(unit) == cls and hasattr(unit, "copy") and callable(unit.copy):
             return unit.copy()
         elif isinstance(unit, pocommon.pounit):
-            newunit = cls(unit.source)
+            newunit = cls(unit.source, encoding)
             newunit.target = unit.target
             #context
             newunit.msgidcomment = unit._extract_msgidcomments()
             context = unit.getcontext()
             if not newunit.msgidcomment and context:
-                gpo.po_message_set_msgctxt(newunit._gpo_message, context)
+                newunit.setcontext(context)
 
             locations = unit.getlocations()
             if locations:
@@ -547,19 +606,35 @@ class pounit(pocommon.pounit):
 class pofile(pocommon.pofile):
     UnitClass = pounit
 
-    def __init__(self, inputfile=None, encoding=None, unitclass=pounit):
+    def __init__(self, inputfile=None, encoding=None, unitclass=pounit, noheader=False):
         self._gpo_memory_file = None
         self._gpo_message_iterator = None
-        super(pofile, self).__init__(inputfile=inputfile, encoding=encoding)
+        self.units = []
+        self.sourcelanguage = None
+        self.targetlanguage = None
+        self._encoding = 'utf-8'
         if inputfile is None:
             self._gpo_memory_file = gpo.po_file_create()
             self._gpo_message_iterator = gpo.po_message_iterator(self._gpo_memory_file, None)
+            if not noheader:
+                self.init_headers()
+        else:
+            super(pofile, self).__init__(inputfile=inputfile, encoding=encoding)
 
     def addunit(self, unit, new=True):
         if new:
             gpo.po_message_insert(self._gpo_message_iterator, unit._gpo_message)
         super(pofile, self).addunit(unit)
 
+    def _insert_header(self, header):
+        header._store = self
+        self.units.insert(0, header)
+        gpo.po_message_iterator_free(self._gpo_message_iterator)
+        self._gpo_message_iterator = gpo.po_message_iterator(self._gpo_memory_file, None)
+        gpo.po_message_insert(self._gpo_message_iterator, header._gpo_message)
+        while gpo.po_next_message(self._gpo_message_iterator):
+            pass
+
     def removeduplicates(self, duplicatestyle="merge"):
         """make sure each msgid is unique ; merge comments etc from duplicates into original"""
         # TODO: can we handle consecutive calls to removeduplicates()? What
@@ -569,6 +644,7 @@ class pofile(pocommon.pofile):
         # TODO: this is using a list as the pos aren't hashable, but this is slow.
         # probably not used frequently enough to worry about it, though.
         markedpos = []
+
         def addcomment(thepo):
             thepo.msgidcomment = " ".join(thepo.getlocations())
             markedpos.append(thepo)
@@ -587,16 +663,16 @@ class pofile(pocommon.pofile):
                 elif duplicatestyle == "msgctxt":
                     origpo = id_dict[id]
                     if origpo not in markedpos:
-                        gpo.po_message_set_msgctxt(origpo._gpo_message, " ".join(origpo.getlocations()))
+                        origpo.setcontext(" ".join(origpo.getlocations()))
                         markedpos.append(thepo)
-                    gpo.po_message_set_msgctxt(thepo._gpo_message, " ".join(thepo.getlocations()))
+                    thepo.setcontext(" ".join(thepo.getlocations()))
                     uniqueunits.append(thepo)
             else:
                 if not id:
                     if duplicatestyle == "merge":
                         addcomment(thepo)
                     else:
-                        gpo.po_message_set_msgctxt(thepo._gpo_message, " ".join(thepo.getlocations()))
+                        thepo.setcontext(" ".join(thepo.getlocations()))
                 id_dict[id] = thepo
                 uniqueunits.append(thepo)
         new_gpo_memory_file = gpo.po_file_create()
@@ -609,6 +685,7 @@ class pofile(pocommon.pofile):
         self.units = uniqueunits
 
     def __str__(self):
+
         def obsolete_workaround():
             # Remove all items that are not output by msgmerge when a unit is obsolete.  This is a work
             # around for bug in libgettextpo
@@ -620,15 +697,26 @@ class pofile(pocommon.pofile):
                     while location:
                         gpo.po_message_remove_filepos(unit._gpo_message, 0)
                         location = gpo.po_message_filepos(unit._gpo_message, 0)
+
+        def writefile(filename):
+            self._gpo_memory_file = gpo.po_file_write_v2(self._gpo_memory_file, filename, xerror_handler)
+            with open(filename) as tfile:
+                return tfile.read()
+
         outputstring = ""
         if self._gpo_memory_file:
             obsolete_workaround()
             f, fname = tempfile.mkstemp(prefix='translate', suffix='.po')
             os.close(f)
-            self._gpo_memory_file = gpo.po_file_write_v2(self._gpo_memory_file, fname, xerror_handler)
-            f = open(fname)
-            outputstring = f.read()
-            f.close()
+            outputstring = writefile(fname)
+            if self._encoding != pounit.CPO_ENC:
+                try:
+                    outputstring = outputstring.decode(pounit.CPO_ENC).encode(self._encoding)
+                except UnicodeEncodeError:
+                    self._encoding = pounit.CPO_ENC
+                    self.updateheader(content_type="text/plain; charset=UTF-8",
+                                      content_transfer_encoding="8bit")
+                    outputstring = writefile(fname)
             os.remove(fname)
         return outputstring
 
@@ -668,11 +756,12 @@ class pofile(pocommon.pofile):
 
         self._gpo_memory_file = gpo.po_file_read_v3(input, xerror_handler)
         if self._gpo_memory_file is None:
-            print >> sys.stderr, "Error:"
+            logger.error("Error:")
 
         if needtmpfile:
             os.remove(input)
 
+        self.units = []
         # Handle xerrors here
         self._header = gpo.po_file_domain_header(self._gpo_memory_file, None)
         if self._header:
