@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2002-2009 Zuza Software Foundation
+# Copyright 2002-2011 Zuza Software Foundation
 #
 # This file is part of the Translate Toolkit.
 #
@@ -28,16 +28,17 @@ directly, but can be used once cpo has been established to work."""
 # - previous msgid and msgctxt
 # - accept only unicodes everywhere
 
-from translate.misc.multistring import multistring
-from translate.lang import data
-from translate.storage import pocommon, base, cpo, poparser
-from translate.storage.pocommon import encodingToUse
 import re
 import copy
 import cStringIO
 
+from translate.lang import data
+from translate.misc.multistring import multistring
+from translate.storage import pocommon, base, cpo, poparser
+from translate.storage.pocommon import encodingToUse
+
 lsep = " "
-"""Seperator for #: entries"""
+"""Separator for #: entries"""
 
 basic_header = r'''msgid ""
 msgstr ""
@@ -45,13 +46,14 @@ msgstr ""
 "Content-Transfer-Encoding: 8bit\n"
 '''
 
+
 class pounit(pocommon.pounit):
     # othercomments = []      #   # this is another comment
     # automaticcomments = []  #   #. comment extracted from the source code
     # sourcecomments = []     #   #: sourcefile.xxx:35
     # prev_msgctxt = []       #   #| The previous values that msgctxt and msgid held
-    # prev_msgid = []         # 
-    # prev_msgid_plural = []  # 
+    # prev_msgid = []         #
+    # prev_msgid_plural = []  #
     # typecomments = []       #   #, fuzzy
     # msgidcomment = u""      #   _: within msgid
     # msgctxt
@@ -65,7 +67,6 @@ class pounit(pocommon.pounit):
     def __init__(self, source=None, encoding="UTF-8"):
         pocommon.pounit.__init__(self, source)
         self._encoding = encodingToUse(encoding)
-        self.obsolete = False
         self._initallcomments(blankall=True)
         self._msgctxt = u""
 
@@ -97,7 +98,6 @@ class pounit(pocommon.pounit):
             self._source = multistring(source)
     source = property(getsource, setsource)
 
-
     def gettarget(self):
         """Returns the unescaped msgstr"""
         return self._target
@@ -205,6 +205,24 @@ class pounit(pocommon.pounit):
         """
 
         def mergelists(list1, list2, split=False):
+            #decode where necessary
+            if unicode in [type(item) for item in list2] + [type(item) for item in list1]:
+                for position, item in enumerate(list1):
+                    if isinstance(item, str):
+                        list1[position] = item.decode("utf-8")
+                for position, item in enumerate(list2):
+                    if isinstance(item, str):
+                        list2[position] = item.decode("utf-8")
+
+            #Determine the newline style of list2
+            lineend = ""
+            if list2 and list2[0]:
+                for candidate in ["\n", "\r", "\n\r"]:
+                    if list2[0].endswith(candidate):
+                        lineend = candidate
+                if not lineend:
+                    lineend = ""
+
             #Split if directed to do so:
             if split:
                 splitlist1 = []
@@ -218,6 +236,7 @@ class pounit(pocommon.pounit):
                 #Normal merge, but conform to list1 newline style
                 if list1 != list2:
                     for item in list2:
+                        item = item.rstrip(lineend)
                         # avoid duplicate comment lines (this might cause some problems)
                         if item not in list1 or len(item) < 5:
                             list1.append(item)
@@ -237,7 +256,7 @@ class pounit(pocommon.pounit):
         if not self.istranslated() or overwrite:
             # Remove kde-style comments from the translation (if any). XXX - remove
             if pocommon.extract_msgid_comment(otherpo.target):
-                otherpo.target = otherpo.target.replace('_: ' + otherpo._extract_msgidcomments()+ '\n', '')
+                otherpo.target = otherpo.target.replace('_: ' + otherpo._extract_msgidcomments() + '\n', '')
             self.target = otherpo.target
             if self.source != otherpo.source or self.getcontext() != otherpo.getcontext():
                 self.markfuzzy()
@@ -289,26 +308,19 @@ class pounit(pocommon.pounit):
         return super(pounit, self).istranslated() and not self.isobsolete()
 
     def istranslatable(self):
-        return not (self.isheader() or self.isblank() or self.obsolete)
+        return not (self.isheader() or self.isblank() or self.isobsolete())
 
     def isfuzzy(self):
         return self.hastypecomment("fuzzy")
 
-    def markfuzzy(self, present=True):
+    def _domarkfuzzy(self, present=True):
         self.settypecomment("fuzzy", present)
 
-    def isobsolete(self):
-        return self.obsolete
-
     def makeobsolete(self):
         """Makes this unit obsolete"""
-        self.obsolete = True
         self.sourcecomments = []
         self.automaticcomments = []
-
-    def resurrect(self):
-        """Makes an obsolete unit normal"""
-        self.obsolete = False
+        super(pounit, self).makeobsolete()
 
     def hasplural(self):
         """returns whether this pounit contains plural strings..."""
@@ -325,7 +337,7 @@ class pounit(pocommon.pounit):
         return str(_cpo_unit)
 
     def getlocations(self):
-        """Get a list of locations from sourcecomments in the PO unit
+        """Get a list of locations from sourcecomments in the PO unit.
 
         rtype: List
         return: A list of the locations with '#: ' stripped
@@ -335,18 +347,18 @@ class pounit(pocommon.pounit):
         return self.sourcecomments
 
     def addlocation(self, location):
-        """Add a location to sourcecomments in the PO unit
+        """Add a location to sourcecomments in the PO unit.
 
-        @param location: Text location e.g. 'file.c:23' does not include #:
-        @type location: String
+        :param location: Text location e.g. 'file.c:23' does not include #:
+        :type location: String
         """
-        self.sourcecomments.extend(location.split())
+        self.sourcecomments.append(location)
 
     def _extract_msgidcomments(self, text=None):
         """Extract KDE style msgid comments from the unit.
 
-        @rtype: String
-        @return: Returns the extracted msgidcomments found in this unit's msgid.
+        :rtype: String
+        :return: Returns the extracted msgidcomments found in this unit's msgid.
         """
         if text:
             return pocommon.extract_msgid_comment(text)
@@ -357,12 +369,16 @@ class pounit(pocommon.pounit):
         """Get the message context."""
         return self._msgctxt + self.msgidcomment
 
+    def setcontext(self, context):
+        context = data.forceunicode(context or u"")
+        self._msgctxt = context
+
     def getid(self):
         """Returns a unique identifier for this unit."""
         context = self.getcontext()
-        # Gettext does not consider the plural to determine duplicates, only 
+        # Gettext does not consider the plural to determine duplicates, only
         # the msgid. For generation of .mo files, we might want to use this
-        # code to generate the entry for the hash table, but for now, it is 
+        # code to generate the entry for the hash table, but for now, it is
         # commented out for conformance to gettext.
 #        id = '\0'.join(self.source.strings)
         id = self.source
@@ -383,7 +399,7 @@ class pounit(pocommon.pounit):
             #context
             newunit.msgidcomment = unit._extract_msgidcomments()
             if not newunit.msgidcomment:
-                newunit._msgctxt = unit.getcontext()
+                newunit.setcontext(unit.getcontext())
 
             locations = unit.getlocations()
             if locations:
@@ -394,9 +410,9 @@ class pounit(pocommon.pounit):
             notes = unit.getnotes("translator")
             if notes:
                 newunit.addnote(notes, "translator")
+            newunit.markfuzzy(unit.isfuzzy())
             if unit.isobsolete():
                 newunit.makeobsolete()
-            newunit.markfuzzy(unit.isfuzzy())
             for tc in ['python-format', 'c-format', 'php-format']:
                 if unit.hastypecomment(tc):
                     newunit.settypecomment(tc)
@@ -406,6 +422,7 @@ class pounit(pocommon.pounit):
             return base.TranslationUnit.buildfromunit(unit)
     buildfromunit = classmethod(buildfromunit)
 
+
 class pofile(pocommon.pofile):
     """A .po file containing various units"""
     UnitClass = pounit
@@ -461,14 +478,13 @@ class pofile(pocommon.pofile):
 
         A user must ensure that self._cpo_store does not exist, and should
         delete it after using it."""
-        self._cpo_store = cpo.pofile()
+        self._cpo_store = cpo.pofile(noheader=True)
         for unit in self.units:
             if not unit.isblank():
-                self._cpo_store.addunit(cpo.pofile.UnitClass.buildfromunit(unit))
+                self._cpo_store.addunit(cpo.pofile.UnitClass.buildfromunit(unit, self._encoding))
         if not self._cpo_store.header():
             #only add a temporary header
-            self._cpo_store.makeheader(charset="utf-8", encoding="8bit")
-
+            self._cpo_store.makeheader(charset=self._encoding, encoding="8bit")
 
     def parse(self, input):
         """Parses the given file or file source string."""
@@ -482,7 +498,7 @@ class pofile(pocommon.pofile):
 #                input = basic_header + input
 #                tmp_header_added = True
             self.units = []
-            self._cpo_store = cpo.pofile(input)
+            self._cpo_store = cpo.pofile(input, noheader=True)
             self._build_self_from_cpo()
             del self._cpo_store
             if tmp_header_added:
@@ -499,6 +515,7 @@ class pofile(pocommon.pofile):
         # TODO: this is using a list as the pos aren't hashable, but this is slow.
         # probably not used frequently enough to worry about it, though.
         markedpos = []
+
         def addcomment(thepo):
             thepo.msgidcomment = " ".join(thepo.getlocations())
             markedpos.append(thepo)
@@ -516,7 +533,8 @@ class pofile(pocommon.pofile):
                         uniqueunits.append(thepo)
                 elif duplicatestyle == "msgctxt":
                     origpo = id_dict[id]
-                    if origpo not in markedpos:
+                    if origpo not in markedpos and id:
+                        # if it doesn't have an id, we already added msgctxt
                         origpo._msgctxt += " ".join(origpo.getlocations())
                         markedpos.append(thepo)
                     thepo._msgctxt += " ".join(thepo.getlocations())
@@ -533,8 +551,13 @@ class pofile(pocommon.pofile):
 
     def __str__(self):
         """Convert to a string. double check that unicode is handled somehow here"""
-        self._cpo_store = cpo.pofile(encoding=self._encoding)
-        self._build_cpo_from_self()
+        self._cpo_store = cpo.pofile(encoding=self._encoding, noheader=True)
+        try:
+            self._build_cpo_from_self()
+        except UnicodeEncodeError, e:
+            self._encoding = "utf-8"
+            self.updateheader(add=True, Content_Type="text/plain; charset=UTF-8")
+            self._build_cpo_from_self()
         output = str(self._cpo_store)
         del self._cpo_store
         return output
