@@ -20,30 +20,29 @@
 
 """Manage the OmegaT glossary format
 
-   OmegaT glossary format is used by the
-   U{OmegaT<http://www.omegat.org/en/omegat.html>} computer aided
-   translation tool.
+OmegaT glossary format is used by the
+`OmegaT <http://www.omegat.org/en/omegat.html>`_ computer aided
+translation tool.
 
-   It is a bilingual base class derived format with L{OmegaTFile}
-   and L{OmegaTUnit} providing file and unit level access.
+It is a bilingual base class derived format with :class:`OmegaTFile`
+and :class:`OmegaTUnit` providing file and unit level access.
 
-   Format Implementation
-   =====================
-   The OmegaT glossary format is a simple Tab Separated Value (TSV) file
-   with the columns: source, target, comment.
+Format Implementation
+    The OmegaT glossary format is a simple Tab Separated Value (TSV) file
+    with the columns: source, target, comment.
 
-   The dialect of the TSV files is specified by L{OmegaTDialect}.
+    The dialect of the TSV files is specified by :class:`OmegaTDialect`.
 
-   Encoding
-   --------
-   The files are either UTF-8 or encoded using the system default.  UTF-8
-   encoded files use the .utf8 extension while system encoded files use
-   the .tab extension.
+Encoding
+    The files are either UTF-8 or encoded using the system default.  UTF-8
+    encoded files use the .utf8 extension while system encoded files use
+    the .tab extension.
 """
 
 import csv
 import locale
 import sys
+
 from translate.storage import base
 
 OMEGAT_FIELDNAMES = ["source", "target", "comment"]
@@ -56,14 +55,6 @@ class OmegaTDialect(csv.Dialect):
     delimiter = "\t"
     lineterminator = "\r\n"
     quoting = csv.QUOTE_NONE
-    if sys.version_info < (2, 5, 0):
-        # We need to define the following items for csv in Python < 2.5
-        # OmegaT does not quote anything FIXME So why MINIMAL?
-        quoting = csv.QUOTE_MINIMAL
-        doublequote = False
-        skipinitialspace = False
-        escapechar = None
-        quotechar = '"'
 csv.register_dialect("omegat", OmegaTDialect)
 
 
@@ -83,8 +74,8 @@ class OmegaTUnit(base.TranslationUnit):
     def setdict(self, newdict):
         """Set the dictionary of values for a OmegaT line
 
-        @param newdict: a new dictionary with OmegaT line elements
-        @type newdict: Dict
+        :param newdict: a new dictionary with OmegaT line elements
+        :type newdict: Dict
         """
         # TODO First check that the values are OK
         self._dict = newdict
