@@ -1,30 +1,31 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
+#
 # Copyright 2002-2008 Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """
 Classes that hold units of .oo files (oounit) or entire files (oofile).
 
-These are specific .oo files for localisation exported by OpenOffice.org - SDF 
-format (previously knows as GSI files). For an overview of the format, see
-U{http://l10n.openoffice.org/L10N_Framework/Intermediate_file_format.html}
+These are specific .oo files for localisation exported by OpenOffice.org - SDF
+format (previously knows as GSI files).
+
+.. There used to be an overview of the format here
+   http://l10n.openoffice.org/L10N_Framework/Intermediate_file_format.html
 
 The behaviour in terms of escaping is explained in detail in the programming
 comments.
@@ -33,9 +34,10 @@ comments.
 
 import os
 import re
+import warnings
+
 from translate.misc import quote
 from translate.misc import wStringIO
-import warnings
 
 # File normalisation
 
@@ -47,16 +49,20 @@ for i in map(chr, range(256)):
     else:
         normalizetable += "_"
 
+
 class unormalizechar(dict):
+
     def __init__(self, normalchars):
         self.normalchars = {}
         for char in normalchars:
             self.normalchars[ord(char)] = char
+
     def __getitem__(self, key):
         return self.normalchars.get(key, u"_")
 
 unormalizetable = unormalizechar(normalfilenamechars.decode("ascii"))
 
+
 def normalizefilename(filename):
     """converts any non-alphanumeric (standard roman) characters to _"""
     if isinstance(filename, str):
@@ -64,18 +70,19 @@ def normalizefilename(filename):
     else:
         return filename.translate(unormalizetable)
 
+
 def makekey(ookey, long_keys):
     """converts an oo key tuple into a unique identifier
 
-    @param ookey: an oo key
-    @type ookey: tuple
-    @param long_keys: Use long keys
-    @type long_keys: Boolean
-    @rtype: str
-    @return: unique ascii identifier
+    :param ookey: an oo key
+    :type ookey: tuple
+    :param long_keys: Use long keys
+    :type long_keys: Boolean
+    :rtype: str
+    :return: unique ascii identifier
     """
     project, sourcefile, resourcetype, groupid, localid, platform = ookey
-    sourcefile = sourcefile.replace('\\','/')
+    sourcefile = sourcefile.replace('\\', '/')
     if long_keys:
         sourcebase = os.path.join(project, sourcefile)
     else:
@@ -91,18 +98,18 @@ def makekey(ookey, long_keys):
     return normalizefilename(key)
 
 # These are functions that deal with escaping and unescaping of the text fields
-# of the SDF file. These should only be applied to the text column. 
+# of the SDF file. These should only be applied to the text column.
 # The fields quickhelptext and title are assumed to carry no escaping.
-# 
-# The escaping of all strings except those coming from .xhp (helpcontent2) 
+#
+# The escaping of all strings except those coming from .xhp (helpcontent2)
 # sourcefiles work as follows:
 #   (newline)         ->  \n
 #   (carriage return) ->  \r
 #   (tab)             ->  \t
 # Backslash characters (\) and single quotes (') are not consistently escaped,
 # and are therefore left as they are.
-# 
-# For strings coming from .xhp (helpcontent2) sourcefiles the following 
+#
+# For strings coming from .xhp (helpcontent2) sourcefiles the following
 # characters are escaped inside XML tags only:
 #   <  ->  \<  when used with lowercase tagnames (with some exceptions)
 #   >  ->  \>  when used with lowercase tagnames (with some exceptions)
@@ -110,10 +117,12 @@ def makekey(ookey, long_keys):
 # The following is consistently escaped in .xhp strings (not only in XML tags):
 #   \  ->  \\
 
+
 def escape_text(text):
     """Escapes SDF text to be suitable for unit consumption."""
     return text.replace("\n", "\\n").replace("\t", "\\t").replace("\r", "\\r")
-    
+
+
 def unescape_text(text):
     """Unescapes SDF text to be suitable for unit consumption."""
     return text.replace("\\\\", "\a").replace("\\n", "\n").replace("\\t", "\t").\
@@ -121,17 +130,20 @@ def unescape_text(text):
 
 helptagre = re.compile('''<[/]??[a-z_\-]+?(?:| +[a-z]+?=".*?") *[/]??>''')
 
+
 def escape_help_text(text):
     """Escapes the help text as it would be in an SDF file.
 
-    <, >, " are only escaped in <[[:lower:]]> tags. Some HTML tags make it in in 
-    lowercase so those are dealt with. Some OpenOffice.org help tags are not 
+    <, >, " are only escaped in <[[:lower:]]> tags. Some HTML tags make it in in
+    lowercase so those are dealt with. Some OpenOffice.org help tags are not
     escaped.
     """
     text = text.replace("\\", "\\\\")
     for tag in helptagre.findall(text):
         escapethistag = False
-        for escape_tag in ["ahelp", "link", "item", "emph", "defaultinline", "switchinline", "caseinline", "variable", "bookmark_value", "image", "embedvar", "alt"]:
+        for escape_tag in ["ahelp", "link", "item", "emph", "defaultinline",
+                           "switchinline", "caseinline", "variable",
+                           "bookmark_value", "image", "embedvar", "alt"]:
             if tag.startswith("<%s" % escape_tag) or tag == "</%s>" % escape_tag:
                 escapethistag = True
         if tag in ["<br/>", "<help-id-missing/>"]:
@@ -141,10 +153,12 @@ def escape_help_text(text):
             text = text.replace(tag, escaped_tag)
     return text
 
+
 def unescape_help_text(text):
     """Unescapes normal text to be suitable for writing to the SDF file."""
     return text.replace(r"\<", "<").replace(r"\>", ">").replace(r'\"', '"').replace(r"\\", "\\")
 
+
 def encode_if_needed_utf8(text):
     """Encode a Unicode string the the specified encoding"""
     if isinstance(text, unicode):
@@ -154,6 +168,7 @@ def encode_if_needed_utf8(text):
 
 class ooline(object):
     """this represents one line, one translation in an .oo file"""
+
     def __init__(self, parts=None):
         """construct an ooline from its parts"""
         if parts is None:
@@ -171,7 +186,7 @@ class ooline(object):
                     (len(parts), parts))
             newparts = list(parts)
             if len(newparts) < 15:
-                newparts = newparts + [""] * (15-len(newparts))
+                newparts = newparts + [""] * (15 - len(newparts))
             else:
                 newparts = newparts[:15]
             parts = tuple(newparts)
@@ -184,7 +199,7 @@ class ooline(object):
         """return a list of parts in this line"""
         return (self.project, self.sourcefile, self.dummy, self.resourcetype,
                 self.groupid, self.localid, self.helpid, self.platform,
-                self.width, self.languageid, self._text, self.helptext, 
+                self.width, self.languageid, self._text, self.helptext,
                 self.quickhelptext, self.title, self.timestamp)
 
     def gettext(self):
@@ -193,7 +208,7 @@ class ooline(object):
             return unescape_help_text(self._text)
         else:
             return unescape_text(self._text)
-        
+
     def settext(self, text):
         """Sets the text column and handle escaping."""
         if self.sourcefile.endswith(".xhp"):
@@ -216,8 +231,10 @@ class ooline(object):
         return (self.project, self.sourcefile, self.resourcetype, self.groupid,
                 self.localid, self.platform)
 
+
 class oounit:
     """this represents a number of translations of a resource"""
+
     def __init__(self):
         """construct the oounit"""
         self.languages = {}
@@ -232,13 +249,24 @@ class oounit:
         """convert to a string. double check that unicode is handled"""
         return encode_if_needed_utf8(self.getoutput())
 
-    def getoutput(self):
+    def getoutput(self, skip_source=False, fallback_lang=None):
         """return the lines in tab-delimited form"""
-        return "\r\n".join([str(line) for line in self.lines])
+        if skip_source:
+            lines = self.lines[1:]
+            if not lines:
+                # Untranslated, so let's do fall-back: (bug 1883)
+                new_line = ooline(self.lines[0].getparts())
+                new_line.languageid = fallback_lang
+                lines = [new_line]
+        else:
+            lines = self.lines
+        return "\r\n".join([str(line) for line in lines])
+
 
 class oofile:
     """this represents an entire .oo file"""
     UnitClass = oounit
+
     def __init__(self, input=None):
         """constructs the oofile"""
         self.oolines = []
@@ -279,11 +307,11 @@ class oofile:
             thisline = ooline(parts)
             self.addline(thisline)
 
-    def __str__(self):
+    def __str__(self, skip_source=False, fallback_lang=None):
         """convert to a string. double check that unicode is handled"""
-        return encode_if_needed_utf8(self.getoutput())
+        return encode_if_needed_utf8(self.getoutput(skip_source, fallback_lang))
 
-    def getoutput(self):
+    def getoutput(self, skip_source=False, fallback_lang=None):
         """converts all the lines back to tab-delimited form"""
         lines = []
         for oe in self.units:
@@ -291,12 +319,14 @@ class oofile:
                 warnings.warn("contains %d lines (should be 2 at most): languages %r" % (len(oe.lines), oe.languages))
                 oekeys = [line.getkey() for line in oe.lines]
                 warnings.warn("contains %d lines (should be 2 at most): keys %r" % (len(oe.lines), oekeys))
-            oeline = str(oe) + "\r\n"
+            oeline = oe.getoutput(skip_source, fallback_lang) + "\r\n"
             lines.append(oeline)
         return "".join(lines)
 
+
 class oomultifile:
     """this takes a huge GSI file and represents it as multiple smaller files..."""
+
     def __init__(self, filename, mode=None, multifilestyle="single"):
         """initialises oomultifile from a seekable inputfile or writable outputfile"""
         self.filename = filename
@@ -373,6 +403,7 @@ class oomultifile:
 
     def openoutputfile(self, subfile):
         """returns a pseudo-file object for the given subfile"""
+
         def onclose(contents):
             self.multifile.write(contents)
             self.multifile.flush()
@@ -387,4 +418,3 @@ class oomultifile:
         oosubfile.filename = subfile
         oosubfile.parse(subfilesrc)
         return oosubfile
-
