@@ -18,6 +18,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
+from pytest import mark
+
 from translate.storage.placeables import base, general, parse, xliff, StringElem
 
 
@@ -85,7 +87,7 @@ class TestStringElem:
         assert self.elem.find(searchelem) == 46
 
     def test_find_elems_with(self):
-        assert self.elem.find_elems_with(u'Ģët') == [ self.elem.sub[0], self.elem.sub[1] ]
+        assert self.elem.find_elems_with(u'Ģët') == [self.elem.sub[0], self.elem.sub[1]]
         assert len(self.elem.find_elems_with('a')) == 3
 
     def test_flatten(self):
@@ -102,7 +104,7 @@ class TestStringElem:
         # Case 2: An entire element #
         elem = self.elem.copy()
         offset = elem.elem_offset(elem.sub[2])
-        deleted, parent, offset = elem.delete_range(offset, offset+len(elem.sub[2]))
+        deleted, parent, offset = elem.delete_range(offset, offset + len(elem.sub[2]))
         assert deleted == self.elem.sub[2]
         assert parent is elem
         assert offset == len(elem.sub[0]) + len(elem.sub[1])
@@ -171,6 +173,7 @@ class TestStringElem:
 
 
 class TestConverters:
+
     def setup_method(self, method):
         self.elem = parse(TestStringElem.ORIGSTR, general.parsers)
 
@@ -182,13 +185,14 @@ class TestConverters:
         assert unicode(self.elem) == unicode(basetree)
         assert repr(self.elem) != repr(basetree)
 
+    @mark.xfail(reason="Test needs fixing, disabled for now")
     def test_to_general_placeables(self):
         basetree = base.to_base_placeables(self.elem)
         gentree = general.to_general_placeables(basetree)
         assert gentree == self.elem
 
-    # FIXME: Fix and enable the following test
-    def DISABLED_test_to_xliff_placeables(self):
+    @mark.xfail(reason="Test needs fixing, disabled for now")
+    def test_to_xliff_placeables(self):
         basetree = base.to_base_placeables(self.elem)
         xliff_from_base = xliff.to_xliff_placeables(basetree)
         assert unicode(xliff_from_base) != unicode(self.elem)
