@@ -16,62 +16,71 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 
 from lxml import etree
+
 from translate.storage.placeables import lisa, StringElem
 from translate.storage.placeables.xliff import Bx, Ex, G, UnknownXML, X
 
+
 def test_xml_to_strelem():
+    source = etree.fromstring(u'<source>a</source>')
+    elem = lisa.xml_to_strelem(source)
+    assert elem == StringElem(u'a')
+
     source = etree.fromstring(u'<source>a<x id="foo[1]/bar[1]/baz[1]"/></source>')
     elem = lisa.xml_to_strelem(source)
-    assert elem.sub == [ StringElem(u'a'), X(id=u'foo[1]/bar[1]/baz[1]') ]
+    assert elem.sub == [StringElem(u'a'), X(id=u'foo[1]/bar[1]/baz[1]')]
 
     source = etree.fromstring(u'<source>a<x id="foo[1]/bar[1]/baz[1]"/>é</source>')
     elem = lisa.xml_to_strelem(source)
-    assert elem.sub == [ StringElem(u'a'), X(id=u'foo[1]/bar[1]/baz[1]'), StringElem(u'é') ]
+    assert elem.sub == [StringElem(u'a'), X(id=u'foo[1]/bar[1]/baz[1]'), StringElem(u'é')]
 
     source = etree.fromstring(u'<source>a<g id="foo[2]/bar[2]/baz[2]">b<x id="foo[1]/bar[1]/baz[1]"/>c</g>é</source>')
     elem = lisa.xml_to_strelem(source)
-    assert elem.sub == [ StringElem(u'a'), G(id=u'foo[2]/bar[2]/baz[2]', sub=[StringElem(u'b'), X(id=u'foo[1]/bar[1]/baz[1]'), StringElem(u'c')]), StringElem(u'é') ]
+    assert elem.sub == [StringElem(u'a'), G(id=u'foo[2]/bar[2]/baz[2]', sub=[StringElem(u'b'), X(id=u'foo[1]/bar[1]/baz[1]'), StringElem(u'c')]), StringElem(u'é')]
+
 
 def test_xml_space():
     source = etree.fromstring(u'<source xml:space="default"> a <x id="foo[1]/bar[1]/baz[1]"/> </source>')
     elem = lisa.xml_to_strelem(source)
     print elem.sub
-    assert elem.sub == [ StringElem(u'a '), X(id=u'foo[1]/bar[1]/baz[1]'), StringElem(u' ')]
+    assert elem.sub == [StringElem(u'a '), X(id=u'foo[1]/bar[1]/baz[1]'), StringElem(u' ')]
+
 
 def test_chunk_list():
-    left  = StringElem([u'a', G(id='foo[2]/bar[2]/baz[2]', sub=[u'b', X(id='foo[1]/bar[1]/baz[1]'), u'c']), u'é'])
+    left = StringElem([u'a', G(id='foo[2]/bar[2]/baz[2]', sub=[u'b', X(id='foo[1]/bar[1]/baz[1]'), u'c']), u'é'])
     right = StringElem([u'a', G(id='foo[2]/bar[2]/baz[2]', sub=[u'b', X(id='foo[1]/bar[1]/baz[1]'), u'c']), u'é'])
     assert left == right
 
+
 def test_set_strelem_to_xml():
     source = etree.Element(u'source')
     lisa.strelem_to_xml(source, StringElem(u'a'))
-    assert etree.tostring(source, encoding = 'UTF-8') == '<source>a</source>'
+    assert etree.tostring(source, encoding='UTF-8') == '<source>a</source>'
 
     source = etree.Element(u'source')
     lisa.strelem_to_xml(source, StringElem([u'a', u'é']))
-    assert etree.tostring(source, encoding = 'UTF-8') == '<source>aé</source>'
+    assert etree.tostring(source, encoding='UTF-8') == '<source>aé</source>'
 
     source = etree.Element(u'source')
     lisa.strelem_to_xml(source, StringElem(X(id='foo[1]/bar[1]/baz[1]')))
-    assert etree.tostring(source, encoding = 'UTF-8') == '<source><x id="foo[1]/bar[1]/baz[1]"/></source>'
+    assert etree.tostring(source, encoding='UTF-8') == '<source><x id="foo[1]/bar[1]/baz[1]"/></source>'
 
     source = etree.Element(u'source')
     lisa.strelem_to_xml(source, StringElem([u'a', X(id='foo[1]/bar[1]/baz[1]')]))
-    assert etree.tostring(source, encoding = 'UTF-8') == '<source>a<x id="foo[1]/bar[1]/baz[1]"/></source>'
+    assert etree.tostring(source, encoding='UTF-8') == '<source>a<x id="foo[1]/bar[1]/baz[1]"/></source>'
 
     source = etree.Element(u'source')
     lisa.strelem_to_xml(source, StringElem([u'a', X(id='foo[1]/bar[1]/baz[1]'), u'é']))
-    assert etree.tostring(source, encoding = 'UTF-8') == '<source>a<x id="foo[1]/bar[1]/baz[1]"/>é</source>'
+    assert etree.tostring(source, encoding='UTF-8') == '<source>a<x id="foo[1]/bar[1]/baz[1]"/>é</source>'
 
     source = etree.Element(u'source')
     lisa.strelem_to_xml(source, StringElem([u'a', G(id='foo[2]/bar[2]/baz[2]', sub=[u'b', X(id='foo[1]/bar[1]/baz[1]'), u'c']), u'é']))
-    assert etree.tostring(source, encoding = 'UTF-8') == '<source>a<g id="foo[2]/bar[2]/baz[2]">b<x id="foo[1]/bar[1]/baz[1]"/>c</g>é</source>'
+    assert etree.tostring(source, encoding='UTF-8') == '<source>a<g id="foo[2]/bar[2]/baz[2]">b<x id="foo[1]/bar[1]/baz[1]"/>c</g>é</source>'
+
 
 def test_unknown_xml_placeable():
     # The XML below is (modified) from the official XLIFF example file Sample_AlmostEverything_1.2_strict.xlf
@@ -89,15 +98,14 @@ def test_unknown_xml_placeable():
             [
                 StringElem(u'bpt'),
                 UnknownXML(u'sub', xml_node=copy(source[1][0])),
-                StringElem(u'\n               ')
+                StringElem(u'\n               '),
             ],
             id='_1_ski_139',
-            xml_node=copy(source[3])
-        ),
+            xml_node=copy(source[3])),
         StringElem(u'TEXT'),
         UnknownXML(u'ept', id=u'_1_ski_238', xml_node=copy(source[2])),
         StringElem(u'TEXT'),
-        UnknownXML(id='_1_ski_337', xml_node=copy(source[3])), # ph-tag
+        UnknownXML(id='_1_ski_337', xml_node=copy(source[3])),  # ph-tag
         StringElem(u'TEXT'),
         UnknownXML(u'it', id='_1_ski_436', xml_node=copy(source[4])),
         StringElem(u'TEXT'),
@@ -120,6 +128,7 @@ def test_unknown_xml_placeable():
     lisa.strelem_to_xml(xml, elem)
     assert etree.tostring(xml) == etree.tostring(source)
 
+
 if __name__ == '__main__':
     test_chunk_list()
     test_xml_to_strelem()
