@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright 2002-2009 Zuza Software Foundation
+# Copyright 2013 F Wolff
 #
 # This file is part of the Translate Toolkit.
 #
@@ -18,32 +19,35 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""classes that hold units of .po files (pounit) or entire files (pofile)
-gettext-style .po (or .pot) files are used in translations for KDE et al (see kbabel)"""
+"""Classes that hold units of Gettext .po files (pounit) or entire
+files (pofile).
+"""
 
 from __future__ import generators
-from translate.misc.multistring import multistring
-from translate.misc import quote
-from translate.misc import textwrap
-from translate.lang import data
-from translate.storage import pocommon, base, poparser
-from translate.storage.pocommon import encodingToUse
-import re
 import copy
 import cStringIO
+import re
+import textwrap
+
+from translate.lang import data
+from translate.misc.multistring import multistring
+from translate.misc import quote
+from translate.storage import pocommon, base, poparser
+from translate.storage.pocommon import encodingToUse
 
 lsep = "\n#: "
-"""Seperator for #: entries"""
+"""Separator for #: entries"""
 
 # general functions for quoting / unquoting po strings
 
 po_unescape_map = {"\\r": "\r", "\\t": "\t", '\\"': '"', '\\n': '\n', '\\\\': '\\'}
 po_escape_map = dict([(value, key) for (key, value) in po_unescape_map.items()])
 
+
 def escapeforpo(line):
     """Escapes a line for po format. assumes no \n occurs in the line.
 
-    @param line: unescaped text
+    :param line: unescaped text
     """
     special_locations = []
     for special_key in po_escape_map:
@@ -55,66 +59,109 @@ def escapeforpo(line):
     for location in special_locations:
         escaped_line += line[last_location:location]
         escaped_line += po_escape_map[line[location:location+1]]
-        last_location = location+1
+        last_location = location + 1
     escaped_line += line[last_location:]
     return escaped_line
 
-def unescapehandler(escape):
 
+def unescapehandler(escape):
     return po_unescape_map.get(escape, escape)
 
-def wrapline(line):
-    """Wrap text for po files."""
-    wrappedlines = textwrap.wrap(line, 76, replace_whitespace=False, expand_tabs=False, drop_whitespace=False)
 
-    # Lines should not start with a space...
-    if len(wrappedlines) > 1:
-        for index, line in enumerate(wrappedlines[1:]):
-            if line.startswith(' '):
-                # Remove the space at the beginning of the line:
-                wrappedlines[index+1] = line[1:]
+try:
+    wrapper = textwrap.TextWrapper(
+            width=77,
+            replace_whitespace=False,
+            expand_tabs=False,
+            drop_whitespace=False
+    )
+except TypeError:
+    # Python < 2.6 didn't support drop_whitespace
+    from translate.misc import textwrap
+    wrapper = textwrap.TextWrapper(width=77)
+
+wrapper.wordsep_re = re.compile(
+    r'(\s+|'                                  # any whitespace
+    r'\w*\\.|'                                # any escape should not be split
+    r'[\w\!\'\&\.\,\?]+\s+|'                  # space should go with a word
+    r'[^\s\w]*\w+[a-zA-Z]-(?=\w+[a-zA-Z])|'   # hyphenated words
+    r'(?<=[\w\!\"\'\&\.\,\?])-{2,}(?=\w))')   # em-dash
+wrapper.wordsep_re_uni = re.compile(wrapper.wordsep_re.pattern, re.UNICODE)
 
-                # Append a space to the previous line:
-                wrappedlines[index] += ' '
-    return wrappedlines
 
 def quoteforpo(text):
-    """quotes the given text for a PO file, returning quoted and escaped lines"""
-    polines = []
+    """Quotes the given text for a PO file, returning quoted and
+    escaped lines"""
     if text is None:
-        return polines
-    lines = text.split("\n")
-    if len(lines) > 1 or (len(lines) == 1 and len(lines[0]) > 71):
-        if len(lines) != 2 or lines[1]:
-            polines.extend(['""'])
-        for line in lines[:-1]:
-            #TODO: We should only wrap after escaping
-            lns = wrapline(line)
-            if len(lns) > 0:
-                for ln in lns[:-1]:
-                    polines.extend(['"' + escapeforpo(ln) + '"'])
-                if lns[-1]:
-                    polines.extend(['"' + escapeforpo(lns[-1]) + '\\n"'])
-            else:
-                polines.extend(['"\\n"'])
-    if lines[-1]:
-        polines.extend(['"' + escapeforpo(line) + '"' for line in wrapline(lines[-1])])
+        return []
+    text = escapeforpo(text)
+    lines = text.split(u"\\n")
+    for i, l in enumerate(lines[:-1]):
+        lines[i] = l + u"\\n"
+
+    polines = []
+    len_lines = len(lines)
+    if len_lines > 2 or (len_lines == 2 and lines[1]) or len(lines[0]) > 71:
+        polines.append(u'""')
+    for line in lines:
+        lns = wrapper.wrap(line)
+        for ln in lns:
+            polines.append(u'"%s"' % ln)
     return polines
 
+
 def extractpoline(line):
     """Remove quote and unescape line from po file.
 
-    @param line: a quoted line from a po file (msgid or msgstr)
+    :param line: a quoted line from a po file (msgid or msgstr)
+
+    .. deprecated:: 1.10
     """
     extracted = quote.extractwithoutquotes(line, '"', '"', '\\', includeescapes=unescapehandler)[0]
     return extracted
 
+
+def unescape(line):
+    """Unescape the given line.
+
+    Quotes on either side should already have been removed.
+    """
+    escape_places = quote.find_all(line, u"\\")
+    if not escape_places:
+        return line
+
+    # filter escaped escapes
+    true_escape = False
+    true_escape_places = []
+    for escape_pos in escape_places:
+        if escape_pos - 1 in escape_places:
+            true_escape = not true_escape
+        else:
+            true_escape = True
+        if true_escape:
+            true_escape_places.append(escape_pos)
+
+    extracted = u""
+    lastpos = 0
+    for pos in true_escape_places:
+        # everything leading up to the escape
+        extracted += line[lastpos:pos]
+        # the escaped sequence (consuming 2 characters)
+        extracted += unescapehandler(line[pos:pos+2])
+        lastpos = pos+2
+
+    extracted += line[lastpos:]
+    return extracted
+
+
 def unquotefrompo(postr):
-    return u"".join([extractpoline(line) for line in postr])
+    return u"".join([unescape(line[1:-1]) for line in postr])
+
 
 def is_null(lst):
     return lst == [] or len(lst) == 1 and lst[0] == '""'
 
+
 def extractstr(string):
     left = string.find('"')
     right = string.rfind('"')
@@ -123,13 +170,14 @@ def extractstr(string):
     else:
         return string[left:] + '"'
 
+
 class pounit(pocommon.pounit):
     # othercomments = []      #   # this is another comment
     # automaticcomments = []  #   #. comment extracted from the source code
     # sourcecomments = []     #   #: sourcefile.xxx:35
     # prev_msgctxt = []       #   #| The previous values that msgctxt and msgid held
-    # prev_msgid = []         # 
-    # prev_msgid_plural = []  # 
+    # prev_msgid = []         #
+    # prev_msgid_plural = []  #
     # typecomments = []       #   #, fuzzy
     # msgidcomments = []      #   _: within msgid
     # msgctxt
@@ -152,11 +200,6 @@ class pounit(pocommon.pounit):
         self.msgid_pluralcomments = []
         self.msgid_plural = []
         self.msgstr = []
-        self.obsoletemsgctxt = []
-        self.obsoletemsgid = []
-        self.obsoletemsgid_pluralcomments = []
-        self.obsoletemsgid_plural = []
-        self.obsoletemsgstr = []
         pocommon.pounit.__init__(self, source)
 
     def _initallcomments(self, blankall=False):
@@ -167,7 +210,6 @@ class pounit(pocommon.pounit):
             self.sourcecomments = []
             self.typecomments = []
             self.msgidcomments = []
-            self.obsoletemsgidcomments = []
 
     def _get_all_comments(self):
         return [self.othercomments,
@@ -175,18 +217,16 @@ class pounit(pocommon.pounit):
                 self.sourcecomments,
                 self.typecomments,
                 self.msgidcomments,
-                self.obsoletemsgidcomments]
+                ]
 
     allcomments = property(_get_all_comments)
 
     def _get_source_vars(self, msgid, msgid_plural):
-        multi = multistring(unquotefrompo(msgid), self._encoding)
+        singular = unquotefrompo(msgid)
         if self.hasplural():
             pluralform = unquotefrompo(msgid_plural)
-            if isinstance(pluralform, str):
-                pluralform = pluralform.decode(self._encoding)
-            multi.strings.append(pluralform)
-        return multi
+            return multistring([singular, pluralform], self._encoding)
+        return singular
 
     def _set_source_vars(self, source):
         msgid = None
@@ -213,7 +253,7 @@ class pounit(pocommon.pounit):
     def setsource(self, source):
         """Sets the msgid to the given (unescaped) value.
 
-        @param source: an unescaped source string.
+        :param source: an unescaped source string.
         """
         self._rich_source = None
         self.msgid, self.msgid_plural = self._set_source_vars(source)
@@ -226,7 +266,7 @@ class pounit(pocommon.pounit):
     def _set_prev_source(self, source):
         """Sets the msgid to the given (unescaped) value.
 
-        @param source: an unescaped source string.
+        :param source: an unescaped source string.
         """
         self.prev_msgid, self.prev_msgid_plural = self._set_source_vars(source)
     prev_source = property(_get_prev_source, _set_prev_source)
@@ -234,10 +274,9 @@ class pounit(pocommon.pounit):
     def gettarget(self):
         """Returns the unescaped msgstr"""
         if isinstance(self.msgstr, dict):
-            multi = multistring(map(unquotefrompo, self.msgstr.values()), self._encoding)
+            return multistring(map(unquotefrompo, self.msgstr.values()), self._encoding)
         else:
-            multi = multistring(unquotefrompo(self.msgstr), self._encoding)
-        return multi
+            return unquotefrompo(self.msgstr)
 
     def settarget(self, target):
         """Sets the msgstr to the given (unescaped) value"""
@@ -265,13 +304,32 @@ class pounit(pocommon.pounit):
             self.msgstr = quoteforpo(target)
     target = property(gettarget, settarget)
 
+    def getalttrans(self):
+        """Return a list of alternate units.
+
+        Previous msgid and current msgstr is combined to form a single
+        alternative unit."""
+        prev_source = self.prev_source
+        if prev_source and self.isfuzzy():
+            unit = type(self)(prev_source)
+            unit.target = self.target
+            # Already released versions of Virtaal (0.6.x) only supported XLIFF
+            # alternatives, and expect .xmlelement.get().
+            # This can be removed soon:
+            unit.xmlelement = dict()
+            return [unit]
+        return []
+
     def getnotes(self, origin=None):
-        """Return comments based on origin value (programmer, developer, source code and translator)"""
+        """Return comments based on origin value.
+
+        :param origin: programmer, developer, source code, translator or None
+        """
         if origin == None:
             comments = u"".join([comment[2:] for comment in self.othercomments])
             comments += u"".join([comment[3:] for comment in self.automaticcomments])
         elif origin == "translator":
-            comments = u"".join ([comment[2:] for comment in self.othercomments])
+            comments = u"".join([comment[2:] for comment in self.othercomments])
         elif origin in ["programmer", "developer", "source code"]:
             comments = u"".join([comment[3:] for comment in self.automaticcomments])
         else:
@@ -280,7 +338,10 @@ class pounit(pocommon.pounit):
         return comments[:-1]
 
     def addnote(self, text, origin=None, position="append"):
-        """This is modeled on the XLIFF method. See xliff.py::xliffunit.addnote"""
+        """This is modeled on the XLIFF method.
+
+        See :meth:`translate.storage.xliff.xliffunit.addnote`
+        """
         # ignore empty strings and strings without non-space characters
         if not (text and text.strip()):
             return
@@ -338,7 +399,7 @@ class pounit(pocommon.pounit):
 
     def _msgstrlen(self):
         if isinstance(self.msgstr, dict):
-            combinedstr = "\n".join([unquotefrompo(msgstr) for msgstr in self.msgstr.itervalues()])
+            combinedstr = "\n".join(filter(None, [unquotefrompo(msgstr) for msgstr in self.msgstr.itervalues()]))
             return len(combinedstr)
         else:
             return len(unquotefrompo(self.msgstr))
@@ -399,15 +460,16 @@ class pounit(pocommon.pounit):
             mergelists(self.othercomments, otherpo.othercomments)
             mergelists(self.typecomments, otherpo.typecomments)
             if not authoritative:
-                # We don't bring across otherpo.automaticcomments as we consider ourself
-                # to be the the authority.  Same applies to otherpo.msgidcomments
+                # We don't bring across otherpo.automaticcomments as we
+                # consider ourself to be the the authority.  Same applies
+                # to otherpo.msgidcomments
                 mergelists(self.automaticcomments, otherpo.automaticcomments)
                 mergelists(self.msgidcomments, otherpo.msgidcomments)
                 mergelists(self.sourcecomments, otherpo.sourcecomments, split=True)
         if not self.istranslated() or overwrite:
             # Remove kde-style comments from the translation (if any).
             if self._extract_msgidcomments(otherpo.target):
-                otherpo.target = otherpo.target.replace('_: ' + otherpo._extract_msgidcomments()+ '\n', '')
+                otherpo.target = otherpo.target.replace('_: ' + otherpo._extract_msgidcomments() + '\n', '')
             self.target = otherpo.target
             if self.source != otherpo.source or self.getcontext() != otherpo.getcontext():
                 self.markfuzzy()
@@ -426,8 +488,7 @@ class pounit(pocommon.pounit):
         return (is_null(self.msgid)
                         and not is_null(self.msgstr)
                         and self.msgidcomments == []
-                        and is_null(self.msgctxt)
-        )
+                        and is_null(self.msgctxt))
 
     def isblank(self):
         if self.isheader() or len(self.msgidcomments):
@@ -441,11 +502,21 @@ class pounit(pocommon.pounit):
 
     def hastypecomment(self, typecomment):
         """Check whether the given type comment is present"""
-        # check for word boundaries properly by using a regular expression...
-        return sum(map(lambda tcline: len(re.findall("\\b%s\\b" % typecomment, tcline)), self.typecomments)) != 0
+        if not self.typecomments:
+            return False
+        for tc in self.typecomments:
+            # check for word boundaries properly by using a regular expression
+            if re.search("\\b%s\\b" % typecomment, tc):
+                return True
+        return False
 
     def hasmarkedcomment(self, commentmarker):
-        """Check whether the given comment marker is present as # (commentmarker) ..."""
+        """Check whether the given comment marker is present.
+
+        These should appear as::
+
+                # (commentmarker) ...
+        """
         commentmarker = "(%s)" % commentmarker
         for comment in self.othercomments:
             if comment.replace("#", "", 1).strip().startswith(commentmarker):
@@ -455,64 +526,51 @@ class pounit(pocommon.pounit):
     def settypecomment(self, typecomment, present=True):
         """Alters whether a given typecomment is present"""
         if self.hastypecomment(typecomment) != present:
+            typecomments = re.findall(r"\b[-\w]+\b", "\n".join(self.typecomments))
             if present:
-                if len(self.typecomments):
-                    # There is already a comment, so we have to add onto it
-                    self.typecomments[0] = "%s, %s\n" % (self.typecomments[0][:-1], typecomment)
-                else:
-                    self.typecomments.append("#, %s\n" % typecomment)
+                typecomments.append(typecomment)
             else:
-                # this should handle word boundaries properly ...
-                typecomments = map(lambda tcline: re.sub("\\b%s\\b[ \t,]*" % typecomment, "", tcline), self.typecomments)
-                self.typecomments = filter(lambda tcline: tcline.strip() != "#,", typecomments)
+                typecomments.remove(typecomment)
+            if typecomments:
+                typecomments.sort()
+                self.typecomments = ["#, %s\n" % ", ".join(typecomments)]
+            else:
+                self.typecomments = []
 
     def isfuzzy(self):
-        return self.hastypecomment("fuzzy")
+        return self.hastypecomment('fuzzy')
 
     def markfuzzy(self, present=True):
+        if present:
+            self.set_state_n(self.STATE[self.S_FUZZY][0])
+        elif self.hasplural() and not self._msgstrlen() or is_null(self.msgstr):
+            self.set_state_n(self.STATE[self.S_UNTRANSLATED][0])
+        else:
+            self.set_state_n(self.STATE[self.S_TRANSLATED][0])
+
+    def _domarkfuzzy(self, present=True):
         self.settypecomment("fuzzy", present)
 
+    def infer_state(self):
+        if self.obsolete:
+            self.makeobsolete()
+        else:
+            self.markfuzzy(self.hastypecomment('fuzzy'))
+
     def isobsolete(self):
         return self.obsolete
 
     def makeobsolete(self):
         """Makes this unit obsolete"""
+        super(pounit, self).makeobsolete()
         self.obsolete = True
-        if self.msgctxt:
-            self.obsoletemsgctxt = self.msgctxt
-        if self.msgid:
-            self.obsoletemsgid = self.msgid
-            self.msgid = []
-        if self.msgidcomments:
-            self.obsoletemsgidcomments = self.msgidcomments
-            self.msgidcomments = []
-        if self.msgid_plural:
-            self.obsoletemsgid_plural = self.msgid_plural
-            self.msgid_plural = []
-        if self.msgstr:
-            self.obsoletemsgstr = self.msgstr
-            self.msgstr = []
         self.sourcecomments = []
         self.automaticcomments = []
 
     def resurrect(self):
         """Makes an obsolete unit normal"""
+        super(pounit, self).resurrect()
         self.obsolete = False
-        if self.obsoletemsgctxt:
-            self.msgid = self.obsoletemsgctxt
-            self.obsoletemsgctxt = []
-        if self.obsoletemsgid:
-            self.msgid = self.obsoletemsgid
-            self.obsoletemsgid = []
-        if self.obsoletemsgidcomments:
-            self.msgidcomments = self.obsoletemsgidcomments
-            self.obsoletemsgidcomments = []
-        if self.obsoletemsgid_plural:
-            self.msgid_plural = self.obsoletemsgid_plural
-            self.obsoletemsgid_plural = []
-        if self.obsoletemsgstr:
-            self.msgstr = self.obsoletemsgstr
-            self.obsoletemgstr = []
 
     def hasplural(self):
         """returns whether this pounit contains plural strings..."""
@@ -565,63 +623,65 @@ class pounit(pocommon.pounit):
         return partstr
 
     def _encodeifneccessary(self, output):
-        """encodes unicode strings and returns other strings unchanged"""
+        """Encodes unicode strings and returns other strings unchanged"""
         if isinstance(output, unicode):
-            encoding = encodingToUse(getattr(self, "encoding", "UTF-8"))
+            encoding = encodingToUse(getattr(self, "_encoding", "UTF-8"))
             return output.encode(encoding)
         return output
 
     def __str__(self):
-        """convert to a string. double check that unicode is handled somehow here"""
+        """Convert to a string. Double check that unicode is handled
+        somehow here"""
         output = self._getoutput()
         return self._encodeifneccessary(output)
 
     def _getoutput(self):
         """return this po element as a string"""
-        def add_prev_msgid_lines(lines, header, var):
-            if len(var) > 0:
-                lines.append("#| %s %s\n" % (header, var[0]))
-                lines.extend("#| %s\n" % line for line in var[1:])
 
-        def add_prev_msgid_info(lines):
-            add_prev_msgid_lines(lines, 'msgctxt', self.prev_msgctxt)
-            add_prev_msgid_lines(lines, 'msgid', self.prev_msgid)
-            add_prev_msgid_lines(lines, 'msgid_plural', self.prev_msgid_plural)
+        def add_prev_msgid_lines(lines, prefix, header, var):
+            if len(var) > 0:
+                lines.append("%s %s %s\n" % (prefix, header, var[0]))
+                lines.extend("%s %s\n" % (prefix, line) for line in var[1:])
+
+        def add_prev_msgid_info(lines, prefix):
+            add_prev_msgid_lines(lines, prefix, 'msgctxt', self.prev_msgctxt)
+            add_prev_msgid_lines(lines, prefix, 'msgid', self.prev_msgid)
+            add_prev_msgid_lines(lines, prefix, 'msgid_plural', self.prev_msgid_plural)
 
         lines = []
         lines.extend(self.othercomments)
         if self.isobsolete():
             lines.extend(self.typecomments)
             obsoletelines = []
-            if self.obsoletemsgctxt:
-                obsoletelines.append(self._getmsgpartstr("#~ msgctxt", self.obsoletemsgctxt))
-            obsoletelines.append(self._getmsgpartstr("#~ msgid", self.obsoletemsgid, self.obsoletemsgidcomments))
-            if self.obsoletemsgid_plural or self.obsoletemsgid_pluralcomments:
-                obsoletelines.append(self._getmsgpartstr("#~ msgid_plural", self.obsoletemsgid_plural, self.obsoletemsgid_pluralcomments))
-            obsoletelines.append(self._getmsgpartstr("#~ msgstr", self.obsoletemsgstr))
+            add_prev_msgid_info(obsoletelines, prefix="#~|")
+            if self.msgctxt:
+                obsoletelines.append(self._getmsgpartstr("#~ msgctxt", self.msgctxt))
+            obsoletelines.append(self._getmsgpartstr("#~ msgid", self.msgid, self.msgidcomments))
+            if self.msgid_plural or self.msgid_pluralcomments:
+                obsoletelines.append(self._getmsgpartstr("#~ msgid_plural", self.msgid_plural, self.msgid_pluralcomments))
+            obsoletelines.append(self._getmsgpartstr("#~ msgstr", self.msgstr))
             for index, obsoleteline in enumerate(obsoletelines):
                 # We need to account for a multiline msgid or msgstr here
                 obsoletelines[index] = obsoleteline.replace('\n"', '\n#~ "')
             lines.extend(obsoletelines)
-            lines = [self._encodeifneccessary(line) for line in lines]
-            return "".join(lines)
-        # if there's no msgid don't do msgid and string, unless we're the header
-        # this will also discard any comments other than plain othercomments...
+            return u"".join(lines)
+        # if there's no msgid don't do msgid and string, unless we're the
+        # header this will also discard any comments other than plain
+        # othercomments...
         if is_null(self.msgid):
             if not (self.isheader() or self.getcontext() or self.sourcecomments):
-                return "".join(lines)
+                return u"".join(lines)
         lines.extend(self.automaticcomments)
         lines.extend(self.sourcecomments)
         lines.extend(self.typecomments)
-        add_prev_msgid_info(lines)
+        add_prev_msgid_info(lines, prefix="#|")
         if self.msgctxt:
-            lines.append(self._getmsgpartstr("msgctxt", self.msgctxt))
-        lines.append(self._getmsgpartstr("msgid", self.msgid, self.msgidcomments))
+            lines.append(self._getmsgpartstr(u"msgctxt", self.msgctxt))
+        lines.append(self._getmsgpartstr(u"msgid", self.msgid, self.msgidcomments))
         if self.msgid_plural or self.msgid_pluralcomments:
-            lines.append(self._getmsgpartstr("msgid_plural", self.msgid_plural, self.msgid_pluralcomments))
-        lines.append(self._getmsgpartstr("msgstr", self.msgstr))
-        lines = [self._encodeifneccessary(line) for line in lines]
-        postr = "".join(lines)
+            lines.append(self._getmsgpartstr(u"msgid_plural", self.msgid_plural, self.msgid_pluralcomments))
+        lines.append(self._getmsgpartstr(u"msgstr", self.msgstr))
+        postr = u"".join(lines)
         return postr
 
     def getlocations(self):
@@ -634,22 +694,28 @@ class pounit(pocommon.pounit):
         locations = []
         for sourcecomment in self.sourcecomments:
             locations += quote.rstripeol(sourcecomment)[3:].split()
+        for i, loc in enumerate(locations):
+            locations[i] = pocommon.unquote_plus(loc)
         return locations
 
     def addlocation(self, location):
         """Add a location to sourcecomments in the PO unit
 
-        @param location: Text location e.g. 'file.c:23' does not include #:
-        @type location: String
+        :param location: Text location e.g. 'file.c:23' does not include #:
+        :type location: String
 
         """
+        location = data.forceunicode(location)
+        if location.find(" ") != -1:
+            location = pocommon.quote_plus(location)
         self.sourcecomments.append("#: %s\n" % location)
 
     def _extract_msgidcomments(self, text=None):
         """Extract KDE style msgid comments from the unit.
 
-        @rtype: String
-        @return: Returns the extracted msgidcomments found in this unit's msgid.
+        :rtype: String
+        :return: Returns the extracted msgidcomments found in this
+                 unit's msgid.
         """
 
         if not text:
@@ -668,12 +734,16 @@ class pounit(pocommon.pounit):
         """Get the message context."""
         return unquotefrompo(self.msgctxt) + self._extract_msgidcomments()
 
+    def setcontext(self, context):
+        context = data.forceunicode(context)
+        self.msgctxt = quoteforpo(context)
+
     def getid(self):
         """Returns a unique identifier for this unit."""
         context = self.getcontext()
-        # Gettext does not consider the plural to determine duplicates, only 
+        # Gettext does not consider the plural to determine duplicates, only
         # the msgid. For generation of .mo files, we might want to use this
-        # code to generate the entry for the hash table, but for now, it is 
+        # code to generate the entry for the hash table, but for now, it is
         # commented out for conformance to gettext.
 #        id = '\0'.join(self.source.strings)
         id = self.source
@@ -683,13 +753,15 @@ class pounit(pocommon.pounit):
             id = u"%s\04%s" % (context, id)
         return id
 
+
 class pofile(pocommon.pofile):
     """A .po file containing various units"""
     UnitClass = pounit
 
     def parse(self, input):
         """Parses the given file or file source string."""
-        try:
+        if True:
+#        try:
             if hasattr(input, 'name'):
                 self.filename = input.name
             elif not getattr(self, 'filename', ''):
@@ -699,11 +771,12 @@ class pofile(pocommon.pofile):
             # clear units to get rid of automatically generated headers before parsing
             self.units = []
             poparser.parse_units(poparser.ParseState(input, pounit), self)
-        except Exception, e:
-            raise base.ParseError(e)
+#        except Exception, e:
+#            raise base.ParseError(e)
 
     def removeduplicates(self, duplicatestyle="merge"):
-        """Make sure each msgid is unique ; merge comments etc from duplicates into original"""
+        """Make sure each msgid is unique ; merge comments etc from
+        duplicates into original"""
         # TODO: can we handle consecutive calls to removeduplicates()? What
         # about files already containing msgctxt? - test
         id_dict = {}
@@ -711,6 +784,7 @@ class pofile(pocommon.pofile):
         # TODO: this is using a list as the pos aren't hashable, but this is slow.
         # probably not used frequently enough to worry about it, though.
         markedpos = []
+
         def addcomment(thepo):
             thepo.msgidcomments.append('"_: %s\\n"' % " ".join(thepo.getlocations()))
             markedpos.append(thepo)
@@ -728,7 +802,8 @@ class pofile(pocommon.pofile):
                         uniqueunits.append(thepo)
                 elif duplicatestyle == "msgctxt":
                     origpo = id_dict[id]
-                    if origpo not in markedpos:
+                    if origpo not in markedpos and id:
+                        # if it doesn't have an id, we already added msgctxt
                         origpo.msgctxt.append('"%s"' % escapeforpo(" ".join(origpo.getlocations())))
                         markedpos.append(thepo)
                     thepo.msgctxt.append('"%s"' % escapeforpo(" ".join(thepo.getlocations())))
@@ -744,22 +819,31 @@ class pofile(pocommon.pofile):
         self.units = uniqueunits
 
     def __str__(self):
-        """Convert to a string. double check that unicode is handled somehow here"""
+        """Convert to a string. Double check that unicode is handled somehow
+        here"""
         output = self._getoutput()
         if isinstance(output, unicode):
-            return output.encode(getattr(self, "encoding", "UTF-8"))
+            try:
+                return output.encode(getattr(self, "_encoding", "UTF-8"))
+            except UnicodeEncodeError, e:
+                self.updateheader(add=True, Content_Type="text/plain; charset=UTF-8")
+                self._encoding = "UTF-8"
+                for unit in self.units:
+                    unit._encoding = "UTF-8"
+                return self._getoutput().encode("UTF-8")
+
         return output
 
     def _getoutput(self):
         """convert the units back to lines"""
         lines = []
         for unit in self.units:
-            unitsrc = str(unit) + "\n"
+            unitsrc = unit._getoutput() + u"\n"
             lines.append(unitsrc)
-        lines = "".join(self.encode(lines)).rstrip()
+        lines = u"".join(lines).rstrip()
         #After the last pounit we will have \n\n and we only want to end in \n:
         if lines:
-            lines += "\n"
+            lines += u"\n"
         return lines
 
     def encode(self, lines):
@@ -778,11 +862,13 @@ class pofile(pocommon.pofile):
         """decode any non-unicode strings in lines with self._encoding"""
         newlines = []
         for line in lines:
-            if isinstance(line, str) and self._encoding is not None and self._encoding.lower() != "charset":
+            if (isinstance(line, str) and self._encoding is not None and
+                self._encoding.lower() != "charset"):
                 try:
                     line = line.decode(self._encoding)
                 except UnicodeError, e:
-                    raise UnicodeError("Error decoding line with encoding %r: %s. Line is %r" % (self._encoding, e, line))
+                    raise UnicodeError("Error decoding line with encoding %r: %s. Line is %r" %
+                                       (self._encoding, e, line))
             newlines.append(line)
         return newlines
 
