@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2007-2009 Zuza Software Foundation
+# Copyright 2007-2010 Zuza Software Foundation
 #
 # This file is part of the Translate Toolkit.
 #
@@ -21,18 +21,8 @@
 
 """Module to provide a cache of statistics in a database.
 
-@organization: Zuza Software Foundation
-@copyright: 2007 Zuza Software Foundation
-@license: U{GPL <http://www.fsf.org/licensing/licenses/gpl.html>}
 """
 
-from UserDict import UserDict
-
-from translate import __version__ as toolkitversion
-from translate.storage import factory
-from translate.misc.multistring import multistring
-from translate.lang.common import Common
-
 try:
     from sqlite3 import dbapi2
 except ImportError:
@@ -42,17 +32,45 @@ import re
 import sys
 import stat
 import thread
+import logging
+from UserDict import UserDict
 
-kdepluralre = re.compile("^_n: ")
+from translate import __version__ as toolkitversion
+from translate.lang.common import Common
+from translate.misc.multistring import multistring
+from translate.storage import factory
+from translate.storage.workflow import StateEnum
+
+logger = logging.getLogger(__name__)
+
+#kdepluralre = re.compile("^_n: ") #Restore this if you really need support for old kdeplurals
 brtagre = re.compile("<br\s*?/?>")
 xmltagre = re.compile("<[^>]+>")
 numberre = re.compile("\\D\\.\\D")
 
-state_strings = {0: "untranslated", 1: "translated", 2: "fuzzy"}
+extended_state_strings = {
+    StateEnum.EMPTY: "empty",
+    StateEnum.NEEDS_WORK: "needs-work",
+    StateEnum.REJECTED: "rejected",
+    StateEnum.NEEDS_REVIEW: "needs-review",
+    StateEnum.UNREVIEWED: "unreviewed",
+    StateEnum.FINAL: "final",
+    }
+
+UNTRANSLATED = StateEnum.EMPTY
+FUZZY = StateEnum.NEEDS_WORK
+TRANSLATED = StateEnum.UNREVIEWED
+
+state_strings = {
+    UNTRANSLATED: "untranslated",
+    FUZZY: "fuzzy",
+    TRANSLATED: "translated",
+}
+
 
 def wordcount(string):
-    # TODO: po class should understand KDE style plurals
-    string = kdepluralre.sub("", string)
+    # TODO: po class should understand KDE style plurals ##
+    #string = kdepluralre.sub("", string) #Restore this if you really need support for old kdeplurals
     string = brtagre.sub("\n", string)
     string = xmltagre.sub("", string)
     string = numberre.sub(" ", string)
@@ -60,6 +78,7 @@ def wordcount(string):
     #language
     return len(Common.words(string))
 
+
 def wordsinunit(unit):
     """Counts the words in the unit's source and target, taking plurals into
     account. The target words are only counted if the unit is translated."""
@@ -80,8 +99,10 @@ def wordsinunit(unit):
         targetwords += wordcount(s)
     return sourcewords, targetwords
 
+
 class Record(UserDict):
-    def __init__(self, record_keys, record_values=None, compute_derived_values = lambda x: x):
+
+    def __init__(self, record_keys, record_values=None, compute_derived_values=lambda x: x):
         if record_values == None:
             record_values = (0 for _i in record_keys)
         self.record_keys = record_keys
@@ -109,6 +130,7 @@ class Record(UserDict):
     def as_string_for_db(self):
         return ",".join([repr(x) for x in self.to_tuple()])
 
+
 def transaction(f):
     """Modifies f to commit database changes if it executes without exceptions.
     Otherwise it rolls back the database.
@@ -131,7 +153,7 @@ def transaction(f):
             raise
     return decorated_f
 
-UNTRANSLATED, TRANSLATED, FUZZY = 0, 1, 2
+
 def statefordb(unit):
     """Returns the numeric database state for the unit."""
     if unit.istranslated():
@@ -140,6 +162,7 @@ def statefordb(unit):
         return FUZZY
     return UNTRANSLATED
 
+
 class FileTotals(object):
     keys = ['translatedsourcewords',
             'fuzzysourcewords',
@@ -166,7 +189,7 @@ class FileTotals(object):
                 translatedtargetwords   INTEGER NOT NULL);""")
 
     def new_record(cls, state_for_db=None, sourcewords=None, targetwords=None):
-        record = Record(cls.keys, compute_derived_values = cls._compute_derived_values)
+        record = Record(cls.keys, compute_derived_values=cls._compute_derived_values)
         if state_for_db is not None:
             if state_for_db is UNTRANSLATED:
                 record['untranslated'] = 1
@@ -183,13 +206,13 @@ class FileTotals(object):
     new_record = classmethod(new_record)
 
     def _compute_derived_values(cls, record):
-        record["total"]            = record["untranslated"] + \
-                                     record["translated"] + \
-                                     record["fuzzy"]
+        record["total"] = record["untranslated"] + \
+                          record["translated"] + \
+                          record["fuzzy"]
         record["totalsourcewords"] = record["untranslatedsourcewords"] + \
                                      record["translatedsourcewords"] + \
                                      record["fuzzysourcewords"]
-        record["review"]           = 0
+        record["review"] = 0
     _compute_derived_values = classmethod(_compute_derived_values)
 
     def __getitem__(self, fileid):
@@ -209,38 +232,48 @@ class FileTotals(object):
         self.cur.execute("""
             DELETE FROM filetotals
             WHERE fileid=?;
-        """,  (fileid,))
+        """, (fileid,))
+
 
 def emptyfiletotals():
     """Returns a dictionary with all statistics initalised to 0."""
     return FileTotals.new_record()
 
+
 def emptyfilechecks():
     return {}
 
+
 def emptyfilestats():
     return {"total": [], "translated": [], "fuzzy": [], "untranslated": []}
 
+
 def emptyunitstats():
     return {"sourcewordcount": [], "targetwordcount": []}
 
+
 # We allow the caller to specify which value to return when errors_return_empty
 # is True. We do this, since Poolte wants None to be returned when it calls
 # get_mod_info directly, whereas we want an integer to be returned for
 # uses of get_mod_info within this module.
 # TODO: Get rid of empty_return when Pootle code is improved to not require
 #       this.
+
+
 def get_mod_info(file_path):
     file_stat = os.stat(file_path)
     assert not stat.S_ISDIR(file_stat.st_mode)
     return file_stat.st_mtime, file_stat.st_size
 
+
 def suggestion_extension():
     return os.path.extsep + 'pending'
 
+
 def suggestion_filename(filename):
     return filename + suggestion_extension()
 
+
 # ALL PUBLICLY ACCESSIBLE METHODS MUST BE DECORATED WITH THE transaction DECORATOR.
 class StatsCache(object):
     """An object instantiated as a singleton for each statsfile that provides
@@ -254,14 +287,17 @@ class StatsCache(object):
 
     def __new__(cls, statsfile=None):
         current_thread = thread.get_ident()
+
         def make_database(statsfile):
+
             def connect(cache):
-                cache.con = dbapi2.connect(statsfile)
+                # sqlite needs to get the name in utf-8 on all platforms
+                cache.con = dbapi2.connect(statsfile.encode('utf-8'))
                 cache.cur = cache.con.cursor()
 
             def clear_old_data(cache):
                 try:
-                    cache.cur.execute("""SELECT toolkitbuild FROM files""")
+                    cache.cur.execute("""SELECT min(toolkitbuild) FROM files""")
                     val = cache.cur.fetchone()
                     # If the database is empty, we have no idea whether its layout
                     # is correct, so we might as well delete it.
@@ -273,7 +309,7 @@ class StatsCache(object):
                     return False
                 except dbapi2.OperationalError:
                     return False
-            
+
             cache = cls._caches.setdefault(current_thread, {})[statsfile] = object.__new__(cls)
             connect(cache)
             if clear_old_data(cache):
@@ -291,7 +327,8 @@ class StatsCache(object):
                     cachedir = os.path.join(userdir, ".translate_toolkit")
                 if not os.path.exists(cachedir):
                     os.mkdir(cachedir)
-                cls.defaultfile = os.path.realpath(os.path.join(cachedir, "stats.db"))
+                cachedir = cachedir.decode(sys.getfilesystemencoding())
+                cls.defaultfile = os.path.realpath(os.path.join(cachedir, u"stats.db"))
             statsfile = cls.defaultfile
         else:
             statsfile = os.path.realpath(statsfile)
@@ -324,6 +361,7 @@ class StatsCache(object):
             source VARCHAR NOT NULL,
             target VARCHAR,
             state INTEGER,
+            e_state INTEGER,
             sourcewords INTEGER,
             targetwords INTEGER);""")
 
@@ -385,7 +423,7 @@ class StatsCache(object):
             store = store or factory.getobject(realpath)
 
         return self._cachestore(store, realpath, mod_info)
-    
+
     def _getstoredcheckerconfig(self, checker):
         """See if this checker configuration has been used before."""
         config = str(checker.config.__dict__)
@@ -407,15 +445,17 @@ class StatsCache(object):
                 if unitindex:
                     index = unitindex
                 # what about plurals in .source and .target?
+                unit_state_for_db = statefordb(unit)
                 unitvalues.append((unit.getid(), fileid, index, \
                                 unit.source, unit.target, \
                                 sourcewords, targetwords, \
-                                statefordb(unit)))
-                file_totals_record = file_totals_record + FileTotals.new_record(statefordb(unit), sourcewords, targetwords)
+                                unit_state_for_db,
+                                unit.get_state_id()))
+                file_totals_record = file_totals_record + FileTotals.new_record(unit_state_for_db, sourcewords, targetwords)
         # XXX: executemany is non-standard
         self.cur.executemany("""INSERT INTO units
-            (unitid, fileid, unitindex, source, target, sourcewords, targetwords, state)
-            values (?, ?, ?, ?, ?, ?, ?, ?);""",
+            (unitid, fileid, unitindex, source, target, sourcewords, targetwords, state, e_state)
+            values (?, ?, ?, ?, ?, ?, ?, ?, ?);""",
             unitvalues)
         self.file_totals[fileid] = file_totals_record
         if unitindex:
@@ -428,19 +468,39 @@ class StatsCache(object):
         unconditionally."""
         self.cur.execute("""DELETE FROM files WHERE
             path=?;""", (realpath,))
-        self.cur.execute("""INSERT INTO files
+        self.cur.execute("""iNSERT INTO files
             (fileid, path, st_mtime, st_size, toolkitbuild) values (NULL, ?, ?, ?, ?);""",
             (realpath, mod_info[0], mod_info[1], toolkitversion.build))
+        # Unusual capitalisation intended. See bug 2073.
         fileid = self.cur.lastrowid
         self.cur.execute("""DELETE FROM units WHERE
             fileid=?""", (fileid,))
         self._cacheunitstats(store.units, fileid)
         return fileid
 
-    def filetotals(self, filename, store=None):
+    def file_extended_totals(self, filename, store=None):
+        stats = {}
+        fileid = self._getfileid(filename, store=store)
+
+        self.cur.execute("""SELECT e_state, COUNT(id), SUM(sourcewords), SUM(targetwords)
+            FROM units WHERE fileid=? GROUP BY e_state""", (fileid,))
+        values = self.cur.fetchall()
+
+        for value in values:
+            stats[extended_state_strings[value[0]]] = {
+                "units": value[1],
+                "sourcewords": value[2],
+                "targetwords": value[3],
+                }
+        return stats
+
+    def filetotals(self, filename, store=None, extended=False):
         """Retrieves the statistics for the given file if possible, otherwise
         delegates to cachestore()."""
-        return self.file_totals[self._getfileid(filename, store=store)]
+        stats = self.file_totals[self._getfileid(filename, store=store)]
+        if extended:
+            stats["extended"] = self.file_extended_totals(filename, store=store)
+        return stats
 
     @transaction
     def _cacheunitschecks(self, units, fileid, configid, checker, unitindex=None):
@@ -496,8 +556,9 @@ class StatsCache(object):
         if result is not None:
             return result
         else:
-            print >> sys.stderr, """WARNING: Database in inconsistent state. 
-            fileid %d and unitid %s have no entries in the table units.""" % (fileid, unitid)
+            logger.warning("Database in inconsistent state - fileid %d and "
+                           "unitid %s have no entries in the table units.",
+                           fileid, unitid)
             # If values.fetchone() is None, then we return an empty list,
             # to make FileTotals.new_record(*self.get_unit_stats(fileid, unitid))
             # do the right thing.
@@ -532,6 +593,7 @@ class StatsCache(object):
         return state
 
     def _checkerrors(self, filename, fileid, configid, checker, store):
+
         def geterrors():
             self.cur.execute("""SELECT
                 name,
@@ -568,9 +630,10 @@ class StatsCache(object):
         configid = self._getstoredcheckerconfig(checker)
         if configid:
             return configid
-        self.cur.execute("""INSERT INTO checkerconfigs
+        self.cur.execute("""iNSERT INTO checkerconfigs
             (configid, config) values (NULL, ?);""",
             (str(checker.config.__dict__),))
+        # Unusual capitalisation intended. See bug 2073.
         return self.cur.lastrowid
 
     def filechecks(self, filename, checker, store=None):
@@ -584,49 +647,52 @@ class StatsCache(object):
         for value in values:
             if value[1] == -1:
                 continue
-            checkkey = 'check-' + value[0]      #value[0] is the error name
+            checkkey = 'check-' + value[0]      # value[0] is the error name
             if not checkkey in errors:
                 errors[checkkey] = []
-            errors[checkkey].append(value[1])   #value[1] is the unitindex
+            errors[checkkey].append(value[1])   # value[1] is the unitindex
 
         return errors
 
     def file_fails_test(self, filename, checker, name):
         fileid = self._getfileid(filename)
-        configid = self._get_config_id(fileid, checker) 
+        configid = self._get_config_id(fileid, checker)
         self._checkerrors(filename, fileid, configid, checker, None)
         self.cur.execute("""SELECT
             name,
             unitindex
-            FROM uniterrors 
+            FROM uniterrors
             WHERE fileid=? and configid=? and name=?;""", (fileid, configid, name))
         return self.cur.fetchone() is not None
 
-    def filestatestats(self, filename, store=None):
+    def filestatestats(self, filename, store=None, extended=False):
         """Return a dictionary of unit stats mapping sets of unit
         indices with those states"""
         stats = emptyfilestats()
+        if extended:
+            stats["extended"] = {}
+
         fileid = self._getfileid(filename, store=store)
 
-        self.cur.execute("""SELECT
-            state,
-            unitindex
-            FROM units WHERE fileid=?
-            ORDER BY unitindex;""", (fileid,))
+        self.cur.execute("""SELECT state, e_state, unitindex
+            FROM units WHERE fileid=? ORDER BY unitindex;""", (fileid,))
         values = self.cur.fetchall()
 
         for value in values:
-            stats[state_strings[value[0]]].append(value[1])
-            stats["total"].append(value[1])
-
+            stats[state_strings[value[0]]].append(value[2])
+            if extended:
+                if value[1] not in stats["extended"]:
+                    stats["extended"][value[1]] = []
+                stats["extended"][value[1]].append(value[2])
+            stats["total"].append(value[2])
         return stats
 
-    def filestats(self, filename, checker, store=None):
+    def filestats(self, filename, checker, store=None, extended=False):
         """Return a dictionary of property names mapping sets of unit
         indices with those properties."""
         stats = emptyfilestats()
         stats.update(self.filechecks(filename, checker, store))
-        stats.update(self.filestatestats(filename, store))
+        stats.update(self.filestatestats(filename, store, extended=extended))
         return stats
 
     def unitstats(self, filename, _lang=None, store=None):
