@@ -20,17 +20,21 @@
 
 """tests for storage base classes"""
 
-from translate.misc.multistring import multistring
-from translate.storage import base
-from translate.storage.placeables import general, parse as rich_parse
-from py import test
 import os
 import warnings
 
+import pytest
+
+from translate.misc.multistring import multistring
+from translate.storage import base, factory
+from translate.storage.placeables import general, parse as rich_parse
+
+
 def headerless_len(units):
     """return count of translatable (non header) units"""
     return len(filter(lambda x: not x.isheader(), units))
 
+
 def first_translatable(store):
     """returns first translatable unit, skipping header if present"""
     if store.units[0].isheader() and len(store.units) > 1:
@@ -38,24 +42,31 @@ def first_translatable(store):
     else:
         return store.units[0]
 
+
 def test_force_override():
     """Tests that derived classes are not allowed to call certain functions"""
+
     class BaseClass:
+
         def test(self):
             base.force_override(self.test, BaseClass)
             return True
+
         def classtest(cls):
             base.force_override(cls.classtest, BaseClass)
             return True
         classtest = classmethod(classtest)
+
     class DerivedClass(BaseClass):
         pass
+
     baseobject = BaseClass()
     assert baseobject.test()
     assert baseobject.classtest()
     derivedobject = DerivedClass()
-    assert test.raises(NotImplementedError, derivedobject.test)
-    assert test.raises(NotImplementedError, derivedobject.classtest)
+    assert pytest.raises(NotImplementedError, derivedobject.test)
+    assert pytest.raises(NotImplementedError, derivedobject.classtest)
+
 
 class TestTranslationUnit:
     """Tests a TranslationUnit.
@@ -109,13 +120,13 @@ class TestTranslationUnit:
         assert unit.target == ""
 
     def test_escapes(self):
-        """Test all sorts of characters that might go wrong in a quoting and 
+        """Test all sorts of characters that might go wrong in a quoting and
         escaping roundtrip."""
         unit = self.unit
         specials = ['Fish & chips', 'five < six', 'six > five', 'five &lt; six',
                     'Use &nbsp;', 'Use &amp;nbsp;', 'Use &amp;amp;nbsp;',
                     'A "solution"', "skop 'n bal", '"""', "'''", u'µ',
-                    '\n', '\t', '\r', '\r\n', '\\r', '\\', '\\\r'] 
+                    '\n', '\t', '\r', '\r\n', '\\r', '\\', '\\\r']
         for special in specials:
             unit.source = special
             print "unit.source:", repr(unit.source)
@@ -123,7 +134,7 @@ class TestTranslationUnit:
             assert unit.source == special
 
     def test_difficult_escapes(self):
-        """Test difficult characters that might go wrong in a quoting and 
+        """Test difficult characters that might go wrong in a quoting and
         escaping roundtrip."""
 
         unit = self.unit
@@ -195,6 +206,7 @@ class TestTranslationUnit:
         else:
             assert unit.target == u'Tëst <x>string</x>'
 
+
 class TestTranslationStore(object):
     """Tests a TranslationStore.
     Derived classes can reuse these tests by pointing StoreClass to a derived Store"""
@@ -259,7 +271,7 @@ class TestTranslationStore(object):
             store2unit = store2.units[n]
             match = store1unit == store2unit
             if not match:
-                print "match failed between elements %d of %d" % (n+1, headerless_len(store1.units))
+                print "match failed between elements %d of %d" % ((n + 1), headerless_len(store1.units))
                 print "store1:"
                 print str(store1)
                 print "store2:"
@@ -320,3 +332,31 @@ class TestTranslationStore(object):
         assert answer == u"Beziér-kurwe"
         #Just test that __str__ doesn't raise exception:
         src = str(store)
+
+    def test_extensions(self):
+        """Test that the factory knows the extensions for this class."""
+        supported = factory.supported_files()
+        supported_dict = dict([(name, (extensions, mimetypes)) for name, extensions, mimetypes in supported])
+        if not (self.StoreClass.Name and self.StoreClass.Name in supported_dict):
+            return
+        detail = supported_dict[self.StoreClass.Name]  # will start to get problematic once translated
+        print "Factory:", detail[0]
+        print "StoreClass:", self.StoreClass.Extensions
+        for ext in detail[0]:
+            assert ext in self.StoreClass.Extensions
+        for ext in self.StoreClass.Extensions:
+            assert ext in detail[0]
+
+    def test_mimetypes(self):
+        """Test that the factory knows the mimetypes for this class."""
+        supported = factory.supported_files()
+        supported_dict = dict([(name, (extensions, mimetypes)) for name, extensions, mimetypes in supported])
+        if not (self.StoreClass.Name and self.StoreClass.Name in supported_dict):
+            return
+        detail = supported_dict[self.StoreClass.Name]  # will start to get problematic once translated
+        print "Factory:", detail[1]
+        print "StoreClass:", self.StoreClass.Mimetypes
+        for ext in detail[1]:
+            assert ext in self.StoreClass.Mimetypes
+        for ext in self.StoreClass.Mimetypes:
+            assert ext in detail[1]
