@@ -1,11 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from translate.storage import test_po
-from translate.storage import cpo
-from translate.misc.multistring import multistring
+from pytest import raises, mark, importorskip
+
 from translate.misc import wStringIO
-from py.test import raises
+from translate.misc.multistring import multistring
+from translate.storage import test_po
+cpo = importorskip("translate.storage.cpo")
+
 
 class TestCPOUnit(test_po.TestPOUnit):
     UnitClass = cpo.pounit
@@ -23,7 +25,7 @@ class TestCPOUnit(test_po.TestPOUnit):
         assert unit.target.strings == ["Koei", "Koeie"]
         assert unit.target == "Koei"
 
-        unit.target = {0:"Koei", 3:"Koeie"}
+        unit.target = {0: "Koei", 3: "Koeie"}
         assert isinstance(unit.target, multistring)
         assert unit.target.strings == ["Koei", "Koeie"]
         assert unit.target == "Koei"
@@ -72,8 +74,10 @@ class TestCPOUnit(test_po.TestPOUnit):
         unit.addnote("# Double commented comment")
         assert unit.getnotes() == "# Double commented comment"
 
+
 class TestCPOFile(test_po.TestPOFile):
     StoreClass = cpo.pofile
+
     def test_msgidcomments(self):
         """checks that we handle msgid comments"""
         posource = 'msgid "test me"\nmsgstr ""'
@@ -86,53 +90,57 @@ class TestCPOFile(test_po.TestPOFile):
         thepo.msgidcomment = "second comment"
         assert str(pofile).count("_:") == 1
 
-#    def test_merge_duplicates_msgctxt(self):
-#        """checks that merging duplicates works for msgctxt"""
-#        posource = '#: source1\nmsgid "test me"\nmsgstr ""\n\n#: source2\nmsgid "test me"\nmsgstr ""\n'
-#        pofile = self.poparse(posource)
-#        assert len(pofile.units) == 2
-#        pofile.removeduplicates("msgctxt")
-#        print pofile
-#        assert len(pofile.units) == 2
-#        assert str(pofile.units[0]).count("source1") == 2
-#        assert str(pofile.units[1]).count("source2") == 2
-  
-#    def test_merge_blanks(self):
-#        """checks that merging adds msgid_comments to blanks"""
-#        posource = '#: source1\nmsgid ""\nmsgstr ""\n\n#: source2\nmsgid ""\nmsgstr ""\n'
-#        pofile = self.poparse(posource)
-#        assert len(pofile.units) == 2
-#        pofile.removeduplicates("merge")
-#        assert len(pofile.units) == 2
-#        print pofile.units[0].msgidcomments
-#        print pofile.units[1].msgidcomments
-#        assert po.unquotefrompo(pofile.units[0].msgidcomments) == "_: source1\n"
-#        assert po.unquotefrompo(pofile.units[1].msgidcomments) == "_: source2\n"
+    @mark.xfail(reason="Were disabled during port of Pypo to cPO - they might work")
+    def test_merge_duplicates_msgctxt(self):
+        """checks that merging duplicates works for msgctxt"""
+        posource = '#: source1\nmsgid "test me"\nmsgstr ""\n\n#: source2\nmsgid "test me"\nmsgstr ""\n'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 2
+        pofile.removeduplicates("msgctxt")
+        print pofile
+        assert len(pofile.units) == 2
+        assert str(pofile.units[0]).count("source1") == 2
+        assert str(pofile.units[1]).count("source2") == 2
 
-#    def test_msgid_comment(self):
-#        """checks that when adding msgid_comments we place them on a newline"""
-#        posource = '#: source0\nmsgid "Same"\nmsgstr ""\n\n#: source1\nmsgid "Same"\nmsgstr ""\n'
-#        pofile = self.poparse(posource)
-#        assert len(pofile.units) == 2
-#        pofile.removeduplicates("msgid_comment")
-#        assert len(pofile.units) == 2
-#        assert po.unquotefrompo(pofile.units[0].msgidcomments) == "_: source0\n"
-#        assert po.unquotefrompo(pofile.units[1].msgidcomments) == "_: source1\n"
-#        # Now lets check for formating
-#        for i in (0, 1):
-#          expected = '''#: source%d\nmsgid ""\n"_: source%d\\n"\n"Same"\nmsgstr ""\n''' % (i, i)
-#          assert pofile.units[i].__str__() == expected
+    @mark.xfail(reason="Were disabled during port of Pypo to cPO - they might work")
+    def test_merge_blanks(self):
+        """checks that merging adds msgid_comments to blanks"""
+        posource = '#: source1\nmsgid ""\nmsgstr ""\n\n#: source2\nmsgid ""\nmsgstr ""\n'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 2
+        pofile.removeduplicates("merge")
+        assert len(pofile.units) == 2
+        print pofile.units[0].msgidcomments
+        print pofile.units[1].msgidcomments
+        assert po.unquotefrompo(pofile.units[0].msgidcomments) == "_: source1\n"
+        assert po.unquotefrompo(pofile.units[1].msgidcomments) == "_: source2\n"
 
-#    def test_keep_blanks(self):
-#        """checks that keeping keeps blanks and doesn't add msgid_comments"""
-#        posource = '#: source1\nmsgid ""\nmsgstr ""\n\n#: source2\nmsgid ""\nmsgstr ""\n'
-#        pofile = self.poparse(posource)
-#        assert len(pofile.units) == 2
-#        pofile.removeduplicates("keep")
-#        assert len(pofile.units) == 2
-#        # check we don't add msgidcomments
-#        assert po.unquotefrompo(pofile.units[0].msgidcomments) == ""
-#        assert po.unquotefrompo(pofile.units[1].msgidcomments) == ""
+    @mark.xfail(reason="Were disabled during port of Pypo to cPO - they might work")
+    def test_msgid_comment(self):
+        """checks that when adding msgid_comments we place them on a newline"""
+        posource = '#: source0\nmsgid "Same"\nmsgstr ""\n\n#: source1\nmsgid "Same"\nmsgstr ""\n'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 2
+        pofile.removeduplicates("msgid_comment")
+        assert len(pofile.units) == 2
+        assert po.unquotefrompo(pofile.units[0].msgidcomments) == "_: source0\n"
+        assert po.unquotefrompo(pofile.units[1].msgidcomments) == "_: source1\n"
+        # Now lets check for formating
+        for i in (0, 1):
+            expected = '''#: source%d\nmsgid ""\n"_: source%d\\n"\n"Same"\nmsgstr ""\n''' % (i, i)
+            assert pofile.units[i].__str__() == expected
+
+    @mark.xfail(reason="Were disabled during port of Pypo to cPO - they might work")
+    def test_keep_blanks(self):
+        """checks that keeping keeps blanks and doesn't add msgid_comments"""
+        posource = '#: source1\nmsgid ""\nmsgstr ""\n\n#: source2\nmsgid ""\nmsgstr ""\n'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 2
+        pofile.removeduplicates("keep")
+        assert len(pofile.units) == 2
+        # check we don't add msgidcomments
+        assert po.unquotefrompo(pofile.units[0].msgidcomments) == ""
+        assert po.unquotefrompo(pofile.units[1].msgidcomments) == ""
 
     def test_output_str_unicode(self):
         """checks that we can str(pofile) which is in unicode"""
@@ -179,4 +187,3 @@ class TestCPOFile(test_po.TestPOFile):
         print "__str__", str(oldfile)
         assert len(oldfile.units) == 1
         assert str(oldfile).find("# old lonesome comment\nmsgid") >= 0
-    
