@@ -1,12 +1,25 @@
 #!/usr/bin/env python
 
+import os
+import sys
 import StringIO
 import subprocess
-import os.path
 
-from translate.storage import test_base
-from translate.storage import mo
 from translate.storage import factory
+from translate.storage import mo
+from translate.storage import test_base
+
+# get directory of this test
+dir = os.path.dirname(os.path.abspath(__file__))
+# get top-level directory (moral equivalent of ../..)
+dir = os.path.dirname(os.path.dirname(dir))
+# load python modules from top-level
+sys.path.insert(0, dir)
+# add top-level to PYTHONPATH for subprocesses
+os.environ["PYTHONPATH"] = os.pathsep.join(sys.path)
+# add {top-level}/translate/tools to PATH for pocompile
+os.environ["PATH"] = os.pathsep.join([os.path.join(dir, "translate", "tools"),
+                                      os.environ["PATH"]])
 
 class TestMOUnit(test_base.TestTranslationUnit):
     UnitClass = mo.mounit
@@ -96,6 +109,7 @@ msgid_plural "trees"
 msgstr[0] ""
 ''']
 
+
 class TestMOFile(test_base.TestTranslationStore):
     StoreClass = mo.mofile
 
@@ -134,7 +148,7 @@ class TestMOFile(test_base.TestTranslationStore):
             out_file.close()
 
             subprocess.call(['msgfmt', PO_FILE, '-o', MO_MSGFMT])
-            subprocess.call(['pocompile', PO_FILE, MO_POCOMPILE])
+            subprocess.call(['pocompile', '--errorlevel=traceback', PO_FILE, MO_POCOMPILE])
 
             store = factory.getobject(StringIO.StringIO(posource))
             if store.isempty() and not os.path.exists(MO_POCOMPILE):
