@@ -1,18 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+from pytest import raises, mark
+
+from translate.misc import wStringIO
+from translate.misc.multistring import multistring
 from translate.storage import po
 from translate.storage import pypo
 from translate.storage import test_base
-from translate.misc import wStringIO
-from translate.misc.multistring import multistring
-from py.test import raises
+
 
 def test_roundtrip_quoting():
-    specials = ['Fish & chips', 'five < six', 'six > five', 
-                'Use &nbsp;', 'Use &amp;nbsp;' 
-                'A "solution"', "skop 'n bal", '"""', "'''", 
-                '\n', '\t', '\r', 
+    specials = ['Fish & chips', 'five < six', 'six > five',
+                'Use &nbsp;', 'Use &amp;nbsp;'
+                'A "solution"', "skop 'n bal", '"""', "'''",
+                '\n', '\t', '\r',
                 '\\n', '\\t', '\\r', '\\"', '\r\n', '\\r\\n', '\\']
     for special in specials:
         quoted_special = pypo.quoteforpo(special)
@@ -20,8 +22,10 @@ def test_roundtrip_quoting():
         print "special: %r\nquoted: %r\nunquoted: %r\n" % (special, quoted_special, unquoted_special)
         assert special == unquoted_special
 
+
 class TestPOUnit(test_base.TestTranslationUnit):
     UnitClass = po.pounit
+
     def test_istranslatable(self):
         """Tests for the correct behaviour of istranslatable()."""
         unit = self.UnitClass("Message")
@@ -42,6 +46,27 @@ class TestPOUnit(test_base.TestTranslationUnit):
         unit.makeobsolete()
         assert not unit.istranslatable()
 
+    def test_locations(self):
+        """Tests that we can add and retrieve error messages for a unit."""
+
+        def locations_helper(location):
+            unit = self.UnitClass()
+            assert len(unit.getlocations()) == 0
+            unit.addlocation(location)
+            assert len(unit.getlocations()) == 1
+            assert unit.getlocations() == [location]
+        locations_helper("key")
+        locations_helper("file.c:100")
+        locations_helper("I am a key")
+        locations_helper(u"unicoḓe key")
+
+    def test_nongettext_location(self):
+        """test that we correctly handle a non-gettext (file:linenumber) location"""
+        u = self.UnitClass(u"")
+        u.addlocation(u"programming/C/programming.xml:44(para)")
+        assert "programming/C/programming.xml:44(para)" in str(u)
+        assert u"programming/C/programming.xml:44(para)" in u.getlocations()
+
     def test_adding_empty_note(self):
         unit = self.UnitClass("bla")
         print str(unit)
@@ -143,8 +168,10 @@ class TestPOUnit(test_base.TestTranslationUnit):
 #         unit.rich_target = [['a', G('42', ['b']), 'c']]
 #         assert unit.rich_target == [['abc']]
 
+
 class TestPOFile(test_base.TestTranslationStore):
     StoreClass = po.pofile
+
     def poparse(self, posource):
         """helper that parses po source without requiring files"""
         dummyfile = wStringIO.StringIO(posource)
@@ -168,6 +195,15 @@ class TestPOFile(test_base.TestTranslationStore):
         print oldunit
         return str(oldunit)
 
+    def poreflow(self, posource):
+        """Helper to parse and reflow all text according to our code."""
+        pofile = self.poparse(posource)
+        for u in pofile.units:
+            # force rewrapping:
+            u.source = u.source
+            u.target = u.target
+        return str(pofile)
+
     def test_context_only(self):
         """Checks that an empty msgid with msgctxt is handled correctly."""
         posource = '''msgctxt "CONTEXT"
@@ -261,13 +297,25 @@ msgstr[1] "Kóeie"
         assert isinstance(unit.source, multistring)
         assert isinstance(unit.source.strings[1], unicode)
 
+    def test_nongettext_location(self):
+        """test that we correctly handle a non-gettext (file:linenumber) location"""
+        posource = '#: programming/C/programming.xml:44(para)\nmsgid "test"\nmsgstr "rest"\n'
+        pofile = self.poparse(posource)
+        u = pofile.units[-1]
 
-    def wtest_kde_plurals(self):
+        locations = u.getlocations()
+        print locations
+        assert len(locations) == 1
+        assert locations[0] == u"programming/C/programming.xml:44(para)"
+        assert isinstance(locations[0], unicode)
+
+    @mark.xfail(reason="Not Implemented")
+    def test_kde_plurals(self):
         """Tests kde-style plurals. (Bug: 191)"""
-        posource = '''msgid "_n Singular\n"
+        posource = '''msgid "_n Singular\\n"
 "Plural"
-msgstr "Een\n"
-"Twee\n"
+msgstr "Een\\n"
+"Twee\\n"
 "Drie"
 '''
         pofile = self.poparse(posource)
@@ -309,7 +357,7 @@ msgstr "POT-Creation-Date: 2006-03-08 17:30+0200\n"
 
         posource = '#, fuzzy, python-format\nmsgid "ball"\nmsgstr "bal"\n'
         expectednonfuzzy = '#, python-format\nmsgid "ball"\nmsgstr "bal"\n'
-        expectedfuzzyagain = '#, python-format, fuzzy\nmsgid "ball"\nmsgstr "bal"\n'
+        expectedfuzzyagain = '#, fuzzy, python-format\nmsgid "ball"\nmsgstr "bal"\n'  # must be sorted
         pofile = self.poparse(posource)
         print pofile
         assert pofile.units[0].isfuzzy()
@@ -318,13 +366,30 @@ msgstr "POT-Creation-Date: 2006-03-08 17:30+0200\n"
         assert str(pofile) == expectednonfuzzy
         pofile.units[0].markfuzzy()
         print str(pofile)
-        assert str(pofile) == expectedfuzzyagain or str(pofile) == posource
+        assert str(pofile) == expectedfuzzyagain
 
-    def xtest_makeobsolete_untranslated(self):
+        # test the same, but with flags in a different order
+        posource = '#, python-format, fuzzy\nmsgid "ball"\nmsgstr "bal"\n'
+        expectednonfuzzy = '#, python-format\nmsgid "ball"\nmsgstr "bal"\n'
+        expectedfuzzyagain = '#, fuzzy, python-format\nmsgid "ball"\nmsgstr "bal"\n'  # must be sorted
+        pofile = self.poparse(posource)
+        print pofile
+        assert pofile.units[0].isfuzzy()
+        pofile.units[0].markfuzzy(False)
+        assert not pofile.units[0].isfuzzy()
+        print str(pofile)
+        assert str(pofile) == expectednonfuzzy
+        pofile.units[0].markfuzzy()
+        print str(pofile)
+        assert str(pofile) == expectedfuzzyagain
+
+    @mark.xfail(reason="Check differing behaviours between pypo and cpo")
+    def test_makeobsolete_untranslated(self):
         """Tests making an untranslated unit obsolete"""
         posource = '#. The automatic one\n#: test.c\nmsgid "test"\nmsgstr ""\n'
         pofile = self.poparse(posource)
         unit = pofile.units[0]
+        print str(pofile)
         assert not unit.isobsolete()
         unit.makeobsolete()
         assert str(unit) == ""
@@ -342,7 +407,7 @@ msgstr "POT-Creation-Date: 2006-03-08 17:30+0200\n"
         posource = 'msgid "thing\nmsgstr "ding"\nmsgid "Second thing"\nmsgstr "Tweede ding"\n'
         pofile = self.poparse(posource)
         assert len(pofile.units) == 2
-        print `pofile.units[0].source`
+        print repr(pofile.units[0].source)
         assert pofile.units[0].source == u"thing"
 
     def test_malformed_obsolete_units(self):
@@ -404,13 +469,45 @@ msgstr "een"
         unit.resurrect()
         assert unit.hasplural()
 
+    def test_obsolete_with_prev_msgid(self):
+        """Tests that obsolete messages work"""
+        # Bug 1429
+        posource = r'''msgid ""
+msgstr ""
+"PO-Revision-Date: 2006-02-09 23:33+0200\n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=UTF-8\n"
+"Content-Transfer-Encoding: 8-bit\n"
+
+msgid "one"
+msgstr "een"
+
+#, fuzzy
+#~| msgid ""
+#~| "You cannot read anything except web pages with\n"
+#~| "this plugin, sorry."
+#~ msgid "You cannot read anything except web pages with this plugin, sorry."
+#~ msgstr ""
+#~ "Mit diesem Modul können leider ausschließlich Webseiten vorgelesen werden."
+'''
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 3
+        unit = pofile.units[2]
+        print str(unit)
+        assert unit.isobsolete()
+        assert unit.isfuzzy()
+        assert not unit.istranslatable()
+
+        print posource
+        print str(pofile)
+        assert str(pofile) == posource
 
     def test_header_escapes(self):
         pofile = self.StoreClass()
         pofile.updateheader(add=True, **{"Report-Msgid-Bugs-To": r"http://qa.openoffice.org/issues/enter_bug.cgi?subcomponent=ui&comment=&short_desc=Localization%20issue%20in%20file%3A%20dbaccess\source\core\resource.oo&component=l10n&form_name=enter_issue"})
         filecontents = str(pofile)
         print filecontents
-        # We need to make sure that the \r didn't get misrepresented as a 
+        # We need to make sure that the \r didn't get misrepresented as a
         # carriage return, but as a slash (escaped) followed by a normal 'r'
         assert r'\source\core\resource' in pofile.header().target
         assert r're\\resource' in filecontents
@@ -449,7 +546,6 @@ msgstr[1] "Koeie"
         print pofile
         assert str(unit) == poexpected
 
-
     def test_makeobsolete_msgctxt(self):
         """Tests making a unit with msgctxt obsolete"""
         posource = '#: test.c\nmsgctxt "Context"\nmsgid "test"\nmsgstr "rest"\n'
@@ -673,6 +769,48 @@ msgstr "b"
         for line in pofile.units[0].getnotes():
             assert isinstance(line, unicode)
 
+    def test_non_ascii_header_comments(self):
+        posource = r'''
+# Copyright bla.
+msgid ""
+msgstr ""
+"PO-Revision-Date: 2006-02-09 23:33+0200\n"
+"MIME-Version: 1.0\n"
+"Last-Translator: Tránslátór\n"
+"Content-Type: text/plain; charset=UTF-8\n"
+"Content-Transfer-Encoding: 8-bit\n"
+
+msgid "a"
+msgstr "b"
+'''
+        pofile = self.poparse(posource)
+        assert u"Tránslátór" in pofile.units[0].target
+        header_dict = pofile.parseheader()
+        assert u"Last-Translator" in header_dict
+        assert header_dict[u"Last-Translator"] == u"Tránslátór"
+
+
+        # let's test the same with latin-1:
+        posource = r'''
+# Copyright bla.
+msgid ""
+msgstr ""
+"PO-Revision-Date: 2006-02-09 23:33+0200\n"
+"MIME-Version: 1.0\n"
+"Last-Translator: Tránslátór\n"
+"Content-Type: text/plain; charset=ISO-8859-1\n"
+"Content-Transfer-Encoding: 8-bit\n"
+
+msgid "a"
+msgstr "b"
+'''.decode('utf-8').encode('ISO-8859-1')
+
+        pofile = self.poparse(posource)
+        assert u"Tránslátór" in pofile.units[0].target
+        header_dict = pofile.parseheader()
+        assert u"Last-Translator" in header_dict
+        assert header_dict[u"Last-Translator"] == u"Tránslátór"
+
     def test_final_slash(self):
         """Test that \ as last character is correcly interpreted (bug 960)."""
         posource = r'''
@@ -704,9 +842,137 @@ msgstr "start thing dingis fish"
 "
 '''
         pofile1 = self.poparse(posource)
-        print `pofile1.units[1].target`
+        print repr(pofile1.units[1].target)
         assert pofile1.units[1].target == u"start thing dingis fish"
         pofile2 = self.poparse(str(pofile1))
         assert pofile2.units[1].target == u"start thing dingis fish"
         print str(pofile2)
         assert str(pofile1) == str(pofile2)
+
+    def test_encoding_change(self):
+        posource = ur'''
+msgid ""
+msgstr ""
+"PO-Revision-Date: 2006-02-09 23:33+0200\n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=ISO-8859-1\n"
+"Content-Transfer-Encoding: 8-bit\n"
+
+msgid "a"
+msgstr "d"
+'''.encode('iso-8859-1')
+        pofile = self.poparse(posource)
+        unit = pofile.units[1]
+        unit.target = u"ḓ"
+        contents = str(pofile)
+        assert 'msgstr "\341\270\223"' in contents
+        assert 'charset=UTF-8' in contents
+
+    def test_istranslated(self):
+        """checks that istranslated works ok."""
+        posource = ur'''
+msgid ""
+msgstr ""
+"PO-Revision-Date: 2006-02-09 23:33+0200\n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=ISO-8859-1\n"
+"Content-Transfer-Encoding: 8-bit\n"
+
+msgid "a"
+msgid_plural "aa"
+msgstr[0] ""
+'''
+        pofile = self.poparse(posource)
+        unit = pofile.units[1]
+        print str(unit)
+        assert "msgid_plural" in str(unit)
+        assert not unit.istranslated()
+        assert unit.get_state_n() == 0
+
+    def test_wrapping(self):
+        """This tests that we wrap like gettext."""
+        posource = r'''#: file.h:1
+msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345"
+msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345"
+'''
+        # should be unchanged:
+        assert self.poreflow(posource) == posource
+
+        posource = r'''#: 2
+msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
+msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
+'''
+        posource_wanted = r'''#: 2
+msgid ""
+"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
+msgstr ""
+"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
+'''
+        assert self.poreflow(posource) == posource_wanted
+
+        posource = r'''#: 7
+msgid "bla\t12345 12345 12345 12345 12345 12 12345 12345 12345 12345 12345 12345 123"
+msgstr "bla\t12345 12345 12345 12345 12345 15 12345 12345 12345 12345 12345 12345 123"
+'''
+        posource_wanted =r'''#: 7
+msgid ""
+"bla\t12345 12345 12345 12345 12345 12 12345 12345 12345 12345 12345 12345 123"
+msgstr ""
+"bla\t12345 12345 12345 12345 12345 15 12345 12345 12345 12345 12345 12345 123"
+'''
+        assert self.poreflow(posource) == posource_wanted
+
+        posource = r'''#: 7
+msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
+msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
+'''
+        posource_wanted = r'''#: 7
+msgid ""
+"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 "
+"1"
+msgstr ""
+"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 "
+"1"
+'''
+        assert self.poreflow(posource) == posource_wanted
+
+        posource = r'''#: 8
+msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1234\n1234"
+msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1234\n1234"
+
+#: 9
+msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345\n12345"
+msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345\n12345"
+'''
+        posource_wanted = r'''#: 8
+msgid ""
+"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1234\n"
+"1234"
+msgstr ""
+"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1234\n"
+"1234"
+
+#: 9
+msgid ""
+"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 "
+"12345\n"
+"12345"
+msgstr ""
+"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 "
+"12345\n"
+"12345"
+'''
+        assert self.poreflow(posource) == posource_wanted
+
+        posource =r'''#: 10
+msgid "\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"
+msgstr "\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"
+'''
+        posource_wanted = r'''#: 10
+msgid ""
+"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"
+"\\"
+msgstr ""
+"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"
+"\\"
+'''
