@@ -1,11 +1,66 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from translate.storage import test_po
-from translate.storage import pypo
-from translate.misc.multistring import multistring
+from pytest import raises
+
 from translate.misc import wStringIO
-from py.test import raises
+from translate.misc.multistring import multistring
+from translate.storage import pypo
+from translate.storage import test_po
+
+
+class TestHelpers():
+    def test_unescape(self):
+        assert pypo.unescape(r"koei") == "koei"
+        assert pypo.unescape(r"koei\n") == "koei\n"
+        assert pypo.unescape(r"koei\\") == "koei\\"
+        assert pypo.unescape(r"koei\"") == "koei\""
+        assert pypo.unescape(r"koei\r") == "koei\r"
+
+        assert pypo.unescape(r"\nkoei\n") == "\nkoei\n"
+        assert pypo.unescape(r"\\koei\\") == "\\koei\\"
+        assert pypo.unescape(r"\"koei\"") == "\"koei\""
+        assert pypo.unescape(r"\rkoei\r") == "\rkoei\r"
+
+        assert pypo.unescape(r"\n\nkoei\n") == "\n\nkoei\n"
+        assert pypo.unescape(r"\\\nkoei\\\n") == "\\\nkoei\\\n"
+        assert pypo.unescape(r"\"\\koei\"\\") == "\"\\koei\"\\"
+        assert pypo.unescape(r"\\\rkoei\r\\") == "\\\rkoei\r\\"
+
+    def test_quoteforpo(self):
+        """Special escaping routine to manage newlines and linewrap in PO"""
+        # Simple case
+        assert pypo.quoteforpo("Some test") == ['"Some test"']
+        # Newline handling
+        assert pypo.quoteforpo("One\nTwo\n") == ['""', '"One\\n"', '"Two\\n"']
+        # First line wrapping
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A ver") == \
+                             ['"A very long sentence. A very long sentence. A very long sentence. A ver"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very") == \
+                              ['""',
+                               '"A very long sentence. A very long sentence. A very long sentence. A very"']
+        # Long line with a newline
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very lon\n") == \
+                             ['""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"lon\\n"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very 123\n") == \
+                             ['""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"123\\n"']
+        # Special 77 char failure.
+        assert pypo.quoteforpo("Ukuba uyayiqonda into eyenzekayo, \nungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. \n<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu \nobhucabhuca ukudibanisa kwakho.</b>") == \
+                             ['""',
+                              '"Ukuba uyayiqonda into eyenzekayo, \\n"',
+                              '"ungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. "',
+                              '"\\n"',
+                              '"<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu "',
+                              '"\\n"',
+                              '"obhucabhuca ukudibanisa kwakho.</b>"']
+
+    def test_quoteforpo_escaped_quotes(self):
+        """Ensure that we don't break \" in two when wrapping
+
+	See :bug:`3140`
+	"""
+        assert pypo.quoteforpo('''You can get a copy of your Recovery Key by going to &syncBrand.shortName.label; Options on your other device, and selecting  "My Recovery Key" under "Manage Account".''') == [u'""', u'"You can get a copy of your Recovery Key by going to "', u'"&syncBrand.shortName.label; Options on your other device, and selecting  \\""', u'"My Recovery Key\\" under \\"Manage Account\\"."']
+
 
 class TestPYPOUnit(test_po.TestPOUnit):
     UnitClass = pypo.pounit
@@ -23,7 +78,7 @@ class TestPYPOUnit(test_po.TestPOUnit):
         assert unit.target.strings == ["Koei", "Koeie"]
         assert unit.target == "Koei"
 
-        unit.target = {0:"Koei", 3:"Koeie"}
+        unit.target = {0: "Koei", 3: "Koeie"}
         assert isinstance(unit.target, multistring)
         assert unit.target.strings == ["Koei", "Koeie"]
         assert unit.target == "Koei"
@@ -141,8 +196,10 @@ msgstr ""
         print str(unit)
         assert str(unit) == expected
 
+
 class TestPYPOFile(test_po.TestPOFile):
     StoreClass = pypo.pofile
+
     def test_combine_msgidcomments(self):
         """checks that we don't get duplicate msgid comments"""
         posource = 'msgid "test me"\nmsgstr ""'
