@@ -1,17 +1,17 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+import pytest
+
 from translate.storage import test_base
 from translate.storage import qm
 
-from py import test
 
-
-class TestOtUnit(test_base.TestTranslationUnit):
+class TestQtUnit(test_base.TestTranslationUnit):
     UnitClass = qm.qmunit
 
 
-class TestOtFile(test_base.TestTranslationStore):
+class TestQtFile(test_base.TestTranslationStore):
     StoreClass = qm.qmfile
 
     def test_parse(self):
@@ -21,10 +21,20 @@ class TestOtFile(test_base.TestTranslationStore):
 
     def test_save(self):
         # QM does not implement saving
-        assert test.raises(Exception, self.StoreClass.savefile,
+        assert pytest.raises(Exception, self.StoreClass.savefile,
                            self.StoreClass())
 
     def test_files(self):
         # QM does not implement saving
-        assert test.raises(Exception, self.StoreClass.savefile,
+        assert pytest.raises(Exception, self.StoreClass.savefile,
+                           self.StoreClass())
+
+    def test_nonascii(self):
+        # QM does not implement serialising
+        assert pytest.raises(Exception, self.StoreClass.__str__,
+                           self.StoreClass())
+
+    def test_add(self):
+        # QM does not implement serialising
+        assert pytest.raises(Exception, self.StoreClass.__str__,
                            self.StoreClass())
