@@ -4,6 +4,7 @@ from translate.storage import tmx
 from translate.storage import test_base
 from translate.misc import wStringIO
 
+
 class TestTMXUnit(test_base.TestTranslationUnit):
     UnitClass = tmx.tmxunit
 
@@ -27,6 +28,7 @@ class TestTMXUnitFromParsedString(TestTMXUnit):
         store = tmx.tmxfile.parsestring(self.tmxsource)
         self.unit = store.units[0]
 
+
 class TestTMXfile(test_base.TestTranslationStore):
     StoreClass = tmx.tmxfile
 
@@ -50,6 +52,15 @@ class TestTMXfile(test_base.TestTranslationStore):
         newfile = self.tmxparse(str(tmxfile))
         print str(tmxfile)
         assert newfile.translate("A string of characters") == "'n String karakters"
+        
+    def test_withcomment(self):
+        """tests that addtranslation() stores string's comments correctly"""
+        tmxfile = tmx.tmxfile()
+        tmxfile.addtranslation("A string of chars",
+                               "en", "'n String karakters", "af", "comment")
+        newfile = self.tmxparse(str(tmxfile))
+        print str(tmxfile)
+        assert newfile.findunit("A string of chars").getnotes() == "comment"
 
     def test_withnewlines(self):
         """test addtranslation() with newlines"""
@@ -73,4 +84,3 @@ class TestTMXfile(test_base.TestTranslationStore):
         assert tmxfile.translate('Five < ten') == 'Vyf < tien'
         assert xmltext.index('Five &lt; ten')
         assert xmltext.find('Five < ten') == -1
-
