@@ -1,7 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2009 Zuza Software Foundation
+# Copyright 2009, 2013 Zuza Software Foundation
+# Copyright 2013 F Wolff
 #
 # This file is part of translate.
 #
@@ -16,28 +17,29 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """Module to provide a translation memory database."""
-import math
-import time
+
 import logging
+import math
 import re
 import threading
-
+import time
 try:
     from sqlite3 import dbapi2
 except ImportError:
     from pysqlite2 import dbapi2
 
-from translate.search.lshtein import LevenshteinComparer
 from translate.lang import data
+from translate.search.lshtein import LevenshteinComparer
 
 
 STRIP_REGEXP = re.compile("\W", re.UNICODE)
 
+
 class LanguageError(Exception):
+
     def __init__(self, value):
         self.value = value
 
@@ -47,12 +49,16 @@ class LanguageError(Exception):
 
 class TMDB(object):
     _tm_dbs = {}
-    def __init__(self, db_file, max_candidates=3, min_similarity=75, max_length=1000):
+
+    def __init__(self, db_file, max_candidates=3, min_similarity=75,
+                 max_length=1000):
 
         self.max_candidates = max_candidates
         self.min_similarity = min_similarity
         self.max_length = max_length
 
+        if not isinstance(db_file, unicode):
+            db_file = unicode(db_file)  # don't know which encoding
         self.db_file = db_file
         # share connections to same database file between different instances
         if db_file not in self._tm_dbs:
@@ -71,15 +77,14 @@ class TMDB(object):
     def _get_connection(self, index):
         current_thread = threading.currentThread()
         if current_thread not in self._tm_db:
-            connection = dbapi2.connect(self.db_file)
+            connection = dbapi2.connect(self.db_file.encode('utf-8'))
             cursor = connection.cursor()
             self._tm_db[current_thread] = (connection, cursor)
         return self._tm_db[current_thread][index]
-    
+
     connection = property(lambda self: self._get_connection(0))
     cursor = property(lambda self: self._get_connection(1))
 
-    
     def init_database(self):
         """creates database tables and indices"""
 
@@ -120,7 +125,8 @@ CREATE UNIQUE INDEX IF NOT EXISTS targets_uniq_idx ON targets (sid, text, lang);
     def init_fulltext(self):
         """detects if fts3 fulltext indexing module exists, initializes fulltext table if it does"""
 
-        #HACKISH: no better way to detect fts3 support except trying to construct a dummy table?!
+        #HACKISH: no better way to detect fts3 support except trying to
+        # construct a dummy table?!
         try:
             script = """
 DROP TABLE IF EXISTS test_for_fts3;
@@ -129,8 +135,8 @@ DROP TABLE test_for_fts3;
 """
             self.cursor.executescript(script)
             logging.debug("fts3 supported")
-            # for some reason CREATE VIRTUAL TABLE doesn't support IF NOT EXISTS syntax
-            # check if fulltext index table exists manually
+            # for some reason CREATE VIRTUAL TABLE doesn't support IF NOT
+            # EXISTS syntax check if fulltext index table exists manually
             self.cursor.execute("SELECT name FROM sqlite_master WHERE name = 'fulltext'")
             if not self.cursor.fetchone():
                 # create fulltext index table, and index all strings in sources
@@ -142,7 +148,7 @@ CREATE VIRTUAL TABLE fulltext USING fts3(text);
                 logging.debug("created fulltext table")
             else:
                 logging.debug("fulltext table already exists")
-                
+
             # create triggers that would sync sources table with fulltext index
             script = """
 INSERT INTO fulltext (rowid, text) SELECT sid, text FROM sources WHERE sid NOT IN (SELECT rowid FROM fulltext);
@@ -201,10 +207,10 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
         if not target_lang:
             raise LanguageError("undefined target language")
 
-        unitdict = {"source" : unit.source,
-                    "target" : unit.target,
-                    "context": unit.getcontext()
-                    }
+        unitdict = {"source": unit.source,
+                    "target": unit.target,
+                    "context": unit.getcontext(),
+                   }
         self.add_dict(unitdict, source_lang, target_lang, commit)
 
     def add_dict(self, unit, source_lang, target_lang, commit=True):
@@ -267,7 +273,7 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
         if commit:
             self.connection.commit()
         return count
-    
+
     def translate_unit(self, unit_source, source_langs, target_langs):
         """return TM suggestions for unit_source"""
         if isinstance(unit_source, str):
@@ -282,9 +288,10 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
             target_langs = ','.join(target_langs)
         else:
             target_langs = data.normalize_code(target_langs)
-        
+
         minlen = min_levenshtein_length(len(unit_source), self.min_similarity)
-        maxlen = max_levenshtein_length(len(unit_source), self.min_similarity, self.max_length)
+        maxlen = max_levenshtein_length(len(unit_source), self.min_similarity,
+                                        self.max_length)
 
         # split source into words, remove punctuation and special
         # chars, keep words that are at least 3 chars long
@@ -297,23 +304,27 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
                        WHERE s.lang IN (?) AND t.lang IN (?) AND s.length BETWEEN ? AND ?
                        AND fulltext MATCH ?"""
             search_str = " OR ".join(unit_words)
-            self.cursor.execute(query, (source_langs, target_langs, minlen, maxlen, search_str))
+            self.cursor.execute(query, (source_langs, target_langs, minlen,
+                                maxlen, search_str))
         else:
             logging.debug("nonfulltext matching")
             query = """SELECT s.text, t.text, s.context, s.lang, t.lang FROM sources s JOIN targets t ON s.sid = t.sid
-            WHERE s.lang IN (?) AND t.lang IN (?) 
+            WHERE s.lang IN (?) AND t.lang IN (?)
             AND s.length >= ? AND s.length <= ?"""
-            self.cursor.execute(query, (source_langs, target_langs, minlen, maxlen))
+            self.cursor.execute(query, (source_langs, target_langs, minlen,
+                                        maxlen))
 
         results = []
         for row in self.cursor:
-            result = {}
-            result['source'] = row[0]
-            result['target'] = row[1]
-            result['context'] = row[2]
-            result['quality'] = self.comparer.similarity(unit_source, result['source'], self.min_similarity)
-            if result['quality'] >= self.min_similarity:
-                results.append(result)
+            quality = self.comparer.similarity(unit_source, row[0],
+                                               self.min_similarity)
+            if quality >= self.min_similarity:
+                results.append({
+                    'source': row[0],
+                    'target': row[1],
+                    'context': row[2],
+                    'quality': quality,
+                })
         results.sort(key=lambda match: match['quality'], reverse=True)
         results = results[:self.max_candidates]
         logging.debug("results: %s", unicode(results))
@@ -321,7 +332,8 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
 
 
 def min_levenshtein_length(length, min_similarity):
-    return math.ceil(max(length * (min_similarity/100.0), 2))
+    return math.ceil(max(length * (min_similarity / 100.0), 2))
+
 
 def max_levenshtein_length(length, min_similarity, max_length):
-    return math.floor(min(length / (min_similarity/100.0), max_length))
+    return math.floor(min(length / (min_similarity / 100.0), max_length))
