@@ -20,10 +20,11 @@
 
 """module for parsing TMX translation memeory files"""
 
-from translate.storage import lisa
 from lxml import etree
 
 from translate import __version__
+from translate.storage import lisa
+
 
 class tmxunit(lisa.LISAunit):
     """A single unit in the TMX file."""
@@ -62,17 +63,20 @@ class tmxunit(lisa.LISAunit):
         note = etree.SubElement(self.xmlelement, self.namespaced("note"))
         note.text = text.strip()
 
-    def getnotelist(self, origin=None):
-        """Private method that returns the text from notes.
+    def _getnotelist(self, origin=None):
+        """Returns the text from notes.
 
-        The origin parameter is ignored.."""
+        :param origin: Ignored
+        :return: The text from notes
+        :rtype: List
+        """
         note_nodes = self.xmlelement.iterdescendants(self.namespaced("note"))
         note_list = [lisa.getText(note) for note in note_nodes]
 
         return note_list
 
     def getnotes(self, origin=None):
-        return '\n'.join(self.getnotelist(origin=origin))
+        return '\n'.join(self._getnotelist(origin=origin))
 
     def removenotes(self):
         """Remove all the translator notes."""
@@ -83,13 +87,15 @@ class tmxunit(lisa.LISAunit):
     def adderror(self, errorname, errortext):
         """Adds an error message to this unit."""
         #TODO: consider factoring out: some duplication between XLIFF and TMX
-        text = errorname + ': ' + errortext
+        text = errorname
+        if errortext:
+            text += ': ' + errortext
         self.addnote(text, origin="pofilter")
 
     def geterrors(self):
         """Get all error messages."""
         #TODO: consider factoring out: some duplication between XLIFF and TMX
-        notelist = self.getnotelist(origin="pofilter")
+        notelist = self._getnotelist(origin="pofilter")
         errordict = {}
         for note in notelist:
             errorname, errortext = note.split(': ')
@@ -111,7 +117,7 @@ class tmxfile(lisa.LISAfile):
     """Class representing a TMX file store."""
     UnitClass = tmxunit
     Name = _("TMX Translation Memory")
-    Mimetypes  = ["application/x-tmx"]
+    Mimetypes = ["application/x-tmx"]
     Extensions = ["tmx"]
     rootNode = "tmx"
     bodyNode = "body"
@@ -135,10 +141,14 @@ class tmxfile(lisa.LISAfile):
         #headernode.set("creationdate", "YYYYMMDDTHHMMSSZ"
         #headernode.set("creationid", "CodeSyntax"
 
-    def addtranslation(self, source, srclang, translation, translang):
+    def addtranslation(self, source, srclang, translation, translang,
+                       comment=None):
         """addtranslation method for testing old unit tests"""
         unit = self.addsourceunit(source)
         unit.target = translation
+        if comment is not None and len(comment) > 0:
+            unit.addnote(comment)
+
         tuvs = unit.xmlelement.iterdescendants(self.namespaced('tuv'))
         lisa.setXMLlang(tuvs.next(), srclang)
         lisa.setXMLlang(tuvs.next(), translang)
