@@ -2,40 +2,41 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2007 Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 
 """Module for parsing Qt .ts files for translation.
 
 Currently this module supports the old format of .ts files. Some applictaions
 use the newer .ts format which are documented here:
-U{TS file format 4.3<http://doc.trolltech.com/4.3/linguist-ts-file-format.html>}, 
-U{Example<http://svn.ez.no/svn/ezcomponents/trunk/Translation/docs/linguist-format.txt>}
+`TS file format 4.3 <http://doc.qt.digia.com/4.3/linguist-ts-file-format.html>`_,
+`Example <http://svn.ez.no/svn/ezcomponents/trunk/Translation/docs/linguist-format.txt>`_
 
-U{Specification of the valid variable entries <http://doc.trolltech.com/4.3/qstring.html#arg>}, 
-U{2 <http://doc.trolltech.com/4.3/qstring.html#arg-2>}
+`Specification of the valid variable entries <http://qt-project.org/doc/qt-5.0/qtcore/qstring.html#arg>`_,
+`2 <http://qt-project.org/doc/qt-5.0/qtcore/qstring.html#arg-2>`_
 """
 
 from translate.misc import ourdom
 
+
 class QtTsParser:
     contextancestors = dict.fromkeys(["TS"])
     messageancestors = dict.fromkeys(["TS", "context"])
+
     def __init__(self, inputfile=None):
         """make a new QtTsParser, reading from the given inputfile if required"""
         self.filename = getattr(inputfile, "filename", None)
@@ -148,4 +149,3 @@ class QtTsParser:
         """clean up the document if required"""
         if hasattr(self, "document"):
             self.document.unlink()
-
