@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2008-2009 Zuza Software Foundation
+# Copyright 2008-2011 Zuza Software Foundation
 #
 # This file is part of the Translate Toolkit.
 #
@@ -24,20 +24,22 @@ This will eventually replace the older ts.py which only supports the older
 format. While converters haven't been updated to use this module, we retain
 both.
 
-U{TS file format 4.3<http://doc.trolltech.com/4.3/linguist-ts-file-format.html>},
-U{4.5<http://doc.trolltech.com/4.5/linguist-ts-file-format.html>},
-U{Example<http://svn.ez.no/svn/ezcomponents/trunk/Translation/docs/linguist-format.txt>},
-U{Plurals forms<http://www.koders.com/cpp/fidE7B7E83C54B9036EB7FA0F27BC56BCCFC4B9DF34.aspx#L200>}
+`TS file format 4.3 <http://doc.qt.digia.com/4.3/linguist-ts-file-format.html>`_,
+`4.8 <http://qt-project.org/doc/qt-4.8/linguist-ts-file-format.html>`_,
+`5.0 <http://qt-project.org/doc/qt-5.0/qtlinguist/linguist-ts-file-format.html>`_.
+`Example <http://svn.ez.no/svn/ezcomponents/trunk/Translation/docs/linguist-format.txt>`_.
 
-U{Specification of the valid variable entries <http://doc.trolltech.com/4.3/qstring.html#arg>},
-U{2 <http://doc.trolltech.com/4.3/qstring.html#arg-2>}
+`Specification of the valid variable entries <http://qt-project.org/doc/qt-5.0/qtcore/qstring.html#arg>`_,
+`2 <http://qt-project.org/doc/qt-5.0/qtcore/qstring.html#arg-2>`_
 """
 
+from lxml import etree
+
+from translate.lang import data
+from translate.misc.multistring import multistring
 from translate.storage import base, lisa
 from translate.storage.placeables import general
-from translate.misc.multistring import multistring
-from translate.lang import data
-from lxml import etree
+from translate.storage.workflow import StateEnum as state
 
 # TODO: handle translation types
 
@@ -62,7 +64,7 @@ NPLURALS = {
 
 
 class tsunit(lisa.LISAunit):
-    """A single term in the xliff file."""
+    """A single term in the TS file."""
 
     rootNode = "message"
     languageNode = "source"
@@ -70,6 +72,28 @@ class tsunit(lisa.LISAunit):
     namespace = ''
     rich_parsers = general.parsers
 
+    S_OBSOLETE = state.OBSOLETE
+    S_UNTRANSLATED = state.EMPTY
+    S_FUZZY = state.NEEDS_WORK
+    S_TRANSLATED = state.UNREVIEWED
+
+    statemap = {
+                "obsolete": S_OBSOLETE,
+                "unfinished": S_FUZZY,
+                "": S_TRANSLATED,
+                None: S_TRANSLATED,
+    }
+    """This maps the unit "type" attribute to state."""
+
+    STATE = {
+        S_OBSOLETE: (state.OBSOLETE, state.EMPTY),
+        S_UNTRANSLATED: (state.EMPTY, state.NEEDS_WORK),
+        S_FUZZY: (state.NEEDS_WORK, state.UNREVIEWED),
+        S_TRANSLATED: (state.UNREVIEWED, state.MAX),
+    }
+
+    statemap_r = dict((i[1], i[0]) for i in statemap.iteritems())
+
     def createlanguageNode(self, lang, text, purpose):
         """Returns an xml Element setup with given parameters."""
 
@@ -91,6 +115,7 @@ class tsunit(lisa.LISAunit):
 
     def getlanguageNodes(self):
         """We override this to get source and target nodes."""
+
         def not_none(node):
             return not node is None
         return filter(not_none, [self._getsourcenode(), self._gettargetnode()])
@@ -132,8 +157,11 @@ class tsunit(lisa.LISAunit):
             for string in strings:
                 numerus = etree.SubElement(targetnode, self.namespaced("numerusform"))
                 numerus.text = data.forceunicode(string) or u""
+                # manual, nasty pretty printing. See bug 1420.
+                numerus.tail = u"\n        "
         else:
             targetnode.text = data.forceunicode(text) or u""
+            targetnode.tail = u"\n    "
 
     def gettarget(self):
         targetnode = self._gettargetnode()
@@ -152,7 +180,7 @@ class tsunit(lisa.LISAunit):
         return self.xmlelement.get("numerus") == "yes"
 
     def addnote(self, text, origin=None, position="append"):
-        """Add a note specifically in a "comment" tag"""
+        """Add a note specifically in the appropriate *comment* tag"""
         if isinstance(text, str):
             text = text.decode("utf-8")
         current_notes = self.getnotes(origin)
@@ -170,24 +198,18 @@ class tsunit(lisa.LISAunit):
         #TODO: consider only responding when origin has certain values
         comments = []
         if origin in ["programmer", "developer", "source code", None]:
-            notenode = self.xmlelement.find(self.namespaced("comment"))
-            if notenode is not None:
-                comments.append(notenode.text)
             notenode = self.xmlelement.find(self.namespaced("extracomment"))
-            if notenode is not None:
+            if notenode is not None and notenode.text is not None:
                 comments.append(notenode.text)
         if origin in ["translator", None]:
             notenode = self.xmlelement.find(self.namespaced("translatorcomment"))
-            if notenode is not None:
+            if notenode is not None and notenode.text is not None:
                 comments.append(notenode.text)
         return '\n'.join(comments)
 
     def removenotes(self, origin=None):
         """Remove all the translator notes."""
         if origin in ["programmer", "developer", "source code", None]:
-            note = self.xmlelement.find(self.namespaced("comment"))
-            if not note is None:
-                self.xmlelement.remove(note)
             note = self.xmlelement.find(self.namespaced("extracomment"))
             if not note is None:
                 self.xmlelement.remove(note)
@@ -217,7 +239,7 @@ class tsunit(lisa.LISAunit):
         return self._gettype() == "unfinished"
 
     def isfuzzy(self):
-        return self._gettype() == "unfinished"
+        return self._gettype() == "unfinished" and bool(self.target)
 
     def markfuzzy(self, value=True):
         if value:
@@ -226,6 +248,8 @@ class tsunit(lisa.LISAunit):
             self._settype(None)
 
     def getid(self):
+        if self.source is None:
+            return None
         context_name = self.getcontext()
         #XXX: context_name is not supposed to be able to be None (the <name>
         # tag is compulsary in the <context> tag)
@@ -242,7 +266,7 @@ class tsunit(lisa.LISAunit):
         # like we do with PO.
         return bool(self.getid()) and not self.isobsolete()
 
-    def getcontext(self):
+    def getcontextname(self):
         parent = self.xmlelement.getparent()
         if parent is None:
             return None
@@ -251,6 +275,14 @@ class tsunit(lisa.LISAunit):
             return None
         return context.text
 
+    def getcontext(self):
+        contexts = [self.getcontextname()]
+        commentnode = self.xmlelement.find(self.namespaced("comment"))
+        if commentnode is not None and commentnode.text is not None:
+            contexts.append(commentnode.text)
+        contexts = filter(None, contexts)
+        return '\n'.join(contexts)
+
     def addlocation(self, location):
         if isinstance(location, str):
             location = location.decode("utf-8")
@@ -270,8 +302,11 @@ class tsunit(lisa.LISAunit):
         for location_tag in location_tags:
             location = location_tag.get("filename")
             line = location_tag.get("line")
-            if line is not None:
-                location += ':' + line
+            if line:
+                if location:
+                    location += ':' + line
+                else:
+                    location = line
             locations.append(location)
         return locations
 
@@ -280,13 +315,36 @@ class tsunit(lisa.LISAunit):
         #TODO: check if this is necessary:
         if otherunit.isfuzzy():
             self.markfuzzy()
+        else:
+            self.markfuzzy(False)
 
     def isobsolete(self):
         return self._gettype() == "obsolete"
 
+    def get_state_n(self):
+        type = self._gettype()
+        if type == "unfinished":
+            # We want to distinguish between fuzzy and untranslated, which the
+            # format doesn't really do
+            if self.target:
+                return self.S_FUZZY
+            else:
+                return self.S_UNTRANSLATED
+        return self.statemap[type]
+
+    def set_state_n(self, value):
+        if value not in self.statemap_r:
+            value = self.get_state_id(value)
+
+        if value == self.S_UNTRANSLATED:
+            # No real way of representing that in the format, so we just
+            # handle it the same as unfinished
+            value = self.S_FUZZY
+        self._settype(self.statemap_r[value])
+
 
 class tsfile(lisa.LISAfile):
-    """Class representing a XLIFF file store."""
+    """Class representing a TS file store."""
     UnitClass = tsunit
     Name = _("Qt Linguist Translation File")
     Mimetypes = ["application/x-linguist"]
@@ -309,7 +367,7 @@ class tsfile(lisa.LISAfile):
         self.namespace = self.document.getroot().nsmap.get(None, None)
         self.header = self.document.getroot()
         if self._contextname:
-            self.body = self.getcontextnode(self._contextname)
+            self.body = self._getcontextnode(self._contextname)
         else:
             self.body = self.document.getroot()
 
@@ -323,8 +381,8 @@ class tsfile(lisa.LISAfile):
         altering the source language in .ts files, it should be set correctly
         by the extraction tools.
 
-        @return: ISO code e.g. af, fr, pt_BR
-        @rtype: String
+        :return: ISO code e.g. af, fr, pt_BR
+        :rtype: String
         """
         lang = data.normalize_code(self.header.get('sourcelanguage', "en"))
         if lang == 'en-us':
@@ -334,16 +392,16 @@ class tsfile(lisa.LISAfile):
     def gettargetlanguage(self):
         """Get the target language for this .ts file.
 
-        @return: ISO code e.g. af, fr, pt_BR
-        @rtype: String
+        :return: ISO code e.g. af, fr, pt_BR
+        :rtype: String
         """
         return data.normalize_code(self.header.get('language'))
 
     def settargetlanguage(self, targetlanguage):
-        """Set the target language for this .ts file to L{targetlanguage}.
+        """Set the target language for this .ts file to *targetlanguage*.
 
-        @param targetlanguage: ISO code e.g. af, fr, pt_BR
-        @type targetlanguage: String
+        :param targetlanguage: ISO code e.g. af, fr, pt_BR
+        :type targetlanguage: String
         """
         if targetlanguage:
             self.header.set('language', targetlanguage)
@@ -382,7 +440,7 @@ class tsfile(lisa.LISAfile):
         If the contextname is specified, switch to that context (creating it
         if allowed by createifmissing)."""
         if contextname is None:
-            contextname = unit.getcontext()
+            contextname = unit.getcontextname()
 
         if self._contextname != contextname:
             if not self._switchcontext(contextname, createifmissing):
@@ -408,7 +466,7 @@ class tsfile(lisa.LISAfile):
 
     def nplural(self):
         lang = self.header.get("language")
-        if NPLURALS.has_key(lang):
+        if lang in NPLURALS:
             return NPLURALS[lang]
         else:
             return 1
