@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2005-2010 Zuza Software Foundation
+# Copyright 2005-2011 Zuza Software Foundation
 #
 # This file is part of the Translate Toolkit.
 #
@@ -26,14 +26,21 @@ The official recommendation is to use the extention .xlf for XLIFF files.
 from lxml import etree
 
 from translate.misc.multistring import multistring
-from translate.misc.xml_helpers import *
 from translate.storage import base, lisa
 from translate.storage.lisa import getXMLspace
 from translate.storage.placeables.lisa import xml_to_strelem, strelem_to_xml
+from translate.storage.workflow import StateEnum as state
 
 # TODO: handle translation types
 
 ID_SEPARATOR = u"\04"
+# ID_SEPARATOR is commonly used through toolkit to generate compound
+# unit ids (for instance to concatenate msgctxt and msgid in po), but
+# \04 is an illegal char in XML 1.0, ID_SEPARATOR_SAFE will be used
+# instead when converting between xliff and other toolkit supported
+# formats
+ID_SEPARATOR_SAFE = u"__%04__"
+
 
 class xliffunit(lisa.LISAunit):
     """A single term in the xliff file."""
@@ -47,6 +54,35 @@ class xliffunit(lisa.LISAunit):
 
     #TODO: id and all the trans-unit level stuff
 
+    S_UNTRANSLATED = state.EMPTY
+    S_NEEDS_TRANSLATION = state.NEEDS_WORK
+    S_NEEDS_REVIEW = state.NEEDS_REVIEW
+    S_TRANSLATED = state.UNREVIEWED
+    S_SIGNED_OFF = state.FINAL
+
+    statemap = {
+                "new": S_UNTRANSLATED + 1,
+                "needs-translation": S_NEEDS_TRANSLATION,
+                "needs-adaptation": S_NEEDS_TRANSLATION + 1,
+                "needs-l10n": S_NEEDS_TRANSLATION + 2,
+                "needs-review-translation": S_NEEDS_REVIEW,
+                "needs-review-adaptation": S_NEEDS_REVIEW + 1,
+                "needs-review-l10n": S_NEEDS_REVIEW + 2,
+                "translated": S_TRANSLATED,
+                "signed-off": S_SIGNED_OFF,
+                "final": S_SIGNED_OFF + 1,
+                }
+
+    statemap_r = dict((i[1], i[0]) for i in statemap.iteritems())
+
+    STATE = {
+        S_UNTRANSLATED: (state.EMPTY, state.NEEDS_WORK),
+        S_NEEDS_TRANSLATION: (state.NEEDS_WORK, state.NEEDS_REVIEW),
+        S_NEEDS_REVIEW: (state.NEEDS_REVIEW, state.UNREVIEWED),
+        S_TRANSLATED: (state.UNREVIEWED, state.FINAL),
+        S_SIGNED_OFF: (state.FINAL, state.MAX),
+    }
+
     def __init__(self, source, empty=False, **kwargs):
         """Override the constructor to set xml:space="preserve"."""
         super(xliffunit, self).__init__(source, empty, **kwargs)
@@ -57,15 +93,15 @@ class xliffunit(lisa.LISAunit):
     def createlanguageNode(self, lang, text, purpose):
         """Returns an xml Element setup with given parameters."""
 
-        #TODO: for now we do source, but we have to test if it is target, perhaps 
-        # with parameter. Alternatively, we can use lang, if supplied, since an xliff 
-        #file has to conform to the bilingual nature promised by the header.
+        # TODO: for now we do source, but we have to test if it is target,
+        # perhaps with parameter. Alternatively, we can use lang, if
+        # supplied, since an xliff file has to conform to the bilingual
+        # nature promised by the header.
         assert purpose
         langset = etree.Element(self.namespaced(purpose))
-        #TODO: check language
-#        lisa.setXMLlang(langset, lang)
+        # TODO: check language
+        #lisa.setXMLlang(langset, lang)
 
-#        self.createPHnodes(langset, text)
         langset.text = text
         return langset
 
@@ -103,10 +139,15 @@ class xliffunit(lisa.LISAunit):
         #logging.debug('rich source: %s' % (repr(rsrc)))
         #from dubulib.debug.misc import print_stack_funcs
         #print_stack_funcs()
-        return [xml_to_strelem(self.source_dom, getXMLspace(self.xmlelement, self._default_xml_space))]
+        return [
+            xml_to_strelem(self.source_dom,
+                           getXMLspace(self.xmlelement,
+                                       self._default_xml_space))
+        ]
     rich_source = property(get_rich_source, set_rich_source)
 
     def set_rich_target(self, value, lang='xx', append=False):
+        self._rich_target = None
         if value is None:
             self.set_target_dom(self.createlanguageNode(lang, u'', "target"))
             return
@@ -122,21 +163,28 @@ class xliffunit(lisa.LISAunit):
         languageNode.text = None
 
         strelem_to_xml(languageNode, value[0])
+        ### currently giving some issues in Virtaal: self._rich_target = value
 
     def get_rich_target(self, lang=None):
         """retrieves the "target" text (second entry), or the entry in the
         specified language, if it exists"""
-        return [xml_to_strelem(self.get_target_dom(lang), getXMLspace(self.xmlelement, self._default_xml_space))]
+        if self._rich_target is None:
+            self._rich_target = [
+                xml_to_strelem(self.get_target_dom(lang),
+                getXMLspace(self.xmlelement, self._default_xml_space))
+            ]
+        return self._rich_target
     rich_target = property(get_rich_target, set_rich_target)
 
-    def addalttrans(self, txt, origin=None, lang=None, sourcetxt=None, matchquality=None):
+    def addalttrans(self, txt, origin=None, lang=None, sourcetxt=None,
+                    matchquality=None):
         """Adds an alt-trans tag and alt-trans components to the unit.
 
-        @type txt: String
-        @param txt: Alternative translation of the source text.
+        :type txt: String
+        :param txt: Alternative translation of the source text.
         """
 
-        #TODO: support adding a source tag ad match quality attribute.  At 
+        #TODO: support adding a source tag ad match quality attribute.  At
         # the source tag is needed to inject fuzzy matches from a TM.
         if isinstance(txt, str):
             txt = txt.decode("utf-8")
@@ -157,25 +205,27 @@ class xliffunit(lisa.LISAunit):
             lisa.setXMLlang(alttrans, lang)
 
     def getalttrans(self, origin=None):
-        """Returns <alt-trans> for the given origin as a list of units. No 
+        """Returns <alt-trans> for the given origin as a list of units. No
         origin means all alternatives."""
         translist = []
         for node in self.xmlelement.iterdescendants(self.namespaced("alt-trans")):
             if self.correctorigin(node, origin):
-                # We build some mini units that keep the xmlelement. This 
+                # We build some mini units that keep the xmlelement. This
                 # makes it easier to delete it if it is passed back to us.
                 newunit = base.TranslationUnit(self.source)
 
                 # the source tag is optional
                 sourcenode = node.iterdescendants(self.namespaced("source"))
                 try:
-                    newunit.source = lisa.getText(sourcenode.next(), getXMLspace(node, self._default_xml_space))
+                    newunit.source = lisa.getText(sourcenode.next(),
+                                                  getXMLspace(node, self._default_xml_space))
                 except StopIteration:
                     pass
 
                 # must have one or more targets
                 targetnode = node.iterdescendants(self.namespaced("target"))
-                newunit.target = lisa.getText(targetnode.next(), getXMLspace(node, self._default_xml_space))
+                newunit.target = lisa.getText(targetnode.next(),
+                                              getXMLspace(node, self._default_xml_space))
                 #TODO: support multiple targets better
                 #TODO: support notes in alt-trans
                 newunit.xmlelement = node
@@ -203,22 +253,28 @@ class xliffunit(lisa.LISAunit):
         if origin:
             note.set("from", origin)
 
-    def getnotelist(self, origin=None):
-        """Private method that returns the text from notes matching 'origin' or all notes."""
-        notenodes = self.xmlelement.iterdescendants(self.namespaced("note"))
+    def _getnotelist(self, origin=None):
+        """Returns the text from notes matching ``origin`` or all notes.
+
+        :param origin: The origin of the note (or note type)
+        :type origin: String
+        :return: The text from notes matching ``origin``
+        :rtype: List
+        """
+        note_nodes = self.xmlelement.iterdescendants(self.namespaced("note"))
         # TODO: consider using xpath to construct initial_list directly
         # or to simply get the correct text from the outset (just remember to
         # check for duplication.
-        initial_list = [lisa.getText(note, getXMLspace(self.xmlelement, self._default_xml_space)) for note in notenodes if self.correctorigin(note, origin)]
+        initial_list = [lisa.getText(note, getXMLspace(self.xmlelement, self._default_xml_space)) for note in note_nodes if self.correctorigin(note, origin)]
 
         # Remove duplicate entries from list:
         dictset = {}
-        notelist = [dictset.setdefault(note, note) for note in initial_list if note not in dictset]
+        note_list = [dictset.setdefault(note, note) for note in initial_list if note not in dictset]
 
-        return notelist
+        return note_list
 
     def getnotes(self, origin=None):
-        return '\n'.join(self.getnotelist(origin=origin))
+        return '\n'.join(self._getnotelist(origin=origin))
 
     def removenotes(self, origin="translator"):
         """Remove all the translator notes."""
@@ -230,19 +286,60 @@ class xliffunit(lisa.LISAunit):
     def adderror(self, errorname, errortext):
         """Adds an error message to this unit."""
         #TODO: consider factoring out: some duplication between XLIFF and TMX
-        text = errorname + ': ' + errortext
+        text = errorname
+        if errortext:
+            text += ': ' + errortext
         self.addnote(text, origin="pofilter")
 
     def geterrors(self):
         """Get all error messages."""
         #TODO: consider factoring out: some duplication between XLIFF and TMX
-        notelist = self.getnotelist(origin="pofilter")
+        notelist = self._getnotelist(origin="pofilter")
         errordict = {}
         for note in notelist:
             errorname, errortext = note.split(': ')
             errordict[errorname] = errortext
         return errordict
 
+    def get_state_n(self):
+        targetnode = self.getlanguageNode(lang=None, index=1)
+        if targetnode is None:
+            if self.isapproved():
+                return self.S_UNREVIEWED
+            else:
+                return self.S_UNTRANSLATED
+
+        xmlstate = targetnode.get("state", None)
+        state_n = self.statemap.get(xmlstate, self.S_UNTRANSLATED)
+
+        if state_n < self.S_NEEDS_TRANSLATION and self.target:
+            state_n = self.S_NEEDS_TRANSLATION
+
+        if self.isapproved() and state_n < self.S_UNREVIEWED:
+            state_n = self.S_UNREVIEWED
+
+        if not self.isapproved() and state_n > self.S_UNREVIEWED:
+            state_n = self.S_UNREVIEWED
+
+        return  state_n
+
+    def set_state_n(self, value):
+        if value not in self.statemap_r:
+            value = self.get_state_id(value)
+
+        targetnode = self.getlanguageNode(lang=None, index=1)
+
+        #FIXME: handle state qualifiers
+        if value == self.S_UNTRANSLATED:
+            if targetnode is not None and "state" in targetnode.attrib:
+                del targetnode.attrib["state"]
+        else:
+            if targetnode is not None:
+                xmlstate = self.statemap_r.get(value)
+                targetnode.set("state", xmlstate)
+
+        self.markapproved(value > self.S_NEEDS_REVIEW)
+
     def isapproved(self):
         """States whether this unit is approved."""
         return self.xmlelement.get("approved") == "yes"
@@ -256,41 +353,37 @@ class xliffunit(lisa.LISAunit):
 
     def isreview(self):
         """States whether this unit needs to be reviewed"""
-        targetnode = self.getlanguageNode(lang=None, index=1)
-        return not targetnode is None and \
-                "needs-review" in targetnode.get("state", "")
+        return self.get_state_id() == self.S_NEEDS_REVIEW
 
     def markreviewneeded(self, needsreview=True, explanation=None):
-        """Marks the unit to indicate whether it needs review. Adds an optional explanation as a note."""
-        targetnode = self.getlanguageNode(lang=None, index=1)
-        if not targetnode is None:
-            if needsreview:
-                targetnode.set("state", "needs-review-translation")
-                if explanation:
-                    self.addnote(explanation, origin="translator")
-            else:
-                del targetnode.attrib["state"]
+        """Marks the unit to indicate whether it needs review.
+
+        Adds an optional explanation as a note."""
+        state_id = self.get_state_id()
+        if needsreview and state_id != self.S_NEEDS_REVIEW:
+            self.set_state_n(self.S_NEEDS_REVIEW)
+            if explanation:
+                self.addnote(explanation, origin="translator")
+        elif not needsreview and state_id < self.S_UNREVIEWED:
+            self.set_state_n(self.S_UNREVIEWED)
 
     def isfuzzy(self):
 #        targetnode = self.getlanguageNode(lang=None, index=1)
 #        return not targetnode is None and \
 #                (targetnode.get("state-qualifier") == "fuzzy-match" or \
 #                targetnode.get("state") == "needs-review-translation")
-        return not self.isapproved()
+        return not self.isapproved() and bool(self.target)
 
     def markfuzzy(self, value=True):
+        state_id = self.get_state_id()
         if value:
             self.markapproved(False)
+            if state_id != self.S_NEEDS_TRANSLATION:
+                self.set_state_n(self.S_NEEDS_TRANSLATION)
         else:
             self.markapproved(True)
-        targetnode = self.getlanguageNode(lang=None, index=1)
-        if not targetnode is None:
-            if value:
-                targetnode.set("state", "needs-review-translation")
-            else:
-                for attribute in ["state", "state-qualifier"]:
-                    if attribute in targetnode.attrib:
-                        del targetnode.attrib[attribute]
+            if state_id < self.S_UNREVIEWED:
+                self.set_state_n(self.S_UNREVIEWED)
 
     def settarget(self, text, lang='xx', append=False):
         """Sets the target string to the given value."""
@@ -312,19 +405,16 @@ class xliffunit(lisa.LISAunit):
         return True
 
     def marktranslated(self):
-        targetnode = self.getlanguageNode(lang=None, index=1)
-        if targetnode is None:
-            return
-        if self.isfuzzy() and "state-qualifier" in targetnode.attrib:
-            #TODO: consider
-            del targetnode.attrib["state-qualifier"]
-        targetnode.set("state", "translated")
+        state_id = self.get_state_id()
+        if state_id < self.S_UNREVIEWED:
+            self.set_state_n(self.S_UNREVIEWED)
 
     def setid(self, id):
-        self.xmlelement.set("id", id)
+        # sanitize id in case ID_SEPERATOR is present
+        self.xmlelement.set("id", id.replace(ID_SEPARATOR, ID_SEPARATOR_SAFE))
 
     def getid(self):
-        uid = ""
+        uid = u""
         try:
             filename = self.xmlelement.iterancestors(self.namespaced('file')).next().get('original')
             if filename:
@@ -332,15 +422,21 @@ class xliffunit(lisa.LISAunit):
         except StopIteration:
             # unit has no proper file ancestor, probably newly created
             pass
-
-        uid += self.xmlelement.get("id") or ""
+        # hide the fact that we sanitize ID_SEPERATOR
+        uid += unicode(self.xmlelement.get("id") or u"").replace(ID_SEPARATOR_SAFE, ID_SEPARATOR)
         return uid
 
     def addlocation(self, location):
         self.setid(location)
 
     def getlocations(self):
-        return [self.getid()]
+        id_attr = unicode(self.xmlelement.get("id") or u"")
+        # XLIFF files downloaded from PO projects in Pootle
+        # might have id equal to .source, so let's avoid
+        # that:
+        if id_attr and id_attr != self.source:
+            return [id_attr]
+        return []
 
     def createcontextgroup(self, name, contexts=None, purpose=None):
         """Add the context group to the trans-unit with contexts a list with
@@ -375,7 +471,7 @@ class xliffunit(lisa.LISAunit):
                 pairs = []
                 for context in contexts:
                     pairs.append((context.get("context-type"), lisa.getText(context, getXMLspace(self.xmlelement, self._default_xml_space))))
-                groups.append(pairs) #not extend
+                groups.append(pairs)  # not extend
         return groups
 
     def getrestype(self):
@@ -406,8 +502,8 @@ class xliffunit(lisa.LISAunit):
             return False
 
     def multistring_to_rich(cls, mstr):
-        """Override L{TranslationUnit.multistring_to_rich} which is used by the
-            C{rich_source} and C{rich_target} properties."""
+        """Override :meth:`TranslationUnit.multistring_to_rich` which is used
+        by the ``rich_source`` and ``rich_target`` properties."""
         strings = mstr
         if isinstance(mstr, multistring):
             strings = mstr.strings
@@ -418,8 +514,8 @@ class xliffunit(lisa.LISAunit):
     multistring_to_rich = classmethod(multistring_to_rich)
 
     def rich_to_multistring(cls, elem_list):
-        """Override L{TranslationUnit.rich_to_multistring} which is used by the
-            C{rich_source} and C{rich_target} properties."""
+        """Override :meth:`TranslationUnit.rich_to_multistring` which is used
+        by the ``rich_source`` and ``rich_target`` properties."""
         return multistring([unicode(elem) for elem in elem_list])
     rich_to_multistring = classmethod(rich_to_multistring)
 
@@ -428,8 +524,8 @@ class xlifffile(lisa.LISAfile):
     """Class representing a XLIFF file store."""
     UnitClass = xliffunit
     Name = _("XLIFF Translation File")
-    Mimetypes  = ["application/x-xliff", "application/x-xliff+xml"]
-    Extensions = ["xlf", "xliff"]
+    Mimetypes = ["application/x-xliff", "application/x-xliff+xml"]
+    Extensions = ["xlf", "xliff", "sdlxliff"]
     rootNode = "xliff"
     bodyNode = "body"
     XMLskeleton = '''<?xml version="1.0" ?>
@@ -440,6 +536,8 @@ class xlifffile(lisa.LISAfile):
 </file>
 </xliff>'''
     namespace = 'urn:oasis:names:tc:xliff:document:1.1'
+    unversioned_namespace = 'urn:oasis:names:tc:xliff:document:'
+
     suggestions_in_format = True
     """xliff units have alttrans tags which can be used to store suggestions"""
 
@@ -449,7 +547,15 @@ class xlifffile(lisa.LISAfile):
         self._messagenum = 0
 
     def initbody(self):
-        self.namespace = self.document.getroot().nsmap.get(None, None)
+        # detect the xliff namespace, handle both 1.1 and 1.2
+        for prefix, ns in self.document.getroot().nsmap.items():
+            if ns and ns.startswith(self.unversioned_namespace):
+                self.namespace = ns
+                break
+        else:
+            # handle crappy xliff docs without proper namespace declaration
+            # by simply using the xmlns default namespace
+            self.namespace = self.document.getroot().nsmap.get(None, None)
 
         if self._filename:
             filenode = self.getfilenode(self._filename, createifmissing=True)
@@ -461,7 +567,8 @@ class xlifffile(lisa.LISAfile):
         """Initialise the file header."""
         pass
 
-    def createfilenode(self, filename, sourcelanguage=None, targetlanguage=None, datatype='plaintext'):
+    def createfilenode(self, filename, sourcelanguage=None,
+                       targetlanguage=None, datatype='plaintext'):
         """creates a filenode with the given filename. All parameters
         are needed for XLIFF compliance."""
         if sourcelanguage is None:
@@ -562,9 +669,14 @@ class xlifffile(lisa.LISAfile):
         return ""
 
     def getdate(self, filename=None):
-        """Returns the date attribute for the file. If no filename is given,
-        the date of the first file is given. If the date attribute is not
-        specified, None is returned."""
+        """Returns the date attribute for the file.
+
+        If no filename is given, the date of the first file is given.
+        If the date attribute is not specified, None is returned.
+
+        :returns: Date attribute of file
+        :rtype: Date or None
+        """
         if filename:
             node = self.getfilenode(filename)
             if not node is None:
@@ -576,13 +688,13 @@ class xlifffile(lisa.LISAfile):
         return None
 
     def removedefaultfile(self):
-        """We want to remove the default file-tag as soon as possible if we 
+        """We want to remove the default file-tag as soon as possible if we
         know if still present and empty."""
         filenodes = list(self.document.getroot().iterchildren(self.namespaced("file")))
         if len(filenodes) > 1:
             for filenode in filenodes:
-                if filenode.get("original") == "NoName" and \
-                        not list(filenode.iterdescendants(self.namespaced(self.UnitClass.rootNode))):
+                if (filenode.get("original") == "NoName" and
+                    not list(filenode.iterdescendants(self.namespaced(self.UnitClass.rootNode)))):
                     self.document.getroot().remove(filenode)
                 break
 
@@ -624,7 +736,11 @@ class xlifffile(lisa.LISAfile):
         return unit
 
     def switchfile(self, filename, createifmissing=False):
-        """adds the given trans-unit (will create the nodes required if asked). Returns success"""
+        """Adds the given trans-unit (will create the nodes required if asked).
+
+        :returns: Success
+        :rtype: Boolean
+        """
         self._filename = filename
         filenode = self.getfilenode(filename)
         if filenode is None:
