@@ -20,14 +20,19 @@
 
 """Import units from translations files into tmdb."""
 
-import sys
 import os
+import sys
+import logging
+
 from optparse import OptionParser
+
 from translate.storage import factory
 from translate.storage import tmdb
 
+logger = logging.getLogger(__name__)
 
 class Builder:
+
     def __init__(self, tmdbfile, source_lang, target_lang, filenames):
         self.tmdb = tmdb.TMDB(tmdbfile)
         self.source_lang = source_lang
@@ -35,7 +40,7 @@ class Builder:
 
         for filename in filenames:
             if not os.path.exists(filename):
-                print >> sys.stderr, "cannot process %s: does not exist" % filename
+                logger.error("cannot process %s: does not exist", filename)
                 continue
             elif os.path.isdir(filename):
                 self.handledir(filename)
@@ -47,7 +52,7 @@ class Builder:
         try:
             store = factory.getobject(filename)
         except Exception, e:
-            print >> sys.stderr, str(e)
+            logger.error(str(e))
             return
         # do something useful with the store and db
         try:
@@ -73,24 +78,16 @@ class Builder:
 
 
 def main():
-    try:
-        import psyco
-        psyco.full()
-    except Exception:
-        pass
     parser = OptionParser(usage="%prog [options] <input files>")
     parser.add_option(
         "-d", "--tmdb", dest="tmdb_file", default="tm.db",
-        help="translation memory database file (default: tm.db)"
-    )
+        help="translation memory database file (default: tm.db)")
     parser.add_option(
         "-s", "--import-source-lang", dest="source_lang", default="en",
-        help="source language of translation files (default: en)"
-    )
+        help="source language of translation files (default: en)")
     parser.add_option(
         "-t", "--import-target-lang", dest="target_lang",
-        help="target language of translation files"
-    )
+        help="target language of translation files")
     (options, args) = parser.parse_args()
 
     if not options.target_lang:
@@ -99,6 +96,8 @@ def main():
     if len(args) < 1:
         parser.error('No input file(s) specified.')
 
+    logging.basicConfig(format="%(name)s: %(levelname)s: %(message)s")
+
     Builder(options.tmdb_file, options.source_lang, options.target_lang, args)
 
 if __name__ == '__main__':
