@@ -1,40 +1,41 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
-# Copyright 2002-2008 Zuza Software Foundation
-# 
+#
+# Copyright 2002-2011 Zuza Software Foundation
+# Copyright 2013 F Wolff
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""Grep XLIFF, Gettext PO and TMX localization files
+"""Grep XLIFF, Gettext PO and TMX localization files.
 
-Matches are output to snippet files of the same type which can then be reviewed 
-and later merged using pomerge
+Matches are output to snippet files of the same type which can then be reviewed
+and later merged using :doc:`pomerge </commands/pomerge>`.
 
-See: http://translate.sourceforge.net/wiki/toolkit/pogrep for examples and
-usage instructions
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/pogrep.html
+for examples and usage instructions.
 """
 
+import re
+import locale
+
 from translate.storage import factory
 from translate.storage.poheader import poheader
 from translate.misc import optrecurse
 from translate.misc.multistring import multistring
 from translate.lang import data
-import re
-import locale
 
 
 class GrepMatch(object):
@@ -42,11 +43,11 @@ class GrepMatch(object):
 
     # INITIALIZERS #
     def __init__(self, unit, part='target', part_n=0, start=0, end=0):
-        self.unit   = unit
-        self.part   = part
+        self.unit = unit
+        self.part = part
         self.part_n = part_n
-        self.start  = start
-        self.end    = end
+        self.start = start
+        self.end = end
 
     # ACCESSORS #
     def get_getter(self):
@@ -63,10 +64,12 @@ class GrepMatch(object):
                 getter = lambda: self.unit.source
             return getter
         elif self.part == 'notes':
+
             def getter():
                 return self.unit.getnotes()[self.part_n]
             return getter
         elif self.part == 'locations':
+
             def getter():
                 return self.unit.getlocations()[self.part_n]
             return getter
@@ -74,11 +77,13 @@ class GrepMatch(object):
     def get_setter(self):
         if self.part == 'target':
             if self.unit.hasplural():
+
                 def setter(value):
                     strings = self.unit.target.strings
                     strings[self.part_n] = value
                     self.unit.target = strings
             else:
+
                 def setter(value):
                     self.unit.target = value
             return setter
@@ -96,6 +101,7 @@ class GrepMatch(object):
     def __repr__(self):
         return str(self)
 
+
 def real_index(string, nfc_index):
     """Calculate the real index in the unnormalized string that corresponds to
     the index nfc_index in the normalized string."""
@@ -115,15 +121,21 @@ def find_matches(unit, part, strings, re_search):
         if not string:
             continue
         normalized = data.normalize(string)
+        if normalized == string:
+            index_func = lambda s, i: i
+        else:
+            index_func = real_index
         for matchobj in re_search.finditer(normalized):
-            start = real_index(string, matchobj.start())
-            end = real_index(string, matchobj.end())
+            start = index_func(string, matchobj.start())
+            end = index_func(string, matchobj.end())
             matches.append(GrepMatch(unit, part=part, part_n=n, start=start, end=end))
     return matches
 
+
 class GrepFilter:
+
     def __init__(self, searchstring, searchparts, ignorecase=False, useregexp=False,
-            invertmatch=False, accelchar=None, encoding='utf-8',
+            invertmatch=False, keeptranslations=False, accelchar=None, encoding='utf-8',
             max_matches=0):
         """builds a checkfilter using the given checker"""
         if isinstance(searchstring, unicode):
@@ -136,7 +148,7 @@ class GrepFilter:
             # which has a new meaning now.
             self.search_source = ('source' in searchparts) or ('msgid' in searchparts)
             self.search_target = ('target' in searchparts) or ('msgstr' in searchparts)
-            self.search_notes =  ('notes' in searchparts) or ('comment' in searchparts)
+            self.search_notes = ('notes' in searchparts) or ('comment' in searchparts)
             self.search_locations = 'locations' in searchparts
         else:
             self.search_source = True
@@ -150,6 +162,7 @@ class GrepFilter:
         if self.useregexp:
             self.searchpattern = re.compile(self.searchstring)
         self.invertmatch = invertmatch
+        self.keeptranslations = keeptranslations
         self.accelchar = accelchar
         self.max_matches = max_matches
 
@@ -173,7 +186,10 @@ class GrepFilter:
     def filterunit(self, unit):
         """runs filters on an element"""
         if unit.isheader():
-            return []
+            return True
+
+        if self.keeptranslations and unit.target:
+            return True
 
         if self.search_source:
             if isinstance(unit.source, multistring):
@@ -263,8 +279,10 @@ class GrepFilter:
 
         return matches, indexes
 
+
 class GrepOptionParser(optrecurse.RecursiveOptionParser):
     """a specialized Option Parser for the grep tool..."""
+
     def parse_args(self, args=None, values=None):
         """parses the command line options, handling implicit input/output args"""
         (options, args) = optrecurse.optparse.OptionParser.parse_args(self, args, values)
@@ -302,10 +320,17 @@ class GrepOptionParser(optrecurse.RecursiveOptionParser):
         (options, args) = self.parse_args()
         options.inputformats = self.inputformats
         options.outputoptions = self.outputoptions
-        options.checkfilter = GrepFilter(options.searchstring, options.searchparts, options.ignorecase, options.useregexp, options.invertmatch, options.accelchar, locale.getpreferredencoding())
-        self.usepsyco(options)
+        options.checkfilter = GrepFilter(options.searchstring,
+                                         options.searchparts,
+                                         options.ignorecase,
+                                         options.useregexp,
+                                         options.invertmatch,
+                                         options.keeptranslations,
+                                         options.accelchar,
+                                         locale.getpreferredencoding())
         self.recursiveprocess(options)
 
+
 def rungrep(inputfile, outputfile, templatefile, checkfilter):
     """reads in inputfile, filters using checkfilter, writes to outputfile"""
     fromfile = factory.getobject(inputfile)
@@ -315,15 +340,16 @@ def rungrep(inputfile, outputfile, templatefile, checkfilter):
     outputfile.write(str(tofile))
     return True
 
+
 def cmdlineparser():
-    formats = {"po":("po", rungrep), "pot":("pot", rungrep), 
-            "mo":("mo", rungrep), "gmo":("gmo", rungrep), 
-            "tmx":("tmx", rungrep),
-            "xliff":("xliff", rungrep), "xlf":("xlf", rungrep), "xlff":("xlff", rungrep), 
-            None:("po", rungrep)}
+    formats = {"po": ("po", rungrep), "pot": ("pot", rungrep),
+            "mo": ("mo", rungrep), "gmo": ("gmo", rungrep),
+            "tmx": ("tmx", rungrep),
+            "xliff": ("xliff", rungrep), "xlf": ("xlf", rungrep), "xlff": ("xlff", rungrep),
+            None: ("po", rungrep)}
     parser = GrepOptionParser(formats)
     parser.add_option("", "--search", dest="searchparts",
-        action="append", type="choice", choices=["source", "target", "notes", "locations", "msgid", "msgstr", "comment" ],
+        action="append", type="choice", choices=["source", "target", "notes", "locations", "msgid", "msgstr", "comment"],
         metavar="SEARCHPARTS", help="searches the given parts (source, target, notes and locations)")
     parser.add_option("-I", "--ignore-case", dest="ignorecase",
         action="store_true", default=False, help="ignore case distinctions")
@@ -334,14 +360,18 @@ def cmdlineparser():
     parser.add_option("", "--accelerator", dest="accelchar",
         action="store", type="choice", choices=["&", "_", "~"],
         metavar="ACCELERATOR", help="ignores the given accelerator when matching")
+    parser.add_option("-k", "--keep-translations", dest="keeptranslations",
+        action="store_true", default=False, help="always extract units with translations")
     parser.set_usage()
     parser.passthrough.append('checkfilter')
     parser.description = __doc__
     return parser
 
+
 def main():
     parser = cmdlineparser()
     parser.run()
 
+
 if __name__ == '__main__':
     main()
