@@ -14,13 +14,12 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""reads a set of .po or .pot files to produce a pootle-terminology.pot
+"""Create a terminology file by reading a set of .po or .pot files to produce a pootle-terminology.pot.
 
-See: http://translate.sourceforge.net/wiki/toolkit/poterminology for examples and
-usage instructions
+See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/poterminology.html
+for examples and usage instructions.
 """
 import os
 import re
@@ -33,6 +32,8 @@ from translate.storage import po
 from translate.storage import factory
 from translate.misc import file_discovery
 
+logger = logging.getLogger(__name__)
+
 def create_termunit(term, unit, targets, locations, sourcenotes, transnotes, filecounts):
     termunit = po.pounit(term)
     if unit is not None:
@@ -56,7 +57,9 @@ def create_termunit(term, unit, targets, locations, sourcenotes, transnotes, fil
         termunit.addnote("(poterminology) %s (%d)\n" % (filename, count), 'translator')
     return termunit
 
+
 class TerminologyExtractor(object):
+
     def __init__(self, foldtitle=True, ignorecase=False, accelchars="", termlength=3,
                  sourcelanguage="en", invert=False, stopfile=None):
         self.foldtitle = foldtitle
@@ -86,17 +89,17 @@ class TerminologyExtractor(object):
         self.xmlelpat = re.compile(r"<(?:![[-]|[/?]?[A-Za-z_:])[^>]*>")
         # handles XML/HTML entities (&#32; &#x20; &amp; &my_entity;)
         self.xmlentpat = re.compile(r"&(?:#(?:[0-9]+|x[0-9a-f]+)|[a-z_:][\w.-:]*);",
-                               flags=re.UNICODE|re.IGNORECASE)
+                               flags=re.UNICODE | re.IGNORECASE)
 
         self.units = 0
         self.glossary = {}
 
     def parse_stopword_file(self):
 
-        actions = { '+': frozenset(), ':': frozenset(['skip']),
-                    '<': frozenset(['phrase']), '=': frozenset(['word']),
-                    '>': frozenset(['word','skip']),
-                    '@': frozenset(['word','phrase']) }
+        actions = {'+': frozenset(), ':': frozenset(['skip']),
+                   '<': frozenset(['phrase']), '=': frozenset(['word']),
+                   '>': frozenset(['word', 'skip']),
+                   '@': frozenset(['word', 'phrase'])}
 
         stopfile = open(self.stopfile, "r")
         line = 0
@@ -116,14 +119,17 @@ class TerminologyExtractor(object):
                     elif stopline[1] == 'I':
                         self.stopignorecase = True
                     else:
-                        logging.warning("%s line %d - bad case mapping directive", (self.stopfile, line))
+                        logger.warning("%s:%d - bad case mapping directive",
+                                       self.stopfile, line)
                 elif stoptype == '/':
-                    self.stoprelist.append(re.compile(stopline[1:-1]+'$'))
+                    self.stoprelist.append(re.compile(stopline[1:-1] + '$'))
                 else:
                     self.stopwords[stopline[1:-1]] = actions[stoptype]
         except KeyError, character:
-            logging.warning("%s line %d - bad stopword entry starts with", (self.stopfile, line))
-            logging.warning("%s line %d all lines after error ignored",  (self.stopfile, line + 1))
+            logger.warning("%s:%d - bad stopword entry starts with '%s'",
+                           self.stopfile, line, str(character))
+            logger.warning("%s:%d all lines after error ignored",
+                           self.stopfile, line + 1)
         stopfile.close()
 
     def clean(self, string):
@@ -164,14 +170,12 @@ class TerminologyExtractor(object):
 
     def processunits(self, units, fullinputpath):
         sourcelang = lang_factory.getlanguage(self.sourcelanguage)
-        rematchignore = frozenset(('word','phrase'))
+        rematchignore = frozenset(('word', 'phrase'))
         defaultignore = frozenset()
         for unit in units:
             self.units += 1
             if unit.isheader():
                 continue
-            if unit.hasplural():
-                continue
             if not self.invert:
                 source = self.clean(unit.source)
                 target = self.clean(unit.target)
@@ -235,8 +239,7 @@ class TerminologyExtractor(object):
     def extract_terms(self, create_termunit=create_termunit, inputmin=1, fullmsgmin=1, substrmin=2, locmin=2):
         terms = {}
         locre = re.compile(r":[0-9]+$")
-        print >> sys.stderr, ("%d terms from %d units" %
-                              (len(self.glossary), self.units))
+        logger.info("%d terms from %d units", len(self.glossary), self.units)
         for term, translations in self.glossary.iteritems():
             if len(translations) <= 1:
                 continue
@@ -273,7 +276,7 @@ class TerminologyExtractor(object):
             numsources = len(sources)
             numfiles = len(filecounts)
             numlocs = len(locations)
-            if numfiles < inputmin or numlocs < locmin:
+            if numfiles < inputmin or 0 < numlocs < locmin:
                 continue
             if fullmsg:
                 if numsources < fullmsgmin:
@@ -291,14 +294,20 @@ class TerminologyExtractor(object):
             terms[term] = ((10 * numfiles) + numsources, termunit)
         return terms
 
-    def filter_terms(self, terms, sortorders=[ "frequency", "dictionary", "length" ]):
+    sortorders_default = ["frequency", "dictionary", "length"]
+
+    def filter_terms(self, terms, nonstopmin=1, sortorders=sortorders_default):
         """reduce subphrases from extracted terms"""
         # reduce subphrase
         termlist = terms.keys()
-        print >> sys.stderr, "%d terms after thresholding" % len(termlist)
+        logger.info("%d terms after thresholding", len(termlist))
         termlist.sort(lambda x, y: cmp(len(x), len(y)))
         for term in termlist:
             words = term.split()
+            nonstop = [word for word in words if not self.stopword(word)]
+            if len(nonstop) < nonstopmin and  len(nonstop) != len(words):
+                del terms[term]
+                continue
             if len(words) <= 2:
                 continue
             while len(words) > 2:
@@ -310,8 +319,10 @@ class TerminologyExtractor(object):
                 words.pop(0)
                 if terms[term][0] == terms.get(' '.join(words), [0])[0]:
                     del terms[' '.join(words)]
-        print >> sys.stderr, "%d terms after subphrase reduction" % len(terms.keys())
+        logger.info("%d terms after subphrase reduction", len(terms.keys()))
         termitems = terms.values()
+        if sortorders is None:
+            sortorders = self.sortorders_default
         while len(sortorders) > 0:
             order = sortorders.pop()
             if order == "frequency":
@@ -321,7 +332,7 @@ class TerminologyExtractor(object):
             elif order == "length":
                 termitems.sort(lambda x, y: cmp(len(x[1].source), len(y[1].source)))
             else:
-                logging.warning("unknown sort order %s", order)
+                logger.warning("unknown sort order %s", order)
         return termitems
 
 
@@ -387,7 +398,6 @@ class TerminologyOptionParser(optrecurse.RecursiveOptionParser):
         (options, args) = self.parse_args()
         options.inputformats = self.inputformats
         options.outputoptions = self.outputoptions
-        self.usepsyco(options)
         self.extractor = TerminologyExtractor(foldtitle=options.foldtitle, ignorecase=options.ignorecase,
                                               accelchars=options.accelchars, termlength=options.termlength,
                                               sourcelanguage=options.sourcelanguage,
@@ -408,7 +418,7 @@ class TerminologyOptionParser(optrecurse.RecursiveOptionParser):
             else:
                 inputfiles = [options.input]
         if os.path.isdir(options.output):
-            options.output = os.path.join(options.output,"pootle-terminology.pot")
+            options.output = os.path.join(options.output, "pootle-terminology.pot")
 
         self.initprogressbar(inputfiles, options)
         for inputpath in inputfiles:
@@ -435,23 +445,26 @@ class TerminologyOptionParser(optrecurse.RecursiveOptionParser):
     def outputterminology(self, options):
         """saves the generated terminology glossary"""
         termfile = po.pofile()
-        print >> sys.stderr, ("scanned %d files" % self.files)
+        logger.info("scanned %d files", self.files)
         terms = self.extractor.extract_terms(inputmin=options.inputmin, fullmsgmin=options.fullmsgmin,
                                    substrmin=options.substrmin, locmin=options.locmin)
-        termitems = self.extractor.filter_terms(terms, sortorders=options.sortorders)
+        termitems = self.extractor.filter_terms(terms, nonstopmin=options.nonstopmin, sortorders=options.sortorders)
         for count, unit in termitems:
             termfile.units.append(unit)
         open(options.output, "w").write(str(termfile))
 
+
 def fold_case_option(option, opt_str, value, parser):
     parser.values.ignorecase = False
     parser.values.foldtitle = True
 
+
 def preserve_case_option(option, opt_str, value, parser):
     parser.values.ignorecase = parser.values.foldtitle = False
 
+
 def main():
-    formats = {"po":("po", None), "pot": ("pot", None), None:("po", None)}
+    formats = {"po": ("po", None), "pot": ("pot", None), None: ("po", None)}
     parser = TerminologyOptionParser(formats)
 
     parser.add_option("-u", "--update", type="string", dest="update",
@@ -461,7 +474,7 @@ def main():
                       help="read stopword (term exclusion) list from STOPFILE (default %s)" %
                       file_discovery.get_abs_data_filename('stoplist-en'))
 
-    parser.set_defaults(foldtitle = True, ignorecase = False)
+    parser.set_defaults(foldtitle=True, ignorecase=False)
     parser.add_option("-F", "--fold-titlecase", callback=fold_case_option,
         action="callback", help="fold \"Title Case\" to lowercase (default)")
     parser.add_option("-C", "--preserve-case", callback=preserve_case_option,
@@ -470,10 +483,12 @@ def main():
         action="store_true", help="make all terms lowercase")
 
     parser.add_option("", "--accelerator", dest="accelchars", default="",
-        metavar="ACCELERATORS", help="ignores the given accelerator characters when matching")
+        metavar="ACCELERATORS", help="ignore the given accelerator characters when matching")
 
     parser.add_option("-t", "--term-words", type="int", dest="termlength", default="3",
         help="generate terms of up to LENGTH words (default 3)", metavar="LENGTH")
+    parser.add_option("", "--nonstop-needed", type="int", dest="nonstopmin", default="1",
+        help="omit terms with less than MIN nonstop words (default 1)", metavar="MIN")
     parser.add_option("", "--inputs-needed", type="int", dest="inputmin",
         help="omit terms appearing in less than MIN input files (default 2, or 1 if only one input file)", metavar="MIN")
     parser.add_option("", "--fullmsg-needed", type="int", dest="fullmsgmin", default="1",
@@ -483,10 +498,9 @@ def main():
     parser.add_option("", "--locs-needed", type="int", dest="locmin", default="2",
         help="omit terms appearing in less than MIN different original source files (default 2)", metavar="MIN")
 
-    sortorders_default = [ "frequency", "dictionary", "length" ]
     parser.add_option("", "--sort", dest="sortorders", action="append",
-        type="choice", choices=sortorders_default, metavar="ORDER", default=sortorders_default,
-        help="output sort order(s): %s (default is all orders in the above priority)" % ', '.join(sortorders_default))
+        type="choice", choices=TerminologyExtractor.sortorders_default, metavar="ORDER",
+        help="output sort order(s): %s (may repeat option, default is all in above order)" % ', '.join(TerminologyExtractor.sortorders_default))
 
     parser.add_option("", "--source-language", dest="sourcelanguage", default="en",
         help="the source language code (default 'en')", metavar="LANG")
