@@ -1,10 +1,17 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+import StringIO
+from translate.tools import pocount
+
+from pytest import mark
+
 from translate.storage import po
 from translate.storage import statsdb
 
-class TestPOCount:
+
+class TestCount:
+
     def count(self, source, expectedsource, target=None, expectedtarget=None):
         """simple helper to check the respective word counts"""
         poelement = po.pounit(source)
@@ -62,6 +69,7 @@ class TestPOCount:
         # msgstr[0] "e paletšwe go hwetša sešireletši sa seswantšho sa paete ya %u"
         # msgstr[1] "e paletšwe go hwetša sešireletši sa seswantšho sa dipaete tša %u"
 
+    @mark.xfail(reason="Support commented out pending removal")
     def test_plurals_kde(self):
         """test that we correcly count old style KDE plurals"""
         self.count("_n: Singular\\n\nPlural", 2, "Een\\n\ntwee\\n\ndrie", 3)
@@ -73,10 +81,79 @@ class TestPOCount:
     # Counting strings
     #  We need to check how we count strings also and if we call it translated or untranslated
     # ie an all spaces msgid should be translated if there are spaces in the msgstr
-   
+
     # Make sure we don't count obsolete messages
 
     # Do we correctly identify a translated yet blank message?
 
     # Need to test that we can differentiate between fuzzy, translated and untranslated
 
+
+class TestPOCount:
+    """This only tests the old (memory-based) pocount method, not the current
+    code based on statsdb."""
+
+    inputdata = r'''
+msgid "translated unit"
+msgstr "translated unit"
+
+#, fuzzy
+msgid "fuzzy unit"
+msgstr "fuzzy unit"
+
+# untranslated
+msgid "untranslated unit"
+msgstr ""
+
+# obsolete
+#~ msgid "obsolete translated unit"
+#~ msgstr "obsolete translated unit"
+
+#, fuzzy
+#~ msgid "obsolete fuzzy unit"
+#~ msgstr "obsolete fuzzy unit"
+
+# untranslated
+#~ msgid "obsolete untranslated unit"
+#~ msgstr ""
+'''
+
+    def test_translated(self):
+        pofile = StringIO.StringIO(self.inputdata)
+        stats = pocount.calcstats_old(pofile)
+        assert stats['translated'] == 1
+
+    def test_fuzzy(self):
+        pofile = StringIO.StringIO(self.inputdata)
+        stats = pocount.calcstats_old(pofile)
+        assert stats['fuzzy'] == 1
+
+    def test_untranslated(self):
+        pofile = StringIO.StringIO(self.inputdata)
+        stats = pocount.calcstats_old(pofile)
+        assert stats['untranslated'] == 1
+
+    def test_total(self):
+        pofile = StringIO.StringIO(self.inputdata)
+        stats = pocount.calcstats_old(pofile)
+        assert stats['total'] == 3
+
+    def test_translatedsourcewords(self):
+        pofile = StringIO.StringIO(self.inputdata)
+        stats = pocount.calcstats_old(pofile)
+        assert stats['translatedsourcewords'] == 2
+
+    def test_fuzzysourcewords(self):
+        pofile = StringIO.StringIO(self.inputdata)
+        stats = pocount.calcstats_old(pofile)
+        assert stats['fuzzysourcewords'] == 2
+
+    def test_untranslatedsourcewords(self):
+        pofile = StringIO.StringIO(self.inputdata)
+        stats = pocount.calcstats_old(pofile)
+        assert stats['untranslatedsourcewords'] == 2
+
+    def test_totalsourcewords(self):
+        pofile = StringIO.StringIO(self.inputdata)
+        stats = pocount.calcstats_old(pofile)
+        assert stats['totalsourcewords'] == 6
