@@ -1,12 +1,27 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+import pytest
+from pytest import mark
+
 from translate.tools import pomerge
 from translate.storage import factory
 from translate.storage import po
-from translate.storage import xliff 
+from translate.storage import xliff
 from translate.misc import wStringIO
 
+
+def test_str2bool():
+    """test the str2bool function"""
+    assert pomerge.str2bool("yes")
+    assert pomerge.str2bool("true")
+    assert pomerge.str2bool("1")
+    assert not pomerge.str2bool("no")
+    assert not pomerge.str2bool("false")
+    assert not pomerge.str2bool("0")
+    pytest.raises(ValueError, pomerge.str2bool, "2")
+
+
 class TestPOMerge:
     xliffskeleton = '''<?xml version="1.0" ?>
 <xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
@@ -17,22 +32,35 @@ class TestPOMerge:
   </file>
 </xliff>'''
 
-    def mergestore(self, templatesource, inputsource):
-        """merges the sources of the given files and returns a new pofile object"""
+    def mergestore(self, templatesource, inputsource, mergeblanks="yes",
+                   mergefuzzy="yes",
+                   mergecomments="yes"):
+        """merges the sources of the given files and returns a new pofile
+        object"""
         templatefile = wStringIO.StringIO(templatesource)
         inputfile = wStringIO.StringIO(inputsource)
         outputfile = wStringIO.StringIO()
-        assert pomerge.mergestore(inputfile, outputfile, templatefile)
+        assert pomerge.mergestore(inputfile, outputfile, templatefile,
+                                  mergeblanks=mergeblanks,
+                                  mergefuzzy=mergefuzzy,
+                                  mergecomments=mergecomments,
+        )
         outputpostring = outputfile.getvalue()
         outputpofile = po.pofile(outputpostring)
         return outputpofile
 
-    def mergexliff(self, templatesource, inputsource):
-        """merges the sources of the given files and returns a new xlifffile object"""
+    def mergexliff(self, templatesource, inputsource, mergeblanks="yes",
+                   mergefuzzy="yes",
+                   mergecomments="yes"):
+        """merges the sources of the given files and returns a new xlifffile
+        object"""
         templatefile = wStringIO.StringIO(templatesource)
         inputfile = wStringIO.StringIO(inputsource)
         outputfile = wStringIO.StringIO()
-        assert pomerge.mergestore(inputfile, outputfile, templatefile)
+        assert pomerge.mergestore(inputfile, outputfile, templatefile,
+                                  mergeblanks=mergeblanks,
+                                  mergefuzzy=mergefuzzy,
+                                  mergecomments=mergecomments)
         outputxliffstring = outputfile.getvalue()
         print "Generated XML:"
         print outputxliffstring
@@ -47,10 +75,21 @@ class TestPOMerge:
             return len(pofile.units)
 
     def singleunit(self, pofile):
-        """checks that the pofile contains a single non-header unit, and returns it"""
+        """checks that the pofile contains a single non-header unit, and
+        returns it"""
         assert self.countunits(pofile) == 1
         return pofile.units[-1]
 
+    def test_mergesore_bad_data(self):
+        """Test that we catch bad options sent to mergestore"""
+        templatefile = wStringIO.StringIO("")
+        inputfile = wStringIO.StringIO("")
+        outputfile = wStringIO.StringIO()
+        pytest.raises(ValueError, pomerge.mergestore, inputfile, outputfile,
+                    templatefile, mergeblanks="yay")
+        pytest.raises(ValueError, pomerge.mergestore, inputfile, outputfile,
+                    templatefile, mergecomments="yay")
+
     def test_simplemerge(self):
         """checks that a simple po entry merges OK"""
         templatepo = '''#: simple.test\nmsgid "Simple String"\nmsgstr ""\n'''
@@ -60,6 +99,17 @@ class TestPOMerge:
         assert pounit.source == "Simple String"
         assert pounit.target == "Dimpled Ring"
 
+    def test_simplemerge_no_locations(self):
+        """checks that a simple po entry merges OK"""
+        templatepo = '''msgid "Simple String"
+msgstr ""'''
+        inputpo = '''msgid "Simple String"
+msgstr "Dimpled Ring"'''
+        pofile = self.mergestore(templatepo, inputpo)
+        pounit = self.singleunit(pofile)
+        assert pounit.source == "Simple String"
+        assert pounit.target == "Dimpled Ring"
+
     def test_replacemerge(self):
         """checks that a simple po entry merges OK"""
         templatepo = '''#: simple.test\nmsgid "Simple String"\nmsgstr "Dimpled Ring"\n'''
@@ -69,8 +119,37 @@ class TestPOMerge:
         assert pounit.source == "Simple String"
         assert pounit.target == "Dimpled King"
 
+    def test_merging_blanks(self):
+        """By default we will merge blanks, but we can also override that"""
+        templatepo = '''#: simple.test\nmsgid "Simple String"\nmsgstr "Dimpled Ring"\n'''
+        inputpo = '''#: simple.test\nmsgid "Simple String"\nmsgstr ""\n'''
+        pofile = self.mergestore(templatepo, inputpo)
+        pounit = self.singleunit(pofile)
+        assert pounit.source == "Simple String"
+        assert pounit.target == ""
+        pofile = self.mergestore(templatepo, inputpo, mergeblanks="no")
+        pounit = self.singleunit(pofile)
+        assert pounit.source == "Simple String"
+        assert pounit.target == "Dimpled Ring"
+
+    def test_merging_fuzzies(self):
+        """By default we will merge fuzzies, but can can also override that."""
+        templatepo = '''#: simple.test\nmsgid "Simple String"\nmsgstr "Dimpled Ring"\n'''
+        inputpo = '''#: simple.test\n#, fuzzy\nmsgid "Simple String"\nmsgstr "changed fish"\n'''
+        pofile = self.mergestore(templatepo, inputpo)
+        pounit = self.singleunit(pofile)
+        assert pounit.source == "Simple String"
+        assert pounit.target == "changed fish"
+        assert pounit.isfuzzy()
+        pofile = self.mergestore(templatepo, inputpo, mergefuzzy="no")
+        pounit = self.singleunit(pofile)
+        assert pounit.source == "Simple String"
+        assert pounit.target == "Dimpled Ring"
+        assert pounit.isfuzzy() == False
+
     def test_merging_locations(self):
-        """check that locations on separate lines are output in Gettext form of all on one line"""
+        """check that locations on separate lines are output in Gettext form
+        of all on one line"""
         templatepo = '''#: location.c:1\n#: location.c:2\nmsgid "Simple String"\nmsgstr ""\n'''
         inputpo = '''#: location.c:1\n#: location.c:2\nmsgid "Simple String"\nmsgstr "Dimpled Ring"\n'''
         expectedpo = '''#: location.c:1%slocation.c:2\nmsgid "Simple String"\nmsgstr "Dimpled Ring"\n''' % po.lsep
@@ -78,8 +157,45 @@ class TestPOMerge:
         print pofile
         assert str(pofile) == expectedpo
 
+    def test_unit_missing_in_template_with_locations(self):
+        """If the unit is missing in the template we should raise an error"""
+        templatepo = '''#: location.c:1
+msgid "Simple String"
+msgstr ""'''
+        inputpo = '''#: location.c:1
+msgid "Simple String"
+msgstr "Dimpled Ring"
+
+#: location.c:1
+msgid "Extra string"
+msgstr "Perplexa ring"'''
+        expectedpo = '''#: location.c:1
+msgid "Simple String"
+msgstr "Dimpled Ring"
+'''
+        pofile = self.mergestore(templatepo, inputpo)
+        print pofile
+        assert str(pofile) == expectedpo
+
+    def test_unit_missing_in_template_no_locations(self):
+        """If the unit is missing in the template we should raise an error"""
+        templatepo = '''msgid "Simple String"
+msgstr ""'''
+        inputpo = '''msgid "Simple String"
+msgstr "Dimpled Ring"
+
+msgid "Extra string"
+msgstr "Perplexa ring"'''
+        expectedpo = '''msgid "Simple String"
+msgstr "Dimpled Ring"
+'''
+        pofile = self.mergestore(templatepo, inputpo)
+        print pofile
+        assert str(pofile) == expectedpo
+
     def test_reflowed_source_comments(self):
-        """ensure that we don't duplicate source comments (locations) if they have been reflowed"""
+        """ensure that we don't duplicate source comments (locations) if they
+        have been reflowed"""
         templatepo = '''#: newMenu.label\n#: newMenu.accesskey\nmsgid "&New"\nmsgstr ""\n'''
         newpo = '''#: newMenu.label newMenu.accesskey\nmsgid "&New"\nmsgstr "&Nuwe"\n'''
         expectedpo = '''#: newMenu.label%snewMenu.accesskey\nmsgid "&New"\nmsgstr "&Nuwe"\n''' % po.lsep
@@ -98,14 +214,15 @@ msgid "bla"
 msgstr "blabla"
 '''
         newpo = templatepo
-        expectedpo =  templatepo
+        expectedpo = templatepo
         pofile = self.mergestore(templatepo, newpo)
         pounit = self.singleunit(pofile)
         print pofile
         assert str(pofile) == expectedpo
 
     def test_merge_dont_delete_unassociated_comments(self):
-        """ensure that we do not delete comments in the PO file that are not assocaited with a message block"""
+        """ensure that we do not delete comments in the PO file that are not
+        assocaited with a message block"""
         templatepo = '''# Lonely comment\n\n# Translation comment\nmsgid "Bob"\nmsgstr "Toolmaker"\n'''
         mergepo = '''# Translation comment\nmsgid "Bob"\nmsgstr "Builder"\n'''
         expectedpo = '''# Lonely comment\n# Translation comment\nmsgid "Bob"\nmsgstr "Builder"\n'''
@@ -126,12 +243,14 @@ msgstr "blabla"
         templatepo = '''msgid ""\n"Simple string\\n"\nmsgstr ""\n'''
         mergepo = '''msgid ""\n"Simple string\\n"\nmsgstr ""\n"Dimpled ring\\n"\n'''
         expectedpo = '''msgid ""\n"Simple string\\n"\nmsgstr "Dimpled ring\\n"\n'''
+        expectedpo2 = '''msgid "Simple string\\n"\nmsgstr "Dimpled ring\\n"\n'''
         pofile = self.mergestore(templatepo, mergepo)
         print "Expected:\n%s\n\nMerged:\n%s" % (expectedpo, str(pofile))
-        assert str(pofile) == expectedpo
+        assert str(pofile) == expectedpo or str(pofile) == expectedpo2
 
     def test_preserve_format_minor_start_and_end_of_sentence_changes(self):
-        """Test that we are not too fussy about large diffs for simple changes at the start or end of a sentence"""
+        """Test that we are not too fussy about large diffs for simple
+        changes at the start or end of a sentence"""
         templatepo = '''msgid "Target type:"\nmsgstr "Doelsoort"\n\n'''
         mergepo = '''msgid "Target type:"\nmsgstr "Doelsoort:"\n'''
         expectedpo = mergepo
@@ -154,7 +273,8 @@ msgstr "blabla"
         assert str(pofile) == expectedpo
 
     def test_preserve_format_last_entry_in_a_file(self):
-        """The last entry in a PO file is usualy not followed by an empty line.  Test that we preserve this"""
+        """The last entry in a PO file is usualy not followed by an empty
+        line.  Test that we preserve this"""
         templatepo = '''msgid "First"\nmsgstr ""\n\nmsgid "Second"\nmsgstr ""\n'''
         mergepo = '''msgid "First"\nmsgstr "Eerste"\n\nmsgid "Second"\nmsgstr "Tweede"\n'''
         expectedpo = '''msgid "First"\nmsgstr "Eerste"\n\nmsgid "Second"\nmsgstr "Tweede"\n'''
@@ -169,13 +289,15 @@ msgstr "blabla"
         print "Expected:\n%s\n\nMerged:\n%s" % (expectedpo, str(pofile))
         assert str(pofile) == expectedpo
 
-    def xtest_escape_tabs(self):
-        """Ensure that input tabs are escaped in the output, like gettext does."""
+    @mark.xfail(reason="Not Implemented")
+    def test_escape_tabs(self):
+        """Ensure that input tabs are escaped in the output, like
+        gettext does."""
 
         # The strings below contains the tab character, not spaces.
         templatepo = '''msgid "First	Second"\nmsgstr ""\n\n'''
         mergepo = '''msgid "First	Second"\nmsgstr "Eerste	Tweede"\n'''
-        expectedpo = r'''imsgid "First\tSecond"
+        expectedpo = r'''msgid "First\tSecond"
 msgstr "Eerste\tTweede"
 '''
         pofile = self.mergestore(templatepo, mergepo)
@@ -183,7 +305,8 @@ msgstr "Eerste\tTweede"
         assert str(pofile) == expectedpo
 
     def test_preserve_comments_layout(self):
-        """Ensure that when we merge with new '# (poconflict)' or other comments that we don't mess formating"""
+        """Ensure that when we merge with new '# (poconflict)' or other
+        comments that we don't mess formating"""
         templatepo = '''#: filename\nmsgid "Desktop Background.bmp"\nmsgstr "Desktop Background.bmp"\n\n'''
         mergepo = '''# (pofilter) unchanged: please translate\n#: filename\nmsgid "Desktop Background.bmp"\nmsgstr "Desktop Background.bmp"\n'''
         expectedpo = mergepo
@@ -192,9 +315,10 @@ msgstr "Eerste\tTweede"
         assert str(pofile) == expectedpo
 
     def test_merge_dos2unix(self):
-        """Test that merging a comment line with dos newlines doesn't add a new line"""
+        """Test that merging a comment line with dos newlines doesn't add a
+        new line"""
         templatepo = '''# User comment\n# (pofilter) Translate Toolkit comment\n#. Automatic comment\n#: location_comment.c:110\nmsgid "File"\nmsgstr "File"\n\n'''
-        mergepo =  '''# User comment\r\n# (pofilter) Translate Toolkit comment\r\n#. Automatic comment\r\n#: location_comment.c:110\r\nmsgid "File"\r\nmsgstr "Ifayile"\r\n\r\n'''
+        mergepo = '''# User comment\r\n# (pofilter) Translate Toolkit comment\r\n#. Automatic comment\r\n#: location_comment.c:110\r\nmsgid "File"\r\nmsgstr "Ifayile"\r\n\r\n'''
         expectedpo = '''# User comment\n# (pofilter) Translate Toolkit comment\n#. Automatic comment\n#: location_comment.c:110\nmsgid "File"\nmsgstr "Ifayile"\n'''
         pofile = self.mergestore(templatepo, mergepo)
         assert str(pofile) == expectedpo
@@ -208,7 +332,7 @@ msgstr "Eerste\tTweede"
 
         # New comment
         templatepo = '''#: location_comment.c:110\nmsgid "File"\nmsgstr "File"\n\n'''
-        mergepo =  '''# User comment\r\n# (pofilter) Translate Toolkit comment\r\n#: location_comment.c:110\r\nmsgid "File"\r\nmsgstr "Ifayile"\r\n\r\n'''
+        mergepo = '''# User comment\r\n# (pofilter) Translate Toolkit comment\r\n#: location_comment.c:110\r\nmsgid "File"\r\nmsgstr "Ifayile"\r\n\r\n'''
         expectedpo = '''# User comment\n# (pofilter) Translate Toolkit comment\n#: location_comment.c:110\nmsgid "File"\nmsgstr "Ifayile"\n'''
         pofile = self.mergestore(templatepo, mergepo)
         assert str(pofile) == expectedpo
@@ -239,7 +363,7 @@ msgstr "Eerste\tTweede"
         unit = xlifffile.units[0]
         assert unit.source == "red"
         assert unit.target == "rooi"
-        
+
     def test_xliff_into_po(self):
         templatepo = '# my comment\nmsgid "red"\nmsgstr ""'
         mergexliff = self.xliffskeleton % '''<trans-unit>
@@ -251,7 +375,8 @@ msgstr "Eerste\tTweede"
         assert str(pofile) == expectedpo
 
     def test_merging_dont_merge_kde_comments_found_in_translation(self):
-        """If we find a KDE comment in the translation (target) then do not merge it."""
+        """If we find a KDE comment in the translation (target) then do not
+        merge it."""
 
         templatepo = '''msgid "_: KDE comment\\n"\n"File"\nmsgstr "File"\n\n'''
         mergepo = '''msgid "_: KDE comment\\n"\n"File"\nmsgstr "_: KDE comment\\n"\n"Ifayile"\n\n'''
@@ -259,7 +384,7 @@ msgstr "Eerste\tTweede"
         pofile = self.mergestore(templatepo, mergepo)
         print "Expected:\n%s\n\nMerged:\n%s" % (expectedpo, str(pofile))
         assert str(pofile) == expectedpo
-        
+
         # Translated kde comment.
         mergepo = '''msgid "_: KDE comment\\n"\n"File"\nmsgstr "_: KDE kommentaar\\n"\n"Ifayile"\n\n'''
         print "Expected:\n%s\n\nMerged:\n%s" % (expectedpo, str(pofile))
@@ -274,7 +399,8 @@ msgstr "Eerste\tTweede"
         assert str(pofile) == expectedpo
 
     def test_merging_untranslated_with_kde_disambiguation(self):
-        """test merging untranslated messages that are the same except for KDE disambiguation"""
+        """test merging untranslated messages that are the same except for
+        KDE disambiguation"""
         templatepo = r'''#: sendMsgTitle
 #: sendMsgTitle.accesskey
 msgid "_: sendMsgTitle sendMsgTitle.accesskey\n"
@@ -305,7 +431,8 @@ msgstr "Stuur"
         assert str(pofile) == expectedpo
 
     def test_merging_header_entries(self):
-        """Check that we do the right thing if we have header entries in the input PO."""
+        """Check that we do the right thing if we have header entries in the
+        input PO."""
 
         templatepo = r'''#, fuzzy
 msgid ""
@@ -365,3 +492,60 @@ msgstr "Dimpled Ring"
         pofile = self.mergestore(templatepo, mergepo)
         print "Expected:\n%s\n---\nMerged:\n%s\n---" % (expectedpo, str(pofile))
         assert str(pofile) == expectedpo
+
+    def test_merging_different_locations(self):
+        """Test when merging units that are unchanged except for changed
+        locations that we don't go fuzzy (bug 1583)"""
+
+        templatepo = r'''#, fuzzy
+msgid ""
+msgstr ""
+"Content-Type: text/plain; charset=utf-8\n"
+"Content-Transfer-Encoding: 8bit\n"
+
+#: sentinelheadline
+msgctxt "sentinelheadline"
+msgid "DESTROY SENTINELS"
+msgstr "ZERSTÖRE WACHPOSTEN"
+
+#: sentinelheadline1
+msgctxt "sentinelheadline1"
+msgid "DESTROY SENTINELS"
+msgstr "ZERSTÖRE WACHPOSTEN"
+'''
+        mergepo = r'''msgid ""
+msgstr ""
+"Content-Type: text/plain; charset=utf-8\n"
+"Content-Transfer-Encoding: 8bit\n"
+
+#: sentinelheadline
+#: sentinelheadline1
+msgctxt "sentinelheadline"
+msgid "DESTROY SENTINELS"
+msgstr "ZERSTÖRE WACHPOSTEN"
+
+#: sentinelheadline1
+msgctxt "sentinelheadline1"
+msgid "DESTROY SENTINELS"
+msgstr "ZERSTÖRE WACHPOSTEN"
+'''
+        expectedpo2 = r'''msgid ""
+msgstr ""
+"Content-Type: text/plain; charset=utf-8\n"
+"Content-Transfer-Encoding: 8bit\n"
+
+#: sentinelheadline sentinelheadline1
+msgctxt "sentinelheadline"
+msgid "DESTROY SENTINELS"
+msgstr "ZERSTÖRE WACHPOSTEN"
+
+#: sentinelheadline1
+msgctxt "sentinelheadline1"
+msgid "DESTROY SENTINELS"
+msgstr "ZERSTÖRE WACHPOSTEN"
+'''
+
+        expectedpo = mergepo
+        pofile = self.mergestore(templatepo, mergepo)
+        print "Expected:\n%s\n---\nMerged:\n%s\n---" % (expectedpo, str(pofile))
+        assert str(pofile) == expectedpo or str(pofile) == expectedpo2
